package com.qunzhixinxi.hnqz.admin.api.constant;

import com.baomidou.mybatisplus.annotation.EnumValue;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * Upms 服务枚举
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2022-08-15 15:24
 */
public final class UpmsType {

	private UpmsType() {
	}

	/**
	 * 任务大类
	 */
	@Getter
	@AllArgsConstructor
	public enum TaskCategory {

		FORWARD("FORWARD", "转发类任务"),
		CONFERENCES_INTERVIEWS("CONFERENCES_INTERVIEWS", "会议与专访"),
		CUSTOMER_VISIT("CUSTOMER_VISIT", "客户拜访"),
		MARKET_INFO_COLLECTION("MARKET_INFO_COLLECTION", "市场信息搜集"),
		SCORE_LIMIT("SCORE_LIMIT", "积分限制");

		@EnumValue
		private final String type;

		private final String description;


	}


	/**
	 * 任务二级分类
	 */
	@Getter
	@AllArgsConstructor
	public enum TaskSubCategory {

		PRODUCT_ALBUM("PRODUCT_ALBUM", TaskCategory.FORWARD, "产品相册"),
		ACADEMIC_INFO_EXCHANGE("ACADEMIC_INFO_EXCHANGE", TaskCategory.FORWARD, "学术信息交流"),
		CONFERENCE("CONFERENCE", TaskCategory.CONFERENCES_INTERVIEWS, "会议举办"),
		ONE_ON_ONE_INTERVIEW("ONE_ON_ONE_INTERVIEW", TaskCategory.CONFERENCES_INTERVIEWS, "一对一专访"),
		TRAINING("TRAINING", TaskCategory.CONFERENCES_INTERVIEWS, "培训"),
		BUSINESS_COMPANY_VISIT("BUSINESS_COMPANY_VISIT", TaskCategory.CUSTOMER_VISIT, "商业公司拜访"),
		HOSPITAL_VISIT("HOSPITAL_VISIT", TaskCategory.CUSTOMER_VISIT, "医院拜访"),
		PHARMACY_VISIT("PHARMACY_VISIT", TaskCategory.CUSTOMER_VISIT, "药店拜访"),
		INVENTORY_INFO_COLLECTION("INVENTORY_INFO_COLLECTION", TaskCategory.MARKET_INFO_COLLECTION, "库存信息搜集"),
		HOSPITAL_INFO_COLLECTION("HOSPITAL_INFO_COLLECTION", TaskCategory.MARKET_INFO_COLLECTION, "医院信息搜集"),
		SCORE_LIMIT("SCORE_LIMIT", TaskCategory.SCORE_LIMIT, "积分限制");

		@EnumValue
		private final String type;

		private final TaskCategory category;

		private final String description;

	}

	/**
	 * 任务二级分类2
	 */
	@Getter
	@AllArgsConstructor
	public enum TaskSubCategory2 {

//		PRODUCT_ALBUM("PRODUCT_ALBUM", TaskCategory.FORWARD, "产品相册"),
		ACADEMIC_INFO_EXCHANGE("ACADEMIC_INFO_EXCHANGE", TaskCategory.FORWARD, "学术信息交流"),
		CONFERENCE("CONFERENCE", TaskCategory.CONFERENCES_INTERVIEWS, "会议举办"),
		ONE_ON_ONE_INTERVIEW("ONE_ON_ONE_INTERVIEW", TaskCategory.CONFERENCES_INTERVIEWS, "一对一专访"),
		TRAINING("TRAINING", TaskCategory.CONFERENCES_INTERVIEWS, "培训"),
//		BUSINESS_COMPANY_VISIT("BUSINESS_COMPANY_VISIT", TaskCategory.CUSTOMER_VISIT, "商业公司拜访"),
//		HOSPITAL_VISIT("HOSPITAL_VISIT", TaskCategory.CUSTOMER_VISIT, "医院拜访"),
//		PHARMACY_VISIT("PHARMACY_VISIT", TaskCategory.CUSTOMER_VISIT, "药店拜访"),
		INVENTORY_INFO_COLLECTION("INVENTORY_INFO_COLLECTION", TaskCategory.MARKET_INFO_COLLECTION, "库存信息搜集"),
		HOSPITAL_INFO_COLLECTION("HOSPITAL_INFO_COLLECTION", TaskCategory.MARKET_INFO_COLLECTION, "医院信息搜集"),
		CUSTOMER_VISIT("CUSTOMER_VISIT", TaskCategory.CUSTOMER_VISIT, "客户拜访"),
		PRODUCT_PROMOTION("PRODUCT_PROMOTION", TaskCategory.FORWARD, "产品推广"),
		MARKET_INFO_COLLECTION("MARKET_INFO_COLLECTION", TaskCategory.MARKET_INFO_COLLECTION, "市场信息搜集"),
		SPECIAL_SERVICE("SPECIAL_SERVICE", null, "专项服务"),
		BUSINESS_COMPANY_CUSTOMER_MANAGEMENT("BUSINESS_COMPANY_CUSTOMER_MANAGEMENT", null, "商业公司客户管理"),
		AGENT_CUSTOMER_MANAGEMENT("AGENT_CUSTOMER_MANAGEMENT", null, "代理商客户管理"),
		ENT_TASK("ENT_TASK", null, "企业任务"),
		TASK_TYPE_CONFIG("TASK_TYPE_CONFIG", null, "任务类型配置");



		@EnumValue
		private final String type;

		private final TaskCategory category;

		private final String description;

	}

	// 企业类型
	@Getter
	@AllArgsConstructor
	public enum EnterpriseType {

		CMO("CMO", "药企"),
		CSO("CSO", "CSO"),
		NET_MED("NET_MED", "互联网医疗"),
		MED_MED("MED_MED", "医学传媒"),
		MEETING("MEETING", "科室会");

		@EnumValue
		private final String type;
		private final String description;
	}

	// 产品线
	@Getter
	@AllArgsConstructor
	public enum ProductLine {

		CSO1("CSO1", "CSO1"),
		CSO2("CSO2", "CSO2"),
		HCP1("HCP1", "HCP1"),
		HCP2("HCP2", "HCP2"),
		HCP3("HCP3", "HCP3"),
		HCP4("HCP4", "HCP4");

		@EnumValue
		private final String type;
		private final String description;
	}

	/**
	 * 终止积分包类型
	 *
	 * @author lixuesong
	 * @date 2023/08/28
	 */
	@Getter
	@AllArgsConstructor
	public enum TerminatePackageType {
		ALL("ALL", "1", "全部终止"),
		REMAIN("REMAIN", "2", "剩余回收");

		@EnumValue
		private final String type;
		private final String enableType;
		private final String description;

		/**
		 * 根据enableType返回枚举
		 *
		 * @param enableType 终止类型
		 * @return {@link TerminatePackageType}
		 */
		public static TerminatePackageType getByEnableType(String enableType) {
			if ("1".equals(enableType)) {
				return ALL;
			} else if ("2".equals(enableType)) {
				return REMAIN;
			}
			return null;
		}
	}
}
