package com.qunzhixinxi.hnqz.admin.api.dto;

import com.qunzhixinxi.hnqz.admin.api.entity.WmQuestion;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuiz;
import com.qunzhixinxi.hnqz.common.core.constant.enums.UpmsState;
import com.qunzhixinxi.hnqz.common.core.constant.enums.UpmsType;
import lombok.AccessLevel;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

/**
 * 试卷dto
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-16 16:07
 */
@NoArgsConstructor(access = AccessLevel.NONE)
public final class WmQuizDTO {

	@Data
	public static class OnQuizCreate {

		/**
		 * 标题
		 */
		@NotBlank(message = "标题必填")
		@Length(max = 64, message = "标题最大支持字符数{max}")
		private String title;

		/**
		 * 简介
		 */
		private String introduction;

		/**
		 * 问卷来源类型
		 */
		@NotNull(message = "问卷类型必填")
		private UpmsType.QuizSourceType sourceType;

	}

	@Data
	public static class OnQuizUpdate {

		@NotNull(message = "试卷编号必填")
		private String sn;

		@Valid
		private WmQuiz.Expand expand;

		/**
		 * 标题
		 */
		@Length(max = 64, message = "标题最大支持字符数{max}")
		private String title;

		/**
		 * 简介
		 */
		private String introduction;

	}

	@Data
	public static class OnQuizState {

		@NotNull(message = "试卷ID必填")
		private Integer quizId;

		@NotNull(message = "操作状态必填")
		private UpmsState.QuizState state;
	}


	@Data
	public static class OnItemCreate {

		@NotBlank(message = "标题必填")
		private String label;

		@NotBlank(message = "试卷编号")
		private String quizSerialNum;

		@NotEmpty(message = "选项不能为空")
		private WmQuestion.Option[] options;

		@NotBlank(message = "答案必填")
		private String answer;

		@NotNull(message = "排序权重必填")
		private Long widget;

		@NotNull(message = "分数必填")
		@Min(value = 0, message = "分数最小为{value}")
		private Integer mark;
	}

	@Data
	public static class OnItemUpdate {

		@NotNull(message = "试卷项ID必填")
		private Integer itemId;

		private String label;

		private WmQuestion.Option[] options;

		private String answer;

		private Long widget;

		@Min(value = 0, message = "分数最小为{value}")
		private Integer mark;
	}

	@Data
	public static class OnItemDelete {

		@NotNull(message = "试卷项ID必填")
		private Integer itemId;
	}


	@Data
	public static class OnQuestionCreate {
		@NotBlank(message = "试题标题必填")
		@Length(max = 64, message = "试卷标题最多{max}个字符")
		private String label;

		@Valid
		@NotEmpty(message = "试题选项必填")
		private WmQuestion.Option[] options;

		@NotBlank(message = "试题答案必填")
		@Length(max = 255, message = "试卷标题最多{max}个字符")
		private String answer;

	}

	@Data
	public static class OnQuestionUpdate {

		@NotNull(message = "试题ID必填")
		private Integer questionId;

		private String label;

		private WmQuestion.Option[] options;

		private String answer;

	}

	@Data
	public static class OnQuestionDel {
		@NotNull(message = "试题ID必填")
		private Integer questionId;
	}

}
