package com.qunzhixinxi.hnqz.admin.api.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.common.core.constant.enums.CommonFlag;
import lombok.Data;
import lombok.ToString;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * 任务提交百分比规则
 *
 * @author lixuesong
 * @version 1.0.0
 * @date 2022-12-22 14:43
 */
@Data
@ToString
@TableName(value = "wm_task_sub_percent_rule", autoResultMap = true)
public class WmTaskSubmissionPercentRule extends Model<WmTaskSubmissionPercentRule> {

	private static final long serialVersionUID = -2370239048359430285L;

	/**
	 * 提交规则id
	 */
	@TableId(value = "rule_id", type = IdType.AUTO)
	private Integer ruleId;

	/**
	 * 企业id
	 */
	@NotNull(message = "deptId不能为空")
	private Integer deptId;

	/**
	 * 批量保存编号
	 */
	private String batchId;

	/**
	 * 二级分类
	 */
	@NotNull(message = "subCategory不能为空")
	@TableField(value = "sub_cate")
	private UpmsType.TaskSubCategory2 subCategory;

	/**
	 * 任务规则
	 */
	@Valid
	@NotNull(message = "rule不能为空")
	@TableField(value = "sub_rule", typeHandler = FastjsonTypeHandler.class)
	private PercentRule rule;

	/**
	 * 可操作标记
	 */
	private CommonFlag.OptFlag optFlag;

	/**
	 * 创建时间
	 */
	private LocalDateTime createTime;

	/**
	 * 创建时间
	 */
	private LocalDateTime updateTime;


	/**
	 * 提交规则实体
	 */
	@Data
	@ToString
	public static class PercentRule implements Serializable {

		private static final long serialVersionUID = -6254889506697833544L;

		/**
		 * 任务类型列表
		 */
		@NotEmpty(message = "taskTypeIds不能为空")
		private String[] taskTypeIds;

		/**
		 * 积分包值：积分包值大于这个值的包，才限制这个规则
		 */
		@NotNull(message = "startScore不能为空")
		private Integer startScore;

		/**
		 * 百分比：达到要求的积分包，上述所选的任务总完成积分值不得大于积分包值*百分比
		 */
		@NotBlank(message = "limitPercent不能为空")
		private String limitPercent;
	}


}
