package com.qunzhixinxi.hnqz.admin.api.vo;

import cn.hutool.core.util.StrUtil;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 个人累计积分记录
 *
 * @author snows
 * @date 2023/3/27 10:37
 */
@Data
public class PersonalScoreRecordVO implements Serializable {
	private static final long serialVersionUID = -6628168954572943648L;

	/**
	 * 积分包id
	 */
	private String scorePackageId;

	/**
	 * 积分包名称
	 */
	private String scorePackageName;

	/**
	 * 任务类型id
	 */
	private String taskTypeId;

	/**
	 * 任务类型名称
	 */
	private String taskTypeName;

	/**
	 * 任务完成时间
	 */
	private LocalDateTime lookintoDate;

	/**
	 * 任务积分值
	 */
	private Integer score;

	/**
	 * 积分记录状态枚举
	 */
	@Getter
	@AllArgsConstructor
	public enum PersonalScoreRecordStatus {
		PAID("PAID", "6", "1", "已结算积分"),
		UNPAID("UNPAID", "4,5,7", "0,2,3,4,5,9", "待结算积分");

		/**
		 * 记录统计状态
		 */
		private final String status;

		/**
		 * 对应积分包状态
		 */
		private final String scorePackageStatus;

		/**
		 * 结算状态
		 */
		private final String settleStatus;

		/**
		 * 描述
		 */
		private final String description;

		/**
		 * 根据积分包状态获取枚举
		 *
		 * @param scorePackageStatus 积分包状态
		 * @return 枚举对象
		 */
		public static PersonalScoreRecordStatus getByScorePackageStatus(String scorePackageStatus) {
			for (PersonalScoreRecordStatus scoreRecordStatus : PersonalScoreRecordStatus.values()) {
				if (StrUtil.split(scoreRecordStatus.scorePackageStatus, StrUtil.COMMA).contains(scorePackageStatus)) {
					return scoreRecordStatus;
				}
			}
			return null;
		}
	}

}
