package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.dto.OladingCommonRequest;
import com.qunzhixinxi.hnqz.admin.entity.HuiqiyunInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.LingcaiaiInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.OladingInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.QifuInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.entity.XinbadaInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.ZhongyiyunInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.ZhongzhiInputRecord;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.mapper.QifuInputRecordMapper;
import com.qunzhixinxi.hnqz.admin.service.HuiqiyunInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.LingcaiaiInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptSubService;
import com.qunzhixinxi.hnqz.admin.service.XinbadaInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.ZhongyiyunInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.ZhongzhiInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.gig.GigThirdApiService;
import com.qunzhixinxi.hnqz.admin.service.gig.OladingInputRecordService;
import com.qunzhixinxi.hnqz.admin.util.SaasApiDemoV2;
import com.qunzhixinxi.hnqz.admin.util.ZhongyiyunUtils;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * 第三方零工接口
 *
 * @author lixuesong
 * @date 2022年04月17日 11:31
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/gig-third-api")
public class GigThirdApiController {

	private final GigThirdApiService gigThirdApiService;

	private final OladingInputRecordService oladingInputRecordService;

	private final XinbadaInputRecordService xinbadaInputRecordService;

	private final LingcaiaiInputRecordService lingcaiaiInputRecordService;

	private final HuiqiyunInputRecordService huiqiyunInputRecordService;

	private final ZhongyiyunInputRecordService zhongyiyunInputRecordService;

	private final ZhongzhiInputRecordService zhongzhiInputRecordService;

	private final QifuInputRecordMapper qifuInputRecordMapper;

	private final SysDeptSubService sysDeptSubService;

	private final RedisTemplate redisTemplate;


	/**
	 * 福建企赋结算异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("福建企赋结算异步通知回调")
	@PostMapping("/qifu/pay-notify")
	public String qiFuSettleNotifyRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("福建企赋结算请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);

		// 请求序号
		String reqId = bodyJson.getStr("reqId");

		String redisKey = String.format("%s_%s", CacheConstants.QI_FU_SETTLE_NOTIFY_KEY, reqId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqId, 30, TimeUnit.MINUTES);
		if (Boolean.FALSE.equals(absent)) {
			log.warn("存在在途操作reqId={}", reqId);
			return "fail";
		}

		String decryptDataStr = null;
		com.alibaba.fastjson.JSONObject unPackMessage = null;
		try {
			// 查询deptSub获取desKey
			SysDeptSub deptSub = sysDeptSubService.getOne(Wrappers.<SysDeptSub>lambdaQuery()
					.eq(SysDeptSub::getEnableFlag, "1")
					.eq(SysDeptSub::getSubjectLocation, SubjectLocation.QI_FU)
					.last("limit 1"));
			if (deptSub == null || StrUtil.isBlank(deptSub.getAppId())) {
				log.warn("未配置福建企赋deptSub reqId={}", reqId);
				return "fail";
			}

			String publicKey = deptSub.getRsaPublicKey();

			// 验签/解密
			unPackMessage =
					SaasApiDemoV2.unPackMessage(JSON.parseObject(body), com.alibaba.fastjson.JSONObject.class, publicKey);
			decryptDataStr = com.alibaba.fastjson.JSONObject.toJSONString(unPackMessage);
		} catch (Exception e) {
			log.error("福建企赋结算回调参数异常", e);
			return "fail";
		} finally {
			// 请求记录到数据库
			QifuInputRecord inputRecord = new QifuInputRecord();
			inputRecord.setReqId(reqId);
			inputRecord.setReqType(QifuInputRecord.ReqType.SETTLE_NOTIFY.getType());
			inputRecord.setReqBody(body);
			inputRecord.setDecryptBody(decryptDataStr);
			qifuInputRecordMapper.insert(inputRecord);
		}

		// 业务处理
		gigThirdApiService.qiFuSettleNotify(unPackMessage);

		return "success";
	}

	/**
	 * 福建企赋签约异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("福建企赋签约异步通知回调")
	@PostMapping("/qifu/sign-notify")
	public String qiFuSignNotifyRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("福建企赋签约请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);

		// 请求序号
		String reqId = bodyJson.getStr("reqId");
		String data = bodyJson.getStr("data");

		String redisKey = String.format("%s_%s", CacheConstants.QI_FU_SIGN_NOTIFY_KEY, reqId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqId, 30, TimeUnit.MINUTES);
		if (Boolean.FALSE.equals(absent)) {
			log.warn("存在在途操作reqId={}", reqId);
			return "fail";
		}

		String decryptDataStr = null;
		com.alibaba.fastjson.JSONObject unPackMessage = null;
		try {
			// 查询deptSub获取desKey
			SysDeptSub deptSub = sysDeptSubService.getOne(Wrappers.<SysDeptSub>lambdaQuery()
					.eq(SysDeptSub::getEnableFlag, "1")
					.eq(SysDeptSub::getSubjectLocation, SubjectLocation.QI_FU)
					.last("limit 1"));
			if (deptSub == null || StrUtil.isBlank(deptSub.getAppId())) {
				log.warn("未配置福建企赋deptSub reqId={}", reqId);
				return "fail";
			}

			String publicKey = deptSub.getRsaPublicKey();

			// 验签/解密
			unPackMessage =
					SaasApiDemoV2.unPackMessage(JSON.parseObject(body), com.alibaba.fastjson.JSONObject.class, publicKey);
			decryptDataStr = com.alibaba.fastjson.JSONObject.toJSONString(unPackMessage);
		} catch (Exception e) {
			log.error("福建企赋签约回调参数异常", e);
			return "fail";
		} finally {
			// 请求记录到数据库
			QifuInputRecord inputRecord = new QifuInputRecord();
			inputRecord.setReqId(reqId);
			inputRecord.setReqType(QifuInputRecord.ReqType.SIGN_NOTIFY.getType());
			inputRecord.setReqBody(body);
			inputRecord.setDecryptBody(decryptDataStr);
			qifuInputRecordMapper.insert(inputRecord);
		}

		// 业务处理
		gigThirdApiService.qiFuSignNotify(unPackMessage);

		return "success";
	}

	/**
	 * 中智回调异步通知记录
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("中智结算异步通知回调")
	@PostMapping("/zhongzhi/pay-notify")
	public Map<String, Object> zhongZhiSettleNotify(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("中智结算请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);

		// 三方订单号
		String thirdOrderNo = bodyJson.getStr("thirdOrderNo");

		Map<String, Object> result = new HashMap<>();

		String redisKey = String.format("%s_%s", CacheConstants.ZHONG_ZHI_SETTLE_NOTIFY_KEY, thirdOrderNo);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, thirdOrderNo, 30, TimeUnit.MINUTES);
		if (Boolean.FALSE.equals(absent)) {
			log.warn("存在在途操作orderNo={}", thirdOrderNo);
			result.put("code", "00000400");
			result.put("msg", "存在在途操作");
			return result;
		}

		// 请求记录到数据库
		ZhongzhiInputRecord inputRecord = new ZhongzhiInputRecord();
		inputRecord.setThirdOrderNo(thirdOrderNo);
		inputRecord.setReqType(ZhongzhiInputRecord.ReqType.SETTLE_NOTIFY.getType());
		inputRecord.setReqBody(body);
		zhongzhiInputRecordService.save(inputRecord);

		// 业务处理
		gigThirdApiService.zhongzhiSettleNotify(bodyJson);

		result.put("code", "00000200");
		result.put("msg", "成功");

		return result;
	}

	/**
	 * 众蚁云结算异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("众蚁云结算异步通知回调")
	@PostMapping("/zhongyiyun/pay-notify")
	public String zhongYiYunSettleNotifyRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("众蚁云结算请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);

		// 请求序号
		String reqId = bodyJson.getStr("reqId");

		String redisKey = String.format("%s_%s", CacheConstants.ZHONG_YI_YUN_SETTLE_NOTIFY_KEY, reqId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqId, 30, TimeUnit.MINUTES);
		if (Boolean.FALSE.equals(absent)) {
			log.warn("存在在途操作reqId={}", reqId);
			return "fail";
		}

		// 请求记录到数据库
		ZhongyiyunInputRecord inputRecord = new ZhongyiyunInputRecord();
		inputRecord.setReqId(reqId);
		inputRecord.setReqType(ZhongyiyunInputRecord.ReqType.SETTLE_NOTIFY.getType());
		inputRecord.setReqBody(body);
		inputRecord.setDecryptBody(body);
		zhongyiyunInputRecordService.save(inputRecord);

		// 业务处理
		gigThirdApiService.zhongYiYunSettleNotify(bodyJson);

		return "success";
	}

	/**
	 * 众蚁云签约异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("众蚁云签约异步通知回调")
	@PostMapping("/zhongyiyun/sign-notify")
	public String zhongYiYunSignNotifyRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("众蚁云签约请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);

		// 请求序号
		String reqId = bodyJson.getStr("reqId");

		String redisKey = String.format("%s_%s", CacheConstants.ZHONG_YI_YUN_SIGN_NOTIFY_KEY, reqId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqId, 30, TimeUnit.MINUTES);
		if (Boolean.FALSE.equals(absent)) {
			log.warn("存在在途操作reqId={}", reqId);
			return "fail";
		}

		JSONObject newBodyJson = null;
		try {
			String merId = bodyJson.getStr("merId");
			String bankCard = bodyJson.getStr("bankCard");
			String idCard = bodyJson.getStr("idCard");
			String mobile = bodyJson.getStr("mobile");
			String name = bodyJson.getStr("name");
			String remark3 = bodyJson.getStr("remark3");

			// 查询deptSub获取desKey
			SysDeptSub deptSub = sysDeptSubService.getOne(Wrappers.<SysDeptSub>lambdaQuery()
					.likeRight(SysDeptSub::getAppId, merId)
					.eq(SysDeptSub::getEnableFlag, "1")
					.eq(SysDeptSub::getSubjectLocation, SubjectLocation.ZHONG_YI_YUN));
			if (deptSub == null || StrUtil.isBlank(deptSub.getAppId())) {
				log.warn("未配置众蚁云deptSub merId={}", merId);
				return "fail";
			}

			String desKey = StrUtil.split(deptSub.getAppId(), "|").get(2);

			newBodyJson = BeanUtil.copyProperties(bodyJson, JSONObject.class);
			newBodyJson.set("bankCard", ZhongyiyunUtils.decrypt(bankCard, desKey));
			newBodyJson.set("idCard", ZhongyiyunUtils.decrypt(idCard, desKey));
			newBodyJson.set("mobile", ZhongyiyunUtils.decrypt(mobile, desKey));
			newBodyJson.set("name", ZhongyiyunUtils.decrypt(name, desKey));
			newBodyJson.set("remark3", ZhongyiyunUtils.decrypt(remark3, desKey));
		} catch (Exception e) {
			log.error("众蚁云签约回调参数异常", e);
			return "fail";
		} finally {
			// 请求记录到数据库
			ZhongyiyunInputRecord inputRecord = new ZhongyiyunInputRecord();
			inputRecord.setReqId(reqId);
			inputRecord.setReqType(ZhongyiyunInputRecord.ReqType.SIGN_NOTIFY.getType());
			inputRecord.setReqBody(body);
			inputRecord.setDecryptBody(JSONUtil.toJsonStr(newBodyJson));
			zhongyiyunInputRecordService.save(inputRecord);
		}

		// 业务处理
		gigThirdApiService.zhongYiYunSignNotify(newBodyJson);

		return "success";
	}

	/**
	 * 汇企云异步通知回调
	 *
	 * @param body
	 * @param headers
	 * @return
	 */
	@Inner(value = false)
	@SysLog("汇企云异步通知回调")
	@PostMapping("/huiqiyun")
	public String huiQiYunRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {
		log.info("汇企云请求方法参数:body={}, headers={}", body, headers);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);


		String reqMsgId = bodyJson.getStr("reqMsgId");
		String redisKey = String.format("%s_%s", CacheConstants.HUI_QI_YUN_NOTIFY_KEY, reqMsgId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqMsgId, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.warn("存在在途操作reqMsgId={}", reqMsgId);
			return "FAIL";
		}

		// 请求记录到数据库
		HuiqiyunInputRecord record = new HuiqiyunInputRecord();
		record.setReqMsgId(reqMsgId);
		record.setReqBody(body);
		huiqiyunInputRecordService.save(record);

		gigThirdApiService.huiqiyunNotify(reqMsgId, bodyJson, record.getId());

		return "SUCCESS";
	}

	/**
	 * 浪潮(灵才接好活)结算回调
	 *
	 * @param request
	 * @return
	 */
	@Inner(value = false)
	@SysLog("浪潮异步通知回调")
	@PostMapping("/lingcaiai")
	public String lingcaiaiNotify(HttpServletRequest request) {
		Map<String, String[]> parameterMap = request.getParameterMap();
		log.info("浪潮结算回调请求方法参数:parameterMap={}", JSONUtil.toJsonStr(parameterMap));

		// 签名
		String[] verifies = parameterMap.get("verify");
		String verify = ArrayUtil.isEmpty(verifies) ? "" : verifies[0];
		// 通知类型
		String[] channels = parameterMap.get("channel");
		String channel = ArrayUtil.isEmpty(channels) ? "" : channels[0];
		// 项目单号
		String[] batchNos = parameterMap.get("batch_no");
		String batchNo = ArrayUtil.isEmpty(batchNos) ? "" : batchNos[0];
		// 订单状态
		String[] statuses = parameterMap.get("status");
		String orderStatus = ArrayUtil.isEmpty(statuses) ? "" : statuses[0];
		// project_accounts
		String[] projectAccountsArr = parameterMap.get("project_accounts");
		// 账号单号
//		if (StrUtil.isBlank(projectAccounts)) {
//			log.warn("project_accounts为空");
//			return "fail";
//		}

		String redisKey = String.format("%s_%s_%s", CacheConstants.ZHEGN_QI_ZHI_XING_NOTIFY_KEY, channel, batchNo);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, batchNo, 30, TimeUnit.MINUTES);
		if (!absent) {
			return "success";
		}

		// 请求存入数据库
		LingcaiaiInputRecord record = new LingcaiaiInputRecord();
		record.setVerify(verify);
		record.setChannel(channel);
		record.setReqBody(JSONUtil.toJsonStr(parameterMap));
		record.setBatchNo(batchNo);
		record.setOrderStatus(NumberUtil.isNumber(orderStatus) ? Integer.valueOf(orderStatus) : -1);
		record.setCreateTime(LocalDateTime.now());
		lingcaiaiInputRecordService.save(record);

		return gigThirdApiService.lingcaiaiNotify(channel, batchNo, Integer.valueOf(orderStatus));
	}

	/**
	 * 正启之星异步通知回调
	 *
	 * 接口用于接收批量订单创建结果通知和下发结果通知。
	 * 请注意回调请求头默认的”Content-Type”: “application/octet-stream”
	 * 企业需要给我方响应。通知超时时间10秒，如果没有收到正确响应视为通知失败，失败后重试3次，每次间隔10分钟。
	 *
	 * @param body 请求body
	 * @param headers 请求头
	 */
	@Inner(value = false)
	@SysLog("正启之星异步通知回调")
	@PostMapping("/xinbada")
	public Map<String, Object> xinbadaRequest(@RequestBody String body, @RequestHeader HttpHeaders headers) {

		log.info("正启之星请求方法参数:body={}, headers={}", body, headers);

		// 	签名
		String signature = Objects.requireNonNull(headers.get("signature")).get(0);
		// 通知类型（order_check_result_notify：订单校验结果通知，order_call_back_notification：订单下发回调通知，task_redispatch_notification：单笔重试回调通知）
		String actionName = Objects.requireNonNull(headers.get("action_name")).get(0);

		// 请求body json
		JSONObject bodyJson = JSONUtil.parseObj(body);
		Integer orderStatus = bodyJson.getInt("order_status");
		String orderUuid = bodyJson.getStr("order_uuid");
		String orderNo = bodyJson.getStr("order_no");
		String customerUserUuid = bodyJson.getStr("customer_user_uuid");
		String serverUserUuid = bodyJson.getStr("server_user_uuid");

		Map<String, Object> successResult = new HashMap<>();
		if (!StrUtil.isAllNotBlank(signature, actionName, orderNo) || orderStatus == null) {
			log.warn("必传参数为空");
			successResult.put("resp_code", 400);
			successResult.put("resp_message", "必传参数为空");
			successResult.put("success", "false");
			return successResult;
		}

		String redisKey = String.format("%s_%s_%s", CacheConstants.ZHEGN_QI_ZHI_XING_NOTIFY_KEY, actionName, orderNo);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, orderNo, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.warn("存在在途操作orderNo={}", orderNo);
			successResult.put("resp_code", 400);
			successResult.put("resp_message", "存在在途操作");
			successResult.put("success", "false");
			return successResult;
		}

		// 请求记录到数据库
		XinbadaInputRecord xinbadaInputRecord = new XinbadaInputRecord();
		xinbadaInputRecord.setActionName(actionName);
		xinbadaInputRecord.setSignature(signature);
		xinbadaInputRecord.setReqBody(body);
		xinbadaInputRecord.setCreateTime(LocalDateTime.now());
		// body中的部分字段
		xinbadaInputRecord.setOrderStatus(orderStatus);
		xinbadaInputRecord.setOrderUuid(orderUuid);
		xinbadaInputRecord.setOrderNo(orderNo);
		xinbadaInputRecord.setCustomerUserUuid(customerUserUuid);
		xinbadaInputRecord.setServerUserUuid(serverUserUuid);
		xinbadaInputRecordService.save(xinbadaInputRecord);

		gigThirdApiService.xinbadaNotify(actionName, orderUuid, orderStatus);

		successResult.put("resp_code", 200);
		successResult.put("resp_message", "SUCCESS");
		successResult.put("success", "true");

		return successResult;
	}

	/**
	 * 自由职家统一请求地址
	 *
	 * @return
	 */
	@Inner(value = false)
	@SysLog("自由职家统一请求地址")
	@PostMapping("/olading")
	public void oladingRequest(@RequestParam(value = "req", required = false) String reqStr,
							   @RequestParam(required = false) String sign, HttpServletResponse response) {

		log.info("自由职家请求方法参数:req={}, sign={}", reqStr, sign);

		// 请求数据
		OladingCommonRequest.Req req = JSONUtil.toBean(reqStr, OladingCommonRequest.Req.class);
		// 请求接口名称
		String interfaceName = req.getName();
		// 请求data
		Map<String, Object> data = req.getData();
		// 请求requestId
		String requestId = (String) data.get("requestId");
		if (StrUtil.isBlank(requestId)) {
			requestId = (String) data.get("request_id");
		}
		String redisKey = String.format("%s_%s_%s", CacheConstants.OLADING_NOTIFY_KEY, interfaceName, requestId);
		Boolean absent = redisTemplate.opsForValue().setIfAbsent(redisKey, reqStr, 30, TimeUnit.MINUTES);
		if (!absent) {
			log.error("存在在途操作requestId={}", requestId);
			try {
				response.getWriter().write("SUCCESS");
				response.getWriter().flush();
			} catch (Exception e) {
				log.error("response返回异常", e);
			}
			return;
		}

		// 1.请求记录
		OladingInputRecord record = new OladingInputRecord();
		record.setRequestId(requestId);
		record.setInterfaceName(interfaceName);
		record.setReqBody(String.format("%s,%s", reqStr, sign));
		record.setPrivateKey("");
		record.setPublicKey("");
		oladingInputRecordService.save(record);

		try {
			// 2. 验签 TODO 暂不验签
//			boolean verfyResult =
//					hMACSignProvide.verfy(ALGORITHM_TYPE, PUBLIC_KEY, JSONUtil.toJsonStr(req.getData()), sign);
//			log.info("验签结果: {}", verfyResult);

			// 3.实际操作

			if (StrUtil.isNotBlank(requestId)) {
				try {
					// 睡3000ms，解决阿拉钉回调过快，找不到requestId的问题
					Thread.sleep(3000);
				} catch (Exception e) {
					log.warn("sleep异常", e);
				}

				gigThirdApiService.oladingNotify(requestId, req);
			}
		} catch (Exception e) {
			log.error("自由职家统一请求异常", e);
		} finally {
			redisTemplate.delete(redisKey);
			try {
				response.getWriter().write("SUCCESS");
				response.getWriter().flush();
			} catch (Exception e) {
				log.error("response返回异常", e);
			}
		}

	}

}
