/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserRole;
import com.qunzhixinxi.hnqz.admin.service.MobileService;
import com.qunzhixinxi.hnqz.admin.service.SysUserRoleService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author hnqz
 * @date 2018/11/14
 * <p>
 * 手机验证码
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/mobile")
@Api(value = "mobile", tags = "手机管理模块")
public class MobileController {

	private static final Integer FINA_ROLE_CODE = 7;

	private final MobileService mobileService;
	private final SysUserService userService;
	private final SysUserRoleService userRoleService;

	@Inner(value = false)
	@GetMapping("/{mobile}")
	public R sendSmsCode(@PathVariable String mobile) {
		return mobileService.sendSmsCode(mobile);
	}


	@Inner(value = false)
	@GetMapping("/pc/{mobile}")
	public R sendSmsCodePc(@PathVariable String mobile) {
		return mobileService.sendSmsCodePc(mobile);
	}

	/**
	 * 财务管理员发送验证码
	 *
	 * @return 发送结果
	 */
	@Inner(value = false)
	@GetMapping("/for-fina")
	public R<?> sendSmsCodeForFina(@RequestParam(value = "userId") Integer userId) {

		List<SysUserRole> userRoles = userRoleService.list(Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getUserId, userId));

		Set<Integer> roles = userRoles.stream().mapToInt(SysUserRole::getRoleId).boxed().collect(Collectors.toSet());
		log.info("roles:{}", roles);
		if (!roles.contains(FINA_ROLE_CODE)) {
			return R.failed("请联系管理员开通财务管理员权限");
		}

		// 获取用户信息判断，是否需要发送验证码
		SysUser sysUser = userService.getById(userId);
		log.info("是否需要发送验证码temp3：{}", sysUser.getTemp3());
		boolean need = StrUtil.isBlank(sysUser.getTemp3());
		Boolean smsSucc = mobileService.sendSmsCodeForFina(sysUser.getUsername(), need);
		return R.ok(smsSucc, smsSucc ? "成功" : "失败");

	}

}
