/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.csp.sentinel.util.StringUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.constant.MqConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReportOpt;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.input.WmPackageExportApiInput;
import com.qunzhixinxi.hnqz.admin.enums.ChannelEnum;
import com.qunzhixinxi.hnqz.admin.enums.ReportEnum;
import com.qunzhixinxi.hnqz.admin.service.*;
import com.qunzhixinxi.hnqz.common.core.entity.BaseMap;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.rabbitmq.client.RabbitMqClient;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * <p>
 * 报告审核导出
 * </p>
 *
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/reportExport")
@Api(value = "reportExport", tags = "导出管理模块")
public class ReportExportController {

	private final WmTaskService wmTaskService;

	private final WmScorePackageService wmScorePackageService;

	private final WmReportService wmReportService;

	private final ReportExportService reportExportService;

	private final SysDeptService sysDeptService;

	private final JavaMailSender sender;

	private final UpmsConfig upmsConfig;
	private final RedisTemplate<String,String> redisTemplate;
	private final RabbitMqClient rabbitMqClient;



	/**
	 * 报告导出--个人报告导出分页查询
	 *
	 * @param page           分页对象
	 * @param wmScorePackage 积分包
	 * @return
	 */
	@ApiOperation(value = "分页查询", notes = "分页查询")
	@GetMapping("/getExportPackageZbTotalList")
	public R getExportPackageZbTotalList(Page page, WmScorePackage wmScorePackage) {
		return reportExportService.getExportPackageZbTotalList(page,wmScorePackage);
	}

	/**
	 * 报告导出--查看任务
	 *
	 * @param page           分页对象
	 * @param wmTask
	 * @return
	 */
	@ApiOperation(value = "分页查询", notes = "分页查询")
	@GetMapping("/getExportPackageZbTaskList")
	public R getExportPackageZbTaskList(Page page, WmTask wmTask) {
		if(StringUtil.isEmpty(wmTask.getScorePackageId())){
			return R.failed("积分包ID不能为空");
		}
		return reportExportService.getExportPackageZbTaskList(page,wmTask);
	}




	// /**
	//  * 结算报告管理--导出汇总或详情
	//  * @param wmReport
	//  * @return
	//  * @throws IOException
	//  */
	// @GetMapping("/exportNbReport")
	// public R exportNbReport(WmReport wmReport ) throws IOException {
	// 	wmReport=wmReportService.getById(wmReport.getId());
	// 	List<WmTask> taskList=new ArrayList<>();//该报告关联的所有未提交的任务
	// 	//更新任务表审批意见
	// 	WmScorePackage scorePackage=wmScorePackageService.getById(wmReport.getScoreId());
	// 	WmTask wmTask=new WmTask();
	// 	wmTask.setDelFlag("0");
	// 	wmTask.setEnableFlag("0");
	// 	wmTask.setRealFlag("0");
	//
	// 	SysDept sysDept = sysDeptService.getById(scorePackage.getSendPackageDeptId());
	//
	//
	// 	if("0".equals(scorePackage.getTypeid())){//药企
	// 		wmTask.setReportDrugId(wmReport.getId());
	// 		wmTask.setReportDrugApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());//审核通过的状态
	// 		taskList=wmTaskService.list(Wrappers.query(wmTask));
	// 	}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
	// 		wmTask.setReportOneId(wmReport.getId());
	// 		if(sysDept.getLevel() == 2){
	// 			wmTask.setReportDrugApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());//审核通过的状态
	// 		}else {
	// 			wmTask.setReportOneApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());//审核通过的状态
	// 		}
	//
	// 		taskList=wmTaskService.list(Wrappers.query(wmTask));
	// 	}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
	// 		wmTask.setReportSecondId(wmReport.getId());
	// 		wmTask.setReportSecondApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());//审核通过的状态
	// 		taskList=wmTaskService.list(Wrappers.query(wmTask));
	// 	}
	// 	if(null==taskList||taskList.size()<=0){
	// 		return R.failed("未查询到该报告关联的审批通过的任务");
	// 	}
	// 	String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));
	// 	wmTask.setScorePackageId(wmReport.getScoreId().toString());
	// 	wmTask.setId(ids);
	//
	// 		return reportExportService.exportNb(wmTask,wmReport);//报告汇总
	//
	//
	// }

	/**
	 * 结算报告管理--导出汇总或详情
	 * @param wmReport
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportNbReviewReport")
	public R exportNbReviewReport(WmReport wmReport) {


		String key = String.format(CacheConstants.EXPORT_NB_REVIEW_REPORT_CACHE, wmReport.getId());

		String o = redisTemplate.opsForValue().get(key);

		if (StringUtils.isNotEmpty(o) && WmReportOpt.WmReportOptStatus.GENERATING.name().equals(o)){

			return R.failed("已有在途生成记录，能耐心等待");

		}


		HnqzUser user = SecurityUtils.getUser();
		BaseMap map = new BaseMap();
		map.put("requestBody", JSONUtil.toJsonStr(wmReport));
		map.put("username",user.getUsername());
		map.put("deptId",user.getDeptId());
		map.put("approvalOpinion", null);

		rabbitMqClient.sendMessage(MqConstants.DIRECT_MODE_REPORT_EXPORT_NB_REVIEW_QUEUE,map);

		redisTemplate.opsForValue().set(key, WmReportOpt.WmReportOptStatus.GENERATING.name(),
				CacheConstants.DEF_REPORT_CREATING_TTL, TimeUnit.MILLISECONDS);

		return R.ok("开始导出，请耐心等待");


	}

	/**
	 * 结算报告管理--导出汇总或详情
	 *
	 * @param wmReport
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportNbReviewReport/status")
	public R exportNbReviewReportStatus(WmReport wmReport) {

		String key = String.format(CacheConstants.EXPORT_NB_REVIEW_REPORT_CACHE, wmReport.getId());

		String o = redisTemplate.opsForValue().get(key);
		WmReportOpt opt = new WmReportOpt();

		if (StringUtils.isNotEmpty(o)) {
			// 生成中的
			if ("GENERATING".equals(o)) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
			}
			// 生成失败的
			else if (o.startsWith("ERROR")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o.split(StrUtil.UNDERLINE)[1]);
			}
			// 生成失败的2
			else if (!o.startsWith("http")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o);
			}
			// 成功的
			else {
				Long expire = redisTemplate.opsForValue().getOperations().getExpire(key, TimeUnit.SECONDS);
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
				opt.setTtl(LocalDateTime.now().plusSeconds(expire));
				opt.setLatestUrl(o);
			}

		} else {
			opt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
		}

		return R.ok(opt);

	}

	/**
	 * 结算报告管理--导出汇总或详情
	 * @param wmReport
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportNbReport")
	public R exportNbReport(WmReport wmReport ) throws IOException {


		String key = String.format(CacheConstants.EXPORT_NB_REPORT_CACHE, wmReport.getId());

		String o = redisTemplate.opsForValue().get(key);

		if (StringUtils.isNotEmpty(o) && WmReportOpt.WmReportOptStatus.GENERATING.name().equals(o)){

			return R.failed("已有在途生成记录，能耐心等待");

		}


		HnqzUser user = SecurityUtils.getUser();
		BaseMap map = new BaseMap();
		map.put("requestBody", JSONUtil.toJsonStr(wmReport));
		map.put("username",user.getUsername());
		map.put("deptId",user.getDeptId());

		rabbitMqClient.sendMessage(MqConstants.DIRECT_MODE_REPORT_EXPORT_NB_QUEUE,map);

		redisTemplate.opsForValue().set(key, WmReportOpt.WmReportOptStatus.GENERATING.name(),
				CacheConstants.DEF_REPORT_CREATING_TTL, TimeUnit.MILLISECONDS);

		return R.ok("开始导出，请耐心等待");


	}



	// /**
	//  * 个人报告导出--导出汇总或详情  汇总和详情合为一个模版
	//  * @param wmTask
	//  * @return
	//  * @throws IOException
	//  */
	// @GetMapping("/exportZbReport")
	// public R exportZbReport(WmTask wmTask) throws IOException {
	// 	String packageId=wmTask.getScorePackageDrugId();
	// 	if (null == packageId) {
	// 		return R.failed("积分包ID不能为空");
	// 	}
	// 	LambdaQueryWrapper<WmTask> queryWrapper=new LambdaQueryWrapper();
	// 	queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(",")));
	// 	queryWrapper.eq(WmTask::getTaskStatus,"3");
	// 	queryWrapper.eq(WmTask::getDelFlag,"0");
	// 	queryWrapper.eq(WmTask::getEnableFlag,"0");
	// 	queryWrapper.eq(WmTask::getRealFlag,"0");
	// 	List<WmTask> taskList=wmTaskService.list(queryWrapper);
	// 	if(taskList.isEmpty()){
	// 		return R.failed("未查询到审核通过的任务");
	// 	}
	// 	String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));
	// 	wmTask.setScorePackageId(packageId);
	// 	wmTask.setId(ids);
	//
	// 	return reportExportService.exportZb(wmTask);//报告汇总
	//
	//
	// }

	/**
	 * 个人报告导出状态
	 *
	 * @param wmTask
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbReport/status")
	public R exportZbReportStatus(WmTask wmTask) throws IOException {

		String scorePackageDrugId = wmTask.getScorePackageDrugId();

		// 判断是否是合并导出
		if (scorePackageDrugId.contains(StrUtil.COMMA)) {
			String[] idArr = scorePackageDrugId.split(StrUtil.COMMA);
			scorePackageDrugId = Arrays.stream(idArr).sorted().collect(Collectors.joining(StrUtil.COMMA));
			wmTask.setScorePackageDrugId(scorePackageDrugId);

		}

		String key = String.format(CacheConstants.EXPORT_ZB_REPORT_CACHE, wmTask.getScorePackageDrugId());

		String o = redisTemplate.opsForValue().get(key);
		WmReportOpt opt = new WmReportOpt();

		if (StringUtils.isNotEmpty(o)) {
			// 生成中的
			if ("GENERATING".equals(o)) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
			}
			// 生成失败的
			else if (o.startsWith("ERROR")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o.split(StrUtil.UNDERLINE)[1]);
			}
			// 生成失败的2
			else if (!o.startsWith("http")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o);
			}
			// 成功的
			else {
				Long expire = redisTemplate.opsForValue().getOperations().getExpire(key, TimeUnit.SECONDS);
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
				opt.setTtl(LocalDateTime.now().plusSeconds(expire));
				opt.setLatestUrl(o);
			}

		} else {
			opt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
		}

		return R.ok(opt);

	}

	/**
	 * 个人报告导出状态
	 *
	 * @param wmTask
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbReviewReport/status")
	public R exportZbReviewReportStatus(WmTask wmTask) {

		String scorePackageDrugId = wmTask.getScorePackageDrugId();

		// 判断是否是合并导出
		if (scorePackageDrugId.contains(StrUtil.COMMA)) {
			String[] idArr = scorePackageDrugId.split(StrUtil.COMMA);
			scorePackageDrugId = Arrays.stream(idArr).sorted().collect(Collectors.joining(StrUtil.COMMA));
			wmTask.setScorePackageDrugId(scorePackageDrugId);

		}

		String key = String.format(CacheConstants.EXPORT_ZB_REVIEW_REPORT_CACHE, wmTask.getScorePackageDrugId());

		String o = redisTemplate.opsForValue().get(key);
		WmReportOpt opt = new WmReportOpt();

		if (StringUtils.isNotEmpty(o)) {
			// 生成中的
			if ("GENERATING".equals(o)) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
			}
			// 生成失败的
			else if (o.startsWith("ERROR")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o.split(StrUtil.UNDERLINE)[1]);
			}
			// 生成失败的2
			else if (!o.startsWith("http")) {
				opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
				opt.setErrorMsg(o);
			}
			// 成功的
			else {
				Long expire = redisTemplate.opsForValue().getOperations().getExpire(key, TimeUnit.SECONDS);
				opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
				opt.setTtl(LocalDateTime.now().plusSeconds(expire));
				opt.setLatestUrl(o);
			}

		} else {
			opt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
		}

		return R.ok(opt);

	}

	/**
	 * 个人报告导出--导出汇总或详情  汇总和详情合为一个模版（只导出待审核的任务）
	 * @param wmTask
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbReviewReport")
	public R<?> exportZbReviewReport(WmTask wmTask) {

		String scorePackageDrugId = wmTask.getScorePackageDrugId();

		// 判断是否是合并导出
		if (scorePackageDrugId.contains(StrUtil.COMMA)){
			String[] idArr = scorePackageDrugId.split(StrUtil.COMMA);
			scorePackageDrugId = Arrays.stream(idArr).sorted().collect(Collectors.joining(StrUtil.COMMA));
			wmTask.setScorePackageDrugId(scorePackageDrugId);

		}

		String key = String.format(CacheConstants.EXPORT_ZB_REVIEW_REPORT_CACHE, wmTask.getScorePackageDrugId());

		String o = redisTemplate.opsForValue().get(key);

		if (StringUtils.isNotEmpty(o) && WmReportOpt.WmReportOptStatus.GENERATING.name().equals(o)){

			return R.failed("已有在途生成记录，能耐心等待");

		}


		HnqzUser user = SecurityUtils.getUser();
		BaseMap map = new BaseMap();
		map.put("requestBody", JSONUtil.toJsonStr(wmTask));
		map.put("username",user.getUsername());
		map.put("deptId",user.getDeptId());
		map.put("taskStatus", "2");

		SysDept dept = sysDeptService.getById(user.getDeptId());
		map.put("isGuangshengtang", "913500007297027606".equals(dept.getTaxCode()));

		rabbitMqClient.sendMessage(MqConstants.DIRECT_MODE_REPORT_EXPORT_ZB_REVIEW_QUEUE, map);

		redisTemplate.opsForValue().set(key, WmReportOpt.WmReportOptStatus.GENERATING.name(),
				CacheConstants.DEF_REPORT_CREATING_TTL, TimeUnit.MILLISECONDS);

		return R.ok("开始导出，请耐心等待");

	}

	/**
	 * 个人报告导出--导出汇总或详情  汇总和详情合为一个模版
	 * @param wmTask
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbReport")
	public R exportZbReport(WmTask wmTask) throws IOException {

		String scorePackageDrugId = wmTask.getScorePackageDrugId();

		// 判断是否是合并导出
		if (scorePackageDrugId.contains(StrUtil.COMMA)){
			String[] idArr = scorePackageDrugId.split(StrUtil.COMMA);
			scorePackageDrugId = Arrays.stream(idArr).sorted().collect(Collectors.joining(StrUtil.COMMA));
			wmTask.setScorePackageDrugId(scorePackageDrugId);

		}

		String key = String.format(CacheConstants.EXPORT_ZB_REPORT_CACHE, wmTask.getScorePackageDrugId());

		String o = redisTemplate.opsForValue().get(key);

		if (StringUtils.isNotEmpty(o) && WmReportOpt.WmReportOptStatus.GENERATING.name().equals(o)){

			return R.failed("已有在途生成记录，能耐心等待");

		}


		HnqzUser user = SecurityUtils.getUser();
		BaseMap map = new BaseMap();
		map.put("requestBody", JSONUtil.toJsonStr(wmTask));
		map.put("username",user.getUsername());
		map.put("deptId",user.getDeptId());

		SysDept dept = sysDeptService.getById(user.getDeptId());
		map.put("isGuangshengtang", "913500007297027606".equals(dept.getTaxCode()));

		rabbitMqClient.sendMessage(MqConstants.DIRECT_MODE_REPORT_EXPORT_ZB_QUEUE,map);

		redisTemplate.opsForValue().set(key, WmReportOpt.WmReportOptStatus.GENERATING.name(),
				CacheConstants.DEF_REPORT_CREATING_TTL, TimeUnit.MILLISECONDS);

		return R.ok("开始导出，请耐心等待");

	}

	/**
	 * 个人报告导出--导出汇总或详情  汇总和详情合为一个模版
	 * @param packageId 积分包id
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportHCPReport")
	public R exportHCPReport(String packageId) throws IOException {
		if (null == packageId) {
			return R.failed("积分包ID不能为空");
		}

		return reportExportService.exportHCP(packageId);
	}

	/**
	 * 个人报告导出--导出汇总或详情  汇总和详情合为一个模版
	 * @param input
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbInfoFormWeiNew")
	public R exportZbInfoFormWeiNew(WmPackageExportApiInput input) {
		String packageId=input.getId();
		if (null == packageId) {
			return R.failed("积分包ID不能为空");
		}
		LambdaQueryWrapper<WmTask> queryWrapper=new LambdaQueryWrapper();
		queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(",")));
		queryWrapper.eq(WmTask::getTaskStatus,"3");
		queryWrapper.eq(WmTask::getDelFlag,"0");
		queryWrapper.eq(WmTask::getEnableFlag,"0");
		queryWrapper.eq(WmTask::getRealFlag,"0");
		List<WmTask> taskList=wmTaskService.list(queryWrapper);
		if(taskList.isEmpty()){
			return R.failed("未查询到审核通过的任务");
		}
		String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));

		WmTask wmTask = new WmTask();
		wmTask.setScorePackageId(packageId);
		wmTask.setId(ids);

//		return reportExportService.exportZb(wmTask);//报告汇总

		R r = R.ok();
		try {
			r = reportExportService.exportZb(wmTask);//报告汇总

			if (r.getCode() == 0) {
				log.info("文件生成成功");
			} else {
				return r;
			}
			Map<String, String> resultMap = (Map<String, String>) r.getData();

			MimeMessage mail = sender.createMimeMessage();
			try {

				MimeMessageHelper helper = new MimeMessageHelper(mail, true);

				helper.setFrom(upmsConfig.getFormMail());
				helper.setTo(input.getEmail());

				helper.setSubject("要易云平台学术信息沟通专员业绩报告");

				helper.setText("要易云平台学术信息沟通专员业绩报告","尊敬的学术信息沟通专员：" +
				"<br/>&nbsp;&nbsp;&nbsp;&nbsp;感谢您使用要易平台，下载业绩报告请<a href=" + resultMap.get("url") + ">点击链接</a >");
				/**
				 * 附件
				 */
//				helper.addAttachment("众包代表业绩报告明细", multipartFile);

				sender.send(mail);
			} catch (MessagingException ex) {
				log.error("邮件发送失败");
			}

		} catch (Exception e) {
			e.printStackTrace();
			return R.failed();
		}
		return r;


	}





	// /**
	//  * 个人报告导出--导出汇总或详情
	//  * @param wmTask
	//  *@param exportType  1 导出汇总  2导出详情
	//  * @return
	//  * @throws IOException
	//  */
//	@GetMapping("/exportZbReport")
//	public R exportZbReport(WmTask wmTask, String exportType) throws IOException {
//		String packageId=wmTask.getScorePackageDrugId();
//		if (null == packageId) {
//			return R.failed("积分包ID不能为空");
//		}
//		LambdaQueryWrapper<WmTask> queryWrapper=new LambdaQueryWrapper();
//		queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(",")));
//		queryWrapper.eq(WmTask::getTaskStatus,"3");
//		queryWrapper.eq(WmTask::getDelFlag,"0");
//		queryWrapper.eq(WmTask::getEnableFlag,"0");
//		List<WmTask> taskList=wmTaskService.list(queryWrapper);
//		if(taskList.isEmpty()){
//			return R.failed("未查询到审核通过的任务");
//		}
//		String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));
//		wmTask.setScorePackageId(packageId);
//		wmTask.setId(ids);
//
//		if("1".equals(exportType)){
//			return reportExportService.exportZb(wmTask);//报告汇总
//		}else  if("2".equals(exportType)){
//			return reportExportService.exportZbInfo(wmTask);
//		}else {
//			return R.failed("参数类型不对");
//		}
//
//	}



	/**
	 * 结算包-报告导出--导出汇总或详情
	 * @param wmTask
	 *@param exportType  1 导出汇总  2导出详情
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportZbInfoJs")
	public R exportZbInfoJs(WmTask wmTask, String exportType) throws IOException {
		String packageId=wmTask.getScorePackageDrugId();
		if (null == packageId) {
			return R.failed("积分包ID不能为空");
		}
		LambdaQueryWrapper<WmTask> queryWrapper=new LambdaQueryWrapper();
		queryWrapper.in(WmTask::getScorePackageId, Arrays.asList(packageId.split(",")));
		queryWrapper.eq(WmTask::getTaskStatus,"3");
		queryWrapper.eq(WmTask::getDelFlag,"0");
		queryWrapper.eq(WmTask::getEnableFlag,"0");
		List<WmTask> taskList=wmTaskService.list(queryWrapper);
		if(taskList.isEmpty()){
			return R.failed("未查询到审核通过的任务");
		}
		String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));
		wmTask.setScorePackageId(packageId);
		wmTask.setId(ids);

		//if("1".equals(exportType)){
			return reportExportService.exportZbJs(wmTask);//报告汇总
//		}else  if("2".equals(exportType)){
//			return reportExportService.exportZbInfoJs(wmTask);
//		}else {
//			return R.failed("参数类型不对");
//		}

	}


	// /**
	//  * 历史数据导出报告
	//  * @param wmScorePackage
	//  * @return
	//  * @throws IOException
	//  */
	// @GetMapping("/exportNbaHistory")
	// public R exportNbaHistory(WmScorePackage wmScorePackage) throws IOException {
	// 	wmScorePackage=wmScorePackageService.getById(wmScorePackage.getId());
	// 	if(wmScorePackage==null){
	// 		return R.failed("未查询到积分包信息");
	// 	}
	// 	LambdaQueryWrapper<WmTask> queryWrapper=new LambdaQueryWrapper();
	// 	queryWrapper.eq(WmTask::getScorePackageDrugId, wmScorePackage.getId());
	// 	queryWrapper.eq(WmTask::getTaskStatus,"3");
	// 	queryWrapper.eq(WmTask::getDelFlag,"0");
	// 	queryWrapper.eq(WmTask::getEnableFlag,"0");
	// 	queryWrapper.eq(WmTask::getRealFlag,"0");
	// 	List<WmTask> taskList=wmTaskService.list(queryWrapper);
	// 	//List<WmTask> taskList = wmTaskMapper.selectListByWmTask(wmScorePackage);
	// 	//更新任务表审批意见
	// 	WmTask wmTask=new WmTask();
	// 	wmTask.setDelFlag("0");
	// 	wmTask.setEnableFlag("0");
	//
	// 	if(null==taskList||taskList.size()<=0){
	// 		return R.failed("未查询到该报告关联的审批通过的任务");
	// 	}
	// 	String ids = taskList.stream().map(WmTask::getId).collect(Collectors.joining("," ));
	// 	wmTask.setScorePackageId(wmScorePackage.getId());
	// 	//wmTask.setId(ids);
	// 	wmTask.setId("(select id from wm_task where del_flag=0 and real_flag=0 and enable_flag=0 and task_status=3  and score_package_drug_id="+wmScorePackage.getId()+")");
	//
	// 	return reportExportService.exportNba(wmTask,wmScorePackage);//报告汇总
	//
	//
	// }


	/**
	 * 历史数据导出报告
	 * @param wmScorePackage
	 * @return
	 * @throws IOException
	 */
	@GetMapping("/exportNbaHistory")
	public R exportNbaHistory(WmScorePackage wmScorePackage) throws IOException {


		String key = String.format(CacheConstants.EXPORT_NBA_HISTORY_REPORT_CACHE,wmScorePackage.getId());

		String o = redisTemplate.opsForValue().get(key);

		if (StringUtils.isNotEmpty(o) && WmReportOpt.WmReportOptStatus.GENERATING.name().equals(o)){

			return R.failed("已有在途生成记录，能耐心等待");

		}


		HnqzUser user = SecurityUtils.getUser();
		BaseMap map = new BaseMap();
		map.put("requestBody",JSONUtil.toJsonStr(wmScorePackage));
		map.put("username",user.getUsername());
		map.put("deptId",user.getDeptId());

		rabbitMqClient.sendMessage(MqConstants.DIRECT_MODE_REPORT_EXPORT_NBA_HISTORY_QUEUE,map);

		redisTemplate.opsForValue().set(key, WmReportOpt.WmReportOptStatus.GENERATING.name(),
				CacheConstants.DEF_REPORT_CREATING_TTL,TimeUnit.MILLISECONDS);

		return R.ok("开始导出，请耐心等待");

	}


}
