package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.entity.WmDaAgent;
import com.qunzhixinxi.hnqz.admin.entity.WmDaDrugEnt;
import com.qunzhixinxi.hnqz.admin.entity.dto.SettleConfigDTO;
import com.qunzhixinxi.hnqz.admin.entity.input.DeptSubLimitAmountInput;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.service.SysDeptRelationService;
import com.qunzhixinxi.hnqz.admin.service.WmDaAgentService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.service.SysDeptSubService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * 机构支付渠道管理
 *
 * @author ryz
 * @date 2021-04-19 16:02:30
 */
@RestController
@AllArgsConstructor
@RequestMapping("/sysdeptsub" )
@Api(value = "sysdeptsub", tags = "机构支付渠道管理")
public class SysDeptSubController {

    private final  SysDeptSubService sysDeptSubService;
	private final SysDeptRelationService sysDeptRelationService;
	private final WmDaAgentService wmDaAgentService;
	private final WmDaDrugEntService wmDaDrugEntService;

	/**
	 * 查询结算渠道（用于结算页面）
	 *
	 * @return 结果
	 */
	@GetMapping("/get-settle-subject")
	public R getSettleSubejct() {
		Integer deptId = SecurityUtils.getUser().getDeptId();
		List<SysDeptSub> resultList = new ArrayList<>();

		// 原有结算渠道
		SysDeptSub sysDeptSub = new SysDeptSub();
		sysDeptSub.setDeptId(deptId);
		sysDeptSub.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		List<SysDeptSub> subList = sysDeptSubService.list(Wrappers.query(sysDeptSub));
		if (CollUtil.isNotEmpty(subList)) {
			subList.forEach(sub -> sub.setSubjectName(sub.getSubjectLocation().getDescription()));
			resultList.addAll(subList);
		}

		// MAH结算渠道-拼接上级药企作为渠道名称
		WmDaAgent daAgent = wmDaAgentService.getOne(Wrappers.<WmDaAgent>lambdaQuery()
				.eq(WmDaAgent::getDeptId, deptId));
		if(daAgent!= null) {
            if (daAgent.getMahSettleDeptId() != null) {
				// 查询上级药企
				List<SysDeptRelation> deptRelations = sysDeptRelationService.list(Wrappers.<SysDeptRelation>lambdaQuery()
						.eq(SysDeptRelation::getDescendant, deptId)
						.ne(SysDeptRelation::getAncestor, deptId)
						.ne(SysDeptRelation::getAncestor, 1));
				if (CollUtil.isNotEmpty(deptRelations)) {
					List<WmDaDrugEnt> drugEnts = wmDaDrugEntService.list(Wrappers.<WmDaDrugEnt>lambdaQuery()
							.in(WmDaDrugEnt::getDeptId, deptRelations.stream().map(SysDeptRelation::getAncestor).collect(Collectors.toSet())));
					drugEnts.forEach(drugEnt -> {
						SysDeptSub newDeptSub = new SysDeptSub();
						newDeptSub.setSubId(Integer.valueOf(drugEnt.getDeptId()));
						newDeptSub.setDeptId(Integer.valueOf(drugEnt.getDeptId()));
						newDeptSub.setSubjectLocation(SubjectLocation.MAH_SETTLE);
						newDeptSub.setSubjectType(String.valueOf(SubjectLocation.MAH_SETTLE.getGigType().getCode()));
						newDeptSub.setSubjectName(drugEnt.getEntname());
						resultList.add(newDeptSub);
					});
                }

			}
        }

		return R.ok(resultList);
	}

	/**
	 * 查询结算渠道列表-无参
	 * @return
	 */
	@ApiOperation(value = "查询结算渠道列表-无参", notes = "查询结算渠道列表-无参")
	@GetMapping("/getSubSelect" )
	public R getSubSelect() {
		SysDeptSub sysDeptSub = new SysDeptSub();
		sysDeptSub.setDeptId(SecurityUtils.getUser().getDeptId());
		sysDeptSub.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		List<SysDeptSub> subList = sysDeptSubService.list(Wrappers.query(sysDeptSub));
		return R.ok(subList);
	}

	/**
	 * 查询结算渠道列表-带参
	 * @param sysDeptSub
	 * @return
	 */
	@ApiOperation(value = "查询结算渠道列表-带参", notes = "查询结算渠道列表-带参")
	@GetMapping("/getSubList" )
	public R getSubList(SysDeptSub sysDeptSub) {
		sysDeptSub.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		return R.ok(sysDeptSubService.list(Wrappers.query(sysDeptSub)));
	}

	/**
	 * 修改限额
	 * @param input
	 * @return
	 */
	@ApiOperation(value = "修改限额", notes = "修改限额")
	@PostMapping("/updateSubLimitAmount" )
	public R updateSubLimitAmount(@RequestBody DeptSubLimitAmountInput input) {
		if(CollectionUtils.isEmpty(input.getSubList())){
			return R.failed("参数异常");
		}
		return sysDeptSubService.updateSubLimitAmount(input);
	}

	/**
	 * 结算管理配置
	 *
	 * @param settleConfigDTO
	 * @return
	 */
	@SysLog("结算管理配置-保存")
	@PostMapping("/save-settle-config")
	public R<?> saveSettleConfig(@Valid @RequestBody SettleConfigDTO settleConfigDTO) {
		for (SettleConfigDTO.DeptSub deptSub : settleConfigDTO.getSubList()) {
			if (deptSub.getLimitAmount() == null || ArrayUtil.isEmpty(deptSub.getSubjectInvoiceCategory())
					|| deptSub.getSubjectLocation() == null) {
				return R.failed("必填参数为空");
			}
		}
		DeptSubLimitAmountInput input = new DeptSubLimitAmountInput();
		input.setDeptId(settleConfigDTO.getDeptId());
		List<SysDeptSub> subList = new ArrayList<>();
		settleConfigDTO.getSubList().forEach(deptSub -> {
			SysDeptSub sysDeptSub =
					BeanUtil.copyProperties(deptSub, SysDeptSub.class, "subjectInvoiceCategory");
			sysDeptSub.setSubjectInvoiceCategory(String.join(StrUtil.COMMA, deptSub.getSubjectInvoiceCategory()));
			subList.add(sysDeptSub);
		});
		input.setSubList(subList);
		sysDeptSubService.saveSettleConfig(input);

		return R.ok();
	}

    /**
     * 分页查询
     * @param page 分页对象
     * @param sysDeptSub 部门管理
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_view')" )
    public R getSysDeptSubPage(Page page, SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.page(page, Wrappers.query(sysDeptSub)));
    }


    /**
     * 通过id查询部门管理
     * @param deptId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{deptId}" )
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_view')" )
    public R getById(@PathVariable("deptId" ) Integer deptId) {
        return R.ok(sysDeptSubService.getById(deptId));
    }

    /**
     * 新增部门管理
     * @param sysDeptSub 部门管理
     * @return R
     */
    @ApiOperation(value = "新增部门管理", notes = "新增部门管理")
    @SysLog("新增部门管理" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_add')" )
    public R save(@RequestBody SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.save(sysDeptSub));
    }

    /**
     * 修改部门管理
     * @param sysDeptSub 部门管理
     * @return R
     */
    @ApiOperation(value = "修改部门管理", notes = "修改部门管理")
    @SysLog("修改部门管理" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_edit')" )
    public R updateById(@RequestBody SysDeptSub sysDeptSub) {
        return R.ok(sysDeptSubService.updateById(sysDeptSub));
    }

    /**
     * 通过id删除部门管理
     * @param deptId id
     * @return R
     */
    @ApiOperation(value = "通过id删除部门管理", notes = "通过id删除部门管理")
    @SysLog("通过id删除部门管理" )
    @DeleteMapping("/{deptId}" )
    @PreAuthorize("@pms.hasPermission('admin_sysdeptsub_del')" )
    public R removeById(@PathVariable Integer deptId) {
        return R.ok(sysDeptSubService.removeById(deptId));
    }

}
