/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.entity.WmCro;
import com.qunzhixinxi.hnqz.admin.service.WmCroService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * CRO管理
 *
 * @author Dhy
 * @date 2020-12-17 14:37:09
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmcro" )
@Api(value = "wmcro", tags = "CRO管理")
public class WmCroController {

    private final WmCroService wmCroService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmCro CRO
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    public R getWmCroPage(Page page, WmCro wmCro) {
        return R.ok(wmCroService.selectCroList(page, wmCro));
    }


    /**
     * 通过id查询CRO
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmCroService.getById(id));
    }

    /**
     * 新增CRO
     * @param wmCro CRO
     * @return R
     */
    @ApiOperation(value = "新增CRO", notes = "新增CRO")
    @SysLog("新增CRO管理" )
    @PostMapping
    public R save(@RequestBody WmCro wmCro) {
		wmCro.setUpdateTime(LocalDateTime.now());
		return R.ok(wmCroService.save(wmCro));
    }

    /**
     * 修改CRO
     * @param wmCro CRO
     * @return R
     */
    @ApiOperation(value = "修改CRO", notes = "修改CRO")
    @SysLog("修改CRO管理" )
    @PutMapping
    public R updateById(@RequestBody WmCro wmCro) {
    	wmCro.setUpdateTime(LocalDateTime.now());
        return R.ok(wmCroService.updateById(wmCro));
    }

    /**
     * 通过id删除CRO
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除CRO", notes = "通过id删除CRO")
    @SysLog("通过id删除CRO" )
    @DeleteMapping("/{id}" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmCroService.removeById(id));
    }

}
