package com.qunzhixinxi.hnqz.admin.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.entity.dto.DeptTaskDTO;
import com.qunzhixinxi.hnqz.admin.entity.vo.WmTaskVO;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.enums.ReportEnum;
import com.qunzhixinxi.hnqz.admin.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDeptTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 企业任务控制器
 *
 * @author lixuesong
 * @date 2023/09/21
 */
@Slf4j
@RestController
@AllArgsConstructor
public class WmDeptTaskController {

    private final WmScorePackageService wmScorePackageService;
    private final WmDeptTaskService wmDeptTaskService;
    private final WmTaskService wmTaskService;
    private final WmTaskTypeService wmTaskTypeService;
    private final SysUserService sysUserService;
    private final RedisTemplate redisTemplate;

    /**
     * 查询关联的企业积分包
     *
     * @return {@link R}<{@link ?}> 结果
     */
    @GetMapping("/dept-task/list-related-ent-package")
    public R<?> listRelatedEntPackage() {
        Integer deptId = SecurityUtils.getUser().getDeptId();

        List<WmScorePackage> packageList = wmScorePackageService.getScorePackagekfpjf(null, deptId, new String[]{"0", "1", "2"}, null);

        List<Map<String, String>> results = new ArrayList<>();
        if (CollUtil.isNotEmpty(packageList)) {
            Set<String> packageIdSet = packageList.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
            // 校验是否超过积分包企业任务上限
            List<WmTask> queryTaskList = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
                    .in(WmTask::getScorePackageId, packageIdSet)
                    .eq(WmTask::getEnableFlag, EnableEnum.ENABLE.val())
                    .eq(WmTask::getDelFlag, DelEnum.NOT_DEL.val())
                    .eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val()));
            Map<String, Integer> packageTasksScoreMap = new HashMap<>();
            if (CollUtil.isNotEmpty(queryTaskList)) {
                packageTasksScoreMap = queryTaskList.stream()
                        .collect(Collectors.groupingBy(WmTask::getScorePackageId,
                                Collectors.summingInt(WmTask::getScore)));
            }

            Map<String, Integer> finalPackageTasksScoreMap = packageTasksScoreMap;
            packageList.stream()
                    .filter(wmScorePackage -> {
                        Integer tasksScore = finalPackageTasksScoreMap.getOrDefault(wmScorePackage.getId(), 0);
                        // 是否超过企业任务上限
                        boolean deptLimitFlag = BigDecimal.valueOf(tasksScore)
                                .compareTo(BigDecimal.valueOf(wmScorePackage.getScore())
                                        .multiply(wmScorePackage.getEntTaskLimit()).divide(new BigDecimal("100"))) < 0;
                        // 同时满足以下条件:
                        // 1.可分配积分值大于0的
                        // 2.没有超过企业任务上限
                        if (wmScorePackage.getKfpjf() > 0 && deptLimitFlag
                                && wmScorePackage.getEntTaskLimit().compareTo(BigDecimal.ZERO) > 0) {
                            return true;
                        }
                        return false;
                    }).forEach(wmScorePackage -> {
                        Map<String, String> result = new HashMap<>();
                        result.put("id", wmScorePackage.getId());
                        result.put("scorePackageName", wmScorePackage.getScorePackageName());
                        result.put("score", String.valueOf(wmScorePackage.getScore()));
                        result.put("kfpjf", String.valueOf(wmScorePackage.getKfpjf()));
                        results.add(result);
                    });
        }

        return R.ok(results);
    }

    /**
     * 分页查询企业任务
     *
     * @param page             分页参数
     * @param scorePackageName 积分包名称
     * @return {@link R}<{@link ?}> 结果
     */
    @GetMapping("/dept-task/page-ent-task")
    public R<?> pageEntTask(Page<WmTaskVO> page, String scorePackageName) {

        Integer deptId = SecurityUtils.getUser().getDeptId();

        // 查询当前企业所有的企业大包(包括上级发给自己的,和自己发给自己的)
        LambdaQueryWrapper<WmScorePackage> queryWrapper = Wrappers.<WmScorePackage>lambdaQuery()
                .eq(WmScorePackage::getDeptId, deptId)
                .in(WmScorePackage::getTypeid, "0", "1", "2")
                .eq(WmScorePackage::getDelFlag, DelEnum.NOT_DEL.val())
                .eq(WmScorePackage::getEnableFlag, EnableEnum.ENABLE.val())
                .eq(WmScorePackage::getOldPackage, "0");
        if (StrUtil.isNotBlank(scorePackageName)) {
            queryWrapper.like(WmScorePackage::getScorePackageName, scorePackageName);
        }
        List<WmScorePackage> relatedPackages = wmScorePackageService.list(queryWrapper);
        if (CollUtil.isEmpty(relatedPackages)) {
            return R.ok(page);
        }
        // 积分包id列表
        Set<String> packageIdSet = relatedPackages.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
        // 积分包名称map
        Map<String, String> packageNameMap = relatedPackages.stream()
                .collect(Collectors.toMap(WmScorePackage::getId, WmScorePackage::getScorePackageName));

        // 查询积分包下的企业任务
        Page<WmTask> taskPage = wmTaskService.page(new Page<>(page.getCurrent(), page.getSize()),
                Wrappers.<WmTask>lambdaQuery()
                        .in(WmTask::getScorePackageId, packageIdSet)
                        .orderByDesc(WmTask::getLookintoDate));

        if (CollUtil.isEmpty(taskPage.getRecords())) {
            return R.ok(page);
        }

        // 查询taskType
        List<WmTaskType> taskTypeList = wmTaskTypeService.list(Wrappers.<WmTaskType>lambdaQuery()
                .eq(WmTaskType::getTaskTypeLevel, "1")
                .eq(WmTaskType::getDelFlag, "0")
                .eq(WmTaskType::getEnableFlag, "0"));
        // taskType通过id分组（key为parentId，value为taskTypeName）
        Map<String, String> taskTypeIdMap = taskTypeList.stream()
                .collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));

        // 查询涉及到的user
        Set<String> userIdSet = taskPage.getRecords().stream().map(WmTask::getTaskUserId).collect(Collectors.toSet());
        List<SysUser> users = sysUserService.listByIds(userIdSet);
        Map<Integer, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

        List<WmTaskVO> vos = taskPage.getRecords().stream().map(wmTask -> {
            WmTaskVO wmTaskVO = BeanUtil.copyProperties(wmTask, WmTaskVO.class);
            // 积分包名称
            wmTaskVO.setScorePackageName(packageNameMap.getOrDefault(wmTask.getScorePackageId(), ""));
            // 任务类型名称
            wmTaskVO.setTaskTypeName(taskTypeIdMap.getOrDefault(wmTask.getTaskTypeId(), ""));
            // 姓名
            wmTaskVO.setTaskUsername(userMap.getOrDefault(Integer.parseInt(wmTask.getTaskUserId()), ""));
            // 报告审核状态
            wmTaskVO.setApprovalOpinionName("");
            if (StrUtil.isNotBlank(wmTask.getReportDrugApprovalOpinion())) {
                if (ReportEnum.APPROVAL_OPINION_YES.getType().equals(wmTask.getReportDrugApprovalOpinion())) {
                    wmTaskVO.setApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());
                    wmTaskVO.setApprovalOpinionName(ReportEnum.APPROVAL_OPINION_YES.getDescription());
                } else if (ReportEnum.APPROVAL_OPINION_NO.getType().equals(wmTask.getReportDrugApprovalOpinion())) {
                    wmTaskVO.setApprovalOpinion(ReportEnum.APPROVAL_OPINION_NO.getType());
                    wmTaskVO.setApprovalOpinionName(ReportEnum.APPROVAL_OPINION_NO.getDescription());
                }

            } else if (StrUtil.isNotBlank(wmTask.getReportOneApprovalOpinion())) {
                if (ReportEnum.APPROVAL_OPINION_YES.getType().equals(wmTask.getReportOneApprovalOpinion())) {
                    wmTaskVO.setApprovalOpinion(ReportEnum.APPROVAL_OPINION_YES.getType());
                    wmTaskVO.setApprovalOpinionName(ReportEnum.APPROVAL_OPINION_YES.getDescription());
                } else if (ReportEnum.APPROVAL_OPINION_NO.getType().equals(wmTask.getReportOneApprovalOpinion())) {
                    wmTaskVO.setApprovalOpinion(ReportEnum.APPROVAL_OPINION_NO.getType());
                    wmTaskVO.setApprovalOpinionName(ReportEnum.APPROVAL_OPINION_NO.getDescription());
                }
            } else {
                wmTaskVO.setUpdateTime(null);
            }

            return wmTaskVO;
        }).collect(Collectors.toList());

        page.setRecords(vos);
        page.setTotal(taskPage.getTotal());
        page.setPages(taskPage.getPages());

        return R.ok(page);
    }

    /**
     * 企业任务保存-企业通用
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-企业通用")
    @PostMapping("/dept-task/save-task-type61")
    public R<?> saveEntTaskForTaskType61(@Validated @RequestBody DeptTaskDTO.TaskType61 params) {
        log.info("企业任务保存-企业通用请求参数:{}", params);

        return R.ok(wmDeptTaskService.saveEntTaskForTaskType61(params));
    }

    /**
     * 企业任务保存-医学会各级年会及卫星会
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-医学会各级年会及卫星会")
    @PostMapping("/dept-task/save-task-type62")
    public R<?> saveEntTaskForTaskType62(@Validated @RequestBody DeptTaskDTO.TaskType62 params) {
        log.info("企业任务保存-医学会各级年会及卫星会请求参数:{}", params);

        return R.ok(wmDeptTaskService.saveEntTaskForTaskType62(params));
    }

    /**
     * 企业任务保存-专家共识巡讲会
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-专家共识巡讲会")
    @PostMapping("/dept-task/save-task-type63")
    public R<?> saveEntTaskForTaskType63(@Validated @RequestBody DeptTaskDTO.TaskType63 params) {
        log.info("企业任务保存-专家共识巡讲会请求参数:{}", params);

        return R.ok(wmDeptTaskService.saveEntTaskForTaskType63(params));
    }

    /**
     * 企业任务保存-企业通用-缓存
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-企业通用-缓存")
    @PostMapping("/dept-task/save-task-type61-temp")
    public R<?> saveEntTaskForTaskType61Temp(@RequestBody DeptTaskDTO.TaskType61 params) {
        log.info("企业任务保存-企业通用-缓存:{}", params);

        String cacheKey = String.format("%s-%s-%s", CacheConstants.ENT_TASK_CACHE_KEY, "61", SecurityUtils.getUser().getId());
        redisTemplate.opsForValue().set(cacheKey, JSONUtil.toJsonStr(params));

        return R.ok();
    }

    /**
     * 企业任务保存-医学会各级年会及卫星会-缓存
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-医学会各级年会及卫星会-缓存")
    @PostMapping("/dept-task/save-task-type62-temp")
    public R<?> saveEntTaskForTaskType62Temp(@RequestBody DeptTaskDTO.TaskType62 params) {
        log.info("企业任务保存-医学会各级年会及卫星会-缓存:{}", params);

        String cacheKey = String.format("%s-%s-%s", CacheConstants.ENT_TASK_CACHE_KEY, "62", SecurityUtils.getUser().getId());
        redisTemplate.opsForValue().set(cacheKey, JSONUtil.toJsonStr(params));

        return R.ok();
    }

    /**
     * 企业任务保存-专家共识巡讲会-缓存
     *
     * @param params 参数
     * @return {@link R}<{@link ?}> 结果
     */
    @SysLog("企业任务保存-专家共识巡讲会-缓存")
    @PostMapping("/dept-task/save-task-type63-temp")
    public R<?> saveEntTaskForTaskType63Temp(@RequestBody DeptTaskDTO.TaskType63 params) {
        log.info("企业任务保存-专家共识巡讲会-缓存:{}", params);

        String cacheKey = String.format("%s-%s-%s", CacheConstants.ENT_TASK_CACHE_KEY, "63", SecurityUtils.getUser().getId());
        redisTemplate.opsForValue().set(cacheKey, JSONUtil.toJsonStr(params));

        return R.ok();
    }

    /**
     * 获取企业任务保存缓存数据
     *
     * @param taskTypeId 任务类型 ID
     * @return {@link R}<{@link ?}> 结果
     */
    @GetMapping("/dept-task/task-type-temp/{taskTypeId}")
    public R<?> getEntTaskTemp(@PathVariable("taskTypeId") String taskTypeId) {
        String cacheKey = String.format("%s-%s-%s", CacheConstants.ENT_TASK_CACHE_KEY, taskTypeId, SecurityUtils.getUser().getId());
        String jsonStr = (String) redisTemplate.opsForValue().get(cacheKey);
        if (StrUtil.isBlank(jsonStr)) {
            return R.ok();
        }
        switch (taskTypeId) {
            case "61":
                return R.ok(JSONUtil.toBean(jsonStr, DeptTaskDTO.TaskType61.class));
            case "62":
                return R.ok(JSONUtil.toBean(jsonStr, DeptTaskDTO.TaskType62.class));
            case "63":
                return R.ok(JSONUtil.toBean(jsonStr, DeptTaskDTO.TaskType63.class));
            default:
                return R.ok();
        }
    }
}
