/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmSettleAccountReceive;
import com.qunzhixinxi.hnqz.admin.service.WmSettleAccountReceiveService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 *
 *
 * @author zhangwt
 * @date 2021-02-01 18:06:37
 */
@RestController
@AllArgsConstructor
@RequestMapping("/accountReceive" )
@Api(value = "accountReceive", tags = "入账通知")
public class WmSettleAccountReceiveController {

    private final  WmSettleAccountReceiveService wmSettleAccountReceiveService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmSettleAccountReceive
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    public R getWmSettleAccountReceivePage(Page page, WmSettleAccountReceive wmSettleAccountReceive) {
        return R.ok(wmSettleAccountReceiveService.page(page, Wrappers.query(wmSettleAccountReceive)));
    }


    /**
     * 通过id查询
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmSettleAccountReceiveService.getById(id));
    }

    /**
     * 新增
     * @param wmSettleAccountReceive
     * @return R
     */
    @ApiOperation(value = "新增", notes = "新增")
    @SysLog("新增" )
    @PostMapping
    public R save(@RequestBody WmSettleAccountReceive wmSettleAccountReceive) {
        return R.ok(wmSettleAccountReceiveService.save(wmSettleAccountReceive));
    }

    /**
     * 修改
     * @param wmSettleAccountReceive
     * @return R
     */
    @ApiOperation(value = "修改", notes = "修改")
    @SysLog("修改" )
    @PutMapping
    public R updateById(@RequestBody WmSettleAccountReceive wmSettleAccountReceive) {
        return R.ok(wmSettleAccountReceiveService.updateById(wmSettleAccountReceive));
    }

    /**
     * 通过id删除
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除", notes = "通过id删除")
    @SysLog("通过id删除" )
    @DeleteMapping("/{id}" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmSettleAccountReceiveService.removeById(id));
    }

}
