/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.admin.entity.WmWkArticleShare;
import com.qunzhixinxi.hnqz.admin.service.WmWkArticleShareService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 文章分享
 *
 * @author pigx code generator
 * @date 2020-07-09 22:34:30
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmwkarticleshare" )
@Api(value = "wmwkarticleshare", tags = "文章分享管理")
public class WmWkArticleShareController {

    private final  WmWkArticleShareService wmWkArticleShareService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmWkArticleShare 文章分享
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_view')" )
    public R getWmWkArticleSharePage(Page page, WmWkArticleShare wmWkArticleShare) {

		OrderItem orderItem=new OrderItem();
		orderItem.setColumn("create_time");
		orderItem.setAsc(false);
		page.addOrder(orderItem);

        return R.ok(wmWkArticleShareService.page(page, Wrappers.query(wmWkArticleShare)));
    }


    /**
     * 通过id查询文章分享
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_view')" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmWkArticleShareService.getById(id));
    }

    /**
     * 新增文章分享
     * @param wmWkArticleShare 文章分享
     * @return R
     */
    @ApiOperation(value = "新增文章分享", notes = "新增文章分享")
    @SysLog("新增文章分享" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_add')" )
    public R save(@RequestBody WmWkArticleShare wmWkArticleShare) {
		wmWkArticleShare.setCreateTime(LocalDateTime.now());
        return R.ok(wmWkArticleShareService.save(wmWkArticleShare));
    }

    /**
     * 修改文章分享
     * @param wmWkArticleShare 文章分享
     * @return R
     */
    @ApiOperation(value = "修改文章分享", notes = "修改文章分享")
    @SysLog("修改文章分享" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_edit')" )
    public R updateById(@RequestBody WmWkArticleShare wmWkArticleShare) {
        return R.ok(wmWkArticleShareService.updateById(wmWkArticleShare));
    }

    /**
     * 通过id删除文章分享
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除文章分享", notes = "通过id删除文章分享")
    @SysLog("通过id删除文章分享" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmwkarticleshare_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmWkArticleShareService.removeById(id));
    }

}
