/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 任务表
 *
 * @author gaoyanng
 * @date 2020-06-25 23:35:25
 */
@Data
@TableName(value = "wm_task", autoResultMap = true)
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "任务表")
public class WmTask extends Model<WmTask> {
	private static final long serialVersionUID = 1L;

	/**
	 *
	 */
	@TableId
	@ApiModelProperty(value="")
	private String id;

	/**
	 * 任务编号
	 */
	private String taskNumber;

	/**
	 * 任务类型ID
	 */
	@ApiModelProperty(value="任务类型ID")
	private String taskTypeId;

	/**
	 * 任务类型名称
	 */
	@TableField(exist = false)
	private String taskTypeName;

	@ApiModelProperty(value="任务得分")
	Integer score;

	@ApiModelProperty(value="任务提交到的积分包ID")
	String scorePackageDrugId;

	@ApiModelProperty(value="任务提交到的积分包ID")
	String scorePackageLevel1Id;

	@ApiModelProperty(value="任务类型ID")
	String scorePackageId;

	@ApiModelProperty(value="积分分配")
	String taskRuleId;

	/**
	 * 调研时间
	 */
	@ApiModelProperty(value="调研时间")
	private LocalDateTime lookintoDate;
	/**
	 * 省
	 */
	@ApiModelProperty(value="省")
	private String province;
	/**
	 * 市
	 */
	@ApiModelProperty(value="市")
	private String city;
	/**
	 * 区
	 */
	@ApiModelProperty(value="区")
	private String area;
	/**
	 * 地址
	 */
	@ApiModelProperty(value="地址")
	private String address;
	/**
	 * 调研类型ID
	 */
	@ApiModelProperty(value="调研类型ID")
	private String lookintoTypeId;
	/**
	 * 任务内容ID
	 */
	@ApiModelProperty(value="任务内容ID")
	private String taskContentId;
	/**
	 * 任务状态
	 */
	@ApiModelProperty(value="任务状态")
	private String taskStatus;

	/**
	 * 提交审核状态
	 */
	@ApiModelProperty(value="提交审核状态")
	private String submitStatus;

	/**
	 * 任务用户ID
	 */
	@ApiModelProperty(value="任务用户ID")
	private String taskUserId;

	/**
	 * 用户类型ID
	 */
	@ApiModelProperty(value="用户类型ID")
	private String taskUserType;

	/**
	 * 服务药企
	 */
	@ApiModelProperty(value="服务药企")
	private String drugEntId;

	/**
	 * 分享图片地址
	 */
	@ApiModelProperty(value="服务药企")
	private String shareImgUrl;

	/**
	 * 组织机构ID
	 */
	@ApiModelProperty(value="组织机构ID")
	private String deptId;
	/**
	 * 是否删除
	 */
	@ApiModelProperty(value="是否删除")
	private String delFlag;
	/**
	 * 是否禁用
	 */
	@ApiModelProperty(value="是否禁用")
	private String enableFlag;
	/**
	 * 所属租户
	 */
	@ApiModelProperty(value="所属租户",hidden=true)
	private Integer tenantId;
	/**
	 * 创建时间
	 */
	@ApiModelProperty(value="创建时间")
	private LocalDateTime createTime;
	/**
	 * 创建人
	 */
	@ApiModelProperty(value="创建人")
	private Integer createUser;
	/**
	 * 更新时间
	 */
	@ApiModelProperty(value="更新时间")
	private LocalDateTime updateTime;
	/**
	 * 更新人
	 */
	@ApiModelProperty(value="更新人")
	private String updateUser;

	private String platAuditStatus;

	@TableField(typeHandler = JacksonTypeHandler.class)
	private List<String> extIds;

	private String taskInfoImg;

	/**
	 * 对比结果
	 */
	@ApiModelProperty(value="对比结果")
	private String compareResult;
	/**
	 * 审批意见
	 */
	@ApiModelProperty(value="审批意见")
	private String approvalOpinion;
	/**
	 * 审批说明
	 */
	@ApiModelProperty(value="审批说明")
	private String approvalInfo;

	private String realFlag;

	@ApiModelProperty(value="任务提交到一级cso的报告ID")
	private Integer reportOneId;
	@ApiModelProperty(value="任务提交到一级cso的报告审核状态")
	private String reportOneApprovalStatus;
	@ApiModelProperty(value="任务提交到一级cso的报告审批意见")
	private String reportOneApprovalOpinion;
	@ApiModelProperty(value="任务提交到一级cso的报告审批说明")
	private String reportOneApprovalInfo;

	@ApiModelProperty(value="任务提交到二级cso的报告ID")
	private Integer reportSecondId;
	@ApiModelProperty(value="任务提交到二级cso的报告审核状态")
	private String reportSecondApprovalStatus;
	@ApiModelProperty(value="任务提交到二级cso的报告审批意见")
	private String reportSecondApprovalOpinion;
	@ApiModelProperty(value="任务提交到二级cso的报告审批说明")
	private String reportSecondApprovalInfo;

	@ApiModelProperty(value="任务提交到药企的报告ID")
	private Integer reportDrugId;
	@ApiModelProperty(value="任务提交到药企的报告审核状态")
	private String reportDrugApprovalStatus;
	@ApiModelProperty(value="任务提交到药企的报告审批意见")
	private String reportDrugApprovalOpinion;
	@ApiModelProperty(value="任务提交到药企的报告审批说明")
	private String reportDrugApprovalInfo;

	/**
	 * 接单对象list
	 */
	@TableField(exist = false)
	private WmTaskContent wmTaskContent;

	@TableField(exist = false)
	private String type;

	/**
	 * cso审批说明
	 */
	@ApiModelProperty(value="cso审批说明")
	private String taskStatusInfo;


	private String taskFrom;

	/**
	 * 复审次数记录
	 */
	private Integer reviewTaskCount;

	/**
	 * 企业复审次数记录
	 */
	private Integer deptReviewTaskCount;

	/**
	 * 任务用户用户名
	 */
	@TableField(exist = false)
	private String taskUsername;

	/**
	 * 关联的用户手机号
	 */
	@TableField(exist = false)
	private String username;

	/**
	 * 结算状态
	 */
	@TableField(exist = false)
	private Integer settleStatus;

	/**
	 * 提交时间
	 */
	@TableField(exist = false)
	private LocalDateTime subTime;

	/**
	 * 提交到零工时间
	 */
	@TableField(exist = false)
	private LocalDateTime subToGigTime;

	/**
	 * 结算时间
	 */
	@TableField(exist = false)
	private LocalDateTime notifyDate;
}
