package com.qunzhixinxi.hnqz.admin.entity.dto;

import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

/**
 * 企业任务请求参数DTO
 *
 * @author lixuesong
 * @date 2023/09/21
 */
@Data
public class DeptTaskDTO implements Serializable {
    private static final long serialVersionUID = 410601826934557948L;

    /**
     * 任务类型61-企业通用
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType61 implements Serializable {
        private static final long serialVersionUID = -8969888154230924285L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 主题
         */
        @NotBlank(message = "主题必填")
        private String meetingtTitle;

        /**
         * 时间
         */
        @NotBlank(message = "时间必填")
        private String meetingTime;

        /**
         * 文件上传
         */
        @NotEmpty(message = "文件上传必填")
        private List<FileUrlObj> files;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 任务积分
         */
        @NotNull(message = "任务积分必填")
        private Integer score;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

    /**
     * 任务类型62-医学会各级年会及卫星会
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType62 implements Serializable {
        private static final long serialVersionUID = -2282200603021121878L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 会议主题
         */
        @NotBlank(message = "会议主题必填")
        private String meetingtTitle;

        /**
         * 会议时间
         */
        @NotBlank(message = "会议时间必填")
        private String meetingTime;

        /**
         * 会议人次
         */
        @NotBlank(message = "会议人次必填")
        private String meetingNumber;

        /**
         * 会议邀请函
         */
        @NotEmpty(message = "会议邀请函必填")
        private List<FileUrlObj> meetingInvitationUrl;

        /**
         * 会议计划表
         */
        @NotEmpty(message = "会议计划表必填")
        private List<FileUrlObj> meetingPlanUrl;

        /**
         * 会议签到表
         */
        @NotEmpty(message = "会议签到表必填")
        private List<FileUrlObj> meetingCheckInUrl;

        /**
         * 现场照片
         */
        @NotEmpty(message = "现场照片必填")
        private List<FileUrlObj> livePhotosUrl;

        /**
         * 会议内容
         */
        @NotEmpty(message = "会议内容必填")
        private List<FileUrlObj> meetingContentUrl;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

    /**
     * 任务类型63-专家共识巡讲会
     *
     * @author lixuesong
     * @date 2023/09/21
     */
    @Data
    public static class TaskType63 implements Serializable {
        private static final long serialVersionUID = -1918885625462400849L;

        /**
         * 积分包id
         */
        @NotBlank(message = "积分包id必填")
        private String packageId;

        /**
         * 主题
         */
        @NotBlank(message = "主题必填")
        private String meetingtTitle;

        /**
         * 时间
         */
        @NotBlank(message = "时间必填")
        private String meetingTime;

        /**
         * 参加人次
         */
        @NotBlank(message = "参加人次必填")
        private String meetingNumber;

        /**
         * 活动策划书
         */
        @NotEmpty(message = "活动策划书必填")
        private List<FileUrlObj> eventPlannerUrl;

        /**
         * 活动计划表
         */
        @NotEmpty(message = "活动计划表必填")
        private List<FileUrlObj> meetingPlanUrl;

        /**
         * 会议签到表
         */
        @NotEmpty(message = "会议签到表必填")
        private List<FileUrlObj> meetingCheckInUrl;

        /**
         * 现场照片
         */
        @NotEmpty(message = "现场照片必填")
        private List<FileUrlObj> livePhotosUrl;

        /**
         * 会议内容
         */
        @NotEmpty(message = "会议内容必填")
        private List<FileUrlObj> meetingContentUrl;

        /**
         * 外采费用
         */
        private String outsourcingAmount;

        /**
         * 外采费用-凭证
         */
        private List<FileUrlObj> outsourcingAmountProof;

        /**
         * 会议总结
         */
        private String meetingSummary;
    }

    /**
     * 文件 URL obj
     *
     * @author lixuesong
     * @date 2023/09/27
     */
    @Data
    public static class FileUrlObj implements Serializable {
        private static final long serialVersionUID = 7914265352319844404L;

        private String fileName;
        
        private String url;
    }

}
