package com.qunzhixinxi.hnqz.admin.enums;

import com.baomidou.mybatisplus.annotation.EnumValue;
import com.qunzhixinxi.hnqz.common.taxhelper.enums.TaxHelperCertStatus;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * 零工平台类型（订单支付方式类型）
 *
 * @author lixuesong
 * @date 2022年03月02日 11:10
 */
@Getter
@AllArgsConstructor
public enum GigTypeEnum {

	// 零工平台类型
	REN_LI_JIA(0, "人力家", "人力家", RenlijiaCertStatus.class),
	TAX_HELPOR(1, "税邦云", "税邦云", TaxHelperCertStatus.class),
	OLADING(2, "钉灵工", "钉灵工-海南飞亿", OladingCertStatus.class),
	YEE(3, "易联数科", "易联数科", YeeCertStatus.class),
	ZHENG_QI_ZHI_XING(4, "正启之星", "正启之星", ZhengQiZhiXingCertStatus.class),
	LANG_CHAO(5, "浪潮", "浪潮", LangChaoCertStatus.class),
	HUI_QI_YUN(6, "汇企云", "汇企云", HuiQiYunCertStatus.class),
	ZHONG_YI_YUN(7, "众蚁云", "众蚁云", ZhongYiYunCertStatus.class),
	ZHONG_ZHI(8, "中智", "中智", ZhongZhiCertStatus.class),
	QI_FU(9, "福建企赋", "福建企赋", ZhongYiYunCertStatus.class),
	SHEN_ZHEN_REN_CAI(10, "深圳人才", "深圳人才", OladingCertStatus.class),

	MAH_SETTLE(99, "MAH结算", "MAH结算", null);

	@EnumValue
	private int code;

	private String desc;

	private String desc2;

	private Class<?> statusClass;

	public static GigTypeEnum resolve(int code) {
		for (GigTypeEnum gigTypeEnum : GigTypeEnum.values()) {
			if (code == gigTypeEnum.getCode()) {
				return gigTypeEnum;
			}
		}
		return null;
	}

	/**
	 * 根据认证状态code获取对应字符串
	 *
	 * @param gigType
	 * @param certStatus
	 * @return
	 */
	public static String getCertStatusByCode(GigTypeEnum gigType, int certStatus) {
		String certStatusStr = "";
		switch (gigType) {
			case REN_LI_JIA:
				certStatusStr = RenlijiaCertStatus.resolve(certStatus).name();
				break;
			case TAX_HELPOR:
				certStatusStr = TaxHelperCertStatus.resolve(certStatus).name();
				break;
			case YEE:
				certStatusStr = YeeCertStatus.resolve(String.valueOf(certStatus)).name();
				break;
			case OLADING:
			case SHEN_ZHEN_REN_CAI:
				certStatusStr = OladingCertStatus.resolve(String.valueOf(certStatus)).name();
				break;
			case ZHENG_QI_ZHI_XING:
				certStatusStr = ZhengQiZhiXingCertStatus.resolve(certStatus).name();
				break;
			case LANG_CHAO:
				certStatusStr = LangChaoCertStatus.resolve(certStatus).name();
				break;
			case HUI_QI_YUN:
				certStatusStr = HuiQiYunCertStatus.resolve(certStatus).name();
				break;
			case ZHONG_YI_YUN:
				certStatusStr = ZhongYiYunCertStatus.resolve(certStatus).name();
				break;
			case ZHONG_ZHI:
				certStatusStr = ZhongZhiCertStatus.resolve(certStatus).name();
				break;
			case QI_FU:
				certStatusStr = ZhongYiYunCertStatus.resolve(certStatus).name();
				break;
			default:
		}

		return certStatusStr;
	}

	/**
	 * 根据认证状态code获取对应中文
	 *
	 * @param gigType
	 * @param certStatus
	 * @return
	 */
	public static String getCertStatusNameByCode(GigTypeEnum gigType, int certStatus) {
		String certStatusStr = "";
		switch (gigType) {
			case REN_LI_JIA:
				certStatusStr = RenlijiaCertStatus.resolve(certStatus).getMessage();
				break;
			case TAX_HELPOR:
				certStatusStr = TaxHelperCertStatus.resolve(certStatus).getDesc();
				break;
			case YEE:
				certStatusStr = YeeCertStatus.resolve(String.valueOf(certStatus)).getMessage();
				break;
			case OLADING:
			case SHEN_ZHEN_REN_CAI:
				certStatusStr = OladingCertStatus.resolve(String.valueOf(certStatus)).getMessage();
				break;
			case ZHENG_QI_ZHI_XING:
				certStatusStr = ZhengQiZhiXingCertStatus.resolve(certStatus).getMessage();
				break;
			case LANG_CHAO:
				certStatusStr = LangChaoCertStatus.resolve(certStatus).getMessage();
				break;
			case HUI_QI_YUN:
				certStatusStr = HuiQiYunCertStatus.resolve(certStatus).getMessage();
				break;
			case ZHONG_YI_YUN:
				certStatusStr = ZhongYiYunCertStatus.resolve(certStatus).getMessage();
				break;
			case ZHONG_ZHI:
				certStatusStr = ZhongZhiCertStatus.resolve(certStatus).getMessage();
				break;
			case QI_FU:
				certStatusStr = ZhongYiYunCertStatus.resolve(certStatus).getMessage();
				break;
			default:
		}

		return certStatusStr;
	}
}
