package com.qunzhixinxi.hnqz.admin.enums;

import com.baomidou.mybatisplus.annotation.EnumValue;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 税源地
 */
@Getter
@AllArgsConstructor
public enum SubjectLocation {

	// 仁励家通道
	REN_LI_JIA("REN_LI_JIA", GigTypeEnum.REN_LI_JIA, "人力家", 0, "renlijia_invoice"),

	// 税邦云通道
	ZHONG_YING("ZHONG_YING", GigTypeEnum.TAX_HELPOR, "税邦云", 1, "tax_helper_invoice"),
	LUO_SHU_YUN("LUO_SHU_YUN", GigTypeEnum.TAX_HELPOR, "税邦云", 2, "tax_helper_invoice"),

	// 钉零工通道
	FEI_YI("FEI_YI", GigTypeEnum.OLADING, "钉灵工-海南飞亿", 3, "olading_invoice"),
	JIN_YUAN("JIN_YUAN", GigTypeEnum.OLADING, "金园数科", 4, "olading_jinyuan_invoice"),
	XIN_TAI_ZI("XIN_TAI_ZI", GigTypeEnum.OLADING, "钉灵工-河南薪泰梓", 5, "olading_henan_invoice"),

	// 易联
	YI_LIAN("YI_LIAN", GigTypeEnum.YEE, "易联数科", 6, "yee_invoice"),

	// 正启之星
	ZHENG_QI_ZHI_XING("ZHENG_QI_ZHI_XING",  GigTypeEnum.ZHENG_QI_ZHI_XING, "正启之星", 7, "zheng_qi_zhi_xing_invoice"),

	// 浪潮
	LANG_CHAO("LANG_CHAO", GigTypeEnum.LANG_CHAO, "浪潮", 8, "langchao_invoice"),

	// 正启之星-江西
	JIANG_XI("JIANG_XI",  GigTypeEnum.ZHENG_QI_ZHI_XING, "正启之星-江西", 9, "zheng_qi_zhi_xing_jiang_xi_invoice"),

	// 汇企云
	HUI_QI_YUN("HUI_QI_YUN", GigTypeEnum.HUI_QI_YUN, "汇企云", 10, "hui_qi_yun_invoice"),

	// 众蚁云
	ZHONG_YI_YUN("ZHONG_YI_YUN", GigTypeEnum.ZHONG_YI_YUN, "众蚁云", 11, "zhong_yi_yun_invoice"),
	// 中智
	ZHONG_ZHI("ZHONG_ZHI", GigTypeEnum.ZHONG_ZHI, "中智", 12, "zhongzhi_invoice"),
	ZHAO_YU("ZHAO_YU", GigTypeEnum.OLADING, "钉灵工-河南兆宇", 13, "olading_zhaoyu_invoice"),
	QI_FU("QI_FU", GigTypeEnum.QI_FU, "福建企赋", 14, "qifu_invoice"),
	SHEN_ZHEN_REN_CAI("SHEN_ZHEN_REN_CAI", GigTypeEnum.SHEN_ZHEN_REN_CAI, "深圳之光", 14, "shenzhenrencai_invoice"),


	MAH_SETTLE("MAH_SETTLE", GigTypeEnum.MAH_SETTLE, "MAH结算", 99, "");

	/**
	 * 税源地类型
	 */
	@EnumValue
	private final String type;

	/**
	 * 渠道类型
	 */
	private final GigTypeEnum gigType;

	/**
	 * 描述
	 */
	private final String description;

	/**
	 * 排序
	 */
	private final Integer sort;

	/**
	 * 发票类目
	 */
	private final String invoiceType;

	/**
	 * 根据gigType获取枚举
	 *
	 * @param gigType
	 * @return
	 */
	public static List<SubjectLocation> getByGigType(GigTypeEnum gigType) {
		List<SubjectLocation> list = new ArrayList<>();
		for (SubjectLocation subjectLocation : SubjectLocation.values()) {
			if (subjectLocation.gigType.equals(gigType)) {
				list.add(subjectLocation);
			}
		}
		return list;
	}

	/**
	 * 根据type获取枚举
	 *
	 * @param type
	 * @return
	 */
	public static SubjectLocation resolve(String type) {
		for (SubjectLocation subjectLocation: SubjectLocation.values()){
			if (subjectLocation.type.equals(type)){
				return subjectLocation;
			}
		}
		return null;
	}

	/**
	 * 获取所有渠道对应的发票类目type
	 *
	 * @return 发票类目type列表
	 */
	public static List<String> getAllInvoices() {
		return Arrays.stream(SubjectLocation.values()).map(SubjectLocation::getInvoiceType).collect(Collectors.toList());
	}
}