package com.qunzhixinxi.hnqz.admin.recharge.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.recharge.dto.SysDeptInvoiceRequest;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptInvoice;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptInvoiceService;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptInvoiceRelationVO;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptInvoiceVO;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

/**
 * 企业开票前端控制器
 *
 * @author snows
 * @date 2023/6/16 14:49
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/dept/invoice")
public class SysDeptInvoiceController {

    private final SysDeptInvoiceService deptInvoiceService;
    private final SysDeptService deptService;
    private final RedisTemplate redisTemplate;

    /**
     * 开票申请
     *
     * @param toInvoice 开票参数
     * @return 响应结果
     */
    @SysLog("开票申请")
    @PostMapping
    public R<?> toInvoice(@Validated @RequestBody SysDeptInvoiceRequest.ToInvoice toInvoice) {
        log.info("开票申请请求参数：{}", toInvoice);
        return R.ok(deptInvoiceService.toInvoice(toInvoice));
    }

    /**
     * 开票审核
     *
     * @param toApprove 开票审核
     * @return 响应结果
     */
    @SysLog("平台财务-开票审核")
    @PostMapping("/to-approve")
    public R<?> toApprove(@Validated @RequestBody SysDeptInvoiceRequest.ToApprove toApprove) {
        log.info("平台财务-开票审核请求参数：{}", toApprove);
        return R.ok(deptInvoiceService.toApprove(toApprove));
    }

    /**
     * 企业-开票管理列表
     *
     * @param page               分页参数
     * @param toScorePackagePage 请求参数
     * @return 响应结果
     */
    @GetMapping("/page-invoice-for-dept")
    public R<IPage<SysDeptInvoiceRelationVO>> pageInvoiceForDept(Page<SysDeptInvoiceRelationVO> page, SysDeptInvoiceRequest.ToScorePackagePage toScorePackagePage) {

        return R.ok(deptInvoiceService.pageInvoiceForDept(page, toScorePackagePage));
    }

    /**
     * 平台财务管理-开票列表
     *
     * @param page          分页参数
     * @param toInvoicePage 请求参数
     * @return 响应结果
     */
    @GetMapping("/page-invoice-for-admin")
    public R<IPage<SysDeptInvoiceVO>> pageInvoiceForAdmin(Page<SysDeptInvoiceVO> page, SysDeptInvoiceRequest.ToInvoicePage toInvoicePage) {

        return R.ok(deptInvoiceService.pageInvoiceForAdmin(page, toInvoicePage));
    }

    /**
     * 开票记录
     *
     * @param page          分页参数
     * @param toInvoicePage 请求参数
     * @return 响应结果
     */
    @GetMapping("/page-invoice-record")
    public R<IPage<SysDeptInvoiceVO>> pageInvoiceRecord(Page<SysDeptInvoiceVO> page, SysDeptInvoiceRequest.ToInvoicePage toInvoicePage) {

        return R.ok(deptInvoiceService.pageInvoiceRecord(page, toInvoicePage));
    }

    /**
     * 平台财务-关联积分包列表
     *
     * @param page   分页参数
     * @param invoiceId 企业开票表ID
     * @return 响应结果
     */
    @GetMapping("/page-invoice")
    public R<IPage<SysDeptInvoiceRelationVO>> pageInvoice(Page<SysDeptInvoiceRelationVO> page, @RequestParam("invoiceId") Integer invoiceId) {

        return R.ok(deptInvoiceService.pageInvoice(page, invoiceId));
    }

    /**
     * 开票详情（园区）
     *
     * @param invoiceId 开票id
     * @return 结果
     */
    @GetMapping("/detail/{invoiceId}")
    public R<SysDeptInvoiceVO> invoiceDetail(@PathVariable("invoiceId") Integer invoiceId) {
        SysDeptInvoice deptInvoice = deptInvoiceService.getById(invoiceId);
        if (deptInvoice != null) {
            SysDeptInvoiceVO deptInvoiceVO = BeanUtil.copyProperties(deptInvoice, SysDeptInvoiceVO.class);
            SysDept dept = deptService.getById(deptInvoice.getDeptId());
            deptInvoiceVO.setDeptName(dept.getName());
            deptInvoiceVO.setApprovalStatusStr(deptInvoice.getApprovalStatus().getDescription());
            deptInvoiceVO.setInvoiceTypeStr(deptInvoice.getInvoiceType().getDescription());
            return R.ok(deptInvoiceVO);
        }
        return R.ok();
    }

    /**
     * 获取缓存开票信息
     *
     * @return 缓存开票信息
     */
    @GetMapping("/cache-invoice-info")
    public R<?> getCacheInvoiceInfo() {
        String cacheKey = CacheConstants.INVOICE_DEPT_KEY + SecurityUtils.getUser().getDeptId();
        String cacheVal = (String) redisTemplate.opsForValue().get(cacheKey);
        Map<String, String> result = new HashMap<>();
        if (StrUtil.isBlank(cacheVal)) {
            result.put("addresseeName", "");
            result.put("address", "");
            result.put("addresseePhone", "");
            result.put("invoiceType", "");
            return R.ok(result);
        }
        SysDeptInvoice deptInvoice = JSONUtil.toBean(cacheVal, SysDeptInvoice.class);
        result.put("addresseeName", deptInvoice.getAddresseeName());
        result.put("address", deptInvoice.getAddress());
        result.put("addresseePhone", deptInvoice.getAddresseePhone());
        result.put("invoiceType", deptInvoice.getInvoiceType().getType());
        return R.ok(result);
    }
}
