package com.qunzhixinxi.hnqz.admin.recharge.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.recharge.dto.SysDeptRechargeRequest;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptInvoice;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRecharge;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRechargeRecord;
import com.qunzhixinxi.hnqz.admin.recharge.enums.InvoiceApprovalStatus;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptInvoiceService;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptRechargeRecordService;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptRechargeService;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptInvoiceRelationVO;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeRecordForDeptVO;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeRecordVO;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeVO;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 企业充值管理前端控制器
 *
 * @author snows
 * @date 2023/6/12 11:17
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/dept/recharge")
public class SysDeptRechargeController {

    private final SysDeptRechargeService deptRechargeService;

    private final SysDeptRechargeRecordService deptRechargeRecordService;

    private final SysDeptInvoiceService deptInvoiceService;

    /**
     * 配置(权限管理)
     *
     * @param toConfig 配置参数
     * @return 结果
     */
    @SysLog("配置(平台财务-权限管理)")
    @PostMapping("/to-config")
    public R<?> toConfig(@RequestBody @Validated SysDeptRechargeRequest.ToConfig toConfig) {
        log.info("配置(平台财务-权限管理)入参:{}", toConfig);
        if (ArrayUtil.isNotEmpty(toConfig.getPermissions())) {
            if (toConfig.getOverdrawScore() == null) {
                return R.failed("透支额度必填");
            }
        } else {
            if (toConfig.getOverdrawScore() != null) {
                return R.failed("没有配置余额透支权限");
            }
        }
        return R.ok(deptRechargeService.toConfig(toConfig));
    }

    /**
     * 充值(平台财务)
     *
     * @param toRecharge 充值参数
     * @return 充值结果
     */
    @SysLog("充值(平台财务)")
    @PostMapping
    public R<?> toRecharge(@Validated @RequestBody SysDeptRechargeRequest.ToRecharge toRecharge) {
        log.info("充值(平台财务)入参:{}", toRecharge);
        return R.ok(deptRechargeService.recharge(toRecharge));
    }

    /**
     * 分页查询(平台财务)
     *
     * @param page 分页参数
     * @param toPage 条件参数
     * @return 分页结果
     */
    @GetMapping("/page")
    public R<IPage<SysDeptRechargeVO>> pageRechargeForAdmin(Page<SysDeptRechargeVO> page, SysDeptRechargeRequest.ToPage toPage) {
        return R.ok(deptRechargeService.pageRechargeForAdmin(page, toPage));
    }

    /**
     * 积分充值/分配/发包记录(平台财务/园区)
     *
     * @param page 分页参数
     * @return 分页结果
     */
    @GetMapping("/page-recharge-record")
    public R<IPage<SysDeptRechargeRecordVO>> pageRechargeRecord(Page<SysDeptRechargeRecordVO> page, @Validated SysDeptRechargeRequest.ToAdminRecordPage toAdminRecordPage) {
        return R.ok(deptRechargeRecordService.pageRechargeRecord(page, toAdminRecordPage.getDeptId(), toAdminRecordPage.getType()));
    }

    /**
     * 积分分配/发包记录(企业)
     *
     * @param page 分页参数
     * @return 分页结果
     */
    @GetMapping("/page-record-for-dept")
    public R<IPage<SysDeptRechargeRecordForDeptVO>> pageRecordForDept(Page<SysDeptRechargeRecordForDeptVO> page, SysDeptRechargeRequest.ToRecordPage toRecordPage) {
        return R.ok(deptRechargeRecordService.pageRecordForDept(page, toRecordPage.getType(), toRecordPage.getCreateTime()));
    }

    /**
     * 企业-数据总览
     *
     * @return 结果
     */
    @GetMapping("/dept-overview")
    public R<?> deptScoreOverview() {
        Integer deptId = SecurityUtils.getUser().getDeptId();
        SysDeptRecharge deptRecharge = deptRechargeService.getOne(Wrappers.<SysDeptRecharge>lambdaQuery()
                .eq(SysDeptRecharge::getDeptId, deptId)
                .eq(SysDeptRecharge::getDelFlag, DelEnum.NOT_DEL.val()));

        Map<String, Integer> result = new HashMap<>(4);
        if (deptRecharge == null) {
            result.put("score", 0);
            result.put("usedScore", 0);
            result.put("toInvoiceScore", 0);
            result.put("invoicedScore", 0);
        } else {
            // 积分余额
            result.put("score", deptRecharge.getScore());
            // 本月消耗积分
            List<SysDeptRechargeRecord> usedRecords = deptRechargeRecordService.list(Wrappers.<SysDeptRechargeRecord>lambdaQuery()
                    .eq(SysDeptRechargeRecord::getDeptId, deptId)
                    .eq(SysDeptRechargeRecord::getDelFlag, DelEnum.NOT_DEL.val()));
            int usedScore = usedRecords.stream().mapToInt(SysDeptRechargeRecord::getChangeScore).sum();
            result.put("usedScore", Math.abs(usedScore));

            // 可开票记录
            List<SysDeptInvoiceRelationVO> relationVOS = deptInvoiceService.listToInvoiceScore(deptId);
            if (CollUtil.isNotEmpty(relationVOS)) {
                // 可开票积分
                result.put("toInvoiceScore", relationVOS.stream().mapToInt(SysDeptInvoiceRelationVO::getScore).sum());
            } else {
                // 可开票积分
                result.put("toInvoiceScore", 0);
            }

            // 当月时间范围
            LocalDate nowDate = LocalDate.now();
            LocalDateTime startTime = nowDate.with(TemporalAdjusters.firstDayOfMonth()).atStartOfDay();
            LocalDateTime endTime = nowDate.plusMonths(1L).with(TemporalAdjusters.firstDayOfMonth()).atStartOfDay();
            List<SysDeptInvoice> monthInvoices = deptInvoiceService.list(Wrappers.<SysDeptInvoice>lambdaQuery()
                    .eq(SysDeptInvoice::getDeptId, deptId)
                    .eq(SysDeptInvoice::getDelFlag, DelEnum.NOT_DEL.val())
                    .eq(SysDeptInvoice::getApprovalStatus, InvoiceApprovalStatus.PASSED)
                    .between(SysDeptInvoice::getApprovalTime, startTime, endTime));
            if (CollUtil.isNotEmpty(monthInvoices)) {
                // 本月已开票积分
                result.put("invoicedScore", monthInvoices.stream().mapToInt(SysDeptInvoice::getRelationPackageScore).sum());
            } else {
                // 本月已开票积分
                result.put("invoicedScore", 0);
            }
        }

        return R.ok(result);
    }


}
