package com.qunzhixinxi.hnqz.admin.recharge.dto;

import com.qunzhixinxi.hnqz.admin.recharge.enums.InvoiceApprovalStatus;
import com.qunzhixinxi.hnqz.admin.recharge.enums.InvoiceStatus;
import com.qunzhixinxi.hnqz.admin.recharge.enums.InvoiceType;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 开票相关请求
 *
 * @author snows
 * @date 2023/6/16 15:13
 */
public class SysDeptInvoiceRequest {

    /**
     * 开票请求参数
     */
    @Data
    public static class ToInvoice implements Serializable {
        private static final long serialVersionUID = -6254095346012051281L;

        /**
         * 积分包id列表
         */
        @NotEmpty(message = "积分包id列表必填")
        private String[] packageIds;

        /**
         * 开票主体
         */
//        @NotNull(message = "开票主体必填")
//        private Integer deptId;

        /**
         * 发票类型
         */
        @NotNull(message = "发票类型必填")
        private InvoiceType invoiceType;

        /**
         * 邮寄地址
         */
        private String address;

        /**
         * 收件人姓名
         */
        private String addresseeName;

        /**
         * 收件人手机号
         */
        private String addresseePhone;
    }

    /**
     * 开票审核
     */
    @Data
    public static class ToApprove implements Serializable {
        private static final long serialVersionUID = 6518694561688554330L;

        /**
         * 开票id
         */
        @NotNull(message = "开票id必填")
        private Integer invoiceId;

        /**
         * 审核状态
         */
        @NotNull(message = "审核状态必填")
        private InvoiceApprovalStatus approvalStatus;

    }

    /**
     * 开票管理请求参数
     */
    @Data
    public static class ToScorePackagePage implements Serializable {
        private static final long serialVersionUID = -7835160794495661536L;

        /**
         * 发布时间
         */
        private LocalDateTime[] createTime;

        /**
         * 发包企业
         */
        private String deptName;

        /**
         * 开票状态
         */
        private InvoiceStatus invoiceStatus;
    }

    /**
     * 开票管理请求参数(平台管理员)
     */
    @Data
    public static class ToInvoicePage implements Serializable {
        private static final long serialVersionUID = -7645242691493270586L;

        /**
         * 申请时间
         */
        private LocalDateTime[] createTime;

        /**
         * 开票主体
         */
        private String deptName;

        /**
         * 开票状态
         */
        private InvoiceApprovalStatus approvalStatus;
    }
}
