package com.qunzhixinxi.hnqz.admin.recharge.dto;

import com.qunzhixinxi.hnqz.admin.recharge.enums.RechargeRecordType;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * 积分充值请求
 *
 * @author snows
 * @date 2023/6/12 11:23
 */
public class SysDeptRechargeRequest implements Serializable {
    private static final long serialVersionUID = -7104300633204592299L;

    /**
     * 积分充值分页参数
     */
    @Data
    public static class ToPage implements Serializable {
        private static final long serialVersionUID = 9102286943907340580L;

        /**
         * 企业名称
         */
        private String deptName;

        /**
         * 企业类型（级别）
         */
        private Integer deptLevel;
    }

    /**
     * 积分记录分页参数
     */
    @Data
    public static class ToRecordPage implements Serializable {
        private static final long serialVersionUID = -7774101561321194648L;

        /**
         * 企业名称
         */
        private String deptName;

        /**
         * 操作类型
         */
        private RechargeRecordType type;

        /**
         * 操作时间范围
         */
        private LocalDateTime[] createTime;
    }

    /**
     * 积分充值记录分页参数(平台财务、园区)
     */
    @Data
    public static class ToAdminRecordPage implements Serializable {
        private static final long serialVersionUID = -7774101561321194648L;

        /**
         * 企业id
         */
        @NotNull(message = "企业id必填")
        private Integer deptId;

        /**
         * 积分充值记录操作类型
         */
        private RechargeRecordType type;
    }

    /**
     * 充值参数
     */
    @Data
    public static class ToRecharge implements Serializable {
        private static final long serialVersionUID = 1444649528911261490L;

        /**
         * 企业id
         */
        @NotNull(message = "企业id必填")
        private Integer deptId;

        /**
         * 充值金额
         */
        @NotNull(message = "充值金额必填")
        private BigDecimal rechargeAmount;

        /**
         * 充值积分
         */
        @NotNull(message = "充值积分必填")
        private Integer rechargeScore;
    }

    /**
     * 配置(权限管理)参数
     */
    @Data
    public static class ToConfig implements Serializable {
        private static final long serialVersionUID = 3654419027796689016L;

        /**
         * 企业id
         */
        @NotNull(message = "企业id必填")
        private Integer deptId;

        /**
         * 服务费率
         */
        @NotNull(message = "服务费率必填")
        private BigDecimal serviceCharge;

        /**
         * 权限
         */
        private String[] permissions;

        /**
         * 可透支积分
         */
        private Integer overdrawScore;
    }

    /**
     * 积分分配参数
     */
    @Data
    public static class ToDistribute implements Serializable {
        private static final long serialVersionUID = -5449648571357801707L;

        /**
         * 企业id
         */
        @NotNull(message = "企业id必填")
        private Integer deptId;

        /**
         * 分配积分
         */
        @NotNull(message = "分配积分必填")
        private Integer distributeScore;

    }
}
