package com.qunzhixinxi.hnqz.admin.recharge.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.DeptLevelEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmScorePackageStatusMapper;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRecharge;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRechargeRecord;
import com.qunzhixinxi.hnqz.admin.recharge.enums.RechargeRecordPackageType;
import com.qunzhixinxi.hnqz.admin.recharge.enums.RechargeRecordType;
import com.qunzhixinxi.hnqz.admin.recharge.mapper.SysDeptRechargeMapper;
import com.qunzhixinxi.hnqz.admin.recharge.mapper.SysDeptRechargeRecordMapper;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptRechargeRecordService;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeRecordForDeptVO;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeRecordVO;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
* @author snows
* @description 针对表【sys_dept_recharge_record(积分充值/使用记录表)】的数据库操作Service实现
* @createDate 2023-06-12 14:40:23
*/
@Service
@AllArgsConstructor
public class SysDeptRechargeRecordServiceImpl extends ServiceImpl<SysDeptRechargeRecordMapper, SysDeptRechargeRecord>
    implements SysDeptRechargeRecordService {

    private final SysDeptService deptService;
    private final WmScorePackageMapper wmScorePackageMapper;
    private final SysUserService userService;
    private final WmScorePackageStatusMapper wmScorePackageStatusMapper;
    private final SysDeptRechargeMapper deptRechargeMapper;

    /**
     * 充值、分配记录
     *
     * @param page       分页参数
     * @param deptId     企业id
     * @param type       操作类型
     * @return 分页结果
     */
    @Override
    public IPage<SysDeptRechargeRecordVO> pageRechargeRecord(Page<SysDeptRechargeRecordVO> page, Integer deptId, RechargeRecordType type) {
        LambdaQueryWrapper<SysDeptRechargeRecord> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SysDeptRechargeRecord::getDeptId, deptId);
        queryWrapper.eq(SysDeptRechargeRecord::getDelFlag, DelEnum.NOT_DEL.val());
        queryWrapper.orderByDesc(SysDeptRechargeRecord::getCreateTime);
        if (type != null) {
            queryWrapper.eq(SysDeptRechargeRecord::getType, type);
        }

        Page<SysDeptRechargeRecord> recordPage = this.page(new Page<>(page.getCurrent(), page.getSize()), queryWrapper);

        // 查询余额信息
        SysDeptRecharge deptRecharge = deptRechargeMapper.selectOne(Wrappers.<SysDeptRecharge>lambdaQuery()
                .eq(SysDeptRecharge::getDeptId, deptId));

        if (CollUtil.isNotEmpty(recordPage.getRecords())) {
            // 查询企业
            Set<Integer> deptIds = recordPage.getRecords().stream().map(SysDeptRechargeRecord::getDeptId).collect(Collectors.toSet());
            List<SysDept> sysDepts = deptService.listByIds(deptIds);
            Map<Integer, SysDept> deptMap = sysDepts.stream().collect(Collectors.toMap(SysDept::getDeptId, Function.identity()));
            // 查询用户
            Set<Integer> userIds = recordPage.getRecords().stream().map(SysDeptRechargeRecord::getCreateUser).collect(Collectors.toSet());
            List<SysUser> users = userService.listByIds(userIds);
            Map<Integer, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

            List<SysDeptRechargeRecordVO> recordVOList = new ArrayList<>();
            recordPage.getRecords().forEach(record -> {
                SysDeptRechargeRecordVO recordVO = BeanUtil.copyProperties(record, SysDeptRechargeRecordVO.class);
                SysDept dept = deptMap.get(record.getDeptId());
                if (dept != null) {
                    recordVO.setDeptName(dept.getName());
                    recordVO.setDeptLevel(dept.getLevel());
                    recordVO.setDeptLevelStr(Objects.requireNonNull(DeptLevelEnum.getEnumByVal(dept.getLevel())).getName());
                }
                recordVO.setCreaUserStr(userMap.get(record.getCreateUser()));

                if (deptRecharge != null) {
                    // 金额 = 积分 * （服务费率/100）
                    BigDecimal changeAmount = deptRecharge.getServiceCharge().divide(new BigDecimal("100.00"))
                            .multiply(BigDecimal.valueOf(record.getChangeScore())).setScale(2, RoundingMode.HALF_EVEN);
                    recordVO.setChangeAmount(changeAmount);
                    BigDecimal currentBalance = deptRecharge.getServiceCharge().divide(new BigDecimal("100.00"))
                            .multiply(BigDecimal.valueOf(record.getCurrentScore())).setScale(2, RoundingMode.HALF_EVEN);
                    recordVO.setCurrentBalance(currentBalance);
                }
                recordVOList.add(recordVO);
            });

            page.setPages(recordPage.getPages());
            page.setRecords(recordVOList);
            page.setTotal(recordPage.getTotal());
        }

        return page;
    }

    /**
     * 分配、发包记录(企业)
     *
     * @param page       分页参数
     * @param type       操作类型
     * @param createTime 创建时间范围
     * @return 分页结果
     */
    @Override
    public IPage<SysDeptRechargeRecordForDeptVO> pageRecordForDept(Page<SysDeptRechargeRecordForDeptVO> page, RechargeRecordType type, LocalDateTime[] createTime) {
        Integer deptId = SecurityUtils.getUser().getDeptId();
        // 是否管理员或平台管理员
        boolean isAdmin = CollUtil.containsAny(SecurityUtils.getRoles(), Stream.of(1, 2).collect(Collectors.toList()));

        LambdaQueryWrapper<SysDeptRechargeRecord> queryWrapper = Wrappers.lambdaQuery();
        if (!isAdmin) {
            queryWrapper.eq(SysDeptRechargeRecord::getDeptId, deptId);
        }
        if (type != null) {
            queryWrapper.eq(SysDeptRechargeRecord::getType, type);
        }
        if (createTime != null && createTime.length == 2) {
            queryWrapper.between(SysDeptRechargeRecord::getCreateTime, createTime[0], createTime[1]);
        }

        queryWrapper.eq(SysDeptRechargeRecord::getDelFlag, DelEnum.NOT_DEL.val());
        queryWrapper.orderByDesc(SysDeptRechargeRecord::getCreateTime);
        Page<SysDeptRechargeRecord> recordPage = this.page(new Page<>(page.getCurrent(), page.getSize()), queryWrapper);

        if (CollUtil.isNotEmpty(recordPage.getRecords())) {
            // 查询企业
            SysDept dept = deptService.getById(deptId);
            // 查询所有企业
            List<SysDept> depts = deptService.list();
            Map<Integer, String> deptMap = depts.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getName));
            // 查询用户
            Set<Integer> userIds = recordPage.getRecords().stream().map(SysDeptRechargeRecord::getCreateUser).collect(Collectors.toSet());
            List<SysUser> users = userService.listByIds(userIds);
            Map<Integer, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));
            // 查询发放的积分包
            List<WmScorePackage> scorePackages = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                    .select(WmScorePackage::getId, WmScorePackage::getScorePackageName)
                    .eq(WmScorePackage::getDelFlag, DelEnum.NOT_DEL.val())
                    .eq(WmScorePackage::getSendPackageDeptId, deptId));
            // 积分包map
            Map<String, String> packageMap = new HashMap<>();
            // 积分包接单用户map
            Map<String, String> packageUsersMap = new HashMap<>();
            // 积分包子包企业map
            Map<String, Set<String>> scorePackageToEntsMap = new HashMap<>();
            if (CollUtil.isNotEmpty(scorePackages)) {
                packageMap = scorePackages.stream().collect(Collectors.toMap(WmScorePackage::getId, WmScorePackage::getScorePackageName));
                // 查领包记录
                Set<String> packageIds = scorePackages.stream().map(WmScorePackage::getId).collect(Collectors.toSet());
                List<WmScorePackageStatus> scorePackageStatusList = wmScorePackageStatusMapper.selectList(Wrappers.<WmScorePackageStatus>lambdaQuery()
                        .in(WmScorePackageStatus::getPackageId, packageIds)
                        .eq(WmScorePackageStatus::getStatus, PackageStatusEnum.APPROVED.val()));
                if (CollUtil.isNotEmpty(scorePackageStatusList)) {
                    // 查询领包记录的所有用户
                    Set<String> statusUserIds = scorePackageStatusList.stream().map(WmScorePackageStatus::getUserId).collect(Collectors.toSet());
                    List<SysUser> statusUsers = userService.listByIds(statusUserIds);
                    // 转为map（key-积分包id，value-领包用户逗号分隔）
                    Map<Integer, String> statusUserMap = statusUsers.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));
                    packageUsersMap = scorePackageStatusList.stream()
                            .collect(Collectors.groupingBy(WmScorePackageStatus::getPackageId,
                                    Collectors.mapping(wmScorePackageStatus ->
                                            statusUserMap.getOrDefault(Integer.parseInt(wmScorePackageStatus.getUserId()), ""), Collectors.joining(","))));
                }
                // 查询积分包的子包
                List<WmScorePackage> subScorePackages = wmScorePackageMapper.selectList(Wrappers.<WmScorePackage>lambdaQuery()
                        .in(WmScorePackage::getRelationScoreId, packageIds));
                if (CollUtil.isNotEmpty(subScorePackages)) {
                    // 查询子包的所有企业
                    List<SysDept> subPackageDepts = deptService.listByIds(subScorePackages.stream()
                            .map(wmScorePackage -> Integer.parseInt(wmScorePackage.getSendPackageDeptId())).collect(Collectors.toSet()));
                    // 转为map（key-积分包id，value-子包企业逗号分隔）
                    Map<Integer, String> subPackageDeptsMap = subPackageDepts.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getName));
                    scorePackageToEntsMap = subScorePackages.stream()
                            .collect(Collectors.groupingBy(WmScorePackage::getRelationScoreId,
                                    Collectors.mapping(wmScorePackage ->
                                            subPackageDeptsMap.getOrDefault(Integer.parseInt(wmScorePackage.getSendPackageDeptId()), ""), Collectors.toSet())));
                }
            }


            List<SysDeptRechargeRecordForDeptVO> recordVOList = new ArrayList<>();
            Map<String, String> finalPackageUsersMap = packageUsersMap;
            Map<String, Set<String>> finalScorePackageToEntsMap = scorePackageToEntsMap;
            Map<String, String> finalPackageMap = packageMap;
            recordPage.getRecords().forEach(record -> {
                SysDeptRechargeRecordForDeptVO recordVO = BeanUtil.copyProperties(record, SysDeptRechargeRecordForDeptVO.class);
                if (dept != null) {
                    recordVO.setDeptName(dept.getName());
                    recordVO.setDeptLevel(dept.getLevel());
                    recordVO.setDeptLevelStr(Objects.requireNonNull(DeptLevelEnum.getEnumByVal(dept.getLevel())).getName());
                }
                recordVO.setCreaUserStr(userMap.get(record.getCreateUser()));
                recordVO.setOperatingTarget(recordVO.getDeptName());
                // 关联对象
                if (RechargeRecordType.SEND_PACKAGE_DEPT.equals(record.getType()) || RechargeRecordType.SEND_PACKAGE_PERSONAL.equals(record.getType())
                        || RechargeRecordType.RECOVERY.equals(record.getType())) {
                    if (RechargeRecordPackageType.TO_PERSON.equals(record.getPackageType())) {
                        recordVO.setOperatingTarget(finalPackageUsersMap.get(record.getPackageId()));
                    } else {
                        recordVO.setOperatingTarget(String.join(",", finalScorePackageToEntsMap.getOrDefault(record.getPackageId(), Collections.emptySet())));
                    }
                    // 积分包名称
                    recordVO.setScorePackageName(finalPackageMap.get(record.getPackageId()));
                } else {
                    if (record.getRelationDeptId() != null) {
                        recordVO.setOperatingTarget(deptMap.get(record.getRelationDeptId()));
                    }
                }
                recordVOList.add(recordVO);
            });

            page.setPages(recordPage.getPages());
            page.setRecords(recordVOList);
            page.setTotal(recordPage.getTotal());
        }

        return page;
    }
}




