package com.qunzhixinxi.hnqz.admin.recharge.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.DeptLevelEnum;
import com.qunzhixinxi.hnqz.admin.recharge.dto.SysDeptRechargeRequest;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRecharge;
import com.qunzhixinxi.hnqz.admin.recharge.entity.SysDeptRechargeRecord;
import com.qunzhixinxi.hnqz.admin.recharge.enums.RechargeRecordType;
import com.qunzhixinxi.hnqz.admin.recharge.mapper.SysDeptRechargeMapper;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptRechargeRecordService;
import com.qunzhixinxi.hnqz.admin.recharge.service.SysDeptRechargeService;
import com.qunzhixinxi.hnqz.admin.recharge.vo.SysDeptRechargeVO;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
* @author snows
* @description 针对表【sys_dept_recharge(积分充值表)】的数据库操作Service实现
* @createDate 2023-06-12 10:47:24
*/
@Service
@AllArgsConstructor
public class SysDeptRechargeServiceImpl extends ServiceImpl<SysDeptRechargeMapper, SysDeptRecharge>
    implements SysDeptRechargeService {

    private final SysDeptService deptService;
    private final SysDeptRechargeRecordService deptRechargeRecordService;

    /**
     * 分页查询
     *
     * @param page 分页参数
     * @param toPage 条件参数
     * @return 分页结果
     */
    @Override
    public IPage<SysDeptRechargeVO> pageRechargeForAdmin(Page<SysDeptRechargeVO> page, SysDeptRechargeRequest.ToPage toPage) {
        LambdaQueryWrapper<SysDept> queryWrapper = Wrappers.lambdaQuery();
        if (StrUtil.isNotBlank(toPage.getDeptName())) {
            queryWrapper.like(SysDept::getName, toPage.getDeptName());
        }
        if (toPage.getDeptLevel() != null) {
            queryWrapper.eq(SysDept::getLevel, toPage.getDeptLevel());
        }
        // 只显示除管理员的企业
        queryWrapper.ne(SysDept::getLevel, DeptLevelEnum.ADMIN.getVal());

        Page<SysDept> deptPage = deptService.page(new Page<>(page.getCurrent(), page.getSize()), queryWrapper);

        List<SysDeptRechargeVO> rechargeVOList = new ArrayList<>();
        if (CollUtil.isEmpty(deptPage.getRecords())) {
            return page;
        }

        // 查询企业积分充值表
        List<SysDeptRecharge> deptRecharges = this.list(Wrappers.<SysDeptRecharge>lambdaQuery()
                .eq(SysDeptRecharge::getDelFlag, DelEnum.NOT_DEL.val()));
        Map<Integer, SysDeptRecharge> deptRechargeMap = new HashMap<>();
        if (CollUtil.isNotEmpty(deptRecharges)) {
            deptRechargeMap = deptRecharges.stream()
                    .collect(Collectors.toMap(SysDeptRecharge::getDeptId, Function.identity()));
        }
        // 查询积分充值记录表
        List<SysDeptRechargeRecord> usedRecords = deptRechargeRecordService.list(Wrappers.<SysDeptRechargeRecord>lambdaQuery()
                .eq(SysDeptRechargeRecord::getType, RechargeRecordType.RECHARGE)
                .eq(SysDeptRechargeRecord::getDelFlag, DelEnum.NOT_DEL.val()));
        Map<Integer, Integer> usedRecordsMap = new HashMap<>();
        if (CollUtil.isNotEmpty(usedRecords)) {
            usedRecordsMap = usedRecords.stream()
                    .collect(Collectors.groupingBy(SysDeptRechargeRecord::getDeptId, Collectors.summingInt(SysDeptRechargeRecord::getChangeScore)));
        }

        Map<Integer, SysDeptRecharge> finalDeptRechargeMap = deptRechargeMap;
        Map<Integer, Integer> finalUsedRecordsMap = usedRecordsMap;
        deptPage.getRecords().forEach(sysDept -> {
            SysDeptRechargeVO deptRechargeVO = new SysDeptRechargeVO();
            deptRechargeVO.setDeptName(sysDept.getName());
            deptRechargeVO.setDeptId(sysDept.getDeptId());
            deptRechargeVO.setDeptLevel(sysDept.getLevel());
            deptRechargeVO.setDeptLevelStr(Objects.requireNonNull(DeptLevelEnum.getEnumByVal(sysDept.getLevel())).getName());
            SysDeptRecharge deptRecharge = finalDeptRechargeMap.get(sysDept.getDeptId());
            if (deptRecharge != null) {
                deptRechargeVO.setId(deptRecharge.getId());
                deptRechargeVO.setScore(deptRecharge.getScore());

                // 金额 = 积分 * （服务费率/100）
                BigDecimal balance = deptRecharge.getServiceCharge().divide(new BigDecimal("100.00"))
                        .multiply(BigDecimal.valueOf(deptRecharge.getScore())).setScale(2, RoundingMode.HALF_EVEN);
                deptRechargeVO.setBalance(balance);
                deptRechargeVO.setServiceCharge(deptRecharge.getServiceCharge());

                // 金额 = 积分 * （服务费率/100）
                BigDecimal overdrawAmount = deptRecharge.getServiceCharge().divide(new BigDecimal("100.00"))
                        .multiply(BigDecimal.valueOf(deptRecharge.getOverdrawScore())).setScale(2, RoundingMode.HALF_EVEN);
                deptRechargeVO.setOverdrawAmount(overdrawAmount);
                deptRechargeVO.setOverdrawScore(deptRecharge.getOverdrawScore());
                deptRechargeVO.setPermissions(deptRecharge.getPermissions());
                // 根据充值记录计算
                Integer totalRechargeScore = finalUsedRecordsMap.getOrDefault(sysDept.getDeptId(), 0);
                deptRechargeVO.setTotalRechargeScore(totalRechargeScore);
                deptRechargeVO.setTotalInvoiceScore(0);
                deptRechargeVO.setTotalInvoiceAmount(BigDecimal.ZERO);
            }
            rechargeVOList.add(deptRechargeVO);
        });

        page.setRecords(rechargeVOList);
        page.setTotal(deptPage.getTotal());
        page.setPages(deptPage.getPages());

        return page;
    }

    /**
     * 充值
     *
     * @param toRecharge 充值参数
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean recharge(SysDeptRechargeRequest.ToRecharge toRecharge) {

        Integer operator = SecurityUtils.getUser().getId();
        Integer deptId = SecurityUtils.getUser().getDeptId();

        // 更新充值数据
        SysDeptRecharge deptRecharge = this.getOne(Wrappers.<SysDeptRecharge>lambdaQuery()
                .eq(SysDeptRecharge::getDeptId, toRecharge.getDeptId())
                .eq(SysDeptRecharge::getDelFlag, DelEnum.NOT_DEL.val()));
        BigDecimal currentBalance = null;
        int currentScore = 0;
        if (deptRecharge == null) {
            throw new RuntimeException("费率不存在");
        } else {
            // 基于 充值金额 计算
            // 充值积分 = 充值金额 / (服务费率/100)
            BigDecimal rechargeScore = toRecharge.getRechargeAmount().divide(deptRecharge.getServiceCharge()
                            .divide(new BigDecimal("100.00")), RoundingMode.HALF_EVEN)
                    .setScale(0, RoundingMode.UP);

            if (!toRecharge.getRechargeScore().equals(rechargeScore.intValue())) {
                throw new RuntimeException("充值积分不正确");
            }
            // 计算更新金额
//            currentBalance  = toRecharge.getRechargeAmount().add(deptRecharge.getBalance());
            currentScore = rechargeScore.intValue() + deptRecharge.getScore();
            // 更新
            boolean update = this.update(Wrappers.<SysDeptRecharge>lambdaUpdate()
                    .eq(SysDeptRecharge::getId, deptRecharge.getId())
                    .eq(SysDeptRecharge::getVersion, deptRecharge.getVersion())
                    .set(SysDeptRecharge::getVersion, deptRecharge.getVersion() + 1)
//                    .set(SysDeptRecharge::getBalance, currentBalance)
                    .set(SysDeptRecharge::getScore, currentScore)
                    .set(SysDeptRecharge::getUpdateTime, LocalDateTime.now())
                    .set(SysDeptRecharge::getUpdateUser, operator));
            if (!update) {
                throw new RuntimeException("更新失败");
            }
        }

        // 新增充值记录
        SysDeptRechargeRecord rechargeRecord = new SysDeptRechargeRecord();
        rechargeRecord.setRechargeId(deptRecharge.getId());
        rechargeRecord.setChangeScore(toRecharge.getRechargeScore());
//        rechargeRecord.setChangeAmount(toRecharge.getRechargeAmount());
//        rechargeRecord.setCurrentBalance(currentBalance);
        rechargeRecord.setCurrentScore(currentScore);
        rechargeRecord.setType(RechargeRecordType.RECHARGE);
        rechargeRecord.setDeptId(deptRecharge.getDeptId());
        rechargeRecord.setRelationDeptId(deptId);
        rechargeRecord.setCreateUser(operator);
        deptRechargeRecordService.save(rechargeRecord);

        return Boolean.TRUE;
    }

    /**
     * 配置(权限管理)
     *
     * @param toConfig 配置参数
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean toConfig(SysDeptRechargeRequest.ToConfig toConfig) {
        Integer operator = SecurityUtils.getUser().getId();
        // 更新配置数据
        SysDeptRecharge deptRecharge = this.getOne(Wrappers.<SysDeptRecharge>lambdaQuery()
                .eq(SysDeptRecharge::getDeptId, toConfig.getDeptId())
                .eq(SysDeptRecharge::getDelFlag, DelEnum.NOT_DEL.val()));
        // 透支金额 = 透支积分 * (服务费率/100)
        BigDecimal overdrawAmount = null;
        if (toConfig.getOverdrawScore() != null && toConfig.getOverdrawScore() > 0) {
            overdrawAmount = BigDecimal.valueOf(toConfig.getOverdrawScore()).multiply(toConfig.getServiceCharge().divide(new BigDecimal("100.00")))
                    .setScale(2, RoundingMode.HALF_EVEN);
        } else {
            overdrawAmount = BigDecimal.ZERO;
            toConfig.setOverdrawScore(0);
        }

        if (deptRecharge == null) {
            // 不存在则新增一条
            deptRecharge = new SysDeptRecharge();
            deptRecharge.setDeptId(toConfig.getDeptId());
            deptRecharge.setServiceCharge(toConfig.getServiceCharge());
            deptRecharge.setOverdrawScore(toConfig.getOverdrawScore());
//            deptRecharge.setOverdrawAmount(overdrawAmount);
            deptRecharge.setPermissions(toConfig.getPermissions());
            deptRecharge.setScore(0);
//            deptRecharge.setBalance(BigDecimal.ZERO);
            deptRecharge.setCreateUser(operator);
            this.save(deptRecharge);
        } else {
            // 更新
            boolean update = this.update(Wrappers.<SysDeptRecharge>lambdaUpdate()
                    .eq(SysDeptRecharge::getId, deptRecharge.getId())
                    .eq(SysDeptRecharge::getVersion, deptRecharge.getVersion())
                    .set(SysDeptRecharge::getVersion, deptRecharge.getVersion() + 1)
                    .set(SysDeptRecharge::getServiceCharge, toConfig.getServiceCharge())
                    .set(SysDeptRecharge::getOverdrawScore, toConfig.getOverdrawScore())
//                    .set(SysDeptRecharge::getOverdrawAmount, overdrawAmount)
                    .set(SysDeptRecharge::getPermissions, toConfig.getPermissions())
                    .set(SysDeptRecharge::getUpdateTime, LocalDateTime.now())
                    .set(SysDeptRecharge::getUpdateUser, operator));
            if (!update) {
                throw new RuntimeException("更新失败");
            }
        }

        return Boolean.TRUE;
    }
}




