/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.entity.dto.CommonUserDTO;
import com.qunzhixinxi.hnqz.admin.entity.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.admin.enums.GigTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.common.core.util.R;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * @author hnqz
 * @date 2017/10/31
 */
public interface SysUserService extends IService<SysUser> {

	/**
	 * 查询用户信息
	 *
	 * @param sysUser 用户
	 * @return userInfo
	 */
	UserInfo findUserInfo(SysUser sysUser);


	/**
	 * 分页查询用户信息（含有角色信息）
	 *
	 * @param page    分页对象
	 * @param userDTO 参数列表
	 * @return
	 */
	IPage getUsersWithRolePage(Page page, UserDTO userDTO);

	IPage getUsersWithRolePageNoLock(Page page, UserDTO userDTO);

	IPage selectUserList(Page page, UserDTO userDTO);

	/**
	 * 删除用户
	 *
	 * @param sysUser 用户
	 * @return boolean
	 */
	Boolean deleteUserById(SysUser sysUser);

	/**
	 * 更新当前用户基本信息
	 *
	 * @param userDto 用户信息
	 * @return Boolean
	 */
	R<Boolean> updateUserInfo(UserDTO userDto);

	/**
	 * 更新指定用户信息
	 *
	 * @param userDto 用户信息
	 * @return
	 */
	Boolean updateUser(UserDTO userDto);


	public R<Boolean> updateUserInfoVo(UserVO userVO);

	/**
	 * 通过ID查询用户信息
	 *
	 * @param id 用户ID
	 * @return 用户信息
	 */
	UserVO selectUserVoById(Integer id);

	/**
	 * 查询上级部门的用户信息
	 *
	 * @param username 用户名
	 * @return R
	 */
	List<SysUser> listAncestorUsers(String username);

	/**
	 * 保存用户信息
	 *
	 * @param userDto DTO 对象
	 * @return success/fail
	 */
	Boolean saveUser(UserDTO userDto);

	List<SysUser> getUserByPlatId(String platId);

	List<UserVO> getUserVoByPlatId(UserDTO userDTO);


	R updatePwd(UserPwd userPwd);

	SysUser getByIdIgnore(String taskUserId);

	/**
	 * 人力家认证
	 *
	 * @param sysUser
	 * @param sysDept
	 * @return
	 */
	Map<String, String> certByRenLiJia(SysUser sysUser, SysDept sysDept);

	R batchChannelCert(UserCertificationInput input);

	R getAuthInfo(UserCertificationInput input);

	/**
	 * 通过excel批量创建用户
	 *
	 * @param commonUserDTOListMap 通用用户dto
	 * @return 结果
	 */
	Map<String, String> batchByExcel(Map<String, List<CommonUserDTO>> commonUserDTOListMap);

	/**
	 * 结算时校验用户信息
	 *
	 * @param userIds         用户id集合
	 * @param subjectLocation 税源地
	 */
	void checkUserInfoToSettle(Collection<Integer> userIds, SubjectLocation subjectLocation);

	/**
	 * 校验是否超60周岁
	 *
	 * @param userVO 用户vo
	 * @return 结果
	 */
	Map<String, String> checkSixtyYearsOld(UserVO userVO);

	/**
	 * 根据用户名获取用户信息
	 *
	 * @param usernames 用户名
	 * @return 用户信息map
	 */
	Map<String, List<SysUser>> mapUsersByUsername(Collection<String> usernames);
}
