package com.qunzhixinxi.hnqz.admin.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuestion;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;

/**
 * 试卷项服务
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-16 15:59
 */
public interface WmQuestionService extends IService<WmQuestion> {

	/**
	 * 校验试卷项
	 *
	 * @param operator 操作人
	 * @param label    标题
	 * @param itemId   试卷项ID
	 * @return 校验结果
	 */
	Boolean checkQuestion(HnqzUser operator, String label, Integer itemId);

	/**
	 * 创建试题
	 *
	 * @param resource 试题信息
	 * @param operator 操作人
	 * @return 创建结果
	 */
	Boolean createQuestion(WmQuizDTO.OnQuestionCreate resource, HnqzUser operator);

	/**
	 * 创建试题
	 *
	 * @param resource 试题信息
	 * @param operator 操作人
	 * @return 创建结果
	 */
	Boolean updateQuestion(WmQuizDTO.OnQuestionUpdate resource, HnqzUser operator);

	/**
	 * 获取试题分页
	 *
	 * @param current 当前页
	 * @param size    每页记录数
	 * @param label   标题
	 * @param entId   企业ID
	 * @return 分页结果
	 */
	Page<WmQuestion> pageQuestions(Integer current, Integer size, Integer entId, String label);

	/**
	 * 获取试题详情
	 *
	 * @param itemId 试题ID
	 * @param sn     试题唯一序列号
	 * @return 试题信息
	 */
	WmQuestion getQuestionDetails(Integer itemId, String sn);

	/**
	 * 删除试题
	 *
	 * @param resource 试题信息
	 * @return 删除结果
	 */
	Boolean deleteQuestion(WmQuizDTO.OnQuestionDel resource);
}
