package com.qunzhixinxi.hnqz.admin.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuiz;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;

import java.util.Map;

/**
 * 试卷服务
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-16 15:54
 */
public interface WmQuizService extends IService<WmQuiz> {

	/**
	 * 获取试卷详情
	 *
	 * @param quizId 试卷ID
	 * @return 试卷详情
	 */
	Map<String, Object> getDetails(Integer quizId);

	/**
	 * 发布试题
	 *
	 * @param quizId   试题ID
	 * @param operator 操作人
	 * @return 发布结果
	 */
	Boolean releaseQuiz(Integer quizId, HnqzUser operator);

	/**
	 * 中止试卷
	 *
	 * @param quizId   试题ID
	 * @param operator 操作人
	 * @return 发布结果
	 */
	Boolean stopQuiz(Integer quizId, HnqzUser operator);

	/**
	 * 创建试卷
	 *
	 * @param resource 试卷信息
	 * @param user     操作人
	 * @return 试卷编号
	 */
	String createQuiz(WmQuizDTO.OnQuizCreate resource, HnqzUser user);

	/**
	 * 更新试卷
	 *
	 * @param resource 试卷信息
	 * @param user     操作人
	 * @return 更新结果
	 */
	Boolean updateQuiz(WmQuizDTO.OnQuizUpdate resource, HnqzUser user);

	/**
	 * 校验试卷信息
	 *
	 * @param expand    试卷统计信息
	 * @param serialNum 序列号
	 */
	void checkQuizExpand(WmQuiz.Expand expand, String serialNum);
}
