/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.admin.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qunzhixinxi.hnqz.admin.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.entity.dto.ReportTaskListPageDTO;
import com.qunzhixinxi.hnqz.admin.entity.vo.WmTaskVO;
import com.qunzhixinxi.hnqz.common.core.util.R;

import java.util.List;
import java.util.Map;

/**
 * 报告审核表
 *
 * @author pigx code generator
 * @date 2020-12-21 20:57:26
 */
public interface WmReportService extends IService<WmReport> {

	public R updateReportStatus(WmReport wmReport);

	R del(WmReport wmReport);

	/**
	 * 结算报告审批-任务复审
	 *
	 * @param reportId        报告id
	 * @param taskId          任务id
	 * @param approvalOpinion 审核状态(1-通过，2-不通过)
	 * @param approvalInfo    审批说明
	 * @return 复审结果
	 */
	Boolean reviewTask(String reportId, String taskId, String approvalOpinion, String approvalInfo);

	/**
	 * 结算报告审批-快速批量任务审核
	 *
	 * @param reportId        报告id
	 * @param taskIdList      任务id列表
	 * @param approvalOpinion 审核状态(1-通过，2-不通过)
	 * @param approvalInfo    审批说明
	 * @return 审核结果
	 */
	Boolean batchApprovalTask(String reportId, List<String> taskIdList, String approvalOpinion, String approvalInfo);

	R approvalReport(WmReport wmReport, List<String> taskIds, String approvalOpinion, String approvalInfo);

	/**
	 * 企业根据任务类型审核任务
	 *
	 * @param taskIds
	 * @param approvalOpinion
	 * @param approvalInfo
	 * @return
	 */
	R<?> approvalReportByTaskType(List<String> taskIds, String approvalOpinion, String approvalInfo);

	/**
	 * 报告管理--查看
	 *
	 * @param page                  页
	 * @param reportTaskListPageDTO 报告任务列表DTO
	 * @return {@link R}
	 */
	R getTaskListPage(Page page, ReportTaskListPageDTO reportTaskListPageDTO);

	/**
	 * 根据积分包id查询待审核任务详情列表
	 *
	 * @param reportId 报告id
	 * @param taskTypeId 任务类型id
	 * @return 列表结果
	 */
	List<Map<String, Object>> listTaskInfo(Integer reportId, String taskTypeId);

	IPage<WmReport> getReportListPage(Page page, WmReport wmReport, String scoreId, String name);

	/**
	 * 根据任务类型查询任务
	 *
	 * @param page
	 * @param taskTypeName
	 * @param entTaskStatus
	 * @return
	 */
	R<?> taskPageByTaskType(Page<WmTaskVO> page, String taskTypeName, String[] entTaskStatus);


}
