package com.qunzhixinxi.hnqz.admin.service.gig.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.dto.OladingCommonRequest;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.config.TaxNotifyConfig;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.HuiqiyunInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.QifuInputRecord;
import com.qunzhixinxi.hnqz.admin.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.entity.SysUserSub;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageSettleNote;
import com.qunzhixinxi.hnqz.admin.entity.dto.UserSignCertDTO;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.GigCallBackStatus;
import com.qunzhixinxi.hnqz.admin.enums.OladingCertStatus;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.ZhongYiYunCertStatus;
import com.qunzhixinxi.hnqz.admin.event.OladingFinishTaskEvent;
import com.qunzhixinxi.hnqz.admin.mapper.QifuInputRecordMapper;
import com.qunzhixinxi.hnqz.admin.service.HuiqiyunInputRecordService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysDeptSubService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.SysUserSubService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageSettleNoteService;
import com.qunzhixinxi.hnqz.admin.service.gig.GigThirdApiService;
import com.qunzhixinxi.hnqz.admin.service.gig.SysUserSignCertService;
import com.qunzhixinxi.hnqz.admin.util.AESUtil;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.exception.ValidateCodeException;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.net.URLDecoder;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author lixuesong
 * @date 2022年04月17日 11:55
 */
@Slf4j
@Service
@AllArgsConstructor
public class GigThirdApiServiceImpl implements GigThirdApiService {

	private final RedisTemplate redisTemplate;
	private final SysUserSubService sysUserSubService;
	private final SysUserSignCertService sysUserSignCertService;
	private final WmScorePackageSettleNoteService scorePackageSettleNoteService;
	private final WmScorePackageService scorePackageService;
	private final SysDeptService sysDeptService;
	private final SysDeptSubService sysDeptSubService;
	private final SysUserService sysUserService;
	private final UpmsConfig upmsConfig;
	private final TaxNotifyConfig taxNotifyConfig;

	private final ApplicationEventPublisher applicationEventPublisher;

	private final HuiqiyunInputRecordService huiqiyunInputRecordService;

	/**
	 * 福建企赋结算回调业务处理
	 *
	 * @param bodyJson
	 * @return
	 */
	@Override
	public boolean qiFuSettleNotify(com.alibaba.fastjson.JSONObject bodyJson) {
		String reqId = bodyJson.getString("reqId");
		String state = bodyJson.getString("state");

		// 根据orderNo查询结算单
		List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
				.eq(WmScorePackageSettleNote::getStreamId, reqId));
		if (CollUtil.isEmpty(settleNoteList)) {
			log.warn("订单reqId={}对应结算单不存在", reqId);
			return false;
		}

		WmScorePackageSettleNote tmp = settleNoteList.get(0);

		int yaoyiStatus = 0;
		// 	state取值：
		// 	SETSUCC结算成功;
		//	SETFAIL:结算失败;
		//	TICREF:已退票
		if ("SETSUCC".equals(state)) {
			// 对应要易结算成功状态
			yaoyiStatus = 1;
		} else if ("SETFAIL".equals(state)) {
			yaoyiStatus = 2;
		} else {
			log.warn("当前结算state不处理");
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());
			return false;
		}

		// 修改结算状态
		final int finalYaoyiStatus = yaoyiStatus;
		List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
			WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSettleNoteStatus(finalYaoyiStatus);
			if (finalYaoyiStatus == 1){
				updateEntity.setNotifyTime(LocalDateTime.now());
			}
			updateEntity.setUpdateTime(LocalDateTime.now());
			return updateEntity;
		}).collect(Collectors.toList());

		scorePackageSettleNoteService.updateBatchById(updateList);

		WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

		// 遥领不修改
		if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
			// 更新积分包状态
			log.info("更新积分包状态:{}", tmp);
			WmScorePackage wmScorePackage = new WmScorePackage();
			wmScorePackage.setId(tmp.getPackageId());
			wmScorePackage.setSettleStatus(yaoyiStatus);
			wmScorePackage.setUpdateTime(LocalDateTime.now());
			scorePackageService.updateById(wmScorePackage);
		}

		redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

		return true;
	}

	/**
	 * 福建企赋签约回调业务处理
	 *
	 * @param newBodyJson
	 * @return
	 */
	@Override
	public boolean qiFuSignNotify(com.alibaba.fastjson.JSONObject newBodyJson) {

		String idCard = newBodyJson.getString("idCard");
		String mobile = newBodyJson.getString("mobile");
		String name = newBodyJson.getString("name");
		String state = newBodyJson.getString("state");
		String resMsg = newBodyJson.getString("resMsg");

		// 查询当前用户
		SysUser sysUser = sysUserService.getOne(Wrappers.<SysUser>lambdaQuery()
				.eq(SysUser::getRealname, name)
				.eq(SysUser::getIdCardNumber, idCard)
				.eq(SysUser::getUsername, mobile)
				.eq(SysUser::getDelFlag, DelEnum.NOT_DEL.val()));
		if (sysUser == null) {
			log.warn("用户不存在name={},idCard={},mobile={}", name, idCard, mobile);
			return false;
		}

		// 查询对应userSub数据
		SysUserSub userSub = sysUserSubService.getOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getUserId, sysUser.getUserId())
				.eq(SysUserSub::getSubjectLocation, SubjectLocation.QI_FU));
		if (userSub == null) {
			log.warn("没有查到对应userSub数据 userId={}", sysUser.getUserId());
			return false;
		}

		// 更新userSub
		SysUserSub updateUserSub = new SysUserSub();
		updateUserSub.setId(userSub.getId());
		if ("SIGN".equals(state)) {
			// 已签约对应要易已认证
			updateUserSub.setCertStatus(ZhongYiYunCertStatus.CERT.getCode());
		}
		updateUserSub.setCertRemark(String.format("%s;%s", userSub.getCertRemark(), String.format("签约回调：%s;", resMsg)));
		updateUserSub.setFailReason(String.format("签约回调：%s;", resMsg));
		updateUserSub.setCallbackStatus(GigCallBackStatus.RETURNED);
		LocalDateTime now = LocalDateTime.now();
		updateUserSub.setUpdateTime(now);
		updateUserSub.setCallbackTime(now);
		updateUserSub.setUpdateUser(0);
		sysUserSubService.updateById(updateUserSub);

		return true;
	}

	/**
	 * 中智结算回调业务处理
	 *
	 * @param bodyJson 请求数据
	 * @return 是否成功
	 */
	@Override
	public boolean zhongzhiSettleNotify(JSONObject bodyJson) {
		// 三方订单号
		String thirdOrderNo = bodyJson.getStr("thirdOrderNo");
		// 支付结果，SUCCESS成功，FAIL失败
		String payResult = bodyJson.getStr("payResult");

		// 根据orderNo查询结算单
		List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
				.eq(WmScorePackageSettleNote::getStreamId, thirdOrderNo));
		if (CollUtil.isEmpty(settleNoteList)) {
			log.warn("订单thirdOrderNo={}对应结算单不存在", thirdOrderNo);
			return false;
		}

		WmScorePackageSettleNote tmp = settleNoteList.get(0);

		int yaoyiStatus = 0;
		if ("SUCCESS".equals(payResult)) {
			// 对应要易结算成功状态
			yaoyiStatus = 1;
		} else if ("FAIL".equals(payResult)) {
			yaoyiStatus = 2;
		} else {
			log.warn("当前结算payResult不处理");
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());
			return false;
		}

		// 修改结算状态
		final int finalYaoyiStatus = yaoyiStatus;
		List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
			WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSettleNoteStatus(finalYaoyiStatus);
			if (finalYaoyiStatus == 1){
				updateEntity.setNotifyTime(LocalDateTime.now());
			}
			updateEntity.setUpdateTime(LocalDateTime.now());
			return updateEntity;
		}).collect(Collectors.toList());

		scorePackageSettleNoteService.updateBatchById(updateList);

		WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

		// 遥领不修改
		if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
			// 更新积分包状态
			log.info("更新积分包状态:{}", tmp);
			WmScorePackage wmScorePackage = new WmScorePackage();
			wmScorePackage.setId(tmp.getPackageId());
			wmScorePackage.setSettleStatus(yaoyiStatus);
			wmScorePackage.setUpdateTime(LocalDateTime.now());
			scorePackageService.updateById(wmScorePackage);
		}

		redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

		return true;
	}

	/**
	 * 众蚁云结算回调业务处理
	 *
	 * @param bodyJson
	 * @return
	 */
	@Override
	public boolean zhongYiYunSettleNotify(JSONObject bodyJson) {
		String reqId = bodyJson.getStr("reqId");
		String state = bodyJson.getStr("state");

		// 根据orderNo查询结算单
		List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
				.eq(WmScorePackageSettleNote::getStreamId, reqId));
		if (CollUtil.isEmpty(settleNoteList)) {
			log.warn("订单reqId={}对应结算单不存在", reqId);
			return false;
		}

		WmScorePackageSettleNote tmp = settleNoteList.get(0);

		int yaoyiStatus = 0;
		// 	state取值：
		// 	SETSUCC结算成功;
		//	SETFAIL:结算失败;
		//	TICREF:已退票
		if ("SETSUCC".equals(state)) {
			// 对应要易结算成功状态
			yaoyiStatus = 1;
		} else if ("SETFAIL".equals(state)) {
			yaoyiStatus = 2;
		} else {
			log.warn("当前结算state不处理");
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());
			return false;
		}

		// 修改结算状态
		final int finalYaoyiStatus = yaoyiStatus;
		List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
			WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSettleNoteStatus(finalYaoyiStatus);
			if (finalYaoyiStatus == 1){
				updateEntity.setNotifyTime(LocalDateTime.now());
			}
			updateEntity.setUpdateTime(LocalDateTime.now());
			return updateEntity;
		}).collect(Collectors.toList());

		scorePackageSettleNoteService.updateBatchById(updateList);

		WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

		// 遥领不修改
		if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
			// 更新积分包状态
			log.info("更新积分包状态:{}", tmp);
			WmScorePackage wmScorePackage = new WmScorePackage();
			wmScorePackage.setId(tmp.getPackageId());
			wmScorePackage.setSettleStatus(yaoyiStatus);
			wmScorePackage.setUpdateTime(LocalDateTime.now());
			scorePackageService.updateById(wmScorePackage);
		}

		redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

		return true;
	}

	/**
	 * 众蚁云签约回调业务处理
	 *
	 * @param newBodyJson
	 * @return
	 */
	@Override
	public boolean zhongYiYunSignNotify(JSONObject newBodyJson) {

		String idCard = newBodyJson.getStr("idCard");
		String mobile = newBodyJson.getStr("mobile");
		String name = newBodyJson.getStr("name");
		String state = newBodyJson.getStr("state");
		String resMsg = newBodyJson.getStr("resMsg");

		// 查询当前用户
		SysUser sysUser = sysUserService.getOne(Wrappers.<SysUser>lambdaQuery()
				.eq(SysUser::getRealname, name)
				.eq(SysUser::getIdCardNumber, idCard)
				.eq(SysUser::getUsername, mobile)
				.eq(SysUser::getDelFlag, DelEnum.NOT_DEL.val()));
		if (sysUser == null) {
			log.warn("用户不存在name={},idCard={},mobile={}", name, idCard, mobile);
			return false;
		}

		// 查询对应userSub数据
		SysUserSub userSub = sysUserSubService.getOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getUserId, sysUser.getUserId())
				.eq(SysUserSub::getSubjectLocation, SubjectLocation.ZHONG_YI_YUN));
		if (userSub == null) {
			log.warn("没有查到对应userSub数据 userId={}", sysUser.getUserId());
			return false;
		}

		// 更新userSub
		SysUserSub updateUserSub = new SysUserSub();
		updateUserSub.setId(userSub.getId());
		if ("SIGN".equals(state)) {
			// 已签约对应要易已认证
			updateUserSub.setCertStatus(ZhongYiYunCertStatus.CERT.getCode());
		}
		updateUserSub.setCertRemark(String.format("%s;%s", userSub.getCertRemark(), String.format("签约回调：%s;", resMsg)));
		updateUserSub.setFailReason(String.format("签约回调：%s;", resMsg));
		updateUserSub.setCallbackStatus(GigCallBackStatus.RETURNED);
		LocalDateTime now = LocalDateTime.now();
		updateUserSub.setUpdateTime(now);
		updateUserSub.setCallbackTime(now);
		updateUserSub.setUpdateUser(0);
		sysUserSubService.updateById(updateUserSub);

		return true;
	}

	/**
	 * 汇企云回调业务处理
	 *
	 * @param reqMsgId
	 * @param reqBody
	 * @param inputId
	 * @return
	 */
	@Override
	public boolean huiqiyunNotify(String reqMsgId, JSONObject reqBody, Integer inputId) {

		// 查询privateKey
		List<SysDeptSub> deptSubList = sysDeptSubService.list(Wrappers.<SysDeptSub>lambdaQuery()
				.eq(SysDeptSub::getEnableFlag, SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode())
				.eq(SysDeptSub::getSubjectLocation, SubjectLocation.HUI_QI_YUN)
				.isNotNull(SysDeptSub::getAppSecret));
		if (CollUtil.isEmpty(deptSubList)) {
			log.warn("汇企云privateKey不存在");
			return false;
		}

		// 解密body
		Map<String, Object> decryptData = this.decryptData(reqBody, deptSubList.get(0).getAppSecret());
		log.info("汇企云解密后body={}", decryptData);

		// 保存解密后的body
		HuiqiyunInputRecord inputRecord = new HuiqiyunInputRecord();
		inputRecord.setId(inputId);
		inputRecord.setDecryptBody(JSONUtil.toJsonStr(decryptData));
		huiqiyunInputRecordService.updateById(inputRecord);

		// 内层body
		String bodyStr = (String) decryptData.get("body");
		JSONObject bodyJson = JSONUtil.parseObj(bodyStr);

		// 批次出款流水号
		String batchNo = bodyJson.getStr("batchNo");
		// 批次成功总笔数
		Integer totalSuccessNum = bodyJson.getInt("totalSuccessNum");
		// 批次成功总金额
		String totalSuccessAmt = bodyJson.getStr("totalSuccessAmt");
		// 批次失败总笔数
		Integer totalFailNum = bodyJson.getInt("totalFailNum");
		// 批次失败总金额
		String totalFailAmt = bodyJson.getStr("totalFailAmt");
		// 批次请求总笔数
		Integer totalNum = bodyJson.getInt("totalNum");
		// 批次请求总金额
		String totalAmt = bodyJson.getStr("totalAmt");

		int yaoyiStatus;
		if (totalSuccessNum.equals(totalNum)) {
			// 批次成功笔数=批次总笔数，则认为是成功
			yaoyiStatus = 1;
		} else {
			yaoyiStatus = 2;
		}

		// 根据orderNo查询结算单
		List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
				.eq(WmScorePackageSettleNote::getStreamId, batchNo));
		if (CollUtil.isEmpty(settleNoteList)) {
			log.warn("订单batchNo={}对应结算单不存在", batchNo);
			return false;
		}

		// 修改结算状态
		List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
			WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSettleNoteStatus(yaoyiStatus);
			if (yaoyiStatus == 1){
				updateEntity.setNotifyTime(LocalDateTime.now());
			}
			updateEntity.setUpdateTime(LocalDateTime.now());
			return updateEntity;
		}).collect(Collectors.toList());

		scorePackageSettleNoteService.updateBatchById(updateList);

		WmScorePackageSettleNote tmp = settleNoteList.get(0);
		WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

		// 遥领不修改
		if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
			// 更新积分包状态
			log.info("更新积分包状态:{}", tmp);
			WmScorePackage wmScorePackage = new WmScorePackage();
			wmScorePackage.setId(tmp.getPackageId());
			wmScorePackage.setSettleStatus(yaoyiStatus);
			wmScorePackage.setUpdateTime(LocalDateTime.now());
			scorePackageService.updateById(wmScorePackage);
		}

		redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

		return true;
	}

	/**
	 * 通用解密方法
	 *
	 * @param encryptData 加密后的数据
	 * @param privateKey  秘钥
	 * @return 解密数据
	 */
	public Map<String, Object> decryptData(final Map<String, Object> encryptData, final String privateKey) {

		Map<String, Object> res = new HashMap<>(encryptData.size());

		try {
			String headStr = (String) encryptData.get("head");
			String srcHead = URLDecoder.decode(headStr, "UTF-8");


			String cipherStr = (String) encryptData.get("cipher");
			String bodyStr = (String) encryptData.get("body");
			String signStr = (String) encryptData.get("sign");
			String srcBody = null;
			if (StringUtils.isNotBlank(bodyStr)) {
				String resBody = AESUtil.decrypt(bodyStr, privateKey);
				srcBody = URLDecoder.decode(resBody, "UTF-8");
			}
			res.put("head", srcHead);
			res.put("cipher", cipherStr);
			res.put("body", srcBody);
			res.put("sign", signStr);

		} catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}

	/**
	 * 浪潮(灵才接好活)结算回调
	 *
	 * @param channel 通知类型（recharge 充值，batchQuery 批量结果，refund  退款）
	 * @param batchNo 项目单号
	 * @param orderStatus 项目订单状态（0未支付，1已支付，2已取消）
	 * @return 是否成功状态
	 */
	@Override
	public String lingcaiaiNotify(String channel, String batchNo, Integer orderStatus) {
		switch (channel) {
			case "recharge":
				// 充值 暂不实现~
				break;
			case "refund":
				// 退款 暂不实现~
				break;
			case "batchQuery":
				// 订单状态转换为
				int yaoyiStatus;
				switch (orderStatus){
					case 1:
						yaoyiStatus = 1;
						break;
					default:
						yaoyiStatus = 2;
						break;
				}

				// 批量结果
				// 根据orderNo查询结算单
				List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
						.eq(WmScorePackageSettleNote::getStreamId, batchNo));
				if (CollUtil.isEmpty(settleNoteList)) {
					log.warn("订单streamId={}对应结算单不存在", batchNo);
					return "fail";
				}

				// 修改结算状态
				List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
					WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
					updateEntity.setId(note.getId());
					updateEntity.setSettleNoteStatus(yaoyiStatus);
					if (yaoyiStatus == 1){
						updateEntity.setNotifyTime(LocalDateTime.now());
					}
					updateEntity.setUpdateTime(LocalDateTime.now());
					return updateEntity;
				}).collect(Collectors.toList());

				scorePackageSettleNoteService.updateBatchById(updateList);

				WmScorePackageSettleNote tmp = settleNoteList.get(0);
				WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

				// 遥领不修改
				if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
					// 更新积分包状态
					log.info("更新积分包状态:{}", tmp);
					WmScorePackage wmScorePackage = new WmScorePackage();
					wmScorePackage.setId(tmp.getPackageId());
					wmScorePackage.setSettleStatus(yaoyiStatus);
					wmScorePackage.setUpdateTime(LocalDateTime.now());
					scorePackageService.updateById(wmScorePackage);
				}

				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

				break;
			default:
		}

		return "success";
	}

	/**
	 * 正启之星回调业务处理
	 *
	 * @param actionName 通知类型 order_check_result_notify：订单校验结果通知，order_call_back_notification：订单下发回调通知，task_redispatch_notification：单笔重试回调通知
	 * @param orderUuid 订单uuid
	 * @param orderStatus
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void xinbadaNotify(String actionName, String orderUuid, Integer orderStatus) {

		switch (actionName) {
			// 订单下发回调通知
			case "order_call_back_notification":

				// 订单状态转换为
				int yaoyiStatus;
				switch (orderStatus){
					case 50:
						yaoyiStatus = 1;
						break;
					default:
						yaoyiStatus = 2;
						break;
				}

				// 根据orderNo查询结算单
				List<WmScorePackageSettleNote> settleNoteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
						.eq(WmScorePackageSettleNote::getStreamId, orderUuid));
				if (CollUtil.isEmpty(settleNoteList)) {
					log.warn("订单orderUuid={}对应结算单不存在", orderUuid);
					return;
				}

				// 修改结算状态
				List<WmScorePackageSettleNote> updateList = settleNoteList.stream().map(note -> {
					WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
					updateEntity.setId(note.getId());
					updateEntity.setSettleNoteStatus(yaoyiStatus);
					if (yaoyiStatus == 1){
						updateEntity.setNotifyTime(LocalDateTime.now());
					}
					updateEntity.setUpdateTime(LocalDateTime.now());
					return updateEntity;
				}).collect(Collectors.toList());

				scorePackageSettleNoteService.updateBatchById(updateList);

				WmScorePackageSettleNote tmp = settleNoteList.get(0);
				WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

				// 遥领不修改
				if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
					// 更新积分包状态
					log.info("更新积分包状态:{}", tmp);
					WmScorePackage wmScorePackage = new WmScorePackage();
					wmScorePackage.setId(tmp.getPackageId());
					wmScorePackage.setSettleStatus(yaoyiStatus);
					wmScorePackage.setUpdateTime(LocalDateTime.now());
					scorePackageService.updateById(wmScorePackage);
				}

				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

				break;
			default:
				log.warn("通知类型暂不处理actionName={}", actionName);
				break;
		}

	}

	/**
	 * 自由职家回调业务处理
	 *
	 * @param requestId
	 * @param req
	 */
	@Override
	public void oladingNotify(String requestId, OladingCommonRequest.Req req) {
		// 请求接口名称
		String interfaceName = req.getName();
		// 请求data
		Map<String, Object> data = req.getData();
		// 接口名称enum
		OladingCommonRequest.InterfaceName interfaceNameEnum = Enum.valueOf(OladingCommonRequest.InterfaceName.class, interfaceName);
		// 状态
		String status = "CREATE_ORDER_DETAIL".equals(interfaceName) ? (String) data.get("remit_status")  : (String) data.get("status");
		// 截取前缀部分，从而获取到系统内部的requestId
		String requestIdPrefix = StrUtil.split(requestId, StrUtil.UNDERLINE).get(0);

		if (StrUtil.isBlank(status)) {
			throw new ValidateCodeException("status为空");
		}

		switch (interfaceNameEnum) {
			case SIGN_UP:
				// 签约
				OladingCommonRequest.SignUpStatus signUpCertStatus = Enum.valueOf(OladingCommonRequest.SignUpStatus.class, status);
				this.oladingSignUp(requestIdPrefix, signUpCertStatus, req);
				break;
			case UPLOAD_ID_CARD_IMAGE:
				// 证照上传
				OladingCommonRequest.UploadIdCardImageStatus uploadIdCertStatus =
						Enum.valueOf(OladingCommonRequest.UploadIdCardImageStatus.class, status);
				this.oladingUploadIdCardImage(requestIdPrefix, uploadIdCertStatus, req);
				break;
			case UPLOAD_TAXPAYER_AGREEMENT:
				// 上传企业纳税人协议
				OladingCommonRequest.UploadTaxpayerAgreementStatus agreementStatus =
						Enum.valueOf(OladingCommonRequest.UploadTaxpayerAgreementStatus.class, status);
				this.oladingUploadTaxpayerAgreement(requestIdPrefix, agreementStatus, req);
				break;
			case CREATE_ORDER_V2:
				// 创建代付订单结果
				this.oladingCreateOrderV2(requestId, Enum.valueOf(OladingCommonRequest.CreateOrderV2Status.class, status), data);
				break;
			case CREATE_ORDER_DETAIL:
				this.oladingCreateOrderDetail(requestId, Enum.valueOf(OladingCommonRequest.RemitStatusStatus.class, status));
				// 打款订单
				break;
			default:
				throw new ValidateCodeException("回调方法处理不存在");
		}
	}

	/**
	 * 创建代付订单回调处理
	 *
	 * @param requestId 流水号
	 * @param status    受理状态
	 */
	@Transactional(rollbackFor = Exception.class)
	public void oladingCreateOrderDetail(String requestId, OladingCommonRequest.RemitStatusStatus status) {

		int yaoyiStatus;
		switch (status){
			case REMIT:
				yaoyiStatus = 1;
				break;
			case FAIL:
				yaoyiStatus = 2;
				break;
			default:
				log.error("订单状态错误：{}", status);
				return;
		}

		List<WmScorePackageSettleNote> noteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery().eq(WmScorePackageSettleNote::getStreamId, requestId));

		if (CollectionUtils.isNotEmpty(noteList)) {

			log.warn("阿拉钉内部打款：{}", requestId);

			List<WmScorePackageSettleNote> updateList = noteList.stream().map(note -> {
				WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
				updateEntity.setId(note.getId());
				updateEntity.setSettleNoteStatus(yaoyiStatus);
				if (yaoyiStatus == 1){
					updateEntity.setNotifyTime(LocalDateTime.now());
				}
				updateEntity.setUpdateTime(LocalDateTime.now());
				return updateEntity;
			}).collect(Collectors.toList());

			scorePackageSettleNoteService.updateBatchById(updateList);

			WmScorePackageSettleNote tmp = noteList.get(0);
			WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

			// 遥领不修改
			if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
				// 更新积分包状态
				log.info("更新积分包状态:{}", tmp);
				WmScorePackage wmScorePackage = new WmScorePackage();
				wmScorePackage.setId(tmp.getPackageId());
				wmScorePackage.setSettleStatus(yaoyiStatus);
				wmScorePackage.setUpdateTime(LocalDateTime.now());
				scorePackageService.updateById(wmScorePackage);
			}
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());

			// 如果结算成功，则异步发起完单操作，是否成功不做处理
			try {
				if (yaoyiStatus == 1) {
					applicationEventPublisher.publishEvent(new OladingFinishTaskEvent(noteList));
				}
			} catch (Exception e) {
				log.warn("阿拉钉异步发起完单操作异常requestId={}", requestId);
				log.error("阿拉钉异步发起完单操作异常", e);
			}
		}
	}

	/**
	 * 创建代付订单回调处理
	 *
	 * @param requestId 流水号
	 * @param status    受理状态
	 */
	@SuppressWarnings("unchecked")
	@Transactional(rollbackFor = Exception.class)
	public void oladingCreateOrderV2(String requestId, OladingCommonRequest.CreateOrderV2Status status, Map<String, Object> data) {

		boolean accept = OladingCommonRequest.CreateOrderV2Status.ACCEPT.equals(status);

		int yaoyiStatus;
		if (!accept) {
			yaoyiStatus = 2;
		} else {
			List<Map<String, Object>> orderList = (List<Map<String, Object>>) data.get("remit_order_list");
			Map<String, Object> o = orderList.get(0);
			String remitStatue = (String) o.get("remit_status");
			OladingCommonRequest.RemitStatusStatus remitStatusStatus = Enum.valueOf(OladingCommonRequest.RemitStatusStatus.class, remitStatue);
			switch (remitStatusStatus){
				case REMIT:
					yaoyiStatus = 1;
					break;
				case FAIL:
					yaoyiStatus = 2;
					break;
				default:
					return;
			}
		}


		List<WmScorePackageSettleNote> noteList = scorePackageSettleNoteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery().eq(WmScorePackageSettleNote::getStreamId, requestId));

		if (CollectionUtils.isNotEmpty(noteList)) {

			log.warn("阿拉丁创建代付订单：{}", requestId);

			List<WmScorePackageSettleNote> updateList = noteList.stream().map(note -> {
				WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
				updateEntity.setId(note.getId());
				updateEntity.setSettleNoteStatus(yaoyiStatus);
				if (yaoyiStatus == 1){
					updateEntity.setNotifyTime(LocalDateTime.now());
				}
				updateEntity.setUpdateTime(LocalDateTime.now());
				return updateEntity;
			}).collect(Collectors.toList());

			scorePackageSettleNoteService.updateBatchById(updateList);

			WmScorePackageSettleNote tmp = noteList.get(0);
			WmScorePackage scorePackage = scorePackageService.getById(tmp.getPackageId());

			// 遥领不修改
			if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
				// 更新积分包状态
				log.info("更新积分包状态:{}", tmp);
				WmScorePackage wmScorePackage = new WmScorePackage();
				wmScorePackage.setId(tmp.getPackageId());
				wmScorePackage.setSettleStatus(yaoyiStatus);
				wmScorePackage.setUpdateTime(LocalDateTime.now());
				scorePackageService.updateById(wmScorePackage);
			}

			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());
		}
	}


	/**
	 * 自由职家签约结果回调
	 *
	 * @param requestId
	 * @param signUpCertStatus
	 * @param req
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void oladingSignUp(String requestId, OladingCommonRequest.SignUpStatus signUpCertStatus, OladingCommonRequest.Req req) {
		// 先查询requestId是否存在
		SysUserSub userSub = sysUserSubService.getOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getRequestId, requestId));
		if (userSub == null) {
			log.error("{} requestId不存在", requestId);
			throw new ValidateCodeException("requestId不存在");
		}

		Map<String, String> subMap = new HashMap<>();
		SysUserSub updateUserSub = new SysUserSub();
		if (OladingCommonRequest.SignUpStatus.SIGN.equals(signUpCertStatus)) {
			subMap.put("certStatus", OladingCertStatus.SIGNED.name());
			updateUserSub.setCertStatus(Integer.valueOf(OladingCertStatus.SIGNED.getCode()));
		} else {
			subMap.put("certStatus", OladingCertStatus.UN_SIGNED.name());
			updateUserSub.setCertStatus(Integer.valueOf(OladingCertStatus.UN_SIGNED.getCode()));
		}
		updateUserSub.setId(userSub.getId());
		updateUserSub.setUpdateTime(LocalDateTime.now());
		updateUserSub.setUpdateUser(0);
		updateUserSub.setCallbackStatus(GigCallBackStatus.RETURNED);
		if ("0".equals(req.getCode())) {
			updateUserSub.setFailReason(String.valueOf(req.getData().get("failReason")));
		} else {
			updateUserSub.setFailReason(req.getMessage());
		}
		// 更新状态
		sysUserSubService.updateById(updateUserSub);

		// 结果放到redis
		String key = CacheConstants.USER_SIGN_CERT_RESPONSE_KEY + userSub.getUserId();
		Map<String, Object> result = MapUtil.newHashMap(2);
		result.put(userSub.getGigType().name(), new HashMap<>(subMap));

		// 查询redis是否有同时发起的标记，查询出对应的状态放到redis
		String oladingRedisKey = CacheConstants.OLADING_CERT_SEND_CHANNEL + userSub.getUserId();
		String subjectLocationJson = (String) redisTemplate.opsForValue().get(oladingRedisKey);
		if (JSONUtil.isJsonArray(subjectLocationJson)) {
			JSONArray subjectLocationArray = JSONUtil.parseArray(subjectLocationJson);
			List<SubjectLocation> subjectLocationList = subjectLocationArray.stream()
					.map(o -> Enum.valueOf(SubjectLocation.class, String.valueOf(o)))
					.collect(Collectors.toList());
			List<SysUserSub> subList = sysUserSubService.list(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, userSub.getUserId())
					.eq(SysUserSub::getDeptId, userSub.getDeptId())
					.in(SysUserSub::getSubjectLocation, subjectLocationList));
			Map<String, Map<String, String>> dbResultMap = subList.stream()
					.collect(Collectors.toMap(sysUserSub -> sysUserSub.getSubjectLocation().name(),
							sysUserSub -> MapUtil.builder("certStatus",
									OladingCertStatus.resolve(String.valueOf(sysUserSub.getCertStatus())).name()).build()));
			result.putAll(dbResultMap);
		}

		log.info("缓存认证结果到redis：{}", JSONUtil.toJsonStr(result));
		redisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(result), 1, TimeUnit.MINUTES);

		// 如果成功，则发送证照上传MQ
//		if (OladingCommonRequest.SignUpStatus.SIGN.equals(signUpCertStatus)) {
//			log.info("=============================发送证照上传MQ==================================");
//			UserSignCertDTO userSignCertDTO = new UserSignCertDTO();
//			userSignCertDTO.setUserId(userSub.getUserId());
//			userSignCertDTO.setGigTypeList(Collections.singletonList(userSub.getGigType()));
//			sysUserSignCertService.batchChannelCert(userSignCertDTO);
//		}
	}

	/**
	 * 自由职家证照上传结果回调
	 *
	 * @param requestId
	 * @param uploadIdCertStatus
	 */
	@Override
	public void oladingUploadIdCardImage(String requestId, OladingCommonRequest.UploadIdCardImageStatus uploadIdCertStatus, OladingCommonRequest.Req req) {
		// 先查询requestId是否存在
		SysUserSub userSub = sysUserSubService.getOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getRequestId, requestId));
		if (userSub == null) {
			log.error("{} requestId不存在", requestId);
			throw new ValidateCodeException("requestId不存在");
		}

		Map<String, String> subMap = new HashMap<>();
		LambdaUpdateWrapper<SysUserSub> updateUserSubWrapper = Wrappers.lambdaUpdate();
		updateUserSubWrapper.eq(SysUserSub::getId, userSub.getId());
		if (OladingCommonRequest.UploadIdCardImageStatus.IDENTIFY_SUCCESS.equals(uploadIdCertStatus)) {
			subMap.put("certStatus", OladingCertStatus.SIGNED.name());
			updateUserSubWrapper.set(SysUserSub::getCertStatus, Integer.valueOf(OladingCertStatus.SIGNED.getCode()));
		} else {
			subMap.put("certStatus", OladingCertStatus.SIGNED.name());
			updateUserSubWrapper.set(SysUserSub::getCertStatus, Integer.valueOf(OladingCertStatus.SIGNED.getCode()));
			updateUserSubWrapper.set(SysUserSub::getIdCardFrontUrl, null);
			updateUserSubWrapper.set(SysUserSub::getIdCardBackUrl, null);
		}
		LocalDateTime now = LocalDateTime.now();
		updateUserSubWrapper.set(SysUserSub::getUpdateTime, now);
		updateUserSubWrapper.set(SysUserSub::getCallbackTime, now);
		updateUserSubWrapper.set(SysUserSub::getUpdateUser, 0);
		updateUserSubWrapper.set(SysUserSub::getCallbackStatus, GigCallBackStatus.RETURNED);
		if ("0".equals(req.getCode())) {
			updateUserSubWrapper.set(SysUserSub::getFailReason, String.valueOf(req.getData().get("failReason")));
		} else {
			updateUserSubWrapper.set(SysUserSub::getFailReason, req.getMessage());
		}
		// 更新状态
		sysUserSubService.update(updateUserSubWrapper);

		// 结果放到redis
		String key = CacheConstants.USER_SIGN_CERT_RESPONSE_KEY + userSub.getUserId();
		Map<String, Object> result = MapUtil.newHashMap(2);
		result.put(userSub.getGigType().name(), new HashMap<>(subMap));

		// 查询redis是否有同时发起的标记，查询出对应的状态放到redis
		String oladingRedisKey = CacheConstants.OLADING_CERT_SEND_CHANNEL + userSub.getUserId();
		String subjectLocationJson = (String) redisTemplate.opsForValue().get(oladingRedisKey);
		if (JSONUtil.isJsonArray(subjectLocationJson)) {
			JSONArray subjectLocationArray = JSONUtil.parseArray(subjectLocationJson);
			List<SubjectLocation> subjectLocationList = subjectLocationArray.stream()
					.map(o -> Enum.valueOf(SubjectLocation.class, String.valueOf(o)))
					.collect(Collectors.toList());
			List<SysUserSub> subList = sysUserSubService.list(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, userSub.getUserId())
					.eq(SysUserSub::getDeptId, userSub.getDeptId())
					.in(SysUserSub::getSubjectLocation, subjectLocationList));
			Map<String, Map<String, String>> dbResultMap = subList.stream()
					.collect(Collectors.toMap(sysUserSub -> sysUserSub.getSubjectLocation().name(),
							sysUserSub -> MapUtil.builder("certStatus",
									OladingCertStatus.resolve(String.valueOf(sysUserSub.getCertStatus())).name()).build()));
			result.putAll(dbResultMap);
		}

		log.info("缓存认证结果到redis：{}", JSONUtil.toJsonStr(result));
		redisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(result), 1, TimeUnit.MINUTES);


		// 如果成功，则发送协议上传MQ
		if (OladingCommonRequest.UploadIdCardImageStatus.IDENTIFY_SUCCESS.equals(uploadIdCertStatus)) {
			log.info("=============================发送协议上传MQ==================================");
			UserSignCertDTO userSignCertDTO = new UserSignCertDTO();
			userSignCertDTO.setUserId(userSub.getUserId());
			userSignCertDTO.setSubjectLocation(userSub.getSubjectLocation());
			sysUserSignCertService.batchChannelCert(userSignCertDTO);
		}
	}

	/**
	 * 自由职家上传企业纳税人协议回调
	 *
	 * @param requestId
	 * @param agreementStatus
	 */
	@Override
	public void oladingUploadTaxpayerAgreement(String requestId, OladingCommonRequest.UploadTaxpayerAgreementStatus agreementStatus, OladingCommonRequest.Req req) {
		// 先查询requestId是否存在
		SysUserSub userSub = sysUserSubService.getOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getRequestId, requestId));
		if (userSub == null) {
			log.error("{} requestId不存在", requestId);
			throw new ValidateCodeException("requestId不存在");
		}

		Map<String, String> subMap = new HashMap<>();
		LambdaUpdateWrapper<SysUserSub> updateUserSubWrapper = Wrappers.lambdaUpdate();
		updateUserSubWrapper.eq(SysUserSub::getId, userSub.getId());
		if (OladingCommonRequest.UploadTaxpayerAgreementStatus.SUCCESS.equals(agreementStatus)) {
			subMap.put("certStatus", OladingCertStatus.CERT.name());
			updateUserSubWrapper.set(SysUserSub::getCertStatus, Integer.valueOf(OladingCertStatus.CERT.getCode()));
		} else {
			subMap.put("certStatus", OladingCertStatus.SIGNED.name());
			updateUserSubWrapper.set(SysUserSub::getCertStatus, Integer.valueOf(OladingCertStatus.SIGNED.getCode()));
		}
		LocalDateTime now = LocalDateTime.now();
		updateUserSubWrapper.set(SysUserSub::getUpdateTime, now);
		updateUserSubWrapper.set(SysUserSub::getCallbackTime, now);
		updateUserSubWrapper.set(SysUserSub::getUpdateUser, 0);
		updateUserSubWrapper.set(SysUserSub::getCallbackStatus, GigCallBackStatus.RETURNED);
		if ("0".equals(req.getCode())) {
			updateUserSubWrapper.set(SysUserSub::getFailReason, String.valueOf(req.getData().get("failReason")));
		} else {
			updateUserSubWrapper.set(SysUserSub::getFailReason, req.getMessage());
		}
		// 更新状态
		sysUserSubService.update(updateUserSubWrapper);

		// 结果放到redis
		String key = CacheConstants.USER_SIGN_CERT_RESPONSE_KEY + userSub.getUserId();
		Map<String, Object> result = MapUtil.newHashMap(2);
		result.put(userSub.getGigType().name(), new HashMap<>(subMap));

		// 查询redis是否有同时发起的标记，查询出对应的状态放到redis
		String oladingRedisKey = CacheConstants.OLADING_CERT_SEND_CHANNEL + userSub.getUserId();
		String subjectLocationJson = (String) redisTemplate.opsForValue().get(oladingRedisKey);
		if (JSONUtil.isJsonArray(subjectLocationJson)) {
			JSONArray subjectLocationArray = JSONUtil.parseArray(subjectLocationJson);
			List<SubjectLocation> subjectLocationList = subjectLocationArray.stream()
					.map(o -> Enum.valueOf(SubjectLocation.class, String.valueOf(o)))
					.collect(Collectors.toList());
			List<SysUserSub> subList = sysUserSubService.list(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, userSub.getUserId())
					.eq(SysUserSub::getDeptId, userSub.getDeptId())
					.in(SysUserSub::getSubjectLocation, subjectLocationList));
			Map<String, Map<String, String>> dbResultMap = subList.stream()
					.collect(Collectors.toMap(sysUserSub -> sysUserSub.getSubjectLocation().name(),
							sysUserSub -> MapUtil.builder("certStatus",
									OladingCertStatus.resolve(String.valueOf(sysUserSub.getCertStatus())).name()).build()));
			result.putAll(dbResultMap);
		}

		log.info("缓存认证结果到redis：{}", JSONUtil.toJsonStr(result));
		redisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(result), 1, TimeUnit.MINUTES);

		if (OladingCommonRequest.UploadTaxpayerAgreementStatus.SUCCESS.equals(agreementStatus)) {
			// 如果成功，则发送核身视频上传MQ
			log.info("=============================发送核身视频上传MQ==================================");
			UserSignCertDTO userSignCertDTO = new UserSignCertDTO();
			userSignCertDTO.setUserId(userSub.getUserId());
			userSignCertDTO.setSubjectLocation(userSub.getSubjectLocation());
			userSignCertDTO.setUploadFaceID(true);
			sysUserSignCertService.batchChannelCert(userSignCertDTO);
		}
	}

	/**
	 * 完单操作
	 *
	 * @param noteList
	 */
	public void finishTask(List<WmScorePackageSettleNote> noteList) {
		// 使用thirdTaskId分组
		Map<String, List<WmScorePackageSettleNote>> noteTaskIdMap = noteList.stream()
				.filter(note -> StrUtil.isNotBlank(note.getThirdTaskId()))
				.collect(Collectors.groupingBy(WmScorePackageSettleNote::getThirdTaskId));
		if (CollUtil.isEmpty(noteTaskIdMap)) {
			log.info("任务id为空，不发起阿拉丁完单操作");
			return;
		}

		noteTaskIdMap.forEach((thirdTaskId, notes) -> {
			WmScorePackageSettleNote note = notes.get(0);
			WmScorePackage wmScorePackage = scorePackageService.getById(note.getPackageId());
			SysDept dept = sysDeptService.getById(wmScorePackage.getDeptId());
			SysUser sysUser = sysUserService.getById(note.getUserId());

			String gigFinishTaskUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement/finish-task";
			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("endType", "CSO");
			params.put("enterpriseCode", dept.getTaxCode());
			params.put("subjectType", note.getSubjectLocation().getGigType().name());
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation());
			params.put("taskId", note.getThirdTaskId());
			params.put("username", sysUser.getUsername());
			String packageIds = notes.stream().map(WmScorePackageSettleNote::getPackageId).collect(Collectors.joining(StrUtil.COMMA));
			params.put("credentialLink", taxNotifyConfig.getTaskUrl() + "/#/h5/indexTask?id=" + packageIds + "&userId=" + note.getUserId());

			// 发起完单操作
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("阿拉丁完单操作req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigFinishTaskUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.warn("阿拉丁完单操作resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				log.warn("完单操作失败：taskId={}", note.getThirdTaskId());
			}
			JSONObject resultObj = JSONUtil.parseObj(result.getBody());
			// {"code":0,"msg":"SUCCESS","data":{"success":true,"message":"成功","result":"4154"}}
			if (Objects.equals(resultObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = resultObj.getJSONObject("data");
				if (dataJsonObj != null && dataJsonObj.getBool("success", false)) {
					// 完单操作成功，更新状态
					WmScorePackageSettleNote updateNote = new WmScorePackageSettleNote();
					updateNote.setId(note.getId());
					updateNote.setFinishTaskFlag(1);
					scorePackageSettleNoteService.updateById(updateNote);
				}
			}
		});
	}
}
