package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.config.TaxNotifyConfig;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.*;
import com.qunzhixinxi.hnqz.admin.entity.input.SettleNoteStatusOutput;
import com.qunzhixinxi.hnqz.admin.entity.input.WmScorePackageSettleInput;
import com.qunzhixinxi.hnqz.admin.enums.PackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.SettleStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserSubMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmPayOffMapper;
import com.qunzhixinxi.hnqz.admin.service.*;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.ding.config.DingConfig;
import com.qunzhixinxi.hnqz.common.ding.entity.*;
import com.qunzhixinxi.hnqz.common.ding.enums.DingEnum;
import com.qunzhixinxi.hnqz.common.ding.service.DingService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.common.taxhelper.config.TaxHelperConfig;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.model.TaxHelperSettlementModel;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.request.TaxHelperSettlementRequest;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.response.TaxHelperResponse;
import com.qunzhixinxi.hnqz.common.taxhelper.enums.TaxHelperCertStatus;
import com.qunzhixinxi.hnqz.common.taxhelper.service.TaxHelperService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 结算表
 *
 * @author Liujh
 * @date 2020-12-1
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmPayOffServiceImpl extends ServiceImpl<WmPayOffMapper, WmPayOff> implements WmPayOffService {

	private final UpmsConfig upmsConfig;
	private final SysUserMapper sysUserMapper;
	private final SysUserService userService;
	private final SysUserSubMapper sysUserSubMapper;
	private final DingService dingService;
	private final WmScorePackageService wmScorePackageService;
	private final WmTaskService wmTaskService;
	private final WmScorePackageStatusService wmScorePackageStatusService;
	private final WmScorePackageSettleNoteService noteService;
	private final SysDeptService sysDeptService;
	private final SysDeptSubService sysDeptSubService;
	private final SysDictItemMapper sysDictItemMapper;
	private final RedisTemplate<String, Object> redisTemplate;
	private final MonitoringIndicatorService monitoringIndicatorService;
	private final TaxHelperService taxHelperService;
	private final TaxNotifyConfig taxNotifyConfig;

	private final MathContext mc = new MathContext(2, RoundingMode.HALF_EVEN);
	private final String GIG_TOKEN_URL = "%s/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
	private final String GIG_SETTLE_URL = "%s/gig/operation/settlement";

	/**
	 * 财务回退操作
	 *
	 * @param input 提交参数
	 * @return 操作结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> settleBack(WmScorePackageSettleInput input) {
		WmScorePackage wmScorePackage = wmScorePackageService.getById(input.getId());

		if (wmScorePackage.getSettleStatus() != 2 && wmScorePackage.getSettleStatus() != 9) {
			return R.failed("积分包结算状态异常,不能回退");
		}
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(wmScorePackage.getId());
		updateEntity.setScorePackageStatus("4");
		updateEntity.setSettleFlag("0");
		wmScorePackageService.updateById(updateEntity);

		WmScorePackageSettleNote deleteQuery = new WmScorePackageSettleNote();
		deleteQuery.setPackageId(input.getId());
		noteService.remove(Wrappers.query(deleteQuery));

		return R.ok(Boolean.TRUE);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> saveSettleInfo(List<WmScorePackage> wsList) {

		return R.ok();
	}

	/**
	 * 获取待结算积分包信息
	 *
	 * @param pkgId 积分包id
	 * @return 待结算积分包信息
	 */
	@Override
	public R<?> settleById(String pkgId) {

		WmScorePackage wmScorePackage = wmScorePackageService.getById(pkgId);

		if (!"1".equals(wmScorePackage.getPackageFinishStatus())) {
			return R.failed("积分包状态异常");
		}

		List<WmScorePackageSettleNote> notes = noteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery().eq(WmScorePackageSettleNote::getPackageId, pkgId));

		if (CollectionUtils.isEmpty(notes)) {

			List<WmScorePackageStatus> statusList = wmScorePackageStatusService.list(Wrappers.<WmScorePackageStatus>lambdaQuery()
					.eq(WmScorePackageStatus::getPackageId, pkgId)
					.eq(WmScorePackageStatus::getStatus, "2"));

			if (CollectionUtils.isEmpty(statusList)) {
				return R.failed("当前积分包不存在领包记录");
			}

			for (WmScorePackageStatus status : statusList) {
				WmScorePackageSettleNote note = new WmScorePackageSettleNote();
				note.setSettleNo(IdUtil.fastSimpleUUID());
				note.setPackageStatusId(String.valueOf(status.getId()));
				note.setPackageId(status.getPackageId());

				if (StrUtil.equals(PackageTypeEnum.TYPE1_SCOTE_PACKAGE.val(), wmScorePackage.getPackageType1())) {

					note.setSettleAmount(StringUtils.isEmpty(status.getUserScore()) ? BigDecimal.ZERO : new BigDecimal(status.getUserScore(), MathContext.UNLIMITED));

				} else if (PackageTypeEnum.TYPE1_HCP_PACKAGE.val().equals(wmScorePackage.getPackageType1())) {

					// 如果是患者教育，则转换积分值 '分'->'元'
					note.setSettleAmount(StringUtils.isEmpty(status.getUserScore()) ? BigDecimal.ZERO : new BigDecimal(status.getUserScore(), MathContext.UNLIMITED).divide(new BigDecimal("100"), MathContext.UNLIMITED));

				} else {

					note.setSettleAmount(StringUtils.isEmpty(wmScorePackage.getUserScore()) ? BigDecimal.ZERO : new BigDecimal(wmScorePackage.getUserScore(), MathContext.UNLIMITED));

				}

				note.setDiscount(BigDecimal.ONE);
				note.setUserId(status.getUserId());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_NO.getType());
				SysUser noteUser = sysUserMapper.selectById(status.getUserId());
				note.setRealName(noteUser.getRealname());
				note.setIdCardNumber(noteUser.getIdCardNumber());
				notes.add(note);
			}
			if (wmScorePackage.getLocation() != null) {
				wmScorePackage.setSubjectLocation(wmScorePackage.getLocation().name());
			}
		} else {

			notes.forEach(item -> {
				SysUser user = sysUserMapper.selectById(item.getUserId());
				item.setRealName(user.getRealname());
				item.setIdCardNumber(user.getIdCardNumber());
				if (item.getSubjectLocation() != null){
					wmScorePackage.setSubjectLocation(item.getSubjectLocation().name());
				}
			});

			if (StrUtil.isBlank(wmScorePackage.getSubjectLocation())){
				return R.failed("结算积分包未勾选税源地");
			}

		}

		Map<String, Object> resultMap = new HashMap<>(3);

		if (PackageTypeEnum.TYPE1_HCP_PACKAGE.val().equals(wmScorePackage.getPackageType1())) {
			// 如果是患者教育，则转换积分值 '分'->'元'
			wmScorePackage.setUserScore(new BigDecimal(wmScorePackage.getUserScore(), MathContext.UNLIMITED).divide(new BigDecimal("100"), MathContext.UNLIMITED).toString());
		}
		resultMap.put("wsp", wmScorePackage);
		resultMap.put("notes", notes);
		// 显示实际完成积分值
		resultMap.put("finishScore", "0");

		if (PackageTypeEnum.TYPE1_SCOTE_PACKAGE.val().equals(wmScorePackage.getPackageType1()) || PackageTypeEnum.TYPE1_HCP_PACKAGE.val().equals(wmScorePackage.getPackageType1())) {

			List<WmTask> tasks = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
					.eq(WmTask::getRealFlag, "0")
					.eq(WmTask::getScorePackageId, wmScorePackage.getId())
					.eq(WmTask::getTaskStatus, "3"));

			BigDecimal total = BigDecimal.ZERO;

			for (WmTask task : tasks) {
				total = total.add(new BigDecimal(task.getScore()));
			}

			if (PackageTypeEnum.TYPE1_HCP_PACKAGE.val().equals(wmScorePackage.getPackageType1())) {
				// 如果是患者教育，则转换积分值 '分'->'元'
				resultMap.put("finishScore", total.divide(new BigDecimal("100"), MathContext.UNLIMITED));
			} else {
				resultMap.put("finishScore", total.intValue());
			}
		}

		return R.ok(resultMap);
	}

	/**
	 * 多人承接-按积分值业务结算-查询
	 *
	 * @param taskIds 任务id
	 * @return 结算信息
	 */
	@Override
	public R<?> settleInfoByTaskIds(String[] taskIds) {
		// 查询任务
		List<WmTask> taskList = wmTaskService.listByIds(Arrays.asList(taskIds));
		Set<String> taskIdSet = taskList.stream().map(WmTask::getScorePackageId).collect(Collectors.toSet());
		if (taskIdSet.size() > 1) {
			return R.failed("所选任务不属于同一个积分包");
		}

		String scorePackageId = taskList.get(0).getScorePackageId();
		WmScorePackage wmScorePackage = wmScorePackageService.getById(scorePackageId);

		// user-taskList map
		Map<String, List<WmTask>> userTaskMap = taskList.stream().collect(Collectors.groupingBy(WmTask::getTaskUserId));
		// taskId-task Map
		Map<String, WmTask> taskMap = taskList.stream().collect(Collectors.toMap(WmTask::getId, Function.identity()));

		// 查询settleNote
		List<WmScorePackageSettleNote> notes = noteService.list(Wrappers.<WmScorePackageSettleNote>lambdaQuery()
				.in(WmScorePackageSettleNote::getTaskId, Arrays.asList(taskIds))
				.eq(WmScorePackageSettleNote::getPackageId, scorePackageId));
		Map<String, WmScorePackageSettleNote> taskSettleNoteMap = notes.stream()
				.filter(note -> StrUtil.isNotBlank(note.getTaskId()))
				.collect(Collectors.toMap(WmScorePackageSettleNote::getTaskId, Function.identity()));

		// 查询user
		Set<String> allUserIdSet = new HashSet<>();
		allUserIdSet.addAll(userTaskMap.keySet());
		allUserIdSet.addAll(notes.stream().map(WmScorePackageSettleNote::getUserId).collect(Collectors.toSet()));
		List<SysUser> sysUsers = sysUserMapper.selectBatchIds(allUserIdSet);
		Map<Integer, SysUser> userMap = sysUsers.stream().collect(Collectors.toMap(SysUser::getUserId, Function.identity()));

		// 页面回显的结算对象列表
		List<WmScorePackageSettleNote> userNotes = new ArrayList<>();
		// 用于结算的note参数
		List<WmScorePackageSettleNote> toSettleNotes = new ArrayList<>();

		if (wmScorePackage.getLocation() != null) {
			wmScorePackage.setSubjectLocation(wmScorePackage.getLocation().name());
		}
//		if (StrUtil.isBlank(wmScorePackage.getSubjectLocation())){
//			return R.failed("结算积分包未勾选税源地");
//		}

		// 按结算对象分组，统计分别对应的结算信息
		for (String userId : userTaskMap.keySet()) {
			WmScorePackageSettleNote oneUserNote = new WmScorePackageSettleNote();
			List<WmTask> userTasks = userTaskMap.get(userId);

			// 计算每个人的总任务积分
			BigDecimal userTotalSettleAmount = BigDecimal.ZERO;
			for (WmTask task : userTasks) {
				WmScorePackageSettleNote settleNote = taskSettleNoteMap.get(task.getId());
				// 存在note，则取note金额，否则取task的积分
				if (settleNote != null) {
					userTotalSettleAmount = userTotalSettleAmount.add(settleNote.getSettleAmount());
				} else {
					userTotalSettleAmount = userTotalSettleAmount.add(BigDecimal.valueOf(task.getScore()));
				}
			}

			oneUserNote.setPackageId(scorePackageId);
			oneUserNote.setSettleAmount(userTotalSettleAmount);
			oneUserNote.setDiscount(BigDecimal.ONE);
			oneUserNote.setUserId(userId);
			oneUserNote.setSettleNoteStatus(Integer.valueOf(SettleStatusEnum.SETTLE_STATUS_NOTSETTLE.getVal()));

			SysUser noteUser = userMap.get(Integer.parseInt(userId));
			oneUserNote.setRealName(noteUser.getRealname());
			oneUserNote.setIdCardNumber(noteUser.getIdCardNumber());
			userNotes.add(oneUserNote);
		}

		// 添加用于结算的note参数
		for (String taskId : taskIds) {
			WmScorePackageSettleNote note = new WmScorePackageSettleNote();
			// note存在则直接获取
			WmScorePackageSettleNote settleNote = taskSettleNoteMap.get(taskId);
			if (settleNote != null) {
				toSettleNotes.add(settleNote);
				continue;
			}

			// note不存在，取task中的信息作为结算信息
			WmTask task = taskMap.get(taskId);
			note.setSettleAmount(BigDecimal.valueOf(task.getScore()));
			note.setSettleNo(IdUtil.fastSimpleUUID());
			note.setPackageId(scorePackageId);
			note.setTaskId(taskId);
			note.setDiscount(BigDecimal.ONE);
			note.setUserId(task.getTaskUserId());
			note.setSettleNoteStatus(Integer.valueOf(SettleStatusEnum.SETTLE_STATUS_NOTSETTLE.getVal()));
			// user信息
			SysUser noteUser = userMap.get(Integer.parseInt(task.getTaskUserId()));
			note.setRealName(noteUser.getRealname());
			note.setIdCardNumber(noteUser.getIdCardNumber());
			toSettleNotes.add(note);
		}

		Map<String, Object> resultMap = new HashMap<>(3);
		resultMap.put("wsp", wmScorePackage);
		resultMap.put("notes", toSettleNotes);
		resultMap.put("userNotes", userNotes);

		return R.ok(resultMap);
	}

	/**
	 * 业务提交结算信息到财务
	 *
	 * @param input 输入信息
	 * @return 提交结果
	 */
	@Override
	public R<?> settleSubmit(WmScorePackageSettleInput input) {

		WmScorePackage wmScorePackage = wmScorePackageService.getById(input.getId());

		// 校验基本包状态
		wmScorePackageService.checkPkgToSettle(wmScorePackage);

		List<WmScorePackageSettleNote> notes = input.getNotes();
		SubjectLocation location = input.getSubjectLocation();

		Integer deptId = SecurityUtils.getUser().getDeptId();
		if (SubjectLocation.MAH_SETTLE.equals(input.getSubjectLocation())) {
			// 默认在当前企业，取一个结算渠道
			SysDeptSub condition = new SysDeptSub();
			condition.setDeptId(deptId);
			condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
			List<SysDeptSub> deptSubs = sysDeptSubService.list(Wrappers.query(condition));
			if (CollUtil.isEmpty(deptSubs)) {
				return R.failed("结算渠道不存在");
			}
			location = deptSubs.get(0).getSubjectLocation();
		}

		// 校验是否配置了结算限制
		SysDeptSub queryDeptSub = sysDeptSubService.getOne(Wrappers.<SysDeptSub>lambdaQuery()
				.eq(SysDeptSub::getDeptId, SecurityUtils.getUser().getDeptId())
				.eq(SysDeptSub::getSubjectLocation, location)
				.eq(SysDeptSub::getEnableFlag, SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode()));
		if (!queryDeptSub.getSettleEnable()) {
			log.info("{}企业配置了结算限制", deptId);
			throw new RuntimeException("系统维护中，请联系管理员");
		}

		// 校验人员信息
		Set<Integer> userIds = notes.stream().map(WmScorePackageSettleNote::getUserId).map(Integer::valueOf).collect(Collectors.toSet());
		userService.checkUserInfoToSettle(userIds, location);

		// 校验结算
		BigDecimal total = monitoringIndicatorService.settleMonitoringIndicator(notes, location);

		LocalDateTime now = LocalDateTime.now();
		SubjectLocation finalLocation = location;
		notes.forEach(note -> {
			note.setUpdateTime(now);
			note.setSubTime(now);
			note.setSubType(String.valueOf(finalLocation.getGigType().getCode()));
			note.setSubjectLocation(finalLocation);
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(input.getCategoryName());
			if (null == note.getId()) {
				note.setCreateTime(LocalDateTime.now());
			}
		});

		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setInvoiceCategory(input.getInvoiceCategory());
		updateEntity.setSettleAmount(total);
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(DingEnum.SETTLE_STATUS_WAIT.getType());
		updateEntity.setScorePackageStatus("5");
		updateEntity.setDescription(input.getDescription());
		updateEntity.setSubType(String.valueOf(input.getSubjectLocation().getGigType().getCode()));
		updateEntity.setLocation(input.getSubjectLocation());
		updateEntity.setSettlementDate(LocalDateTime.now());
		if (SubjectLocation.MAH_SETTLE.equals(input.getSubjectLocation())) {
			updateEntity.setMahSettleDeptId(input.getMahSettleDeptId());
		}

		if (PackageTypeEnum.TYPE2_SETTLE_PACKAGE.val().equals(wmScorePackage.getPackageType2())
				&& !StrUtil.equals(PackageTypeEnum.TYPE1_SCORE_AND_TASK_PACKAGE.val(), wmScorePackage.getPackageType1())) {
			updateEntity.setPackageFinishStatus("1");
			updateEntity.setTaskAddFlag("0");
		}

		if (StrUtil.equals(PackageTypeEnum.TYPE1_SCORE_AND_TASK_PACKAGE.val(), wmScorePackage.getPackageType1())) {
			// 多人承接积分包可以一直结算
			updateEntity.setSettleFlag("0");
		} else {
			updateEntity.setSettleFlag("1");
		}

		wmScorePackageService.updateById(updateEntity);
		noteService.saveOrUpdateBatch(notes);

		return R.ok(Boolean.TRUE);
	}

	/**
	 * 业务保存操作
	 *
	 * @param input 提交信息
	 * @return 保存结果
	 */
	@Override
	public R<?> settleSave(WmScorePackageSettleInput input) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		SubjectLocation location;

		if (SubjectLocation.MAH_SETTLE.equals(input.getSubjectLocation())) {
			// 默认在当前企业，取一个结算渠道
			SysDeptSub condition = new SysDeptSub();
			condition.setDeptId(SecurityUtils.getUser().getDeptId());
			condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
			List<SysDeptSub> deptSubs = sysDeptSubService.list(Wrappers.query(condition));
			if (CollUtil.isEmpty(deptSubs)) {
				return R.failed("结算渠道不存在");
			}
			location = deptSubs.get(0).getSubjectLocation();
		} else {
			location = input.getSubjectLocation();
		}

		// 校验人员信息
		Set<Integer> userIds = notes.stream().map(WmScorePackageSettleNote::getUserId).map(Integer::valueOf).collect(Collectors.toSet());
		userService.checkUserInfoToSettle(userIds, location);

		// 校验结算
		BigDecimal total = monitoringIndicatorService.settleMonitoringIndicator(notes, location);

		LocalDateTime now = LocalDateTime.now();
		notes.forEach(note -> {
			note.setUpdateTime(now);
			note.setSubTime(now);
			note.setSubType(String.valueOf(location.getGigType().getCode()));
			note.setSubjectLocation(location);
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(input.getCategoryName());
			if (null == note.getId()) {
				note.setCreateTime(LocalDateTime.now());
			}
		});
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setInvoiceCategory(input.getInvoiceCategory());
		updateEntity.setSettleAmount(total);
		updateEntity.setId(input.getId());
		updateEntity.setDescription(input.getDescription());
		updateEntity.setSubType(String.valueOf(location.getGigType().getCode()));
		updateEntity.setLocation(input.getSubjectLocation());
		if (SubjectLocation.MAH_SETTLE.equals(input.getSubjectLocation())) {
			updateEntity.setMahSettleDeptId(input.getMahSettleDeptId());
		}

		wmScorePackageService.updateById(updateEntity);
		noteService.saveOrUpdateBatch(notes);

		return R.ok(Boolean.TRUE);
	}

	/**
	 * 重新结算
	 *
	 * @param input 结算信息
	 * @return 提交结果
	 */
	@Override
	public R<?> settleReIssueBySubType(WmScorePackageSettleInput input) {

		WmScorePackageSettleNote note = noteService.getById(input.getId());

		WmScorePackage wmScorePackage = wmScorePackageService.getById(note.getPackageId());
		note.setSettleNo(IdUtil.fastSimpleUUID());

		input.setNotes(Collections.singletonList(note));
		input.setInvoiceCategory(note.getInvoiceType());
		input.setCategoryName(input.getCategoryName());
		input.setSubType(note.getSubType());
		input.setScorePackageName(wmScorePackage.getScorePackageName());

		if (!DingEnum.NOTE_STATUS_FAIL.getType().equals(note.getSettleNoteStatus())) {
			return R.failed("状态异常，请刷新页面后再操作");
		}

		// 结算配置信息
		HnqzUser operator = SecurityUtils.getUser();
		SubjectLocation location = input.getSubjectLocation();
		SysDeptSub sysDeptSub = sysDeptSubService.getDeptSub(operator.getDeptId(), location);
		SysDept sysDept = sysDeptService.getById(operator.getDeptId());
		String taxCode = sysDept.getTaxCode();
		if (StringUtils.isEmpty(taxCode)) {
			return R.failed("未配置企业的统一社会信用代码");
		}
		String subType = String.valueOf(location.getGigType().getCode());

		// 人力家
		if (SubjectTypeEnum.TYPE_RENLIJIA.getCode().equals(subType)) {
			return settleByRenLiJia(input, sysDeptSub, operator);
		}
		// 自由职家
		else if (SubjectTypeEnum.TYPE_OLADING.getCode().equals(subType)) {
			return settleByOlading(input, wmScorePackage, taxCode, operator);
		}
		// 正启之星
		else if (SubjectTypeEnum.TYPE_ZHENG_QI_ZHI_XING.getCode().equals(subType)){
			return settleBySinbaad(input, wmScorePackage, taxCode, operator);
		}
		// 税邦云
		else {
			log.info("税邦云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			return settleByShuiBangYun(input, sysDeptSub, taxCode, operator);
		}

	}

	/**
	 * 提交结算
	 *
	 * @param input 提交结算信息
	 * @return 提交结果
	 */
	@Override
	public R<?> settleIssueBySubType(WmScorePackageSettleInput input) {

		SubjectLocation location = input.getSubjectLocation();
		WmScorePackage wmScorePackage = wmScorePackageService.getById(input.getId());

		if (wmScorePackage.getSettleStatus() != 2 && wmScorePackage.getSettleStatus() != 9) {
			return R.failed("积分包结算状态异常");
		}

		// 结算配置信息
		HnqzUser operator = SecurityUtils.getUser();

		if (SubjectLocation.MAH_SETTLE.equals(wmScorePackage.getLocation())) {
			// MAH结算的情况-标记
			input.setMahSettleDeptId(operator.getDeptId());
		} else {
			input.setMahSettleDeptId(null);
		}

		SysDeptSub sysDeptSub = sysDeptSubService.getDeptSub(operator.getDeptId(), location);
		SysDept sysDept = sysDeptService.getById(sysDeptSub.getDeptId());
		String taxCode = sysDept.getTaxCode();
		String enterpriseName = sysDept.getName();
		if (StringUtils.isEmpty(taxCode)) {
			return R.failed("未配置企业的统一社会信用代码");
		}

		String subType = String.valueOf(location.getGigType().getCode());

		// 人力家
		if (SubjectTypeEnum.TYPE_RENLIJIA.getCode().equals(subType)) {
			return settleByRenLiJia(input, sysDeptSub, operator);
		}
		// 自由职家
		else if (SubjectTypeEnum.TYPE_OLADING.getCode().equals(subType)) {
			return settleByOlading(input, wmScorePackage, taxCode, operator);
		}
		// 正启之星
		else if (SubjectTypeEnum.TYPE_ZHENG_QI_ZHI_XING.getCode().equals(subType)){
			return settleBySinbaad(input, wmScorePackage, taxCode, operator);
		}
		// 浪潮
		else if (SubjectTypeEnum.TYPE_LANG_CHAO.getCode().equals(subType)) {
			return this.settleByLangchao(input, wmScorePackage, taxCode, enterpriseName, operator);
		}
		// 汇企云
		else if (SubjectTypeEnum.TYPE_HUI_QI_YUN.getCode().equals(subType)) {
			return this.settleByHuiQiYun(input, wmScorePackage, taxCode, operator, sysDeptSub.getPathNo());
		}
		// 众蚁云
		else if (SubjectTypeEnum.TYPE_ZHONG_YI_YUN.getCode().equals(subType)) {
			return this.settleByZhongYiYun(input, wmScorePackage, taxCode, operator);
		}
		// 中智
		else if (SubjectTypeEnum.TYPE_ZHONG_ZHI.getCode().equals(subType)) {
			return this.settleByZhongZhi(input, wmScorePackage, taxCode, operator);
		}
		// 福建企赋
		else if (SubjectTypeEnum.TYPE_QI_FU.getCode().equals(subType)) {
			return this.settleByQiFu(input, wmScorePackage, taxCode, operator);
		}
		// 深圳人才
		else if (SubjectTypeEnum.TYPE_SHEN_ZHEN_REN_CAI.getCode().equals(subType)) {
			return settleByOlading(input, wmScorePackage, taxCode, operator);
		}
		// 税邦云
		else {
			log.info("税邦云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			return settleByShuiBangYun(input, sysDeptSub, taxCode, operator);
		}

	}

	/**
	 * 福建企赋结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByQiFu(WmScorePackageSettleInput input,
								  WmScorePackage wmScorePackage,
								  String taxCode,
								  HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getBankPhone());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				// 市场推广
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "QI_FU");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
				params.put("taskDescription", "按要求完成学术推广并用在线工具采集记录");

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("福建企赋结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("福建企赋结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("福建企赋结算提交成功： {}", settleObj.getStr("data"));
					note.setUpdateTime(LocalDateTime.now());
					//streamId 获取不到
					if (dataJsonObj != null) {
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
						}
					} else {
						log.warn("福建企赋结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("福建企赋结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("福建企赋结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 中智结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByZhongZhi(WmScorePackageSettleInput input,
									WmScorePackage wmScorePackage,
									String taxCode,
									HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 查询发票类目对应的字典
				SysDictItem zhongzhiInvoice = sysDictItemMapper.selectOne(Wrappers.<SysDictItem>lambdaQuery()
						.eq(SysDictItem::getType, "zhongzhi_invoice")
						.eq(SysDictItem::getValue, String.valueOf(note.getInvoiceType())));

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getBankPhone());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				// 行业类别id
				params.put("tagId", zhongzhiInvoice.getDescription());
				// 税收项目编号
				params.put("taskInvoiceContent", zhongzhiInvoice.getRemarks());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "ZHONG_ZHI");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("中智结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("中智结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("中智结算提交成功： {}", settleObj.getStr("data"));
					note.setUpdateTime(LocalDateTime.now());
					//streamId 获取不到
					if (dataJsonObj != null) {
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
						}
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						log.warn("中智结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("中智结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("中智结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 众蚁云结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByZhongYiYun(WmScorePackageSettleInput input,
								  WmScorePackage wmScorePackage,
								  String taxCode,
								  HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getBankPhone());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				// 市场推广
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "ZHONG_YI_YUN");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("众蚁云结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("众蚁云结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("众蚁云结算提交成功： {}", settleObj.getStr("data"));
					note.setUpdateTime(LocalDateTime.now());
					//streamId 获取不到
					if (dataJsonObj != null) {
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
						}
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						log.warn("众蚁云结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("众蚁云结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("众蚁云结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 汇企云结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByHuiQiYun(WmScorePackageSettleInput input,
								  WmScorePackage wmScorePackage,
								  String taxCode,
								  HnqzUser operator,
								  String[] pathNo) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				// 市场推广
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "HUI_QI_YUN");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("notes", pathNo[0]);
				params.put("pathNo", pathNo[0]);
				params.put("notifyUrl", String.format("%s/admin/gig-third-api/huiqiyun", upmsConfig.getClientUrl()));

				if (SubjectLocation.JIN_YUAN.equals(input.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("汇企云结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("汇企云结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("汇企云结算提交成功： {}", settleObj.getStr("data"));
					note.setUpdateTime(LocalDateTime.now());
					//streamId 获取不到
					if (dataJsonObj != null) {
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
						}
					} else {
						log.warn("汇企云结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("汇企云结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("汇企云结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 浪潮结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param enterpriseName 企业名称
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByLangchao(WmScorePackageSettleInput input,
								 WmScorePackage wmScorePackage,
								 String taxCode,
								 String enterpriseName,
								 HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("enterpriseName", enterpriseName);
				params.put("idCardType","ID_CARD");
				params.put("subjectType", "LANG_CHAO");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());

				if (SubjectLocation.JIN_YUAN.equals(input.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("浪潮结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("浪潮结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("浪潮结算提交成功： {}", settleObj.getStr("data"));
					//streamId 获取不到
					if (dataJsonObj == null) {
						log.warn("浪潮结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					}
					if (!StrUtil.equals("200", dataJsonObj.getStr("code"))) {
						log.info("浪潮提交成功 code 状态异常");
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					}
					note.setUpdateTime(LocalDateTime.now());

					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("浪潮结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("浪潮结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 正启之星结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleBySinbaad(WmScorePackageSettleInput input,
								 WmScorePackage wmScorePackage,
								 String taxCode,
								 HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("idCardType","ID_CARD");
				params.put("subjectType", "ZHENG_QI_ZHI_XING");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				if (SubjectLocation.JIN_YUAN.equals(input.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("政企之星结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("政企之星结算resp：{}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("政企之星结算提交成功： {}", settleObj.getStr("data"));
					note.setUpdateTime(LocalDateTime.now());
					//streamId 获取不到
					if (dataJsonObj != null) {
						String requestId = dataJsonObj.getStr("requestId");
						String data = dataJsonObj.getStr("data");
						String respMessage = dataJsonObj.getStr("resp_message");
						if (StrUtil.isNotBlank(requestId)) {
							JSONObject dataJson = JSONUtil.parseObj(data);
							String orderStatus = dataJson.getStr("order_status");
							if ("-1".equals(orderStatus) || "-10".equals(orderStatus)) {
								// 订单失败的状态(-1-创建订单成功，子任务状态校验异常，可做失败处理, -10-下发至个人失败)
								note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
								log.warn("政企之星结算提交失败： {}", respMessage);
								noticeInfo.append(sysUser.getRealname()).append("：提交结算失败，请联系管理员；");
							} else {
								note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							}
							if ("2".equals(orderStatus)) {
								//  2-创建订单成功待支付，支付状态待确认，订单被驳回
								//  (一个是订单的支付状态没有成功，还有一种情况就是供应商驳回了你的订单 都是2
								// 订单不是失败的，只是没付款成功  不排除银行极端情况出现，所以你要是把2当成失败的话就10分钟之后再查下状态判断)
								note.setRemark(data);
							}
							// 新增字段streamId
							note.setStreamId(requestId);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
						}
					} else {
						log.warn("政企之星结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("政企之星结算失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.error("", e);
				log.warn("政企之星结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}


	/**
	 * 阿拉丁结算操作
	 *
	 * @param input          结算数据
	 * @param wmScorePackage 积分包
	 * @param taxCode        税号
	 * @param operator       操作员
	 * @return 结果
	 */
	private R<?> settleByOlading(WmScorePackageSettleInput input,
								 WmScorePackage wmScorePackage,
								 String taxCode,
								 HnqzUser operator) {

		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();

		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(StrUtil.isNotBlank(input.getCategoryName()) ? input.getCategoryName() : note.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		for (WmScorePackageSettleNote note : notes) {
			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			try {

				// 获取请求token
				String url = String.format(GIG_TOKEN_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-token：{}", url);
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(url, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				// 封装结算参数
				Map<String, Object> params = new HashMap<>();
				params.put("token", token);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "全国");
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				// 市场推广
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", input.getSubjectLocation().getGigType().name());
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", input.getSubjectLocation().getType());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("acceptanceStandard", "完成不同任务获得不同任务积分，根据不同任务给与0~30000积分，每个积分兑换0.8~1.2元，按次数结算");


				if (SubjectLocation.JIN_YUAN.equals(input.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				// 发起结算
				String url1 = String.format(GIG_SETTLE_URL, upmsConfig.getGigServiceUrl());
				log.info("远端请求gig-settle：{}", url);
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("阿拉丁结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(url1, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("阿拉丁结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}

				JSONObject settleObj = JSONUtil.parseObj(result.getBody());


				if (Objects.equals(settleObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settleObj.getJSONObject("data");
					log.info("自由职家结算提交成功： {}", note.getSettleNo());
					//streamId 获取不到
					if (dataJsonObj == null) {
						log.warn("自由职家结算提交成功 data 获取不到： {}", note.getSettleNo());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());

					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						// 新增字段streamId
						note.setStreamId(requestId);
						// 第三方任务id
						note.setThirdTaskId(dataJsonObj.getStr("taskId"));
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					log.warn("自由职家结算提交失败： {}", note.getSettleNo());
					note.setUpdateTime(LocalDateTime.now());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settleObj.getStr("msg")).append("；");

					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				}
			} catch (Exception e) {
				log.warn("自由职家结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 税邦云
	 *
	 * @param input      结算数据
	 * @param sysDeptSub 结算配置
	 * @param taxCode    税号
	 * @param operator   操作员
	 * @return 结果
	 */
	private R<?> settleByShuiBangYun(WmScorePackageSettleInput input,
									 SysDeptSub sysDeptSub,
									 String taxCode,
									 HnqzUser operator) {
		List<WmScorePackageSettleNote> notes = input.getNotes();
		StringBuilder noticeInfo = new StringBuilder();


		// WmScorePackageSettleNote
		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(input.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);


		for (WmScorePackageSettleNote note : notes) {
			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (!Objects.equals(userSub.getCertStatus(), TaxHelperCertStatus.CERT.getCode())) {
				return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}

		}

		for (WmScorePackageSettleNote note : notes) {

			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
					note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			TaxHelperConfig config = new TaxHelperConfig();
			config.setAppId(sysDeptSub.getAppId());
			config.setAppSecrete(sysDeptSub.getAppSecret());
			config.setQueryUrl(sysDeptSub.getQueryUrl());

			// 开始结算
			TaxHelperSettlementModel taxHelperSettlementModel = new TaxHelperSettlementModel(config);
			taxHelperSettlementModel.setEnterpriseCode(taxCode);

			taxHelperSettlementModel.setExtendFieldNames(Collections.singletonList("报告下载地址"));
			taxHelperSettlementModel.setCount(1);

			taxHelperSettlementModel.setNotifyUrl(String.format("%s/admin/taxhelper/%s/settlementNotify", taxNotifyConfig.getNotifyUrl(), sysUser.getUserId()));
			TaxHelperSettlementModel.DetailedArr detailedArr = new TaxHelperSettlementModel.DetailedArr();
			detailedArr.setIdCard(sysUser.getIdCardNumber());
			detailedArr.setInputAmount(note.getSettleAmount().doubleValue());
			detailedArr.setCategoryName(note.getCategoryName());
			taxHelperSettlementModel.setDetailedArr(Collections.singletonList(detailedArr));

			TaxHelperSettlementModel.AbstractArr abstractArr = new TaxHelperSettlementModel.AbstractArr();

			abstractArr.setIdCard(sysUser.getIdCardNumber());
			abstractArr.setNumber(1);
			abstractArr.setPrice(note.getSettleAmount().doubleValue());
			abstractArr.setAmount(note.getSettleAmount().doubleValue());

			abstractArr.setExtendField1(String.format("%s/#/h5/indexTask?id=%s&userId=%s", taxNotifyConfig.getTaskUrl(), note.getPackageId(), note.getUserId()));

			taxHelperSettlementModel.setAbstractArr(Collections.singletonList(abstractArr));

			TaxHelperSettlementRequest taxHelperSettlementRequest = new TaxHelperSettlementRequest(taxHelperSettlementModel);

			TaxHelperResponse response = taxHelperService.settlement(taxHelperSettlementRequest);

			log.info("税邦云结算： {}", response);

			if (response.success()) {
				log.info("税邦云结算提交成功： {}", note.getSettleNo());
				//streamId 获取不到
				if (StringUtils.isEmpty(response.getMessage())) {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					log.info("税邦云结算提交成功 streamId 获取不到： {}", note.getSettleNo());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				} else {
					// 新增字段streamId
					note.setStreamId(response.getMessage());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
				}
				note.setUpdateTime(LocalDateTime.now());
			} else {
				log.info("税邦云结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(LocalDateTime.now());
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(response.getMessage()).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(Boolean.TRUE, noticeInfo.toString());
	}

	/**
	 * 人力家结算
	 *
	 * @param input      结算数据
	 * @param sysDeptSub 结算配置
	 * @param operator   操作员
	 * @return 结果
	 */
	private R<?> settleByRenLiJia(WmScorePackageSettleInput input, SysDeptSub sysDeptSub, HnqzUser operator) {
		List<WmScorePackageSettleNote> notes = input.getNotes();

		for (WmScorePackageSettleNote note : notes) {
			note.setInvoiceType(input.getInvoiceCategory());
			note.setCategoryName(input.getCategoryName());
			note.setSubToGigTime(LocalDateTime.now());
		}
		noteService.updateBatchById(notes);

		log.info("结算包ID： {} 开始结算-----------", input.getId());
		log.info("结算渠道（税源地）： {} ", input.getSubjectLocation());

		DingConfig config = new DingConfig();
		config.setAppId(sysDeptSub.getAppId());
		config.setAppSecret(sysDeptSub.getAppSecret());
		config.setQueryUrl(sysDeptSub.getQueryUrl());

		for (WmScorePackageSettleNote note : notes) {

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			Integer deptId;
			// MAH结算校验原企业认证信息
			if (input.getMahSettleDeptId() != null) {
				deptId = sysUser.getDeptId();
			} else {
				deptId = operator.getDeptId();
			}
			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getUserId, sysUser.getUserId())
					.eq(SysUserSub::getSubjectLocation, input.getSubjectLocation())
					.eq(SysUserSub::getDeptId, deptId)
			);

			if (userSub == null) {
				log.info("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：认证信息不存在，不能结算");
			}

			if (StringUtils.isEmpty(sysUser.getIdCardNumber())) {
				log.info("任务包众包代表-" + sysUser.getRealname() + "：身份证号未补录");
				// 添加登录提醒
				redisTemplate.opsForValue().set(CacheConstants.USER_NOTICE_KEY + sysUser.getUserId(), "1", CacheConstants.NOTICE_TIME, TimeUnit.DAYS);
				return R.failed("任务包众包代表-" + sysUser.getRealname() + "：身份证号未补录");
			}

			if (!DingEnum.CER_STATUS_1.getType().equals(userSub.getCertStatus())) {
				//认证中
				log.info("结算对象-" + sysUser.getRealname() + "：未认证成功，不能结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：未认证成功，不能结算");
			}

			if (Boolean.TRUE.equals(redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId()))) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			if (DingEnum.NOTE_STATUS_SUCCESS.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已结算，请勿重复结算");
			}
			if (DingEnum.NOTE_STATUS_SUBMIT.getType().equals(note.getSettleNoteStatus())) {
				log.info("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
				return R.failed("结算对象-" + sysUser.getRealname() + "：已提交，请等待耐心结算结果");
			}
		}

		// 进入结算环节  noticeInfo 失败原因
		StringBuilder noticeInfo = new StringBuilder();
		for (WmScorePackageSettleNote note : notes) {

			// 结算失败过的更新 settleNo
			if (DingEnum.NOTE_STATUS_FAIL.getType().equals(note.getSettleNoteStatus())) {
				note.setSettleNo(IdUtil.fastSimpleUUID());
			}

			// 加锁
			redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(), note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

			SysUser sysUser = sysUserMapper.selectById(note.getUserId());

			// 校验认证信息
			if (DingEnum.CER_STATUS_1.getType().equals(sysUser.getRljCertStatus())) {
				//认证成功  发起结算
				//当前登陆人的机构名称
				String subjectName = sysDeptSub.getSubjectName();

				List<Map<String, String>> jsdList = new ArrayList<>(1);
				Map<String, String> jsdMap = new HashMap<>(4);
				// 服务费率
				BigDecimal serviceCharge = sysDeptSub.getServiceCharge() == null ? BigDecimal.ZERO : sysDeptSub.getServiceCharge();
				// 实际结算金额（结算金额*（1+服务费率））
				BigDecimal actualAmount = note.getSettleAmount().multiply(serviceCharge.add(BigDecimal.valueOf(100)))
						.divide(BigDecimal.valueOf(100),2, RoundingMode.HALF_EVEN);
				jsdMap.put("amount", actualAmount.toPlainString());
				jsdMap.put("mobile", sysUser.getPhone());
				jsdMap.put("idCard", sysUser.getIdCardNumber());
				jsdMap.put("name", sysUser.getRealname());
				jsdList.add(jsdMap);
				LocalDateTime today = LocalDateTime.now();
				LocalDateTime startTime = today.minusDays(85);
				EmployPayResponse employPayResponse = dingService.payV3(config, subjectName,
						note.getSettleNo(), input.getScorePackageName(),
						input.getInvoiceCategory(), jsdList, startTime, today);

				//提交结算接口失败
				if (!employPayResponse.isSuccess()) {
					log.info("人力家接口调用失败，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("：提交结算失败").append(employPayResponse.getCode()).append(employPayResponse.getMsg());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());

				} else {
					//结算提交成功数据
					List<EmployPayResultSuccessItem> successResult = employPayResponse.getData().getSuccessResult();
					if (CollectionUtils.isNotEmpty(successResult)) {

						log.info("结算对象-" + sysUser.getRealname() + "：提交结算成功；");
						noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("：提交结算成功；");
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());

						savePayoffFormNote(note);

					}

					//结算提交失败的数据
					List<EmployPayResultfailItem> failResult = employPayResponse.getData().getFailResult();
					if (CollectionUtils.isNotEmpty(failResult)) {

						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

						log.info("结算对象-" + sysUser.getRealname() + "，结算失败：" + failResult.get(0).getCheckRemark() + "；");
						noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("，结算失败：").append(failResult.get(0).getCheckRemark()).append("；");
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					}
				}

			} else {
				log.info("认证状态异常，结算解锁settleNoteId：{}", input.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			}

		}

		noteService.updateBatchById(notes);

		// 更新结算结果
		SettleNoteStatusOutput output = noteService.selectSettleNoteStatus(input.getId());
		WmScorePackage updateEntity = new WmScorePackage();
		updateEntity.setId(input.getId());
		updateEntity.setSettleStatus(output.toSettleStatus());
		wmScorePackageService.updateById(updateEntity);

		return R.ok(Boolean.TRUE, noticeInfo.toString());
	}

	/**
	 * 提交成功的记录
	 *
	 * @param note 结算记录
	 */
	private void savePayoffFormNote(WmScorePackageSettleNote note) {
		WmPayOff wmPayOff = new WmPayOff();
		wmPayOff.setScoreId(note.getPackageId());
		wmPayOff.setNoteId(String.valueOf(note.getId()));
		wmPayOff.setStatementNo(note.getSettleNo());
		wmPayOff.setSettleAmount(note.getSettleAmount());
		wmPayOff.setSettlePeople(note.getUserId());

		this.save(wmPayOff);
	}

}
