/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */
package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDeptRelation;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.WmReportOpt;
import com.qunzhixinxi.hnqz.admin.entity.WmDaPharmacy;
import com.qunzhixinxi.hnqz.admin.entity.WmDeptTaskTypeConfig;
import com.qunzhixinxi.hnqz.admin.entity.WmReport;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackage;
import com.qunzhixinxi.hnqz.admin.entity.WmScorePackageStatus;
import com.qunzhixinxi.hnqz.admin.entity.WmTask;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskContent;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskContentConfig;
import com.qunzhixinxi.hnqz.admin.entity.WmTaskType;
import com.qunzhixinxi.hnqz.admin.entity.WmUserSign;
import com.qunzhixinxi.hnqz.admin.entity.WmUserSignDetail;
import com.qunzhixinxi.hnqz.admin.entity.WmWkAlbum;
import com.qunzhixinxi.hnqz.admin.entity.WmWkArticle;
import com.qunzhixinxi.hnqz.admin.entity.WmWkArticleShare;
import com.qunzhixinxi.hnqz.admin.entity.dto.ReportTaskListPageDTO;
import com.qunzhixinxi.hnqz.admin.entity.dto.WmDeptTaskTypeConfigJsonDTO;
import com.qunzhixinxi.hnqz.admin.entity.vo.WmTaskVO;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.enums.PackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.ReportEnum;
import com.qunzhixinxi.hnqz.admin.enums.ScorePackageStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.TaskStatusEnum;
import com.qunzhixinxi.hnqz.admin.enums.UserSignPurposeEnum;
import com.qunzhixinxi.hnqz.admin.enums.UserSignResultEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDeptRelationMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmReportMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskContentConfigMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskContentMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmTaskMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmUserSignDetailMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmWkAlbumMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmWkArticleMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmWkArticleShareMapper;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDistributionService;
import com.qunzhixinxi.hnqz.admin.service.WmDaHospitalService;
import com.qunzhixinxi.hnqz.admin.service.WmDaPharmacyService;
import com.qunzhixinxi.hnqz.admin.service.WmDeptTaskTypeConfigService;
import com.qunzhixinxi.hnqz.admin.service.WmReportService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageService;
import com.qunzhixinxi.hnqz.admin.service.WmScorePackageStatusService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskService;
import com.qunzhixinxi.hnqz.admin.service.WmTaskTypeService;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 报告审核表
 *
 * @author pigx code generator
 * @date 2020-12-21 20:57:26
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmReportServiceImpl extends ServiceImpl<WmReportMapper, WmReport> implements WmReportService {

	private final WmScorePackageService wmScorePackageService;
	private final WmScorePackageStatusService wmScorePackageStatusService;
	private final WmTaskTypeService wmTaskTypeService;
	private final WmTaskService wmTaskService;
	private final WmTaskMapper wmTaskMapper;
	private final WmTaskContentMapper wmTaskContentMapper;
	private final WmTaskContentConfigMapper wmTaskContentConfigMapper;
	private final WmReportMapper wmReportMapper;
	private final SysUserService sysUserService;
	private final WmDaDistributionService wmDaDistributionService;
	private final WmDaPharmacyService wmDaPharmacyService;
	private final WmDaHospitalService wmDaHospitalService;
	private final WmDeptTaskTypeConfigService wmDeptTaskTypeConfigService;
	private final SysDeptService sysDeptService;
	private final SysDeptRelationMapper sysDeptRelationMapper;
	private final SysDictItemMapper sysDictItemMapper;
	private final WmUserSignMapper wmUserSignMapper;
	private final WmUserSignDetailMapper wmUserSignDetailMapper;
	private final WmWkArticleShareMapper wmWkArticleShareMapper;
	private final WmWkAlbumMapper wmWkAlbumMapper;
	private final WmWkArticleMapper wmWkArticleMapper;
	private final RedisTemplate<String,String> redisTemplate;

	@Override
	public IPage<WmReport> getReportListPage(Page page, WmReport wmReport, String scoreId, String name){
		IPage<WmReport> reportListPage = wmReportMapper.getReportListPage(page, wmReport, scoreId, name);

		List<WmReport> records = reportListPage.getRecords();

		for (WmReport report : records){

			// word报告状态
			WmReportOpt opt = new WmReportOpt();

			// 获取生成记录
			String key = String.format(CacheConstants.EXPORT_NB_REPORT_CACHE, report.getId());
			String o = redisTemplate.opsForValue().get(key);

			if (StrUtil.isNotEmpty(o)) {

				// 生成中的
				if ("GENERATING".equals(o)) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
				}
				// 生成失败的
				else if (o.startsWith("ERROR")) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					opt.setErrorMsg(o.split(StrUtil.UNDERLINE)[1]);
				}
				// 生成失败的2
				else if (!o.startsWith("http")) {
					opt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					opt.setErrorMsg(o);
				}
				// 成功的
				else {
					Long expire = redisTemplate.opsForValue().getOperations().getExpire(key, TimeUnit.SECONDS);
					opt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
					opt.setTtl(LocalDateTime.now().plusSeconds(expire));
					// o = "https://pre.yaoyi.net/admin/sys-file/wmkj/兼职学术信息沟通专员业绩报告(2249)-20211116191527.doc";
					opt.setLatestUrl(o);
				}


			} else {
				opt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
			}
			report.setReportOpt(opt);


			// excel报告状态
			WmReportOpt excelPpt = new WmReportOpt();

			// 获取生成记录
			String excelKey = String.format(CacheConstants.EXCEL_EXPORT_NB_REPORT_CACHE, report.getId());
			String excelO = redisTemplate.opsForValue().get(excelKey);

			if (StrUtil.isNotEmpty(excelO)) {

				// 生成中的
				if ("GENERATING".equals(excelO)) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.GENERATING);
				}
				// 生成失败的
				else if (excelO.startsWith("ERROR")) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					excelPpt.setErrorMsg(excelO.split(StrUtil.UNDERLINE)[1]);
				}
				// 生成失败的2
				else if (!excelO.startsWith("http")) {
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.ERROR);
					excelPpt.setErrorMsg(excelO);
				}
				// 成功的
				else {
					Long expire = redisTemplate.opsForValue().getOperations().getExpire(excelKey, TimeUnit.SECONDS);
					excelPpt.setStatus(WmReportOpt.WmReportOptStatus.GENERATED);
					excelPpt.setTtl(LocalDateTime.now().plusSeconds(expire));
					// o = "https://pre.yaoyi.net/admin/sys-file/wmkj/兼职学术信息沟通专员业绩报告(2249)-20211116191527.doc";
					excelPpt.setLatestUrl(excelO);
				}


			} else {
				excelPpt.setStatus(WmReportOpt.WmReportOptStatus.NOT_GENERATE);
			}
			report.setExcelReportOpt(excelPpt);

		}

		return reportListPage;
	}


	/**
	 * 报告审核状态修改--提交或结单
	 * reportStatus 1:待提交，2:已提交待审批 3：已部分审批  4：已退回 5:已审批待结单 6:已结单
	 * @param wmReport 报告审核表
	 * @return R
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R updateReportStatus(WmReport wmReport) {
		HnqzUser user= SecurityUtils.getUser();
		//提交修改状态，更新时间
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();
		updateWrapper.set(WmReport::getUpdatePeople,user.getId());
		updateWrapper.set(WmReport::getUpdateTime, LocalDateTime.now());
		updateWrapper.set(WmReport::getReportStatus, wmReport.getReportStatus());
		updateWrapper.eq(WmReport::getId,wmReport.getId());
		boolean isupdate=this.update(updateWrapper);
		if(!isupdate){
			return R.failed("更新报告状态失败");

		}
		//报告状态更新之后，要同时更新任务的报告审核状态
		updateTaskStatusByReport(wmReport);

		return R.ok("成功");
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R del(WmReport wmReport)  {
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();
		updateWrapper.set(WmReport::getDelFlag, wmReport.getDelFlag());
		updateWrapper.eq(WmReport::getId,wmReport.getId());
		boolean isupdate=this.update(updateWrapper);
		if(!isupdate){
			return R.failed("更新报告状态失败");
		}
		wmReport.setReportStatus(null);
		updateTaskStatusByReport(wmReport);
		return R.ok("删除成功");
	}



	/**
	 * 报告状态更新之后，要同时更新任务的报告审核状态
	 * @param wmReport
	 */
	@Transactional(rollbackFor = Exception.class)
	public void updateTaskStatusByReport(WmReport wmReport){
		WmScorePackage scorePackage=wmScorePackageService.getById(wmReport.getScoreId());
		//typeid  0 药企  1一级代理商 2二级代理商 3全职学术推广员 4兼职学术推广员
		LambdaUpdateWrapper<WmTask> updateWrapper=new LambdaUpdateWrapper<>();

		if("0".equals(scorePackage.getTypeid())){//药企
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportDrugId,null);
				updateWrapper.set(WmTask::getReportDrugApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportDrugApprovalInfo,null);
			}
			updateWrapper.set(WmTask::getReportDrugApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportDrugId,wmReport.getId());
		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportOneId,null);
				updateWrapper.set(WmTask::getReportOneApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportOneApprovalInfo,null);
			}
			if(SecurityUtils.getRoles().contains(3)){//药企
				updateWrapper.set(WmTask::getReportDrugApprovalStatus,wmReport.getReportStatus());
			}
			updateWrapper.set(WmTask::getReportOneApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportOneId,wmReport.getId());
		}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
			if("1".equals(wmReport.getDelFlag())){
				updateWrapper.set(WmTask::getReportSecondId,null);
				updateWrapper.set(WmTask::getReportSecondApprovalOpinion,null);
				updateWrapper.set(WmTask::getReportSecondApprovalInfo,null);
			}
			if(SecurityUtils.getRoles().contains(4)){// 一级CSO
				updateWrapper.set(WmTask::getReportOneApprovalStatus,wmReport.getReportStatus());
			}
			updateWrapper.set(WmTask::getReportSecondApprovalStatus,wmReport.getReportStatus());
			updateWrapper.eq(WmTask::getReportSecondId,wmReport.getId());
		}
		wmTaskService.update(updateWrapper);
	}

	/**
	 * 结算报告审批-任务复审
	 *
	 * @param reportId        报告id
	 * @param taskId          任务id
	 * @param approvalOpinion 审核状态(1-通过，2-不通过)
	 * @param approvalInfo    审批说明
	 * @return 复审结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean reviewTask(String reportId, String taskId, String approvalOpinion, String approvalInfo) {
		WmReport queryReport = this.getById(reportId);
		if(queryReport == null){
			throw new RuntimeException("未查询到报告信息");
		}
		if (!CollUtil.toList(ReportEnum.REPORT_STATUS_SETTLE.getType(), ReportEnum.REPORT_STATUS_PART_APPROVAL.getType()).contains(queryReport.getReportStatus())) {
			throw new RuntimeException("报告状态不支持复审");
		}

		// 查询任务
		WmTask queryTask = wmTaskService.getById(taskId);

		if (queryTask.getDeptReviewTaskCount() > 0) {
			throw new RuntimeException("不能重复审核");
		}

		LocalDateTime now = LocalDateTime.now();
		Integer operatorId = SecurityUtils.getUser().getId();

		// 查询企业对应的积分包
		Integer reportScorePackageId = queryReport.getScoreId();
		WmScorePackage reportScorePackage = wmScorePackageService.getById(reportScorePackageId);
		// 查询任务对应的积分包
		String taskScorePackageId = queryTask.getScorePackageId();
		WmScorePackage taskScorePackage = wmScorePackageService.getById(taskScorePackageId);
		if (!CollUtil.toList(ScorePackageStatusEnum.TO_BE_SETTLED.val(), ScorePackageStatusEnum.TO_BE_APPROVAL.val(), ScorePackageStatusEnum.IN_PROGRESS.val())
				.contains(taskScorePackage.getScorePackageStatus())) {
			throw new RuntimeException("任务积分包状态不支持复审");
		}

		String taskStatus = "1".equals(approvalOpinion) ?
				TaskStatusEnum.APPROVED.val() : TaskStatusEnum.FAIL_APPROVED.val();
		if (StrUtil.equals(taskStatus, TaskStatusEnum.APPROVED.val())) {
			// 复审通过，才操作
			wmTaskService.task15_30_16_18(queryTask, taskStatus, null);
		}

		// STEP1: 更新任务企业审核状态
		LambdaUpdateWrapper<WmTask> updateTaskWrapper = new LambdaUpdateWrapper<>();
		updateTaskWrapper.eq(WmTask::getId, taskId);
		updateTaskWrapper.eq(queryTask.getUpdateTime() != null, WmTask::getUpdateTime, queryTask.getUpdateTime());
		updateTaskWrapper.eq(queryTask.getUpdateUser() != null, WmTask::getUpdateUser, queryTask.getUpdateUser());
		List<Integer> roles = SecurityUtils.getRoles();
		if (roles.contains(3)) {
			// 药企
			updateTaskWrapper.set(WmTask::getReportDrugApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportDrugApprovalOpinion, approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportDrugApprovalInfo, approvalInfo);
			if ("1".equals(reportScorePackage.getTypeid())) {
				updateTaskWrapper.set(WmTask::getReportOneApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion, approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportOneApprovalInfo, approvalInfo);
			}
		} else if (roles.contains(4)) {
			//一级代理商类型
			updateTaskWrapper.set(WmTask::getReportOneApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion, approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportOneApprovalInfo, approvalInfo);
			if ("2".equals(reportScorePackage.getTypeid())) {
				updateTaskWrapper.set(WmTask::getReportSecondApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportSecondApprovalOpinion, approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportSecondApprovalInfo, approvalInfo);
			}
		}
		// 审核状态同步
		updateTaskWrapper.set(WmTask::getTaskStatus, taskStatus);
		updateTaskWrapper.set(WmTask::getRealFlag, "0");
		updateTaskWrapper.set(WmTask::getDeptReviewTaskCount, 1);
		updateTaskWrapper.set(WmTask::getUpdateTime, now);
		updateTaskWrapper.set(WmTask::getUpdateUser, operatorId);
		log.info("task更新参数：条件id={},updateTime={},updateUser={},更新项approvalOpinion={},approvalInfo={},taskStatus={},realFlag={},deptReviewTaskCount={},updateTime={},updateUser={}",
				taskId, queryTask.getUpdateTime(), queryTask.getUpdateUser(), approvalOpinion, approvalInfo, taskStatus, "0", 1, now, operatorId);
		boolean updateTask = wmTaskService.update(updateTaskWrapper);
		if (!updateTask) {
			throw new RuntimeException("操作失败，请重试");
		}

		WmTask taskSelect = new WmTask();
		if ("0".equals(reportScorePackage.getTypeid())) {//药企
			taskSelect.setReportDrugId(Integer.valueOf(reportId));
			taskSelect.setReportDrugApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		} else if ("1".equals(reportScorePackage.getTypeid())) {//一级代理商类型
			taskSelect.setReportOneId(Integer.valueOf(reportId));
			taskSelect.setReportOneApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		} else if ("2".equals(reportScorePackage.getTypeid())) {//二级代理商类型
			taskSelect.setReportSecondId(Integer.valueOf(reportId));
			taskSelect.setReportSecondApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}
		// 查询提交给企业的待审核的任务
		int toApproveTaskCount = wmTaskMapper.selectCount(Wrappers.lambdaQuery(taskSelect)
				.eq(WmTask::getScorePackageId, taskScorePackageId));

		// STEP2: 更新报告状态
		LambdaUpdateWrapper<WmReport> updateWrapper = new LambdaUpdateWrapper<>();
		updateWrapper.eq(WmReport::getId, Integer.valueOf(reportId));
		updateWrapper.eq(queryReport.getUpdateTime() != null, WmReport::getUpdateTime, queryReport.getUpdateTime());
		updateWrapper.eq(queryReport.getUpdatePeople() != null, WmReport::getUpdatePeople, queryReport.getUpdatePeople());
		// 待审核任务数为空 则说明提交给企业的任务都已审核，否则是部分审核
		String reportStatus = toApproveTaskCount <= 0 ?
				ReportEnum.REPORT_STATUS_SETTLE.getType() : ReportEnum.REPORT_STATUS_PART_APPROVAL.getType();
		updateWrapper.set(WmReport::getReportStatus, reportStatus);
		updateWrapper.set(WmReport::getUpdatePeople, operatorId);
		updateWrapper.set(WmReport::getUpdateTime, now);
		log.info("report更新参数：条件id={},updateTime={},updatePeople={},更新项reportStatus={},updatePeople={},updateTime={}",
				reportId, queryReport.getUpdateTime(), queryReport.getUpdatePeople(), reportStatus, operatorId, now);
		boolean updateReport = this.update(updateWrapper);
		if (!updateReport) {
			throw new RuntimeException("操作失败，请重试");
		}

		// 查询已提交的任务（除去审核不通过的任务）
		List<WmTask> submitedTaskList = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
				.eq(WmTask::getScorePackageId, taskScorePackageId)
				.ne(WmTask::getTaskStatus, TaskStatusEnum.FAIL_APPROVED.val())
				.eq(WmTask::getRealFlag, "0"));
		// 统计判断任务累计积分是否达到积分包的预设值
		int totalTaskScore = submitedTaskList.stream().mapToInt(WmTask::getScore).sum();
		int approvedCount = submitedTaskList.stream()
				.filter(task -> TaskStatusEnum.APPROVED.val().equals(task.getTaskStatus()))
				.map(WmTask::getScore).reduce(0, Integer::sum);

		// STEP3: 更新领包记录
		List<WmScorePackageStatus> packageStatusList = wmScorePackageStatusService.list(Wrappers.<WmScorePackageStatus>lambdaQuery()
				.eq(WmScorePackageStatus::getStatus, PackageStatusEnum.APPROVED.val())
				.eq(WmScorePackageStatus::getPackageId, taskScorePackageId));
		if (CollUtil.isNotEmpty(packageStatusList)) {
			String taskAddFlagStr = totalTaskScore >= taskScorePackage.getScore() ? "0" : "1";
			// 修改领包记录状态
			packageStatusList.forEach(packageStatus -> {
				LambdaUpdateWrapper<WmScorePackageStatus> updateStatusWrapper = Wrappers.<WmScorePackageStatus>lambdaUpdate()
						.eq(WmScorePackageStatus::getId, packageStatus.getId())
						.eq(packageStatus.getUpdateTime() != null, WmScorePackageStatus::getUpdateTime, packageStatus.getUpdateTime())
						.eq(packageStatus.getUpdateUser() != null, WmScorePackageStatus::getUpdateUser, packageStatus.getUpdateUser())
						.set(WmScorePackageStatus::getTaskAddFlag, taskAddFlagStr)
						.set(WmScorePackageStatus::getUpdateTime, now)
						.set(WmScorePackageStatus::getUpdateUser, operatorId);
				log.info("pacakgeStatus更新参数：条件id={},updateTime={},updateUser={},更新项taskAddFlag={},updateTime={},updateUser={}",
						packageStatus.getId(), packageStatus.getUpdateTime(), packageStatus.getUpdateUser(), taskAddFlagStr, now, operatorId);
				boolean pacakgeStatusUpdate = wmScorePackageStatusService.update(updateStatusWrapper);
				if (!pacakgeStatusUpdate) {
					throw new RuntimeException("操作失败，请重试");
				}
			});
		}

		// STEP4: 更新积分包状态
		if ("2".equals(approvalOpinion)) {
			// 审核不通过，则释放积分值，从而直接可以重新做任务
			String scorePackageStatus = approvedCount >= taskScorePackage.getScore() ?
					ScorePackageStatusEnum.TO_BE_APPROVAL.val() : ScorePackageStatusEnum.IN_PROGRESS.val();
			String taskAddFlagStr = approvedCount >= taskScorePackage.getScore() ? "0" : "1";
			// 更新积分包
			log.info("scorePackage更新参数：条件id={},updateTime={},updateUser={},更新项scorePackageStatus={},taskAddFlag={},updateTime={},updateUser={}",
					taskScorePackageId, taskScorePackage.getUpdateTime(), taskScorePackage.getUpdateUser(), scorePackageStatus, taskAddFlagStr, now, operatorId);
			boolean updateScorePackage = wmScorePackageService.update(Wrappers.<WmScorePackage>lambdaUpdate()
					.eq(WmScorePackage::getId, taskScorePackageId)
					.eq(taskScorePackage.getUpdateTime() != null, WmScorePackage::getUpdateTime, taskScorePackage.getUpdateTime())
					.eq(taskScorePackage.getUpdateUser() != null, WmScorePackage::getUpdateUser, taskScorePackage.getUpdateUser())
					.set(WmScorePackage::getScorePackageStatus, scorePackageStatus)
					.set(WmScorePackage::getTaskAddFlag, taskAddFlagStr)
					.set(WmScorePackage::getUpdateTime, now)
					.set(WmScorePackage::getUpdateUser, operatorId));
			if (!updateScorePackage) {
				throw new RuntimeException("操作失败，请重试");
			}
		} else if ("1".equals(approvalOpinion)) {
			// 审核通过，则判断企业审核通过积分值是否达到积分包的预设值
			List<WmTask> approvalTaskList = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
					.eq(WmTask::getScorePackageId, taskScorePackageId)
					.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
					.eq(WmTask::getRealFlag, "0")
					.and(wrapper -> wrapper.eq(WmTask::getReportDrugApprovalOpinion, "1")
							.or().eq(WmTask::getReportOneApprovalOpinion, "1")));
			int totalScore = approvalTaskList.stream().mapToInt(WmTask::getScore).sum();
			String scorePackageStatus = totalScore >= taskScorePackage.getScore() ?
					ScorePackageStatusEnum.TO_BE_SETTLED.val() : ScorePackageStatusEnum.TO_BE_APPROVAL.val();
			String taskAddFlagStr = totalScore >= taskScorePackage.getScore() ? "0" : "1";
			// 更新积分包
			log.info("scorePackage更新参数：条件id={},updateTime={},updateUser={},更新项scorePackageStatus={},taskAddFlag={},updateTime={},updateUser={}",
					taskScorePackageId, taskScorePackage.getUpdateTime(), taskScorePackage.getUpdateUser(), scorePackageStatus, taskAddFlagStr, now, operatorId);
			boolean updateScorePackage = wmScorePackageService.update(Wrappers.<WmScorePackage>lambdaUpdate()
					.eq(WmScorePackage::getId, taskScorePackageId)
					.eq(taskScorePackage.getUpdateTime() != null, WmScorePackage::getUpdateTime, taskScorePackage.getUpdateTime())
					.eq(taskScorePackage.getUpdateUser() != null, WmScorePackage::getUpdateUser, taskScorePackage.getUpdateUser())
					.set(WmScorePackage::getScorePackageStatus, scorePackageStatus)
					.set(WmScorePackage::getTaskAddFlag, taskAddFlagStr)
					.set(WmScorePackage::getUpdateTime, now)
					.set(WmScorePackage::getUpdateUser, operatorId));
			if (!updateScorePackage) {
				throw new RuntimeException("操作失败，请重试");
			}
		}

		return Boolean.TRUE;
	}

	/**
	 * 结算报告审批-快速批量任务审核
	 *
	 * @param reportId        报告id
	 * @param taskIdList      任务id列表
	 * @param approvalOpinion 审核状态(1-通过，2-不通过)
	 * @param approvalInfo    审批说明
	 * @return 审核结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean batchApprovalTask(String reportId, List<String> taskIdList, String approvalOpinion, String approvalInfo) {
		LocalDateTime now = LocalDateTime.now();
		Integer operatorId = SecurityUtils.getUser().getId();

		WmReport queryReport = this.getById(reportId);
		if(queryReport == null){
			throw new RuntimeException("未查询到报告信息");
		}
		if (!CollUtil.toList(ReportEnum.REPORT_STATUS_PART_APPROVAL.getType(), ReportEnum.REPORT_STATUS_APPROVAL.getType()).contains(queryReport.getReportStatus())) {
			throw new RuntimeException("报告状态不支持审核");
		}

		// 批量查询任务
		List<WmTask> wmTasks = wmTaskService.listByIds(taskIdList);
		Map<String, WmTask> taskMap = wmTasks.stream().collect(Collectors.toMap(WmTask::getId, Function.identity()));

		Set<String> scorePackageIds = wmTasks.stream().map(WmTask::getScorePackageId).collect(Collectors.toSet());
		if (scorePackageIds.size() > 1) {
			throw new RuntimeException("只能处理同一个积分包下的任务");
		}

		// 批量查询积分包
		List<WmScorePackage> taskScorePackages = wmScorePackageService.listByIds(scorePackageIds);
		Map<String, WmScorePackage> scorePackageMap = taskScorePackages.stream().collect(Collectors.toMap(WmScorePackage::getId, Function.identity()));

		// 查询企业对应的积分包
		Integer reportScorePackageId = queryReport.getScoreId();
		WmScorePackage reportScorePackage = wmScorePackageService.getById(reportScorePackageId);

		taskIdList.forEach(id -> {
			// 查询这条任务
			WmTask queryTask = taskMap.get(id);
			if (queryTask == null) {
				throw new RuntimeException("任务不存在");
			}

			// STEP4: 如果是源头积分包，则判断是否审核入库调查信息
			if (StringUtils.isBlank(reportScorePackage.getRelationScoreId())) {
				wmTaskService.task15_30_16_18(queryTask, "1".equals(approvalOpinion) ? "3" : "4", approvalInfo);
			}

			// STEP1: 更新任务企业审核状态
			LambdaUpdateWrapper<WmTask> updateTaskWrapper = new LambdaUpdateWrapper<>();
			updateTaskWrapper.eq(WmTask::getId, id);
			updateTaskWrapper.eq(queryTask.getUpdateTime() != null, WmTask::getUpdateTime, queryTask.getUpdateTime());
			updateTaskWrapper.eq(queryTask.getUpdateUser() != null, WmTask::getUpdateUser, queryTask.getUpdateUser());
			List<Integer> roles = SecurityUtils.getRoles();
			if (roles.contains(3)) {
				// 药企
				updateTaskWrapper.set(WmTask::getReportDrugApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportDrugApprovalOpinion, approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportDrugApprovalInfo, approvalInfo);
				if ("1".equals(reportScorePackage.getTypeid())) {
					updateTaskWrapper.set(WmTask::getReportOneApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
					updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion, approvalOpinion);
					updateTaskWrapper.set(WmTask::getReportOneApprovalInfo, approvalInfo);
				}
			} else if (roles.contains(4)) {
				//一级代理商类型
				updateTaskWrapper.set(WmTask::getReportOneApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion, approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportOneApprovalInfo, approvalInfo);
				if ("2".equals(reportScorePackage.getTypeid())) {
					updateTaskWrapper.set(WmTask::getReportSecondApprovalStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
					updateTaskWrapper.set(WmTask::getReportSecondApprovalOpinion, approvalOpinion);
					updateTaskWrapper.set(WmTask::getReportSecondApprovalInfo, approvalInfo);
				}
			}
			// 审核不通过状态同步
			if ("2".equals(approvalOpinion)) {
				updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.FAIL_APPROVED.val());
			} else {
				updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val());
			}
			updateTaskWrapper.set(WmTask::getRealFlag, "0");
			updateTaskWrapper.set(WmTask::getUpdateTime, now);
			updateTaskWrapper.set(WmTask::getUpdateUser, operatorId);
			log.info("task更新参数：条件id={},updateTime={},updateUser={},更新项approvalOpinion={},approvalInfo={},realFlag={},updateTime={},updateUser={}",
					id, queryTask.getUpdateTime(), queryTask.getUpdateUser(), approvalOpinion, approvalInfo, 0, now, operatorId);
			boolean updateTask = wmTaskService.update(updateTaskWrapper);
			if (!updateTask) {
				throw new RuntimeException("操作失败，请重试");
			}

		});

		WmTask taskSelect = new WmTask();
		if ("0".equals(reportScorePackage.getTypeid())) {//药企
			taskSelect.setReportDrugId(Integer.valueOf(reportId));
			taskSelect.setReportDrugApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		} else if ("1".equals(reportScorePackage.getTypeid())) {//一级代理商类型
			taskSelect.setReportOneId(Integer.valueOf(reportId));
			taskSelect.setReportOneApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		} else if ("2".equals(reportScorePackage.getTypeid())) {//二级代理商类型
			taskSelect.setReportSecondId(Integer.valueOf(reportId));
			taskSelect.setReportSecondApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}
		// 查询提交给企业的待审核的任务
		int toApproveTaskCount = wmTaskMapper.selectCount(Wrappers.lambdaQuery(taskSelect));

		// STEP2: 更新报告状态
		LambdaUpdateWrapper<WmReport> updateWrapper = new LambdaUpdateWrapper<>();
		updateWrapper.eq(WmReport::getId, Integer.valueOf(reportId));
		updateWrapper.eq(queryReport.getUpdateTime() != null, WmReport::getUpdateTime, queryReport.getUpdateTime());
		updateWrapper.eq(queryReport.getUpdatePeople() != null, WmReport::getUpdatePeople, queryReport.getUpdatePeople());
		// 待审核任务数为空 则说明提交给企业的任务都已审核，否则是部分审核
		String reportStatus = toApproveTaskCount <= 0 ?
				ReportEnum.REPORT_STATUS_SETTLE.getType() : ReportEnum.REPORT_STATUS_PART_APPROVAL.getType();
		updateWrapper.set(WmReport::getReportStatus, reportStatus);
		updateWrapper.set(WmReport::getUpdatePeople, operatorId);
		updateWrapper.set(WmReport::getUpdateTime, now);
		log.info("report更新参数：条件id={},updateTime={},updatePeople={},更新项reportStatus={},updatePeople={},updateTime={}",
				reportId, queryReport.getUpdateTime(), queryReport.getUpdatePeople(), reportStatus, operatorId, now);
		boolean updateReport = this.update(updateWrapper);
		if (!updateReport) {
			throw new RuntimeException("操作失败，请重试");
		}

		taskScorePackages.forEach(taskScorePackage -> {
			String taskScorePackageId= taskScorePackage.getId();
			// 查询已提交的任务（除去审核不通过的任务）
			List<WmTask> submitedTaskList = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
					.eq(WmTask::getScorePackageId, taskScorePackageId)
					.ne(WmTask::getTaskStatus, TaskStatusEnum.FAIL_APPROVED.val())
					.eq(WmTask::getRealFlag, "0"));
			// 统计判断任务累计积分是否达到积分包的预设值
			int totalTaskScore = submitedTaskList.stream().mapToInt(WmTask::getScore).sum();
			int approvedCount = submitedTaskList.stream()
					.filter(task -> TaskStatusEnum.APPROVED.val().equals(task.getTaskStatus()))
					.map(WmTask::getScore).reduce(0, Integer::sum);

			// STEP3: 更新领包记录
			List<WmScorePackageStatus> packageStatusList = wmScorePackageStatusService.list(Wrappers.<WmScorePackageStatus>lambdaQuery()
					.eq(WmScorePackageStatus::getStatus, PackageStatusEnum.APPROVED.val())
					.eq(WmScorePackageStatus::getPackageId, taskScorePackageId));
			if (CollUtil.isNotEmpty(packageStatusList)) {
				String taskAddFlagStr = totalTaskScore >= taskScorePackage.getScore() ? "0" : "1";
				// 修改领包记录状态
				packageStatusList.forEach(packageStatus -> {
					LambdaUpdateWrapper<WmScorePackageStatus> updateStatusWrapper = Wrappers.<WmScorePackageStatus>lambdaUpdate()
							.eq(WmScorePackageStatus::getId, packageStatus.getId())
							.eq(packageStatus.getUpdateTime() != null, WmScorePackageStatus::getUpdateTime, packageStatus.getUpdateTime())
							.eq(packageStatus.getUpdateUser() != null, WmScorePackageStatus::getUpdateUser, packageStatus.getUpdateUser())
							.set(WmScorePackageStatus::getTaskAddFlag, taskAddFlagStr)
							.set(WmScorePackageStatus::getUpdateTime, now)
							.set(WmScorePackageStatus::getUpdateUser, operatorId);
					log.info("pacakgeStatus更新参数：条件id={},updateTime={},updateUser={},更新项taskAddFlag={},updateTime={},updateUser={}",
							packageStatus.getId(), packageStatus.getUpdateTime(), packageStatus.getUpdateUser(), taskAddFlagStr, now, operatorId);
					boolean pacakgeStatusUpdate = wmScorePackageStatusService.update(updateStatusWrapper);
					if (!pacakgeStatusUpdate) {
						throw new RuntimeException("操作失败，请重试");
					}
				});
			}

			// STEP4: 更新积分包状态
			if ("2".equals(approvalOpinion)) {
				// 审核不通过，则释放积分值，从而直接可以重新做任务
				String scorePackageStatus = approvedCount >= taskScorePackage.getScore() ?
						ScorePackageStatusEnum.TO_BE_APPROVAL.val() : ScorePackageStatusEnum.IN_PROGRESS.val();
				String taskAddFlagStr = approvedCount >= taskScorePackage.getScore() ? "0" : "1";
				// 更新积分包
				log.info("scorePackage更新参数：条件id={},updateTime={},updateUser={},更新项scorePackageStatus={},taskAddFlag={},updateTime={},updateUser={}",
						taskScorePackageId, taskScorePackage.getUpdateTime(), taskScorePackage.getUpdateUser(), scorePackageStatus, taskAddFlagStr, now, operatorId);
				boolean updateScorePackage = wmScorePackageService.update(Wrappers.<WmScorePackage>lambdaUpdate()
						.eq(WmScorePackage::getId, taskScorePackageId)
						.eq(taskScorePackage.getUpdateTime() != null, WmScorePackage::getUpdateTime, taskScorePackage.getUpdateTime())
						.eq(taskScorePackage.getUpdateUser() != null, WmScorePackage::getUpdateUser, taskScorePackage.getUpdateUser())
						.set(WmScorePackage::getScorePackageStatus, scorePackageStatus)
						.set(WmScorePackage::getTaskAddFlag, taskAddFlagStr)
						.set(WmScorePackage::getUpdateTime, now)
						.set(WmScorePackage::getUpdateUser, operatorId));
				if (!updateScorePackage) {
					throw new RuntimeException("操作失败，请重试");
				}
			} else if ("1".equals(approvalOpinion)) {
				// 审核通过，则判断企业审核通过积分值是否达到积分包的预设值
				List<WmTask> approvalTaskList = wmTaskService.list(Wrappers.<WmTask>lambdaQuery()
						.eq(WmTask::getScorePackageId, taskScorePackageId)
						.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
						.eq(WmTask::getRealFlag, "0")
						.and(wrapper -> wrapper.eq(WmTask::getReportDrugApprovalOpinion, "1")
								.or().eq(WmTask::getReportOneApprovalOpinion, "1")));
				int totalScore = approvalTaskList.stream().mapToInt(WmTask::getScore).sum();
				String scorePackageStatus = totalScore >= taskScorePackage.getScore() ?
						ScorePackageStatusEnum.TO_BE_SETTLED.val() : approvedCount >= taskScorePackage.getScore() ?
						ScorePackageStatusEnum.TO_BE_APPROVAL.val() : ScorePackageStatusEnum.IN_PROGRESS.val();
				String taskAddFlagStr = totalScore >= taskScorePackage.getScore() ? "0" : "1";
				// 更新积分包
				log.info("scorePackage更新参数：条件id={},updateTime={},updateUser={},更新项scorePackageStatus={},taskAddFlag={},updateTime={},updateUser={}",
						taskScorePackageId, taskScorePackage.getUpdateTime(), taskScorePackage.getUpdateUser(), scorePackageStatus, taskAddFlagStr, now, operatorId);
				boolean updateScorePackage = wmScorePackageService.update(Wrappers.<WmScorePackage>lambdaUpdate()
						.eq(WmScorePackage::getId, taskScorePackageId)
						.eq(taskScorePackage.getUpdateTime() != null, WmScorePackage::getUpdateTime, taskScorePackage.getUpdateTime())
						.eq(taskScorePackage.getUpdateUser() != null, WmScorePackage::getUpdateUser, taskScorePackage.getUpdateUser())
						.set(WmScorePackage::getScorePackageStatus, scorePackageStatus)
						.set(WmScorePackage::getTaskAddFlag, taskAddFlagStr)
						.set(WmScorePackage::getUpdateTime, now)
						.set(WmScorePackage::getUpdateUser, operatorId));
				if (!updateScorePackage) {
					throw new RuntimeException("操作失败，请重试");
				}
			}
		});

		return Boolean.TRUE;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public R approvalReport(WmReport wmReport, List<String> taskIds, String approvalOpinion, String approvalInfo) {
		List<WmTask> taskList=new ArrayList<>();//该报告关联的所有未提交的任务

		WmScorePackage scorePackage = wmScorePackageService.getById(wmReport.getScoreId());
		// 如果是源头积分包，则判断是否审核入库调查信息
		if (StringUtils.isBlank(scorePackage.getRelationScoreId())) {
			List<WmTask> tasks = wmTaskService.list(Wrappers.<WmTask>lambdaQuery().in(WmTask::getId, taskIds));
			// 调查信息审核（15、30-医院 16-商业公司 18-药店）
			tasks.forEach(task -> {
				if ("15".equals(task.getTaskTypeId()) || "30".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaHospitalService.approvalHospital(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("wmTask");
					Map<String, String> paramMap = BeanUtil.toBean(data.get("newinfo"), Map.class);
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaHospitalService.updateByApproval(paramMap);
				} else if ("16".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaDistributionService.approvalDistribution(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("wmTask");
					Map<String, String> paramMap = BeanUtil.toBean(data.get("newinfo"), Map.class);
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaDistributionService.updateByApproval(paramMap);
				} else if ("18".equals(task.getTaskTypeId())) {
					Map<String, Object> data = (Map<String, Object>) wmDaPharmacyService.approvalPharmacy(task.getId()).getData();
					WmTask _wmTask = (WmTask) data.get("_wmTask");
					WmDaPharmacy info = (WmDaPharmacy) data.get("info");
					Map<String, String> paramMap = MapUtil.newHashMap(4);
					paramMap.put("infoId", info.getId());
					paramMap.put("approvalInfo", approvalInfo);
					paramMap.put("approvalOpinion", approvalOpinion);
					paramMap.put("compareResult", _wmTask.getCompareResult());
					paramMap.put("taskId", task.getId());
					wmDaPharmacyService.updateByapproval(paramMap);
				}
			});
		}

		//更新任务表审批意见,查询所有未审核的任务，判断报告关联任务是否全部审核
		LambdaUpdateWrapper<WmTask> updateTaskWrapper=new LambdaUpdateWrapper<>();
		WmTask wmTask=new WmTask();
		if(SecurityUtils.getRoles().contains(3)){//药企
			updateTaskWrapper.set(WmTask::getReportDrugApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportDrugApprovalOpinion,approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportDrugApprovalInfo,approvalInfo);

			if("1".equals(scorePackage.getTypeid())){
				updateTaskWrapper.set(WmTask::getReportOneApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion,approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportOneApprovalInfo,approvalInfo);
			}
			updateTaskWrapper.in(WmTask::getId,taskIds);

		}else if(SecurityUtils.getRoles().contains(4)){//一级代理商类型

			updateTaskWrapper.set(WmTask::getReportOneApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
			updateTaskWrapper.set(WmTask::getReportOneApprovalOpinion,approvalOpinion);
			updateTaskWrapper.set(WmTask::getReportOneApprovalInfo,approvalInfo);
			if("2".equals(scorePackage.getTypeid())){
				updateTaskWrapper.set(WmTask::getReportSecondApprovalStatus,ReportEnum.REPORT_STATUS_SETTLE.getType());
				updateTaskWrapper.set(WmTask::getReportSecondApprovalOpinion,approvalOpinion);
				updateTaskWrapper.set(WmTask::getReportSecondApprovalInfo,approvalInfo);
			}
			updateTaskWrapper.in(WmTask::getId,taskIds);
		}
		if("0".equals(scorePackage.getTypeid())){//药企
			wmTask.setReportDrugId(wmReport.getId());
			wmTask.setReportDrugApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
			wmTask.setReportOneId(wmReport.getId());
			wmTask.setReportOneApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}else if("2".equals(scorePackage.getTypeid())) {//二级代理商类型
			wmTask.setReportSecondId(wmReport.getId());
			wmTask.setReportSecondApprovalStatus(ReportEnum.REPORT_STATUS_APPROVAL.getType());
		}
//
		if ("1".equals(approvalOpinion)) {
			updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val());
		} else if ("2".equals(approvalOpinion)) {
			// 审核不通过，则设置TaskStatus为不通过
			updateTaskWrapper.set(WmTask::getTaskStatus, TaskStatusEnum.FAIL_APPROVED.val());
		}
		updateTaskWrapper.set(WmTask::getRealFlag, "0");
		wmTaskService.update(updateTaskWrapper);
		taskList = wmTaskService.list(Wrappers.query(wmTask));
		//更新报告状态
		HnqzUser user= SecurityUtils.getUser();
		//提交修改状态，更新时间
		LambdaUpdateWrapper<WmReport> updateWrapper=new LambdaUpdateWrapper<>();

		//判断任务是不是已经全部审核完成，所有的任务都完成才算完成
		if(taskList.isEmpty()){//taskList为空，说明任务都已审核
			updateWrapper.set(WmReport::getReportStatus, ReportEnum.REPORT_STATUS_SETTLE.getType());
		}else {//不为空，说明是部分审批
			updateWrapper.set(WmReport::getReportStatus, ReportEnum.REPORT_STATUS_PART_APPROVAL.getType());
		}
		updateWrapper.set(WmReport::getUpdatePeople, user.getId());
		updateWrapper.set(WmReport::getUpdateTime, LocalDateTime.now());
		updateWrapper.eq(WmReport::getId, wmReport.getId());
		this.update(updateWrapper);

		LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.in(WmTask::getId, taskIds);
		List<WmTask> wmTaskList = wmTaskService.list(queryWrapper);
		if ("2".equals(approvalOpinion)) {
			// 审核不通过，则释放积分值，从而直接可以重新做任务
			wmTaskList.forEach(task -> {
				// 查询积分包状态
				WmScorePackageStatus scorePackageStatus = new WmScorePackageStatus();
				scorePackageStatus.setPackageId(task.getScorePackageId());
				scorePackageStatus.setUserId(task.getTaskUserId());
				scorePackageStatus.setStatus("2");//接单审核通过
				WmScorePackageStatus updateScorePackageStatus = wmScorePackageStatusService.getOne(Wrappers.query(scorePackageStatus));
				// 查询已提交的任务（除去审核不通过的任务）
				Wrapper<WmTask> taskQueryWrapper = Wrappers.<WmTask>lambdaQuery()
						.eq(WmTask::getScorePackageId, task.getScorePackageId())
						.in(WmTask::getTaskStatus, "1", "2", "3")
						.eq(WmTask::getRealFlag, "0");
				List<WmTask> submitedTaskList = wmTaskService.list(taskQueryWrapper);
				// 统计判断任务累计积分是否达到积分包的预设值
				int totalTaskScore = submitedTaskList.stream().mapToInt(WmTask::getScore).sum();
				WmScorePackage taskScorePackage = wmScorePackageService.getById(task.getScorePackageId());
				WmScorePackage updateScorePackage = new WmScorePackage();
				updateScorePackage.setId(task.getScorePackageId());
				if (totalTaskScore >= taskScorePackage.getScore()) {
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.TO_BE_APPROVAL.val());
					updateScorePackage.setTaskAddFlag("0");
					updateScorePackageStatus.setTaskAddFlag("0");
				}else {
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.IN_PROGRESS.val());
					updateScorePackage.setTaskAddFlag("1");
					updateScorePackageStatus.setTaskAddFlag("1");
				}
				wmScorePackageStatusService.updateById(updateScorePackageStatus);
				wmScorePackageService.updateById(updateScorePackage);
			});
		} else if ("1".equals(approvalOpinion)) {
			// 审核通过，则判断企业审核通过积分值是否达到积分包的预设值
			wmTaskList.forEach(task -> {
				Wrapper<WmTask> apporvalWrapper = Wrappers.<WmTask>lambdaQuery()
						.eq(WmTask::getScorePackageId, task.getScorePackageId())
						.eq(WmTask::getTaskStatus, TaskStatusEnum.APPROVED.val())
						.eq(WmTask::getRealFlag, "0")
						.and(wrapper -> wrapper.eq(WmTask::getReportDrugApprovalOpinion, "1")
								.or().eq(WmTask::getReportOneApprovalOpinion, "1"));
				List<WmTask> approvalTaskList = wmTaskService.list(apporvalWrapper);
				int totalScore = approvalTaskList.stream().mapToInt(WmTask::getScore).sum();
				WmScorePackage taskScorePackage = wmScorePackageService.getById(task.getScorePackageId());
				if (totalScore >= taskScorePackage.getScore()) {
					WmScorePackage updateScorePackage = new WmScorePackage();
					updateScorePackage.setId(task.getScorePackageId());
					updateScorePackage.setScorePackageStatus(ScorePackageStatusEnum.TO_BE_SETTLED.val());
					updateScorePackage.setTaskAddFlag("0");
					wmScorePackageService.updateById(updateScorePackage);
				}
			});
		}

		return R.ok("审批成功");
	}

	/**
	 * 企业根据任务类型审核任务
	 *
	 * @param taskIds
	 * @param approvalOpinion
	 * @param approvalInfo
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> approvalReportByTaskType(List<String> taskIds, String approvalOpinion, String approvalInfo) {
		LambdaQueryWrapper<WmTask> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.in(WmTask::getId, taskIds);
		List<WmTask> wmTaskList = wmTaskService.list(queryWrapper);

		// 根据task任务表 反查report报告表
		// 以reportId（reportDrugId或reportOneId字段）为key进行分类
		Map<Integer, List<WmTask>> reportIdMap = wmTaskList.stream()
				.filter(wmTask -> wmTask.getReportDrugId() != null || wmTask.getReportOneId() != null)
				.collect(Collectors.groupingBy(wmTask -> wmTask.getReportDrugId() != null ?
						wmTask.getReportDrugId() : wmTask.getReportOneId()));

		AtomicBoolean successFlag = new AtomicBoolean(true);
		reportIdMap.forEach((reportId, wmTasks) -> {
			WmReport queryReport = this.getById(reportId);
			List<String> taskIdList = wmTasks.stream().map(WmTask::getId).collect(Collectors.toList());
			log.info("企业批量审核任务reportId={}, taskIds={}", reportId, taskIdList);

			// 调用原有审核方法
			R approvalReportResult = this.approvalReport(queryReport, taskIdList, approvalOpinion, approvalInfo);

			if (approvalReportResult.getCode() != 0) {
				successFlag.set(false);
			}
		});

		return successFlag.get() ? R.ok("审批成功") : R.failed("审批失败");
	}

	/**
	 * 报告管理--查看
	 *
	 * @param page                  页
	 * @param reportTaskListPageDTO 报告任务列表DTO
	 * @return {@link R}
	 */
	@Override
	public R getTaskListPage(Page page, ReportTaskListPageDTO reportTaskListPageDTO) {
		Integer isApprovalPage = reportTaskListPageDTO.getIsApprovalPage();
		String reportApprovalOpinion = reportTaskListPageDTO.getReportApprovalOpinion();
		String taskStatus = reportTaskListPageDTO.getTaskStatus();
		String taskNumber = reportTaskListPageDTO.getTaskNumber();
		String taskTypeId = reportTaskListPageDTO.getTaskTypeId();
		WmReport wmReport = reportTaskListPageDTO.getWmReport();

		Page pages=null;
		WmScorePackage scorePackage=wmScorePackageService.getById(wmReport.getScoreId());
		LambdaQueryWrapper<WmTask> lambdaQueryWrapper= Wrappers.<WmTask>query().lambda();
		lambdaQueryWrapper.and(wrapper -> wrapper.eq(WmTask::getReportDrugId, wmReport.getId())
				.or().eq(WmTask::getReportOneId, wmReport.getId())
				.or().eq(WmTask::getReportSecondId, wmReport.getId()));
		// 如果是审批页面，则过滤掉通过和不通过的数据（reportDrugApprovalOpinion或reportOneApprovalOpinion：1通过，2不通过）
		if (isApprovalPage != null && isApprovalPage.equals(1)) {
			if ("0".equals(scorePackage.getTypeid())) {//药企
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
			} else if ("1".equals(scorePackage.getTypeid())) {//一级代理商类型
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
			} else if ("2".equals(scorePackage.getTypeid())) {//二级代理商类型
				lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportOneApprovalOpinion));
			}
		} else {
			// 查看页面, 审核状态筛选条件
			if (StrUtil.equals("0", reportApprovalOpinion)) {
				// 未审核的条件
				if ("0".equals(scorePackage.getTypeid())) {//药企
					lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
				} else if ("1".equals(scorePackage.getTypeid())) {//一级代理商类型
					lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
				} else if ("2".equals(scorePackage.getTypeid())) {//二级代理商类型
					lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportOneApprovalOpinion));
				}
			} else if (StrUtil.equalsAny(reportApprovalOpinion, "1", "2")) {
				// 审核通过/不通过的条件
				if ("0".equals(scorePackage.getTypeid())) {//药企
					lambdaQueryWrapper.eq(WmTask::getReportDrugApprovalOpinion, reportApprovalOpinion);
				} else if ("1".equals(scorePackage.getTypeid())) {//一级代理商类型
					lambdaQueryWrapper.eq(WmTask::getReportDrugApprovalOpinion, reportApprovalOpinion);
				} else if ("2".equals(scorePackage.getTypeid())) {//二级代理商类型
					lambdaQueryWrapper.eq(WmTask::getReportOneApprovalOpinion, reportApprovalOpinion);
				}
			}
		}

		if (StrUtil.isNotBlank(taskNumber)) {
			lambdaQueryWrapper.like(WmTask::getTaskNumber, taskNumber);
		}
		if (StrUtil.isNotBlank(taskTypeId)) {
			lambdaQueryWrapper.eq(WmTask::getTaskTypeId, taskTypeId);
		}
		if (StrUtil.isNotBlank(taskStatus)) {
			lambdaQueryWrapper.eq(WmTask::getTaskStatus, taskStatus);
		}
		pages=wmTaskService.page(page, lambdaQueryWrapper);
//		if("0".equals(scorePackage.getTypeid())){//药企
//			wmTask.setReportDrugId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
//			wmTask.setReportOneId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
//			wmTask.setReportSecondId(wmReport.getId());
//			pages=wmTaskService.page(page, Wrappers.query(wmTask));
//		}
		if(pages==null){
			return R.ok();
		}
		List<WmTask> taskList=pages.getRecords();
		if(!taskList.isEmpty()){
			List<WmScorePackage> taskScorePackages = wmScorePackageService.listByIds(taskList.stream().map(WmTask::getScorePackageId).collect(Collectors.toSet()));
			Map<String, WmScorePackage> scorePackageMap = taskScorePackages.stream().collect(Collectors.toMap(WmScorePackage::getId, Function.identity()));
			List<Map> resList =taskList.stream().map(wmtask -> {
				Map map = JSON.parseObject(JSON.toJSONString(wmtask), Map.class);
				map.put("reportNo",wmReport.getReportNo());
				map.put("scoreId",wmReport.getScoreId());
				if("0".equals(scorePackage.getTypeid())){//药企
					map.put("reportApprovalOpinion",wmtask.getReportDrugApprovalOpinion());
				}else if("1".equals(scorePackage.getTypeid())){//一级代理商类型
//					map.put("reportApprovalOpinion",wmtask.getReportOneApprovalOpinion());
					if (StrUtil.isNotBlank(wmtask.getReportDrugApprovalOpinion())) {
						map.put("reportApprovalOpinion", wmtask.getReportDrugApprovalOpinion());
					} else {
						// 解决一级代理商自己给自己发包的情况
						map.put("reportApprovalOpinion", wmtask.getReportOneApprovalOpinion());
					}
				}else if("2".equals(scorePackage.getTypeid())){//二级代理商类型
//					map.put("reportApprovalOpinion",wmtask.getReportSecondApprovalOpinion());
					map.put("reportApprovalOpinion",wmtask.getReportOneApprovalOpinion());
				}
				// 姓名
				if (StringUtils.isNumeric(wmtask.getTaskUserId())) {
					SysUser user = sysUserService.getById(Integer.parseInt(wmtask.getTaskUserId()));
					if (user != null) {
						map.put("taskUsername", user.getRealname());
					}
				}
				map.put("scorePackageStatus", scorePackageMap.getOrDefault(wmtask.getScorePackageId(), new WmScorePackage()).getScorePackageStatus());
				return map;
			}).collect(Collectors.toList());

			page.setRecords(resList);
		}
		return R.ok(pages);
	}

	/**
	 * 根据任务类型查询任务
	 *
	 * @param page          分页参数
	 * @param taskTypeName  任务类型名称
	 * @param entTaskStatus 审核状态 0待审核 1通过，2不通过
	 * @return
	 */
	@Override
	public R<?> taskPageByTaskType(Page<WmTaskVO> page, String taskTypeName, String[] entTaskStatus) {

		Integer deptId = SecurityUtils.getUser().getDeptId();

		LambdaQueryWrapper<WmTaskType> queryWrapper = Wrappers.<WmTaskType>lambdaQuery()
				.eq(WmTaskType::getTaskTypeLevel, "1")
				.eq(WmTaskType::getEnableFlag, EnableEnum.ENABLE.val());
		if (StrUtil.isNotBlank(taskTypeName)) {
			queryWrapper.eq(WmTaskType::getTaskTypeName, taskTypeName);
		} else {
			// 查询该企业可查询的任务类型权限
			// 本级dept对应任务类型配置
			WmDeptTaskTypeConfig deptTaskTypeConfig = wmDeptTaskTypeConfigService.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
					.eq(WmDeptTaskTypeConfig::getDeptId, deptId));
			List<WmDeptTaskTypeConfigJsonDTO> filterConfigJsonDTOList = null;
			if (deptTaskTypeConfig != null) {
				// 查询该企业当前任务类型下 的任务类型配置数据
				List<WmDeptTaskTypeConfigJsonDTO> configJsonDTOList =
						JSON.parseArray(deptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
				filterConfigJsonDTOList = configJsonDTOList.stream()
						.filter(configJsonDTO -> configJsonDTO.getQualifiedTask())
						.collect(Collectors.toList());
			}

			// 查询上级的deptId
			List<SysDeptRelation> ancestorDeptRelations = sysDeptRelationMapper.selectList(Wrappers.<SysDeptRelation>lambdaQuery()
					.eq(SysDeptRelation::getDescendant, deptId)
					.ne(SysDeptRelation::getAncestor, deptId)
					.ne(SysDeptRelation::getAncestor, 1));
			List<WmDeptTaskTypeConfigJsonDTO> filterAncestorConfigList = new ArrayList<>();
			ancestorDeptRelations.forEach(ancestorDeptRelation -> {
				// 上级dept对应任务类型配置
				if (ancestorDeptRelation != null) {
					WmDeptTaskTypeConfig ancestorDeptTaskTypeConfig = wmDeptTaskTypeConfigService.getOne(Wrappers.<WmDeptTaskTypeConfig>lambdaQuery()
							.eq(WmDeptTaskTypeConfig::getDeptId, ancestorDeptRelation.getAncestor()));
					if (ancestorDeptTaskTypeConfig != null) {
						List<WmDeptTaskTypeConfigJsonDTO> ancestorConfigJsonDTOList =
								JSON.parseArray(ancestorDeptTaskTypeConfig.getTaskTypeJson(), WmDeptTaskTypeConfigJsonDTO.class);
						// 转换为key value形式
						filterAncestorConfigList.addAll(ancestorConfigJsonDTOList.stream()
								.filter(configJsonDTO -> configJsonDTO.getQualifiedTask() && configJsonDTO.getAssociateLower())
								.collect(Collectors.toList()));
					}
				}
			});

			// 任务配置为空，则直接返回空page
			if (CollectionUtil.isEmpty(filterConfigJsonDTOList) && CollectionUtil.isEmpty(filterAncestorConfigList)) {
				log.warn("任务类型权限不存在");
				return R.failed("任务类型不存在");
			}

			Set<String> taskTypeNames = new HashSet<>();
			if (CollUtil.isNotEmpty(filterConfigJsonDTOList)) {
				taskTypeNames.addAll(filterConfigJsonDTOList.stream().map(WmDeptTaskTypeConfigJsonDTO::getTaskTypeName)
						.collect(Collectors.toList()));
			}
			if (CollUtil.isNotEmpty(filterAncestorConfigList)) {
				taskTypeNames.addAll(filterAncestorConfigList.stream().map(WmDeptTaskTypeConfigJsonDTO::getTaskTypeName)
						.collect(Collectors.toList()));
			}
			queryWrapper.in(WmTaskType::getTaskTypeName, taskTypeNames);
		}
		List<WmTaskType> baseTaskTypeList = wmTaskTypeService.list(queryWrapper);
		if (CollUtil.isEmpty(baseTaskTypeList)) {
			log.warn("任务类型不存在tastTypeName={}", baseTaskTypeList);
			return R.failed("任务类型不存在");
		}

		// 根据drugEntId查询WmReport
//		List<WmReport> wmReportList = wmReportMapper.selectList(Wrappers.<WmReport>lambdaQuery()
//				.eq(WmReport::getDrugEntId, SecurityUtils.getUser().getDeptId()));

//		WmReport queryReport = new WmReport();
//		queryReport.setDrugEntId(SecurityUtils.getUser().getDeptId());
//		queryReport.setTaskTypeIdList(baseTaskTypeList.stream().map(WmTaskType::getId).collect(Collectors.toList()));

		Map<String, Object> entTaskStatusMap = new HashMap<>();
		SysDept dept = sysDeptService.getById(deptId);
		entTaskStatusMap.put("drugEntId", deptId);
		entTaskStatusMap.put("taskTypeIdList", baseTaskTypeList.stream().map(WmTaskType::getId).collect(Collectors.toList()));
//		entTaskStatusMap.put("reportIdList", wmReportList.stream().map(WmReport::getId).collect(Collectors.toList()));
		// 用来判断是否为药企deptLevel=2
		entTaskStatusMap.put("deptLevel", dept.getLevel());

		// 拆分是否通过状态、待审核状态，用于sql优化~
		if (ArrayUtil.isNotEmpty(entTaskStatus)) {
			// 通过状态
			String approvedStatus = null;
			// 待审核状态
			Boolean waitFlag = false;
			if (ArrayUtil.containsAll(entTaskStatus, "1", "2")) {
				// 通过+不通过
				approvedStatus = "3";
			} else if (ArrayUtil.contains(entTaskStatus, "1")) {
				// 通过
				approvedStatus = "1";
			} else if (ArrayUtil.contains(entTaskStatus, "2")) {
				// 不通过
				approvedStatus = "2";
			}
			if (ArrayUtil.contains(entTaskStatus, "0")) {
				// 待审核
				waitFlag = true;
			}
			entTaskStatusMap.put("entTaskStatus", entTaskStatus);
			entTaskStatusMap.put("approvedStatus", approvedStatus);
			entTaskStatusMap.put("waitFlag", waitFlag);
//			queryReport.setEntTaskStatus(entTaskStatusMap);
		}
		IPage<WmTaskVO> pages = wmTaskMapper.entTaskPageByTaskType(page, entTaskStatusMap);

		Page<Map<String, Object>> mapPage = new Page<>(pages.getCurrent(), pages.getSize(), pages.getTotal());
		List<WmTaskVO> taskList = pages.getRecords();
		if (CollUtil.isNotEmpty(taskList)) {
			List<Map<String, Object>> resList = taskList.stream().map(wmtask -> {
				Map<String, Object> map = JSON.parseObject(JSON.toJSONString(wmtask), Map.class);
				map.put("reportNo", wmtask.getReportNo());
				map.put("scoreId", wmtask.getScoreId());
				if (dept.getLevel() == 2) {//药企
					map.put("reportApprovalOpinion", wmtask.getReportDrugApprovalOpinion());
				} else if (dept.getLevel() == 3) {//一级代理商类型
					map.put("reportApprovalOpinion", wmtask.getReportDrugApprovalOpinion());
				} else if (dept.getLevel() == 4) {//二级代理商类型
					map.put("reportApprovalOpinion", wmtask.getReportOneApprovalOpinion());
				}
				// 姓名
				if (StringUtils.isNumeric(wmtask.getTaskUserId())) {
					SysUser user = sysUserService.getById(Integer.parseInt(wmtask.getTaskUserId()));
					if (user != null) {
						map.put("taskUsername", user.getRealname());
					}
				}
				map.put("scorePackageName", wmtask.getScorePackageName());
				// 积分包名称取当前企业发的包包名
				WmScorePackage scorePackage = wmScorePackageService.getById(wmtask.getScorePackageId());
				// 判断此包是不是当前企业发的
				if (!deptId.equals(Integer.parseInt(scorePackage.getSendPackageDeptId())) && StrUtil.isNotBlank(scorePackage.getRelationScoreId())) {
					// 不是则继续查询父级包
					WmScorePackage pScorePackage = wmScorePackageService.getById(scorePackage.getRelationScoreId());
					map.put("scorePackageName", pScorePackage.getScorePackageName());
					if (!deptId.equals(Integer.parseInt(pScorePackage.getSendPackageDeptId())) && StrUtil.isNotBlank(pScorePackage.getRelationScoreId())) {
						// 不是则再继续查询父级包
						WmScorePackage ppScorePackage = wmScorePackageService.getById(pScorePackage.getRelationScoreId());
						map.put("scorePackageName", ppScorePackage.getScorePackageName());
					}
				}
				return map;
			}).collect(Collectors.toList());

			mapPage.setRecords(resList);
		}
		return R.ok(mapPage);
	}

	/**
	 * 根据积分包id查询待审核任务详情列表
	 *
	 * @param reportId 报告id
	 * @param taskTypeId 任务类型id
	 * @return 列表结果
	 */
	@Override
	public List<Map<String, Object>> listTaskInfo(Integer reportId, String taskTypeId) {
		WmReport queryReport = this.getById(reportId);
		if (queryReport == null) {
			throw new RuntimeException("未查询到报告信息");
		}
		WmScorePackage reportScorePackage = wmScorePackageService.getById(queryReport.getScoreId());

		LambdaQueryWrapper<WmTask> lambdaQueryWrapper= Wrappers.<WmTask>query().lambda();
		lambdaQueryWrapper.and(wrapper -> wrapper.eq(WmTask::getReportDrugId, reportId)
				.or().eq(WmTask::getReportOneId, reportId)
				.or().eq(WmTask::getReportSecondId, reportId));
		if ("0".equals(reportScorePackage.getTypeid())) {//药企
			lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
		} else if ("1".equals(reportScorePackage.getTypeid())) {//一级代理商类型
			lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportDrugApprovalOpinion));
		} else if ("2".equals(reportScorePackage.getTypeid())) {//二级代理商类型
			lambdaQueryWrapper.and(wrapper -> wrapper.isNull(WmTask::getReportOneApprovalOpinion));
		}
		if (StrUtil.isNotBlank(taskTypeId)) {
			lambdaQueryWrapper.eq(WmTask::getTaskTypeId, taskTypeId);
		}
		lambdaQueryWrapper.eq(WmTask::getDelFlag, DelEnum.NOT_DEL.val());
		lambdaQueryWrapper.orderByDesc(WmTask::getLookintoDate);
		List<WmTask> taskList = wmTaskMapper.selectList(lambdaQueryWrapper);

		// 添加结算详情
		List<Map<String, Object>> resultList = new ArrayList<>();
		if (CollUtil.isNotEmpty(taskList)) {
			Set<String> packageIdSet = taskList.stream().map(WmTask::getScorePackageId).collect(Collectors.toSet());
			// 查询scorePackage
			List<WmScorePackage> scorePackages = wmScorePackageService.listByIds(packageIdSet);
			Map<String, WmScorePackage> scorePackageMap = scorePackages.stream().collect(Collectors.toMap(WmScorePackage::getId, Function.identity()));
			// 查询taskType
			Set<String> taskTypeIdSet = taskList.stream().map(WmTask::getTaskTypeId).collect(Collectors.toSet());
			List<WmTaskType> taskTypeList = wmTaskTypeService.listByIds(taskTypeIdSet);
			Map<String, String> taskTypeNameMap = taskTypeList.stream().collect(Collectors.toMap(WmTaskType::getId, WmTaskType::getTaskTypeName));
			// 查询taskContent
			Set<String> taskConentIdSet = taskList.stream().map(WmTask::getTaskContentId).collect(Collectors.toSet());
			List<WmTaskContent> taskContentList = wmTaskContentMapper.selectBatchIds(taskConentIdSet);
			Map<Integer, WmTaskContent> taskContentMap = taskContentList.stream().collect(Collectors.toMap(WmTaskContent::getId, Function.identity()));
			// 查询打卡记录userSign
			boolean hasSignTask = taskList.stream().anyMatch(wmTask -> "5".equals(wmTask.getTaskTypeId())
					|| "6".equals(wmTask.getTaskTypeId()) || "33".equals(wmTask.getTaskTypeId()));
			Map<Integer, WmUserSign> userSignMap = new HashMap<>();
			Map<Integer, WmUserSignDetail> userSignDetailMap = new HashMap<>();
			Map<String, List<SysDictItem>> userSignPurposeMap = new HashMap<>();
			if (hasSignTask) {
				Set<String> userSignIdSet = taskContentList.stream()
						.map(WmTaskContent::getTemp3)
						.filter(StrUtil::isNotBlank)
						.collect(Collectors.toSet());
				List<WmUserSign> userSignList = wmUserSignMapper.selectBatchIds(userSignIdSet);
				userSignMap.putAll(userSignList.stream().collect(Collectors.toMap(WmUserSign::getId, Function.identity())));
				List<WmUserSignDetail> signDetailList = wmUserSignDetailMapper.selectList(Wrappers.<WmUserSignDetail>lambdaQuery()
						.in(WmUserSignDetail::getSignId, userSignIdSet));
				if (CollUtil.isNotEmpty(signDetailList)) {
					userSignDetailMap.putAll(signDetailList.stream().collect(Collectors.toMap(WmUserSignDetail::getSignId, Function.identity())));
				}

				// 拜访目的字典值
				List<SysDictItem> itemList = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
						.in(SysDictItem::getType, "user_sign_detail_purpose", "user_sign_detail_distribution_purpose", "user_sign_detail_pharmacy_purpose")
						.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
				userSignPurposeMap.putAll(itemList.stream().collect(Collectors.groupingBy(SysDictItem::getType)));
			}

			// 查询任务字段配置
			Set<String> configedTaskTypeId = new HashSet<>();
			taskTypeIdSet.forEach(s -> {
				// 应收账款回款-A类/应收账款回款-B类/应收账款回款-C类 统一设置为 39（应收账款回款-A类）
				if ("39".equals(s) || "40".equals(s) || "41".equals(s)) {
					configedTaskTypeId.add("39");
				}
				else if ("42".equals(s) || "43".equals(s) || "44".equals(s)) {
					// 手术跟台-一级/手术跟台-二级/手术跟台-三级 都取42
					configedTaskTypeId.add("42");
				}
				else if ("47".equals(s) || "48".equals(s) || "49".equals(s) || "50".equals(s)) {
					// 库存调研、竞品调研、培训、使用情况调查
					configedTaskTypeId.add("47");
				} else {
					configedTaskTypeId.add(s);
				}
			});
			WmTaskContentConfig queryConn = new WmTaskContentConfig();
			queryConn.setDelFlag("0");
			queryConn.setEnableFlag("0");
			List<WmTaskContentConfig> listConfig =
					wmTaskContentConfigMapper.selectList(Wrappers.<WmTaskContentConfig>lambdaQuery()
							.eq(WmTaskContentConfig::getDelFlag, DelEnum.NOT_DEL.val())
							.eq(WmTaskContentConfig::getEnableFlag, EnableEnum.ENABLE.val())
							.in(WmTaskContentConfig::getTaskTypeId, configedTaskTypeId));
			Map<String, List<WmTaskContentConfig>> taskContentConfigMap = new HashMap<>();
			Map<String, List<SysDictItem>> selectItemMap = new HashMap<>();
			if (CollUtil.isNotEmpty(listConfig)) {
				taskContentConfigMap.putAll(listConfig.stream().collect(Collectors.groupingBy(WmTaskContentConfig::getTaskTypeId)));
				// 查询任务字段配置-字典值
				Set<String> selectDictSet = listConfig.stream()
						.filter(config -> StrUtil.equals(config.getTaskFiledType(), "select"))
						.map(WmTaskContentConfig::getDictGroupName)
						.collect(Collectors.toSet());
				if (CollUtil.isNotEmpty(selectDictSet)) {
					List<SysDictItem> selectItemList = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
							.in(SysDictItem::getType, selectDictSet)
							.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
					selectItemMap.putAll(selectItemList.stream().collect(Collectors.groupingBy(SysDictItem::getType)));
				}
			}
			// task用户名称
			List<SysUser> userList = sysUserService.listByIds(taskList.stream().map(WmTask::getTaskUserId).collect(Collectors.toSet()));
			Map<Integer, String> userMap = userList.stream().collect(Collectors.toMap(SysUser::getUserId, SysUser::getRealname));

			taskList.forEach(wmTask -> {
				WmScorePackage scorePackage = scorePackageMap.get(wmTask.getScorePackageId());

				Map<String, Object> map = new HashMap<>();
				wmTask.setTaskTypeName(taskTypeNameMap.get(wmTask.getTaskTypeId()));
				map.put("taskType", wmTask.getTaskTypeId());
				map.put("score", wmTask.getScore());
				map.put("taskUserName", userMap.get(Integer.parseInt(wmTask.getTaskUserId())));
				map.put("taskInfo", wmTask);
				map.put("id", wmTask.getId());

				WmTaskContent wmTaskContent = taskContentMap.get(Integer.parseInt(wmTask.getTaskContentId()));
				map.put("wmTaskContent", wmTaskContent);

				if (null != wmTaskContent) {
					if (null != scorePackage) {
						wmTaskContent.setTemp32(scorePackage.getScorePackageName());
					}
					// 设置地图地址
					if ("30".equals(wmTask.getTaskTypeId()) || "15".equals(wmTask.getTaskTypeId())) {
						wmTaskContent.setTemp31(wmTaskContent.getTemp16());
					}
					if ("16".equals(wmTask.getTaskTypeId())) {
						wmTaskContent.setTemp31(wmTaskContent.getTemp4());
					}

					// 医院拜访、商业公司拜访、药店拜访打卡记录
					List<WmUserSign> wmUserSignList = new ArrayList<>();
					if (Arrays.asList("5", "6", "33").contains(wmTask.getTaskTypeId()) && NumberUtil.isNumber(wmTaskContent.getTemp3())) {
						int userSignId = Integer.parseInt(wmTaskContent.getTemp3());
						WmUserSign userSign = userSignMap.get(userSignId);
						// 关联查询签到详情
						WmUserSignDetail userSignDetail = userSignDetailMap.get(userSignId);
						if (userSignDetail != null) {
							if (ArrayUtil.isNotEmpty(userSignDetail.getPurpose())) {
								// 拜访目的字典值匹配
								Map<String, String> dictPurposeMap = userSignPurposeMap.get(UserSignPurposeEnum.getDictTypeBySignEntType(userSign.getSignEntType()))
										.stream().collect(Collectors.toMap(SysDictItem::getValue, SysDictItem::getLabel));

								String[] purposeNameArr = Stream.of(userSignDetail.getPurpose())
										.map(s -> dictPurposeMap.getOrDefault(s, s))
										.toArray(String[]::new);
								userSignDetail.setPurposeName(purposeNameArr);
							}
							if (StrUtil.isNotBlank(userSignDetail.getResult())) {
								userSignDetail.setResultName(UserSignResultEnum.resolve(userSignDetail.getResult()).getName());
							}
							userSign.setUserSignDetail(userSignDetail);
						}
						wmUserSignList.add(userSign);
					}
					map.put("wmUserSignList", wmUserSignList);
				}

				// 转发分享类任务
				if ("8".equals(wmTask.getTaskTypeId()) || "11".equals(wmTask.getTaskTypeId())
						|| "9".equals(wmTask.getTaskTypeId()) || "10".equals(wmTask.getTaskTypeId())) {
					map.put("title", "");
					map.put("score", wmTask.getScore());
					WmWkArticleShare wmWkArticleShare = wmWkArticleShareMapper.selectById(wmTask.getTaskContentId());
					if (null != wmWkArticleShare) {
						if ("8".equals(wmTask.getTaskTypeId()) || "11".equals(wmTask.getTaskTypeId())) {
							WmWkAlbum wmWkAlbum = wmWkAlbumMapper.selectById(wmWkArticleShare.getArctileId());
							if (wmWkAlbum != null) {
								map.put("title", wmWkAlbum.getXcmc());
							}
						} else {
							WmWkArticle wmWkArticle = wmWkArticleMapper.selectById(wmWkArticleShare.getArctileId());
							if (wmWkArticle != null) {
								map.put("title", wmWkArticle.getTitle());
							}
						}
						map.put("shareId", wmWkArticleShare.getArctileId());
						map.put("remark", wmWkArticleShare.getTemp1());
					}
				}

				// 任务类型字段配置
				String configTaskTypeId = wmTask.getTaskTypeId();
				// 应收账款回款-A类/应收账款回款-B类/应收账款回款-C类 统一设置为 39（应收账款回款-A类）
				if ("39".equals(configTaskTypeId) || "40".equals(configTaskTypeId) || "41".equals(configTaskTypeId)) {
					configTaskTypeId = "39";
				}
				else if ("42".equals(configTaskTypeId) || "43".equals(configTaskTypeId) || "44".equals(configTaskTypeId)) {
					// 手术跟台-一级/手术跟台-二级/手术跟台-三级 都取42
					configTaskTypeId = "42";
				}
				else if ("47".equals(configTaskTypeId) || "48".equals(configTaskTypeId) || "49".equals(configTaskTypeId) || "50".equals(configTaskTypeId)) {
					// 库存调研、竞品调研、培训、使用情况调查
					configTaskTypeId = "47";
				}
				List<WmTaskContentConfig> taskContentConfigs = taskContentConfigMap.get(configTaskTypeId);
				if (CollUtil.isNotEmpty(taskContentConfigs)) {
					taskContentConfigs.sort(Comparator.comparing(config -> Optional.ofNullable(config.getSeq()).orElse(0)));
					// 任务类型字段配置-对应字典值
					// select
					Map<String, Object> dictMap = new HashMap<>();
					taskContentConfigs.forEach(config -> {
						Map<String, Object> itemMap = new HashMap<>();
						if (StrUtil.equals(config.getTaskFiledType(), "select")) {
							List<Map<String, Object>> itemMap1List = selectItemMap.get(config.getDictGroupName()).stream().map(sysDictItem -> {
								Map<String, Object> itemMap1 = new HashMap<>();
								itemMap1.put("label", sysDictItem.getLabel());
								itemMap1.put("value", sysDictItem.getValue());
								return itemMap1;
							}).collect(Collectors.toList());

							dictMap.put(config.getDictGroupName(), itemMap1List);
						} else if (StrUtil.equals(config.getTaskFiledType(), "domain")) {
							if (StrUtil.equals(config.getDictGroupName(), "package_list")) {
								itemMap.put("label", scorePackage.getScorePackageName());
								itemMap.put("value", scorePackage.getId());
								dictMap.put("package_list", Collections.singletonList(itemMap));
							}
						}
					});
					map.put("dict", dictMap);
				}

				map.put("config", taskContentConfigs);
				resultList.add(map);
			});
		}

		return resultList;
	}
}



