/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */
package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.aliyuncs.exceptions.ClientException;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.CacheConstants;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDictItem;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.config.TaxNotifyConfig;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.*;
import com.qunzhixinxi.hnqz.admin.entity.input.SettleNoteStatusOutput;
import com.qunzhixinxi.hnqz.admin.entity.output.SettleAmountMonitorOutput;
import com.qunzhixinxi.hnqz.admin.entity.output.WmTaskSettleNoteOutput;
import com.qunzhixinxi.hnqz.admin.enums.DelEnum;
import com.qunzhixinxi.hnqz.admin.enums.GigTypeEnum;
import com.qunzhixinxi.hnqz.admin.enums.SubjectLocation;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.mapper.SysDictItemMapper;
import com.qunzhixinxi.hnqz.admin.mapper.SysUserSubMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmScorePackageMapper;
import com.qunzhixinxi.hnqz.admin.mapper.WmScorePackageSettleNoteMapper;
import com.qunzhixinxi.hnqz.admin.service.*;
import com.qunzhixinxi.hnqz.admin.util.SendSms;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.ding.config.DingConfig;
import com.qunzhixinxi.hnqz.common.ding.entity.EmployPayResponse;
import com.qunzhixinxi.hnqz.common.ding.entity.EmployPayResultSuccessItem;
import com.qunzhixinxi.hnqz.common.ding.entity.EmployPayResultfailItem;
import com.qunzhixinxi.hnqz.common.ding.enums.DingEnum;
import com.qunzhixinxi.hnqz.common.ding.service.DingService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.common.taxhelper.config.TaxHelperConfig;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.model.TaxHelperSettlementModel;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.request.TaxHelperSettlementRequest;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.response.TaxHelperResponse;
import com.qunzhixinxi.hnqz.common.taxhelper.enums.TaxHelperCertStatus;
import com.qunzhixinxi.hnqz.common.taxhelper.service.TaxHelperService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author pigx code generator
 * @date 2021-01-26 15:44:49
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmScorePackageSettleNoteServiceImpl extends ServiceImpl<WmScorePackageSettleNoteMapper, WmScorePackageSettleNote> implements WmScorePackageSettleNoteService {

	private final WmScorePackageSettleNoteMapper noteMapper;
	private final WmScorePackageMapper wmScorePackageMapper;
	private final SysUserService sysUserService;
	private final SysUserSubMapper sysUserSubMapper;
	private final SysDeptService deptService;
	private final SysDeptSubService sysDeptSubService;
	private final RedisTemplate redisTemplate;
	private final DingService dingService;
	private final WmDaAgentService wmDaAgentService;
	private final WmDaDrugEntService wmDaDrugEntService;
	private final TaxHelperService taxHelperService;
	private final TaxNotifyConfig taxNotifyConfig;
	private final MonitoringIndicatorService monitoringIndicatorService;
	private final SysDictItemMapper sysDictItemMapper;
	private final UpmsConfig upmsConfig;

	@Override
	public IPage<WmTaskSettleNoteOutput> getWmTaskSettleInfoPage(Page page, WmScorePackageSettleNote note) {
		return noteMapper.getWmTaskSettleInfoPage(page, note);
	}

	/**
	 * 结算包结算列表（分页）
	 *
	 * @param page
	 * @param note
	 * @return
	 */
	@Override
	public IPage<WmTaskSettleNoteOutput> getWmTaskForSettlePage(Page page, WmScorePackageSettleNote note, String[] settleStatusArr) {
		return noteMapper.getWmTaskForSettlePage(page, note, settleStatusArr);
	}

	/**
	 * 结算包结算列表（分页）
	 *
	 * @param page
	 * @param note
	 * @param settleStatusArr
	 * @return
	 */
	@Override
	public IPage<WmTaskSettleNoteOutput> getWmTaskForSettlePage2(Page page, WmScorePackageSettleNote note, String[] settleStatusArr) {
		return noteMapper.getWmTaskForSettlePage2(page, note, settleStatusArr);
	}

	/**
	 * 结算包结算列表
	 *
	 * @param note
	 * @return
	 */
	@Override
	public List<WmTaskSettleNoteOutput> listWmTaskForSettle(WmScorePackageSettleNote note) {
		return noteMapper.listWmTaskForSettle(note);
	}

	@Override
	public SettleNoteStatusOutput selectSettleNoteStatus(String packageId) {
		return noteMapper.selectSettleNoteStatus(packageId);
	}

	@Override
	public R getSettleInfo(WmScorePackageSettleNote note) {

		SysUser user = sysUserService.getById(note.getUserId());
		if (null != note.getId()) {

			note = this.baseMapper.selectById(note.getId());
			note.setRealName(user.getRealname());
			note.setIdCardNumber(user.getIdCardNumber());
			return R.ok(note);
		}
		WmScorePackageSettleNote query = new WmScorePackageSettleNote();
		query.setPackageId(note.getPackageId());
		query.setTaskId(note.getTaskId());
		query.setUserId(note.getUserId());

		List<WmScorePackageSettleNote> noteList = this.baseMapper.selectList(Wrappers.query(query));
		if (CollectionUtils.isNotEmpty(noteList)) {
			if (noteList.size() > 1) {
				return R.failed("结算数据异常，请联系管理员处理");
			}
			query = noteList.get(0);

		} else {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
			BigDecimal amount = new BigDecimal(wmScorePackage.getScore())
					.divide(new BigDecimal(wmScorePackage.getTaskNum()), 2, BigDecimal.ROUND_HALF_UP);
			query.setSettleAmount(amount);
			query.setSubType(wmScorePackage.getSubType());
			query.setSubjectLocation(wmScorePackage.getLocation());
		}
		query.setRealName(user.getRealname());
		query.setIdCardNumber(user.getIdCardNumber());

		return R.ok(query);
	}


	@Override
	public R saveSettleInfo(WmScorePackageSettleNote note) {

		SysUser user = sysUserService.getById(note.getUserId());
		SubjectLocation location = note.getSubjectLocation();

		if (StringUtils.isEmpty(user.getIdCardNumber())) {
			return R.failed(user.getRealname() + "未认证，不能发起结算");
		}

		if (note.getSubjectLocation() == null) {
			return R.failed(user.getRealname() + "人员结算渠道不存在，不能结算");
		}

		String subType = String.valueOf(note.getSubjectLocation().getGigType().getCode());

		SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getSubjectLocation, note.getSubjectLocation())
				.eq(SysUserSub::getDeptId, user.getDeptId())
				.eq(SysUserSub::getUserId, user.getUserId())
		);
		if (userSub == null) {
			return R.failed(user.getRealname() + "人员认证信息不存在，不能结算");
		}

		SysDeptSub condition = new SysDeptSub();
		condition.setDeptId(user.getDeptId());
		condition.setSubjectType(subType);
		condition.setSubjectLocation(location);
		condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		SysDeptSub sysDeptSub = sysDeptSubService.getOne(Wrappers.query(condition));

		if (null == sysDeptSub || StringUtils.isBlank(sysDeptSub.getAppId())) {
			return R.failed("人员所在机构未配置结算信息");
		}

		List<String> idCard = new ArrayList<>();
		List<String> userIds = new ArrayList<>();
		List<Integer> ids = new ArrayList<>();
		idCard.add(user.getIdCardNumber());
		userIds.add(user.getUserId() + "");
		if (null != note.getId()) {
			ids.add(note.getId());
		}


		List<SettleAmountMonitorOutput> monitorOutputList = sysDeptSubService.getSettleAmountMonitor(Collections.singletonList(user.getIdCardNumber()), ids, location);
		List<SettleAmountMonitorOutput> yaoyiAmountMonitor = sysDeptSubService.getYaoyiAmountMonitor(Collections.singletonList(user.getIdCardNumber()), ids, location);
		Set<Integer> collect = userIds.stream().map(Integer::valueOf).collect(Collectors.toSet());
		List<SettleAmountMonitorOutput> deptAmountMonitor = sysDeptSubService.getDeptAmountMonitor(collect, ids, location);

		SettleAmountMonitorOutput monitorOutput = monitorOutputList.stream().filter(item -> item.getIdCardNumber()
				.equals(user.getIdCardNumber())).findFirst().orElse(null);

		SettleAmountMonitorOutput deptMonitorOutput = deptAmountMonitor.stream().filter(item -> item.getUserId()
				.equals(note.getUserId())).findFirst().orElse(null);

		SettleAmountMonitorOutput yaoyiMonitorOutput = yaoyiAmountMonitor.stream().filter(item -> item.getIdCardNumber()
				.equals(user.getIdCardNumber())).findFirst().orElse(null);

		// 限额
		MonitoringIndicator monitoringIndicator = monitoringIndicatorService.locationLimit(location);
		BigDecimal deptLimit = sysDeptSub.getLimitAmount();
		BigDecimal yaoyiLimit = monitoringIndicator.getYaoyiLimitAmount();
		BigDecimal monitoringLimit = monitoringIndicator.getLimitAmount();
		BigDecimal monitorAmount = note.getSettleAmount();
		BigDecimal deptMonitorAmount = note.getSettleAmount();
		BigDecimal yaoyiMonitorAmount = note.getSettleAmount();
		if (null != monitorOutput) {
			monitorAmount = monitorAmount.add(monitorOutput.getTotalAmount());
		}
		if (null != deptMonitorOutput) {
			deptMonitorAmount = deptMonitorAmount.add(deptMonitorOutput.getTotalAmount());
		}
		if (null != yaoyiMonitorOutput) {
			yaoyiMonitorAmount = yaoyiMonitorAmount.add(yaoyiMonitorOutput.getTotalAmount());
		}
		if (null != deptLimit) {
			if (deptMonitorAmount.compareTo(deptLimit) > 0) {
				return R.failed(note.getRealName() + "：超过当月限额");
			}
		}
		if (null != monitoringLimit) {
			if (monitorAmount.compareTo(monitoringLimit) > 0) {
				return R.failed(note.getRealName() + "：超过结算渠道限额");
			}
		}
		if (null != yaoyiLimit) {
			if (yaoyiMonitorAmount.compareTo(yaoyiLimit) > 0) {
				return R.failed(note.getRealName() + "：超过平台限额");
			}
		}

		LocalDateTime localDateTime = LocalDateTime.now();
		if (null == note.getId()) {

			WmScorePackageSettleNote entity = new WmScorePackageSettleNote();
			entity.setSubType(subType);
			entity.setSubjectLocation(note.getSubjectLocation());
			entity.setInvoiceType(note.getInvoiceType());
			entity.setCategoryName(note.toCategoryName());
			entity.setSettleAmount(note.getSettleAmount());
			entity.setDiscount(new BigDecimal(1));
			entity.setPackageId(note.getPackageId());
			entity.setTaskId(note.getTaskId());
			entity.setUserId(note.getUserId());
			String settleNo = UUID.randomUUID().toString().replace("-", "");
			entity.setSettleNo(settleNo);
			entity.setSettleNoteStatus(0);
			entity.setCreateTime(localDateTime);
			entity.setUpdateTime(localDateTime);
			entity.setIdCardNumber(user.getIdCardNumber());
			this.baseMapper.insert(entity);
		} else {
			WmScorePackageSettleNote oldData = this.baseMapper.selectById(note.getId());
			if (oldData == null) {
				return R.failed("结算数据不存在，不能修改");
			}

			if (0 != oldData.getSettleNoteStatus()) {
				return R.failed("状态异常，不能修改");
			}
			WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSubType(subType);
			updateEntity.setSubjectLocation(note.getSubjectLocation());
			updateEntity.setInvoiceType(note.getInvoiceType());
			updateEntity.setCategoryName(note.getCategoryName());
			updateEntity.setSettleAmount(note.getSettleAmount());
			updateEntity.setUpdateTime(localDateTime);
			updateEntity.setIdCardNumber(user.getIdCardNumber());
			this.baseMapper.updateById(updateEntity);
		}

		return R.ok();
	}

	/**
	 * 批量业务结算保存
	 *
	 * @param noteList note列表
	 * @return 结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> batchSaveSettleInfo(List<WmScorePackageSettleNote> noteList) {
		for (WmScorePackageSettleNote note : noteList) {
			R settleResult = this.saveSettleInfo(note);
			if (settleResult.getCode() != 0) {
				throw new RuntimeException(settleResult.getMsg());
			}
		}
		return R.ok();
	}

	@Override
	public R subSettleInfo(WmScorePackageSettleNote note) {

		SysUser user = sysUserService.getById(note.getUserId());
		SubjectLocation location = note.getSubjectLocation();

		if (StringUtils.isEmpty(user.getIdCardNumber())) {
			return R.failed(user.getRealname() + "未认证，不能发起结算");
		}

		if (note.getSubjectLocation() == null) {
			return R.failed(user.getRealname() + "人员结算渠道不存在，不能结算");
		}

		if (SubjectLocation.MAH_SETTLE.equals(note.getSubjectLocation())) {
			// 默认在当前企业，取一个结算渠道
			SysDeptSub condition = new SysDeptSub();
			condition.setDeptId(user.getDeptId());
			condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
			List<SysDeptSub> deptSubs = sysDeptSubService.list(Wrappers.query(condition));
			if (CollUtil.isEmpty(deptSubs)) {
				return R.failed("结算渠道不存在");
			}
			SysDeptSub sysDeptSub = deptSubs.get(0);
			location = deptSubs.get(0).getSubjectLocation();

			// 默认取渠道对应的第一个发票类目 TODO
			String invoiceType = sysDeptSub.getSubjectLocation().getInvoiceType();
			// 查询发票类目对应的字典
			List<SysDictItem> invoices = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, invoiceType)
					.eq(SysDictItem::getDelFlag, DelEnum.NOT_DEL.val()));
			if (CollUtil.isEmpty(invoices)) {
				return R.failed(user.getRealname() + "发票类目不存在，不能结算");
			}
			note.setInvoiceType(Integer.valueOf(invoices.get(0).getValue()));
			note.setCategoryName(invoices.get(0).getLabel());
		}

		String subType = String.valueOf(location.getGigType().getCode());

		SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getSubjectLocation, location)
				.eq(SysUserSub::getDeptId, user.getDeptId())
				.eq(SysUserSub::getUserId, user.getUserId())
		);
		if (userSub == null) {
			return R.failed(user.getRealname() + "人员认证信息不存在，不能结算");
		}

		SysDeptSub condition = new SysDeptSub();
		condition.setDeptId(user.getDeptId());
//		condition.setSubjectType(subType);
		condition.setSubjectLocation(location);
		condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		SysDeptSub sysDeptSub = sysDeptSubService.getOne(Wrappers.query(condition));

		// 校验是否配置了结算限制
		if (!sysDeptSub.getSettleEnable()) {
			log.info("{}企业配置了结算限制", user.getDeptId());
			throw new RuntimeException("系统维护中，请联系管理员");
		}

		if (null == sysDeptSub || StringUtils.isBlank(sysDeptSub.getAppId())) {
			return R.failed("人员所在机构未配置结算信息");
		}

		List<String> idCard = new ArrayList<>();
		List<String> userIds = new ArrayList<>();
		List<Integer> ids = new ArrayList<>();
		idCard.add(user.getIdCardNumber());
		userIds.add(user.getUserId() + "");
		if (null != note.getId()) {
			ids.add(note.getId());
		}
		List<SettleAmountMonitorOutput> monitorOutputList = sysDeptSubService.getSettleAmountMonitor(Collections.singletonList(user.getIdCardNumber()), ids, location);
		List<SettleAmountMonitorOutput> yaoyiAmountMonitor = sysDeptSubService.getYaoyiAmountMonitor(Collections.singletonList(user.getIdCardNumber()), ids, location);
		Set<Integer> collect = userIds.stream().map(Integer::valueOf).collect(Collectors.toSet());
		List<SettleAmountMonitorOutput> deptAmountMonitor = sysDeptSubService.getDeptAmountMonitor(collect, ids, location);

		SettleAmountMonitorOutput monitorOutput = monitorOutputList.stream().filter(item -> item.getIdCardNumber()
				.equals(user.getIdCardNumber())).findFirst().orElse(null);

		SettleAmountMonitorOutput deptMonitorOutput = deptAmountMonitor.stream().filter(item -> item.getUserId()
				.equals(note.getUserId())).findFirst().orElse(null);

		SettleAmountMonitorOutput yaoyiMonitorOutput = yaoyiAmountMonitor.stream().filter(item -> item.getIdCardNumber()
				.equals(user.getIdCardNumber())).findFirst().orElse(null);

		// 限额
		MonitoringIndicator monitoringIndicator = monitoringIndicatorService.locationLimit(location);
		BigDecimal deptLimit = sysDeptSub.getLimitAmount();
		BigDecimal yaoyiLimit = monitoringIndicator.getYaoyiLimitAmount();
		BigDecimal monitoringLimit = monitoringIndicator.getLimitAmount();
		BigDecimal monitorAmount = note.getSettleAmount();
		BigDecimal deptMonitorAmount = note.getSettleAmount();
		BigDecimal yaoyiMonitorAmount = note.getSettleAmount();
		if (null != monitorOutput) {
			monitorAmount = monitorAmount.add(monitorOutput.getTotalAmount());
		}
		if (null != deptMonitorOutput) {
			deptMonitorAmount = deptMonitorAmount.add(deptMonitorOutput.getTotalAmount());
		}
		if (null != yaoyiMonitorOutput) {
			yaoyiMonitorAmount = yaoyiMonitorAmount.add(yaoyiMonitorOutput.getTotalAmount());
		}
		if (null != deptLimit) {
			if (deptMonitorAmount.compareTo(deptLimit) > 0) {
				return R.failed(note.getRealName() + "：超过当月限额");
			}
		}
		if (null != monitoringLimit) {
			if (monitorAmount.compareTo(monitoringLimit) > 0) {
				return R.failed(note.getRealName() + "：超过结算渠道限额");
			}
		}
		if (null != yaoyiLimit) {
			if (yaoyiMonitorAmount.compareTo(yaoyiLimit) > 0) {
				return R.failed(note.getRealName() + "：超过平台限额");
			}
		}

		WmScorePackageSettleNote updateEntity;
		LocalDateTime localDateTime = LocalDateTime.now();
		if (null != note.getId()) {
			WmScorePackageSettleNote oldData = this.baseMapper.selectById(note.getId());
			if (0 != oldData.getSettleNoteStatus()) {
				return R.failed("状态异常，不能提交");
			}
			updateEntity = new WmScorePackageSettleNote();
			updateEntity.setId(note.getId());
			updateEntity.setSubType(subType);
			updateEntity.setSubjectLocation(location);
			updateEntity.setInvoiceType(note.getInvoiceType());
			updateEntity.setCategoryName(note.getCategoryName());
			updateEntity.setSettleAmount(note.getSettleAmount());
			updateEntity.setActualAmount(note.getActualAmount());
			updateEntity.setSettleNoteStatus(9);
			updateEntity.setSubTime(localDateTime);
			updateEntity.setUpdateTime(localDateTime);
			updateEntity.setIdCardNumber(user.getIdCardNumber());
			this.baseMapper.updateById(updateEntity);
			WmScorePackageSettleNote settleNote = this.baseMapper.selectById(note.getId());

			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(settleNote.getPackageId());
			wmScorePackage.setSettlementDate(LocalDateTime.now());
			if (SubjectLocation.MAH_SETTLE.equals(note.getSubjectLocation())) {
				wmScorePackage.setMahSettleDeptId(note.getMahSettleDeptId());
				wmScorePackage.setLocation(note.getSubjectLocation());
			}
			wmScorePackageMapper.updateById(wmScorePackage);
			return R.ok();
		} else {
			WmScorePackageSettleNote entity = new WmScorePackageSettleNote();
			entity.setSubType(subType);
			entity.setSubjectLocation(location);
			entity.setInvoiceType(note.getInvoiceType());
			entity.setCategoryName(note.getCategoryName());
			entity.setSettleAmount(note.getSettleAmount());
			entity.setActualAmount(note.getActualAmount());
			entity.setDiscount(new BigDecimal(1));
			entity.setPackageId(note.getPackageId());
			entity.setTaskId(note.getTaskId());
			entity.setUserId(note.getUserId());
			String settleNo = UUID.randomUUID().toString().replace("-", "");
			entity.setSettleNo(settleNo);
			entity.setSettleNoteStatus(9);

			entity.setCreateTime(localDateTime);
			entity.setSubTime(localDateTime);
			entity.setUpdateTime(localDateTime);
			entity.setIdCardNumber(user.getIdCardNumber());

			this.baseMapper.insert(entity);

			if (SubjectLocation.MAH_SETTLE.equals(note.getSubjectLocation())) {
				// 更新 MAH结算-药企deptId
				wmScorePackageMapper.update(null, Wrappers.<WmScorePackage>lambdaUpdate()
						.eq(WmScorePackage::getId, note.getPackageId())
						.set(WmScorePackage::getLocation, note.getSubjectLocation())
						.set(WmScorePackage::getMahSettleDeptId, note.getMahSettleDeptId()));
			}
		}

		return R.ok();
	}

	/**
	 * 批量业务结算提交
	 *
	 * @param noteList note列表
	 * @return 结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public R<?> batchSubSettleInfo(List<WmScorePackageSettleNote> noteList) {
		for (WmScorePackageSettleNote note : noteList) {
			R settleResult = this.subSettleInfo(note);
			if (settleResult.getCode() != 0) {
				throw new RuntimeException(settleResult.getMsg());
			}
		}
		return R.ok();
	}

	@Override
	public R revokeSettle(WmScorePackageSettleNote note) {
		WmScorePackageSettleNote updateEntity;
		if (null != note.getId()) {
			WmScorePackageSettleNote oldData = this.baseMapper.selectById(note.getId());
			if (9 != oldData.getSettleNoteStatus() && 2 != oldData.getSettleNoteStatus()) {

				return R.failed("状态异常，不能回退");
			}
//			updateEntity = new WmScorePackageSettleNote();
//			updateEntity.setId(note.getId());
//			updateEntity.setInvoiceType(note.getInvoiceType());
//			updateEntity.setCategoryName(note.getCategoryName());
//			updateEntity.setSettleAmount(note.getSettleAmount());
//			updateEntity.setSettleNoteStatus(0);
//			this.baseMapper.updateById(updateEntity);
			this.baseMapper.deleteById(note.getId());
			return R.ok();
		}

		WmScorePackageSettleNote query = new WmScorePackageSettleNote();
		if (StringUtils.isNotEmpty(note.getPackageId())) {
			query.setPackageId(note.getPackageId());
		} else {
			return R.failed("批量回退失败，参数异常");
		}
		if (StringUtils.isNotEmpty(note.getUserId())) {
			query.setUserId(note.getUserId());
		}
//		query.setSettleNoteStatus(9);
//		updateEntity = new WmScorePackageSettleNote();
//		updateEntity.setSettleNoteStatus(0);
//		this.baseMapper.update(updateEntity, Wrappers.query(query));
		this.baseMapper.delete(Wrappers.query(query));
		return R.ok();
	}

	@Override
	public R singletSettle(WmScorePackageSettleNote input) {
		WmScorePackageSettleNote note = this.baseMapper.selectById(input.getId());
		WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
		SysDept sysDept = deptService.getById(SecurityUtils.getUser().getDeptId());
		if (9 != note.getSettleNoteStatus() && 2 != note.getSettleNoteStatus()) {
			return R.failed("状态异常，不能结算");
		}
		if (StringUtils.isEmpty(note.getUserId())) {
			return R.failed("数据异常，不能结算");
		}

		SysUser sysUser = sysUserService.getById(note.getUserId());

		if (note.getSubjectLocation() == null) {
			return R.failed(sysUser.getRealname() + "人员结算渠道不存在，不能结算");
		}

		// 获取结算零工
		GigTypeEnum gigTypeEnum = note.getSubjectLocation().getGigType();
		String subType = String.valueOf(note.getSubjectLocation().getGigType().getCode());

		/**
		 * 结算配置信息
		 */
		SysDeptSub condition = new SysDeptSub();
		condition.setDeptId(sysDept.getDeptId());
		condition.setSubjectType(subType);
		condition.setSubjectLocation(note.getSubjectLocation());
		condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		SysDeptSub sysDeptSub = sysDeptSubService.getOne(Wrappers.query(condition));

		if (null == sysDeptSub || StringUtils.isEmpty(sysDeptSub.getAppId())) {
			return R.failed("机构appId未维护");
		}

		SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
				.eq(SysUserSub::getSubjectLocation, note.getSubjectLocation())
				.eq(SysUserSub::getDeptId, sysUser.getDeptId())
				.eq(SysUserSub::getUserId, sysUser.getUserId())
		);

		if (userSub == null) {
			return R.failed(sysUser.getRealname() + "人员认证信息不存在，不能结算");
		}

		if (redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId())) {
			return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
		}

		switch (gigTypeEnum) {
			case REN_LI_JIA:
			case OLADING:
			case ZHENG_QI_ZHI_XING:
			case LANG_CHAO:
			case HUI_QI_YUN:
			case ZHONG_YI_YUN:
			case ZHONG_ZHI:
			case QI_FU:
			case SHEN_ZHEN_REN_CAI:
				if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
					return R.failed(sysUser.getRealname() + "未认证，不能发起结算");
				}
				break;
			case TAX_HELPOR:
				if (!Objects.equals(userSub.getCertStatus(), TaxHelperCertStatus.CERT.getCode())) {
					return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
				}
				break;
			default:
				return R.failed(sysUser.getRealname() + "人员结算渠道不存在，不能结算");
		}

		/**
		 * 发起结算
		 */
		log.info("结算明细noteId： {} 开始结算-----------", note.getId());
		log.info("结算渠道： {} ", note.getSubType());

		if (SubjectTypeEnum.TYPE_RENLIJIA.getCode().equals(note.getSubType())) {
			DingConfig config = new DingConfig();
			config.setAppId(sysDeptSub.getAppId());
			config.setAppSecret(sysDeptSub.getAppSecret());
			config.setQueryUrl(sysDeptSub.getQueryUrl());
			String subjectName = sysDeptSub.getSubjectName();
			return settleByRenLiJia(note, config, wmScorePackage, subjectName, sysDeptSub.getServiceCharge());

		}
		// 自由职家
		else if (SubjectTypeEnum.TYPE_OLADING.getCode().equals(subType)
		) {
			String taxCode = sysDept.getTaxCode();
			return settleByOlading(note, wmScorePackage, sysUser, taxCode);
		}
		//政企之星
		else if (SubjectTypeEnum.TYPE_ZHENG_QI_ZHI_XING.getCode().equals(subType)){
			String taxCode = sysDept.getTaxCode();
			return settleBySinbaad(note, wmScorePackage, sysUser, taxCode);
		}
		// 浪潮
		else if (SubjectTypeEnum.TYPE_LANG_CHAO.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			String enterpriseName = sysDept.getName();
			return this.settleByLangchao(note, wmScorePackage, sysUser, taxCode, enterpriseName);
		}
		// 汇企云
		else if (SubjectTypeEnum.TYPE_HUI_QI_YUN.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			return this.settleByHuiQiYun(note, wmScorePackage, sysUser, taxCode, sysDeptSub.getPathNo());
		}
		// 众蚁云
		else if (SubjectTypeEnum.TYPE_ZHONG_YI_YUN.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			return this.settleByZhongYiYun(note, wmScorePackage, sysUser, taxCode);
		}
		// 中智
		else if (SubjectTypeEnum.TYPE_ZHONG_ZHI.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			return this.settleByZhongZhi(note, wmScorePackage, sysUser, taxCode);
		}
		// 福建企赋
		else if (SubjectTypeEnum.TYPE_QI_FU.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			return this.settleByQiFu(note, wmScorePackage, sysUser, taxCode, sysDeptSub.getPathNo());
		}
		// 自由职家
		else if (SubjectTypeEnum.TYPE_SHEN_ZHEN_REN_CAI.getCode().equals(subType)) {
			String taxCode = sysDept.getTaxCode();
			return settleByOlading(note, wmScorePackage, sysUser, taxCode);
		}
		// 税邦云
		else {
			String taxCode = sysDept.getTaxCode();
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("税邦云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			TaxHelperConfig config = new TaxHelperConfig();
			config.setAppId(sysDeptSub.getAppId());
			config.setAppSecrete(sysDeptSub.getAppSecret());
			config.setQueryUrl(sysDeptSub.getQueryUrl());

			return settleByShuiBangYun(note, config, wmScorePackage, sysUser, taxCode);
			// return settleByShuiBangYun();
		}

	}

	@Override
	public R batchSettle(WmScorePackageSettleNote input) {

		SysDept sysDept = deptService.getById(SecurityUtils.getUser().getDeptId());

		SysDeptSub condition = new SysDeptSub();
		condition.setDeptId(sysDept.getDeptId());
		condition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		List<SysDeptSub> subList = sysDeptSubService.list(Wrappers.query(condition));
		Map<String, SysDeptSub> subMap = new HashMap<>();

		subList.forEach(item -> subMap.put(item.getSubjectLocation().name(), item));

		String[] ids = input.getIdStr().split(",");
		List<WmScorePackageSettleNote> noteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> oladingNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> fuJianQiFuNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> shenZhenRenCaiNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> zhengQiZhiXingNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> huiQiYunNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> zhongZhiNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> zhongYiYunNoteList = new ArrayList<>(ids.length);
		List<WmScorePackageSettleNote> langchaoNoteList = new ArrayList<>(ids.length);
		for (int i = 0; i < ids.length; i++) {
			String id = ids[i];
			WmScorePackageSettleNote note = this.baseMapper.selectById(id);
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());

			if (9 != note.getSettleNoteStatus() && 2 != note.getSettleNoteStatus()) {
				continue;
			}
			if (StringUtils.isEmpty(note.getUserId())) {
				continue;
			}

			/**
			 * 结算配置信息
			 */
			SysDeptSub sysDeptSub = subMap.get(note.getSubjectLocation().name());

			if (null == sysDeptSub || StringUtils.isEmpty(sysDeptSub.getAppId())) {
				return R.failed("机构appId未维护");
			}

			SysUser sysUser = sysUserService.getById(note.getUserId());

			if (note.getSubjectLocation() == null) {
				return R.failed(sysUser.getRealname() + "人员结算渠道不存在，不能结算");
			}

			// 获取结算零工
			GigTypeEnum gigTypeEnum = note.getSubjectLocation().getGigType();
			String subType = String.valueOf(note.getSubjectLocation().getGigType().getCode());

			SysUserSub userSub = sysUserSubMapper.selectOne(Wrappers.<SysUserSub>lambdaQuery()
					.eq(SysUserSub::getSubjectLocation, note.getSubjectLocation())
					.eq(SysUserSub::getDeptId, sysUser.getDeptId())
					.eq(SysUserSub::getUserId, sysUser.getUserId())
			);

			if (userSub == null) {
				return R.failed(sysUser.getRealname() + "人员认证信息不存在，不能结算");
			}

			if (redisTemplate.hasKey(CacheConstants.SETTLE_PACKAGE_KEY + note.getId())) {
				return R.failed("结算对象-" + sysUser.getRealname() + "：正在结算，请勿重复结算");
			}

			switch (gigTypeEnum) {
				case REN_LI_JIA:
				case OLADING:
				case ZHENG_QI_ZHI_XING:
				case LANG_CHAO:
				case HUI_QI_YUN:
				case ZHONG_YI_YUN:
				case ZHONG_ZHI:
				case QI_FU:
				case SHEN_ZHEN_REN_CAI:
					if (!Objects.equals(userSub.getCertStatus(), DingEnum.CER_STATUS_1.getType())) {
						return R.failed(sysUser.getRealname() + "未认证，不能发起结算");
					}
					break;
				case TAX_HELPOR:
					if (!Objects.equals(userSub.getCertStatus(), TaxHelperCertStatus.CERT.getCode())) {
						return R.failed(sysUser.getRealname() + "人员未认证或未绑卡，不能结算");
					}
					break;
				default:
					return R.failed(sysUser.getRealname() + "人员结算渠道不存在，不能结算");
			}

			/**
			 * 发起结算
			 */
			log.info("结算明细noteId： {} 开始结算-----------", note.getId());
			log.info("结算渠道： {} ", note.getSubType());

			if (SubjectTypeEnum.TYPE_RENLIJIA.getCode().equals(subType)) {
				DingConfig config = new DingConfig();
				config.setAppId(sysDeptSub.getAppId());
				config.setAppSecret(sysDeptSub.getAppSecret());
				config.setQueryUrl(sysDeptSub.getQueryUrl());
				String subjectName = sysDeptSub.getSubjectName();
				settleByRenLiJia(note, config, wmScorePackage, subjectName, sysDeptSub.getServiceCharge());

			}
			// 自由职家
			else if (SubjectTypeEnum.TYPE_OLADING.getCode().equals(subType)) {
				oladingNoteList.add(note);
			}
			// 汇企云
			else if (SubjectTypeEnum.TYPE_HUI_QI_YUN.getCode().equals(subType)) {
				huiQiYunNoteList.add(note);
			}
			// 正启之星
			else if (SubjectTypeEnum.TYPE_ZHENG_QI_ZHI_XING.getCode().equals(subType)) {
				zhengQiZhiXingNoteList.add(note);
			}
			// 中智
			else if (SubjectTypeEnum.TYPE_ZHONG_ZHI.getCode().equals(subType)) {
				zhongZhiNoteList.add(note);
			}
			// 众蚁云
			else if (SubjectTypeEnum.TYPE_ZHONG_YI_YUN.getCode().equals(subType)) {
				zhongYiYunNoteList.add(note);
			}
			// 福建企赋
			else if (SubjectTypeEnum.TYPE_QI_FU.getCode().equals(subType)) {
				fuJianQiFuNoteList.add(note);
			}
			// 深圳人才
			else if (SubjectTypeEnum.TYPE_SHEN_ZHEN_REN_CAI.getCode().equals(subType)) {
				shenZhenRenCaiNoteList.add(note);
			}
			// 浪潮
			else if (SubjectTypeEnum.TYPE_LANG_CHAO.getCode().equals(subType)) {
				langchaoNoteList.add(note);
			}
			// 税邦云
			else {
				noteList.add(note);
			}
		}

		// 税邦云结算
		String taxCode = sysDept.getTaxCode();
		String enterpriseName = sysDept.getName();
		if (CollectionUtils.isNotEmpty(noteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("税邦云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			SysDeptSub deptSub = subMap.get(SubjectTypeEnum.TYPE_SHUIBANGYUN.getCode());
			TaxHelperConfig config = new TaxHelperConfig();
			config.setAppId(deptSub.getAppId());
			config.setAppSecrete(deptSub.getAppSecret());
			config.setQueryUrl(deptSub.getQueryUrl());
			return settleByTaxHelper(noteList, config, taxCode);
			// return settleByTaxHelper();
		}
		// 政企之星结算
		if (CollectionUtils.isNotEmpty(zhengQiZhiXingNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("政企之星结算结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			return batchSettleBySinbaad(zhengQiZhiXingNoteList, taxCode);
			// return settleByTaxHelper();
		}

		// 自由职家结算
		if (CollectionUtils.isNotEmpty(oladingNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("自由职家结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			return batchSettleByOlading(oladingNoteList, taxCode);
			// return settleByTaxHelper();
		}

		// 浪潮结算
		if (CollectionUtils.isNotEmpty(langchaoNoteList)) {
			return this.batchSettleByLangchao(langchaoNoteList, taxCode, enterpriseName);
		}

		// 汇企云结算
		if (CollectionUtils.isNotEmpty(huiQiYunNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("汇企云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			SysDeptSub sysDeptSub = subMap.get(SubjectLocation.HUI_QI_YUN.name());
			return this.batchSettleByHuiQiYun(huiQiYunNoteList, taxCode, sysDeptSub.getPathNo());
		}

		// 众蚁云
		if (CollectionUtils.isNotEmpty(zhongYiYunNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("众蚁云结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			return this.batchSettleByZHongYiYun(zhongYiYunNoteList, taxCode);
		}

		// 中智
		if (CollectionUtils.isNotEmpty(zhongZhiNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("中智结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			return this.batchSettleByZhongzhi(zhongZhiNoteList, taxCode);
		}

		// 福建企赋
		if (CollectionUtils.isNotEmpty(fuJianQiFuNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("福建企赋结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			SysDeptSub sysDeptSub = subMap.get(SubjectLocation.QI_FU.name());
			return this.batchSettleByQiFu(fuJianQiFuNoteList, taxCode, sysDeptSub.getPathNo());
		}

		// 深圳人才
		if (CollectionUtils.isNotEmpty(shenZhenRenCaiNoteList)) {
			if (StringUtils.isNotEmpty(taxCode)) {
				log.info("深圳人才结算开始-获取会员所属企业的统一社会信用代码：{} ", taxCode);
			} else {
				return R.failed("未配置企业的统一社会信用代码");
			}
			SysDeptSub sysDeptSub = subMap.get(SubjectLocation.QI_FU.name());
			return this.batchSettleByQiFu(shenZhenRenCaiNoteList, taxCode, sysDeptSub.getPathNo());
		}

		return R.ok();
	}


	private R settleByTaxHelper() {
		return R.failed("因系统维护，暂时不能提起支付，有问题请联系客服");
	}

	/**
	 * 福建企赋结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @param pathNo 批次付款渠道编号
	 * @return 结果
	 */
	private R batchSettleByQiFu(List<WmScorePackageSettleNote> noteList, String taxCode, String[] pathNo) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));


				LocalDate startTime = (LocalDate) params.get("taskStartTime");
				LocalDate endTime = (LocalDate) params.get("taskEndTime");
				LocalDate noteCreateTime = wmScorePackage.getCreateTime().toLocalDate();
				LocalDate noteUpdateTime = wmScorePackage.getUpdateTime().toLocalDate();

				if (noteCreateTime.isBefore(startTime)) {
					params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				}
				if (noteUpdateTime.isAfter(endTime)) {
					params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				}

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getBankPhone());
				params.put("idCard", note.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "QI_FU");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("taskDescription", "按要求完成学术推广并用在线工具采集记录");
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("福建企赋结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("福建企赋结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("福建企赋结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("福建企赋提交成功 data 获取不到");
							return;
						}

						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setSubToGigTime(now);
						note.setUpdateTime(now);
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							log.info("福建企赋提交成功 message：{}", dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("福建企赋结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(now);
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("福建企赋结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 中智结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @return 结果
	 */
	private R batchSettleByZhongzhi(List<WmScorePackageSettleNote> noteList, String taxCode) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		// 查询发票类目对应的字典
		List<SysDictItem> zhongzhiInvoices = sysDictItemMapper.selectList(Wrappers.<SysDictItem>lambdaQuery()
				.eq(SysDictItem::getType, "zhongzhi_invoice"));
		Map<String, SysDictItem> dictItemMap = zhongzhiInvoices.stream()
				.collect(Collectors.toMap(SysDictItem::getValue, Function.identity()));

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());

			// 当前发票类目对应的字典信息
			SysDictItem dictItem = dictItemMap.get(String.valueOf(note.getInvoiceType()));

			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));


				LocalDate startTime = (LocalDate) params.get("taskStartTime");
				LocalDate endTime = (LocalDate) params.get("taskEndTime");
				LocalDate noteCreateTime = wmScorePackage.getCreateTime().toLocalDate();
				LocalDate noteUpdateTime = wmScorePackage.getUpdateTime().toLocalDate();

				if (noteCreateTime.isBefore(startTime)) {
					params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				}
				if (noteUpdateTime.isAfter(endTime)) {
					params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				}

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getUsername());
				params.put("idCard", note.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				// 行业类别id
				params.put("tagId", dictItem.getDescription());
				// 税收项目编号
				params.put("taskInvoiceContent", dictItem.getRemarks());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "ZHONG_ZHI");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("中智结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("中智结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("中智结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("中智提交成功 data 获取不到");
							return;
						}

						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setUpdateTime(now);
						note.setSubToGigTime(now);

						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							log.info("中智提交失败 message：{}", dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("中智结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(now);
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("中智结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 众蚁云结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @return 结果
	 */
	private R batchSettleByZHongYiYun(List<WmScorePackageSettleNote> noteList, String taxCode) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));


				LocalDate startTime = (LocalDate) params.get("taskStartTime");
				LocalDate endTime = (LocalDate) params.get("taskEndTime");
				LocalDate noteCreateTime = wmScorePackage.getCreateTime().toLocalDate();
				LocalDate noteUpdateTime = wmScorePackage.getUpdateTime().toLocalDate();

				if (noteCreateTime.isBefore(startTime)) {
					params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				}
				if (noteUpdateTime.isAfter(endTime)) {
					params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				}

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getBankPhone());
				params.put("idCard", note.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("bankPhone", sysUser.getBankPhone());
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "ZHONG_YI_YUN");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("众蚁云结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("众蚁云结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("众蚁云结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("众蚁云提交成功 data 获取不到");
							return;
						}

						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setUpdateTime(now);
						note.setSubToGigTime(now);

						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							log.info("众蚁云提交成功 message：{}", dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("众蚁云结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(now);
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("众蚁云结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 汇企云结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @param pathNo 批次付款渠道编号
	 * @return 结果
	 */
	private R batchSettleByHuiQiYun(List<WmScorePackageSettleNote> noteList, String taxCode, String[] pathNo) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));


				LocalDate startTime = (LocalDate) params.get("taskStartTime");
				LocalDate endTime = (LocalDate) params.get("taskEndTime");
				LocalDate noteCreateTime = wmScorePackage.getCreateTime().toLocalDate();
				LocalDate noteUpdateTime = wmScorePackage.getUpdateTime().toLocalDate();

				if (noteCreateTime.isBefore(startTime)) {
					params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				}
				if (noteUpdateTime.isAfter(endTime)) {
					params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				}

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "要易云");
				params.put("username", sysUser.getUsername());
				params.put("idCard", note.getIdCardNumber());
				params.put("idCardType","ID_CARD");
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", "HUI_QI_YUN");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("notes", pathNo[0]);
				params.put("pathNo", pathNo[0]);
				params.put("notifyUrl", String.format("%s/admin/gig-third-api/huiqiyun", upmsConfig.getClientUrl()));

				if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("汇企云结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("汇企云结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("汇企云结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("汇企云提交成功 data 获取不到");
							return;
						}

						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setSubToGigTime(now);
						note.setUpdateTime(now);
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							log.info("汇企云提交成功 message：{}", dataJsonObj.getStr("message"));
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("汇企云结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(now);
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("汇企云结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 浪潮结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @param enterpriseName  企业名称
	 * @return 结果
	 */
	private R batchSettleByLangchao(List<WmScorePackageSettleNote> noteList, String taxCode, String enterpriseName) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("enterpriseName", enterpriseName);
				params.put("idCardType","ID_CARD");
				params.put("subjectType", "LANG_CHAO");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));

				if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("浪潮结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("浪潮结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("浪潮结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("浪潮提交成功 data 获取不到");
							return;
						}
						if (!StrUtil.equals("200", dataJsonObj.getStr("code"))) {
							log.info("浪潮提交成功 code 状态异常");
							return;
						}
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setUpdateTime(now);
						note.setSubToGigTime(now);

						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("浪潮结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(LocalDateTime.now());
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("浪潮结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(LocalDateTime.now());
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 政企之星结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @return 结果
	 */
	private R batchSettleBySinbaad(List<WmScorePackageSettleNote> noteList, String taxCode) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("username", sysUser.getUsername());
				params.put("idCard", sysUser.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("idCardType","ID_CARD");
				params.put("subjectType", "ZHENG_QI_ZHI_XING");
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());

				if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("政企之星结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("政企之星结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("政企之星结算结算提交成功");
						// data获取不到
						if (dataJsonObj == null) {
							log.info("政企之星提交成功 data 获取不到");
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id);
							return;
						}

						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						LocalDateTime now = LocalDateTime.now();
						note.setId(id);
						note.setUpdateTime(now);
						note.setSubToGigTime(now);

						String requestId = dataJsonObj.getStr("requestId");
						String data = dataJsonObj.getStr("data");
						String respMessage = dataJsonObj.getStr("resp_message");
						if (StrUtil.isNotBlank(requestId)) {
							JSONObject dataJson = JSONUtil.parseObj(data);
							String orderStatus = dataJson.getStr("order_status");
							if ("-1".equals(orderStatus) || "2".equals(orderStatus) || "-10".equals(orderStatus)) {
								// 订单失败的状态(-1-创建订单成功，子任务状态校验异常，可做失败处理, 2-创建订单成功待支付，支付状态待确认，订单被驳回, -10-下发至个人失败)
								note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
								log.warn("政企之星结算提交失败： {}", respMessage);
							} else {
								note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
							}
							if ("2".equals(orderStatus)) {
								//  2-创建订单成功待支付，支付状态待确认，订单被驳回
								//  (一个是订单的支付状态没有成功，还有一种情况就是供应商驳回了你的订单 都是2
								// 订单不是失败的，只是没付款成功  不排除银行极端情况出现，所以你要是把2当成失败的话就10分钟之后再查下状态判断)
								note.setRemark(data);
							}
							// 新增字段streamId
							note.setStreamId(requestId);
						} else {
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id);
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						}

						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("政企之星结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setUpdateTime(now);
						note.setSubToGigTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				log.error("", e);
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("政企之星结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 哦拉丁结算
	 *
	 * @param noteList 结算记录
	 * @param taxCode  企业三合一码
	 * @return 结果
	 */
	private R batchSettleByOlading(List<WmScorePackageSettleNote> noteList, String taxCode) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		List<Map<String, Object>> settleParamsMapper = new ArrayList<>();
		Map<String, Map<String, Object>> detailMap = new HashMap<>();
		noteList.forEach(note -> {
			WmScorePackage wmScorePackage = wmScorePackageMapper.selectById(note.getPackageId());
			// 封装结算参数
			Map<String, Object> params;
			if (detailMap.containsKey(note.getIdCardNumber())) {
				params = detailMap.get(note.getIdCardNumber());
				params.put("amount", String.valueOf(new BigDecimal((String) params.get("amount")).add(note.getSettleAmount()).doubleValue()));


				LocalDate startTime = (LocalDate) params.get("taskStartTime");
				LocalDate endTime = (LocalDate) params.get("taskEndTime");
				LocalDate noteCreateTime = wmScorePackage.getCreateTime().toLocalDate();
				LocalDate noteUpdateTime = wmScorePackage.getUpdateTime().toLocalDate();

				if (noteCreateTime.isBefore(startTime)) {
					params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				}
				if (noteUpdateTime.isAfter(endTime)) {
					params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				}

				Set<Integer> ids = (Set<Integer>) params.get("noteIds");
				ids.add(note.getId());
				params.put("noteIds", ids);
			} else {
				SysUser sysUser = sysUserService.getById(note.getUserId());
				params = new HashMap<>(19);
				params.put("tenantId", operator.getDeptId());
				params.put("operatorId", operator.getId());
				params.put("requestId", RandomStringUtils.randomAlphabetic(12));
				params.put("taskName", String.format("%s(%s)_%s", sysUser.getRealname(), sysUser.getUsername(), System.currentTimeMillis()));
				params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
				params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
				params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
				params.put("taskLocation", "全国");
				params.put("username", sysUser.getUsername());
				params.put("idCard", note.getIdCardNumber());
				params.put("realName", sysUser.getRealname());
				params.put("bankCard", sysUser.getBankCardNumber());
				params.put("tagId", upmsConfig.getTagId());
				params.put("taskInvoiceContent", note.getCategoryName());
				params.put("endType", "CSO");
				params.put("enterpriseCode", taxCode);
				params.put("subjectType", note.getSubjectLocation().getGigType().name());
				params.put("channel", "DEFAULT");
				params.put("subjectLocation", note.getSubjectLocation());
				params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
				params.put("acceptanceStandard", "完成不同任务获得不同任务积分，根据不同任务给与0~30000积分，每个积分兑换0.8~1.2元，按次数结算");

				if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
					params.put("taxMode", "GENERAL");
					params.put("transactionModel", "SUPERVISE");
				}
				// 海南飞亿 和 河南薪泰梓
				else {
					params.put("taxMode", "SPECIAL");
					params.put("transactionModel", "CHANNEL");
				}

				Set<Integer> noteIds = new HashSet<>();
				noteIds.add(note.getId());
				params.put("noteIds", noteIds);
				detailMap.put(note.getIdCardNumber(), params);
			}
		});

		detailMap.forEach((key, value) -> settleParamsMapper.add(value));


		// 开始结算
		for (Map<String, Object> params : settleParamsMapper) {

			Set<Integer> noteIds = (Set<Integer>) params.get("noteIds");
			try {
				// 获取请求token
				ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
						String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
				log.warn("获取结算token：{}", tokenResult.getBody());
				if (!JSONUtil.isJson(tokenResult.getBody())) {
					return R.failed("获取结算token失败");
				}

				JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

				if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
					return R.failed("获取结算token失败");
				}

				String token = jsonObject.getStr("data");

				// 间隔100毫秒
				Thread.sleep(100);

				params.put("token", token);

				// 发起结算
				HttpHeaders headers = new HttpHeaders();
				headers.setContentType(MediaType.APPLICATION_JSON);
				log.info("阿拉丁结算req： {}", params);
				HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
				log.info("阿拉丁结算resp： {}", result.getBody());
				if (!JSONUtil.isJson(result.getBody())) {
					return R.failed("结算失败");
				}
				JSONObject settelObj = JSONUtil.parseObj(result.getBody());

				if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
					JSONObject dataJsonObj = settelObj.getJSONObject("data");

					noteIds.forEach(id -> {
						log.info("自由职家结算提交成功");
						//data获取不到
						if (dataJsonObj == null) {
							log.info("自由职家提交成功 data 获取不到");
							return;
						}
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						LocalDateTime now = LocalDateTime.now();
						note.setSubToGigTime(now);
						note.setUpdateTime(now);
						String requestId = dataJsonObj.getStr("requestId");
						if (StrUtil.isNotBlank(requestId)) {
							// 新增字段streamId
							note.setStreamId(requestId);
							// 第三方任务id
							note.setThirdTaskId(dataJsonObj.getStr("taskId"));
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
						}
						this.baseMapper.updateById(note);
					});

				} else {
					noteIds.forEach(id -> {
						WmScorePackageSettleNote note = new WmScorePackageSettleNote();
						note.setId(id);
						log.info("自由职家结算提交失败： {}", note.getSettleNo());
						LocalDateTime now = LocalDateTime.now();
						note.setSubToGigTime(now);
						note.setUpdateTime(now);
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						this.baseMapper.updateById(note);
						log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
					});

					return R.ok("结算失败");
				}

			} catch (Exception e) {
				noteIds.forEach(id -> {
					WmScorePackageSettleNote note = new WmScorePackageSettleNote();
					note.setId(id);
					log.info("自由职家结算提交失败： {}", note.getSettleNo());
					LocalDateTime now = LocalDateTime.now();
					note.setUpdateTime(now);
					note.setSubToGigTime(now);
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					this.baseMapper.updateById(note);
					log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				});
				return R.ok("结算失败");
			}
		}

		return R.ok("提交结算成功");
	}

	/**
	 * 税帮云结算
	 *
	 * @param noteList 结算记录
	 * @param config   税帮云config
	 * @param taxCode  企业三合一码
	 * @return 结果
	 */
	private R settleByTaxHelper(List<WmScorePackageSettleNote> noteList, TaxHelperConfig config, String taxCode) {

		// 加锁
		List<Integer> noteIdList = noteList.stream().filter(note ->
						redisTemplate.opsForValue().setIfAbsent(
								CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId(),
								noteList, CacheConstants.SETTLE_TIME, TimeUnit.DAYS))
				.mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toList());

		// 加锁记录数不等于提交记录数，说明存在在途交易
		if (CollectionUtils.isEmpty(noteIdList)) {
			log.error("当前用户存在其他结算请求");
			return R.failed("当前用户存在其他结算请求");
		} else {
			if (!Objects.equals(noteList.size(), noteIdList.size())) {
				// 删除锁
				noteIdList.forEach(id -> redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + id));
				log.error("当前用户存在其他结算请求");
				return R.failed("当前用户存在其他结算请求");
			}
		}

		// 开始结算
		TaxHelperSettlementModel taxHelperSettlementModel = new TaxHelperSettlementModel(config);
		taxHelperSettlementModel.setEnterpriseCode(taxCode);
		taxHelperSettlementModel.setExtendFieldNames(Arrays.asList("报告下载地址"));

		Set<String> userIdList = noteList.stream().map(WmScorePackageSettleNote::getUserId).collect(Collectors.toSet());
		taxHelperSettlementModel.setNotifyUrl(taxNotifyConfig.getNotifyUrl() + "/admin/taxhelper/" + String.join("|", userIdList) + "/v1/settlementNotify");

		List<TaxHelperSettlementModel.AbstractArr> abstractArrList = new ArrayList<>();
		Map<String, TaxHelperSettlementModel.DetailedArr> detailArrMap = new HashMap<>();
		noteList.forEach(note -> {
			TaxHelperSettlementModel.DetailedArr detailedArr;
			if (detailArrMap.containsKey(note.getIdCardNumber())) {
				detailedArr = detailArrMap.get(note.getIdCardNumber());
				detailedArr.setInputAmount((BigDecimal.valueOf(detailedArr.getInputAmount())
						.add(note.getSettleAmount())
						.doubleValue()));
			} else {
				detailedArr = new TaxHelperSettlementModel.DetailedArr();
				detailedArr.setIdCard(note.getIdCardNumber());
				detailedArr.setInputAmount(note.getSettleAmount().doubleValue());
				detailedArr.setCategoryName(StringUtils.isEmpty(note.getCategoryName()) ? "专业技术服务" : note.getCategoryName());
				detailArrMap.put(note.getIdCardNumber(), detailedArr);
			}
			TaxHelperSettlementModel.AbstractArr abstractArr = new TaxHelperSettlementModel.AbstractArr();
			abstractArr.setIdCard(note.getIdCardNumber());
			abstractArr.setNumber(1);
			abstractArr.setPrice(note.getSettleAmount().doubleValue());
			abstractArr.setAmount(note.getSettleAmount().doubleValue());
			abstractArr.setExtendField1(taxNotifyConfig.getTaskUrl() + "/#/h5/indexTask?id=" + note.getPackageId() + "&userId=" + note
					.getUserId());
			abstractArrList.add(abstractArr);
		});
		taxHelperSettlementModel.setCount(detailArrMap.size());
		List<TaxHelperSettlementModel.DetailedArr> detailedArrList = new ArrayList<>(detailArrMap.values());
		taxHelperSettlementModel.setDetailedArr(detailedArrList);
		taxHelperSettlementModel.setAbstractArr(abstractArrList);
		TaxHelperSettlementRequest taxHelperSettlementRequest = new TaxHelperSettlementRequest(taxHelperSettlementModel);
		TaxHelperResponse response = taxHelperService.settlement(taxHelperSettlementRequest);
		log.info("税邦云结算： {}", response);
		if (response.success()) {
			noteList.forEach(note -> {
				log.info("税邦云结算提交成功： {}", note.getSettleNo());
				//streamId 获取不到
				if (StringUtils.isEmpty(response.getMessage())) {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					log.info("税邦云结算提交成功 streamId 获取不到： {}", note.getSettleNo());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				} else {
					// 新增字段streamId
					note.setStreamId(response.getMessage());
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
				}
				LocalDateTime now = LocalDateTime.now();
				note.setUpdateTime(now);
				note.setSubToGigTime(now);

				this.baseMapper.updateById(note);
			});
			return R.ok("提交结算成功");
		} else {
			noteList.forEach(note -> {
				log.info("税邦云结算提交失败： {}", note.getSettleNo());
				LocalDateTime now = LocalDateTime.now();
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				this.baseMapper.updateById(note);
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
			});
			return R.ok("结算失败");

		}
	}

	/**
	 * 福建企赋结算
	 *
	 * @param note
	 * @param wmScorePackage
	 * @param sysUser
	 * @param taxCode
	 * @return
	 */
	private R settleByQiFu(WmScorePackageSettleNote note,
							   WmScorePackage wmScorePackage,
							   SysUser sysUser,
							   String taxCode,
							   String[] pathNo) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
			params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
			params.put("taskLocation", "要易云");
			params.put("username", sysUser.getBankPhone());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("idCardType","ID_CARD");
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("bankPhone", sysUser.getBankPhone());
			// 市场推广
			params.put("tagId", upmsConfig.getTagId());
			params.put("taskInvoiceContent", note.getCategoryName());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("subjectType", "QI_FU");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation().getType());
			params.put("taxMode", "SPECIAL");
			params.put("transactionModel", "CHANNEL");
			params.put("taskDescription", "按要求完成学术推广并用在线工具采集记录");

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("福建企赋结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("福建企赋结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("福建企赋结算提交成功： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				//data 获取不到
				if (dataJsonObj != null) {
					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					log.warn("福建企赋结算提交成功 data 获取不到： {}", note.getSettleNo());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				}
			} else {
				log.warn("福建企赋结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("福建企赋结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setUpdateTime(now);
			note.setSubToGigTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 众蚁云结算
	 *
	 * @param note
	 * @param wmScorePackage
	 * @param sysUser
	 * @param taxCode
	 * @return
	 */
	private R settleByZhongZhi(WmScorePackageSettleNote note,
								 WmScorePackage wmScorePackage,
								 SysUser sysUser,
								 String taxCode) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);

			// 查询发票类目对应的字典
			SysDictItem zhongzhiInvoice = sysDictItemMapper.selectOne(Wrappers.<SysDictItem>lambdaQuery()
					.eq(SysDictItem::getType, "zhongzhi_invoice")
					.eq(SysDictItem::getValue, String.valueOf(note.getInvoiceType())));

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
			params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
			params.put("taskLocation", "要易云");
			params.put("username", sysUser.getBankPhone());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("idCardType","ID_CARD");
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("bankPhone", sysUser.getBankPhone());
			// 行业类别id
			params.put("tagId", zhongzhiInvoice.getDescription());
			// 税收项目编号
			params.put("taskInvoiceContent", zhongzhiInvoice.getRemarks());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("subjectType", "ZHONG_ZHI");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation().getType());
			params.put("taxMode", "SPECIAL");
			params.put("transactionModel", "CHANNEL");
			params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("中智结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("中智结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("中智结算提交成功： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				//data 获取不到
				if (dataJsonObj != null) {
					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					log.warn("中智结算提交成功 data 获取不到： {}", note.getSettleNo());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				}
			} else {
				log.warn("中智结算提交失败： {}", note.getSettleNo());
				note.setSubToGigTime(now);
				note.setUpdateTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("中智结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 众蚁云结算
	 *
	 * @param note
	 * @param wmScorePackage
	 * @param sysUser
	 * @param taxCode
	 * @return
	 */
	private R settleByZhongYiYun(WmScorePackageSettleNote note,
								 WmScorePackage wmScorePackage,
								 SysUser sysUser,
								 String taxCode) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
			params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
			params.put("taskLocation", "要易云");
			params.put("username", sysUser.getBankPhone());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("idCardType","ID_CARD");
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("bankPhone", sysUser.getBankPhone());
			// 市场推广
			params.put("tagId", upmsConfig.getTagId());
			params.put("taskInvoiceContent", note.getCategoryName());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("subjectType", "ZHONG_YI_YUN");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation().getType());
			params.put("taxMode", "SPECIAL");
			params.put("transactionModel", "CHANNEL");
			params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("众蚁云结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("众蚁云结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("众蚁云结算提交成功： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				//data 获取不到
				if (dataJsonObj != null) {
					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					log.warn("众蚁云结算提交成功 data 获取不到： {}", note.getSettleNo());
				}
			} else {
				log.warn("众蚁云结算提交失败： {}", note.getSettleNo());
				note.setSubToGigTime(now);
				note.setUpdateTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("众蚁云结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 汇企云结算
	 *
	 * @param note
	 * @param wmScorePackage
	 * @param sysUser
	 * @param taxCode
	 * @return
	 */
	private R settleByHuiQiYun(WmScorePackageSettleNote note,
							   WmScorePackage wmScorePackage,
							   SysUser sysUser,
							   String taxCode,
							   String[] pathNo) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
			params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
			params.put("taskLocation", "要易云");
			params.put("username", sysUser.getUsername());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("idCardType","ID_CARD");
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			// 市场推广
			params.put("tagId", upmsConfig.getTagId());
			params.put("taskInvoiceContent", note.getCategoryName());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("subjectType", "HUI_QI_YUN");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation().getType());
			params.put("notes", pathNo[0]);
			params.put("pathNo", pathNo[0]);
			params.put("notifyUrl", String.format("%s/admin/gig-third-api/huiqiyun", upmsConfig.getClientUrl()));

			if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
				params.put("taxMode", "GENERAL");
				params.put("transactionModel", "SUPERVISE");
			}
			// 海南飞亿 和 河南薪泰梓
			else {
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
			}

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("汇企云结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("汇企云结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("汇企云结算提交成功： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				//data 获取不到
				if (dataJsonObj != null) {
					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：" + dataJsonObj.getStr("message"));
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					log.warn("汇企云结算提交成功 data 获取不到： {}", note.getSettleNo());
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				}
			} else {
				log.warn("汇企云结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("汇企云结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setUpdateTime(now);
			note.setSubToGigTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	/**
	 * 浪潮结算
	 *
	 * @param note
	 * @param wmScorePackage
	 * @param sysUser
	 * @param taxCode
	 * @return
	 */
	private R settleByLangchao(WmScorePackageSettleNote note,
							   WmScorePackage wmScorePackage,
							   SysUser sysUser,
							   String taxCode,
							   String enterpriseName) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);


			// 封装结算参数

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("username", sysUser.getUsername());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("enterpriseName", enterpriseName);
			params.put("idCardType","ID_CARD");
			params.put("subjectType", "LANG_CHAO");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation());
			params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());

			if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
				params.put("taxMode", "GENERAL");
				params.put("transactionModel", "SUPERVISE");
			}
			// 海南飞亿 和 河南薪泰梓
			else {
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
			}

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("浪潮结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("浪潮结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("浪潮结算提交成功： {}", note.getSettleNo());
				//data 获取不到
				if (dataJsonObj == null) {
					log.warn("浪潮结算提交成功 data 获取不到： {}", note.getSettleNo());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
				}
				if (!StrUtil.equals("200", dataJsonObj.getStr("code"))) {
					log.info("浪潮提交成功 code 状态异常");
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
				}
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				String requestId = dataJsonObj.getStr("requestId");
				if (StrUtil.isNotBlank(requestId)) {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
					// 新增字段streamId
					note.setStreamId(requestId);
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				}
			} else {
				log.warn("浪潮结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("浪潮结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setUpdateTime(now);
			note.setSubToGigTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	private R settleBySinbaad(WmScorePackageSettleNote note,
							  WmScorePackage wmScorePackage,
							  SysUser sysUser,
							  String taxCode) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);


			// 封装结算参数

			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("username", sysUser.getUsername());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("idCardType","ID_CARD");
			params.put("subjectType", "ZHENG_QI_ZHI_XING");
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation());
			if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
				params.put("taxMode", "GENERAL");
				params.put("transactionModel", "SUPERVISE");
			}
			// 海南飞亿 和 河南薪泰梓
			else {
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
			}

			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("政企之星结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.info("政企之星结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");

				log.info("政企之星结算提交成功： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				//data 获取不到
				if (dataJsonObj != null) {
					String requestId = dataJsonObj.getStr("requestId");
					String data = dataJsonObj.getStr("data");
					String respMessage = dataJsonObj.getStr("resp_message");
					if (StrUtil.isNotBlank(requestId)) {
						JSONObject dataJson = JSONUtil.parseObj(data);
						String orderStatus = dataJson.getStr("order_status");
						if ("-1".equals(orderStatus) || "2".equals(orderStatus) || "-10".equals(orderStatus)) {
							// 订单失败的状态(-1-创建订单成功，子任务状态校验异常，可做失败处理, 2-创建订单成功待支付，支付状态待确认，订单被驳回, -10-下发至个人失败)
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
							log.warn("政企之星结算提交失败： {}", respMessage);
							noticeInfo.append(sysUser.getRealname()).append("：提交结算失败，请联系管理员；");
						} else {
							note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						}
						if ("2".equals(orderStatus)) {
							//  2-创建订单成功待支付，支付状态待确认，订单被驳回
							//  (一个是订单的支付状态没有成功，还有一种情况就是供应商驳回了你的订单 都是2
							// 订单不是失败的，只是没付款成功  不排除银行极端情况出现，所以你要是把2当成失败的话就10分钟之后再查下状态判断)
							note.setRemark(data);
						}
						// 新增字段streamId
						note.setStreamId(requestId);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						log.warn("政企之星结算提交失败： {}", respMessage);
						noticeInfo.append(sysUser.getRealname()).append("：提交结算失败，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				} else {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					log.warn("政企之星结算提交失败 data 获取不到： {}", note.getSettleNo());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算失败，请联系管理员；");
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
				}
			} else {
				log.warn("政企之星结算提交失败： {}", note.getSettleNo());
				note.setUpdateTime(now);
				note.setSubToGigTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.error("", e);
			log.warn("政企之星结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setUpdateTime(now);
			note.setSubToGigTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}

	private R settleByOlading(WmScorePackageSettleNote note,
							  WmScorePackage wmScorePackage,
							  SysUser sysUser,
							  String taxCode) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */

		HnqzUser operator = SecurityUtils.getUser();
		String gigTokenUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/token?tenantId={tenantId}&action={action}&userId={userId}";
		String gigSettleUrl = upmsConfig.getGigServiceUrl() + "/gig/operation/settlement";

		StringBuilder noticeInfo = new StringBuilder();
		try {
			// 获取请求token
			ResponseEntity<String> tokenResult = new RestTemplate().getForEntity(gigTokenUrl, String.class,
					String.valueOf(operator.getDeptId()), "settle", String.valueOf(operator.getId()));
			log.warn("获取结算token：{}", tokenResult.getBody());
			if (!JSONUtil.isJson(tokenResult.getBody())) {
				return R.failed("获取结算token失败");
			}

			JSONObject jsonObject = JSONUtil.parseObj(tokenResult.getBody());

			if (jsonObject.getInt("code") != 0 || StrUtil.isBlank(jsonObject.getStr("data"))) {
				return R.failed("获取结算token失败");
			}

			String token = jsonObject.getStr("data");

			// 间隔100毫秒
			Thread.sleep(100);


			// 封装结算参数
			Map<String, Object> params = new HashMap<>();
			params.put("token", token);
			params.put("tenantId", operator.getDeptId());
			params.put("operatorId", operator.getId());
			params.put("requestId", RandomStringUtils.randomAlphabetic(12));
			params.put("taskName", wmScorePackage.getScorePackageName() + "_" + System.currentTimeMillis());
			params.put("amount", String.valueOf(note.getSettleAmount().doubleValue()));
			params.put("taskStartTime", wmScorePackage.getCreateTime().toLocalDate());
			params.put("taskEndTime", wmScorePackage.getUpdateTime().toLocalDate());
			params.put("taskLocation", "全国");
			params.put("username", sysUser.getUsername());
			params.put("idCard", sysUser.getIdCardNumber());
			params.put("realName", sysUser.getRealname());
			params.put("bankCard", sysUser.getBankCardNumber());
			params.put("tagId", upmsConfig.getTagId());
			params.put("taskInvoiceContent", note.getCategoryName());
			params.put("endType", "CSO");
			params.put("enterpriseCode", taxCode);
			params.put("subjectType", note.getSubjectLocation().getGigType().name());
			params.put("channel", "DEFAULT");
			params.put("subjectLocation", note.getSubjectLocation());
			params.put("taskDescription", "根据要求，执行、并完成学术推广活动，并以在线平台工具采集、记录活动内容");
			params.put("acceptanceStandard", "完成不同任务获得不同任务积分，根据不同任务给与0~30000积分，每个积分兑换0.8~1.2元，按次数结算");


			if (SubjectLocation.JIN_YUAN.equals(note.getSubjectLocation())) {
				params.put("taxMode", "GENERAL");
				params.put("transactionModel", "SUPERVISE");
			}
			// 海南飞亿 和 河南薪泰梓
			else {
				params.put("taxMode", "SPECIAL");
				params.put("transactionModel", "CHANNEL");
			}


			// 发起结算
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON);
			log.info("阿拉丁结算req： {}", params);
			HttpEntity<String> result = new RestTemplate().exchange(gigSettleUrl, HttpMethod.POST, new HttpEntity<>(params, headers), String.class);
			log.warn("阿拉丁结算resp： {}", result.getBody());
			if (!JSONUtil.isJson(result.getBody())) {
				return R.failed("结算失败");
			}
			JSONObject settelObj = JSONUtil.parseObj(result.getBody());

			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
			if (Objects.equals(settelObj.getInt("code"), CommonConstants.SUCCESS)) {
				JSONObject dataJsonObj = settelObj.getJSONObject("data");
				log.info("自由职家结算提交成功： {}", note.getSettleNo());
				//streamId 获取不到
				if (dataJsonObj == null) {
					note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
					log.warn("自由职家结算提交成功 data 获取不到： {}", note.getSettleNo());
					noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
					redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + ":note_id_" + note.getId());
				} else {
					String requestId = dataJsonObj.getStr("requestId");
					if (StrUtil.isNotBlank(requestId)) {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
						// 新增字段streamId
						note.setStreamId(requestId);
						// 第三方任务id
						note.setThirdTaskId(dataJsonObj.getStr("taskId"));
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
					} else {
						note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
						noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
						redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
					}
				}
			} else {
				log.warn("自由职家结算提交失败： {}", note.getSettleNo());
				note.setSubToGigTime(now);
				note.setUpdateTime(now);
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(settelObj.getStr("msg")).append("；");

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

			}
		} catch (Exception e) {
			log.warn("自由职家结算提交失败： {}", note.getSettleNo());
			log.error("自由职家结算异常", e);
			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败;");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
		}

		this.baseMapper.updateById(note);

		return R.ok(null, noticeInfo.toString());
	}


	private R settleByShuiBangYun() {
		return R.failed("因系统维护，暂时不能提起支付，有问题请联系客服");
	}

	private R settleByShuiBangYun(WmScorePackageSettleNote note,
								  TaxHelperConfig config,
								  WmScorePackage wmScorePackage,
								  SysUser sysUser,
								  String taxCode) {

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		/**
		 * 开始结算
		 */
		TaxHelperSettlementModel taxHelperSettlementModel = new TaxHelperSettlementModel(config);
		taxHelperSettlementModel.setEnterpriseCode(taxCode);

		taxHelperSettlementModel.setExtendFieldNames(Arrays.asList("报告下载地址"));
		taxHelperSettlementModel.setCount(1);

		taxHelperSettlementModel.setNotifyUrl(taxNotifyConfig.getNotifyUrl() + "/admin/taxhelper/" + sysUser.getUserId() + "/v1/settlementNotify");
		TaxHelperSettlementModel.DetailedArr detailedArr = new TaxHelperSettlementModel.DetailedArr();
		detailedArr.setIdCard(sysUser.getIdCardNumber());
		detailedArr.setInputAmount(note.getSettleAmount().doubleValue());
		detailedArr.setCategoryName(note.getCategoryName());
		taxHelperSettlementModel.setDetailedArr(Arrays.asList(detailedArr));

		TaxHelperSettlementModel.AbstractArr abstractArr = new TaxHelperSettlementModel.AbstractArr();

		abstractArr.setIdCard(sysUser.getIdCardNumber());
		abstractArr.setNumber(1);
		abstractArr.setPrice(note.getSettleAmount().doubleValue());
		abstractArr.setAmount(note.getSettleAmount().doubleValue());

		abstractArr.setExtendField1(taxNotifyConfig.getTaskUrl() + "/#/h5/indexTask?id=" + note.getPackageId() + "&userId=" + note
				.getUserId());

		taxHelperSettlementModel.setAbstractArr(Arrays.asList(abstractArr));

		TaxHelperSettlementRequest taxHelperSettlementRequest = new TaxHelperSettlementRequest(taxHelperSettlementModel);

		TaxHelperResponse response = taxHelperService.settlement(taxHelperSettlementRequest);

		log.info("税邦云结算： {}", response);

		StringBuilder noticeInfo = new StringBuilder();
		if (response.success()) {
			log.info("税邦云结算提交成功： {}", note.getSettleNo());
			//streamId 获取不到
			if (StringUtils.isEmpty(response.getMessage())) {
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
				log.info("税邦云结算提交成功 streamId 获取不到： {}", note.getSettleNo());
				noticeInfo.append(sysUser.getRealname()).append("：提交结算成功，业务流水号获取不到，请联系管理员；");
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			} else {
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
				// 新增字段streamId
				note.setStreamId(response.getMessage());
				noticeInfo.append(sysUser.getRealname()).append("：提交结算成功；");
			}
			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
		} else {
			log.info("税邦云结算提交失败： {}", note.getSettleNo());
			LocalDateTime now = LocalDateTime.now();
			note.setSubToGigTime(now);
			note.setUpdateTime(now);
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			noticeInfo.append(sysUser.getRealname()).append("：结算提交失败-").append(response.getMessage()).append("；");

			log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

		}

		this.baseMapper.updateById(note);

		return R.ok(Boolean.TRUE, noticeInfo.toString());
	}

	private R settleByRenLiJia(WmScorePackageSettleNote note, DingConfig config, WmScorePackage wmScorePackage,
							   String subjectName, BigDecimal serviceCharge) {

		SysUser sysUser = sysUserService.getById(note.getUserId());

		/**
		 * 结算失败过的更新 settleNo
		 */
		if (DingEnum.NOTE_STATUS_FAIL.getType().equals(note.getSettleNoteStatus())) {
			String settleNo = UUID.randomUUID().toString().replace("-", "");
			note.setSettleNo(settleNo);
		}

		/**
		 * 加锁
		 */
		redisTemplate.opsForValue().set(CacheConstants.SETTLE_PACKAGE_KEY + note.getId(),
				note.getId(), CacheConstants.SETTLE_TIME, TimeUnit.DAYS);

		List jsdList = new ArrayList();
		Map jsdMap = new HashMap();
		// 服务费率
		BigDecimal _serviceCharge = serviceCharge == null ? BigDecimal.ZERO : serviceCharge;
		// 实际结算金额（结算金额*（1+服务费率））
		BigDecimal actualAmount = note.getSettleAmount().multiply(_serviceCharge.add(BigDecimal.valueOf(100)))
				.divide(BigDecimal.valueOf(100),2, RoundingMode.HALF_EVEN);
		jsdMap.put("amount", actualAmount.toPlainString());
		jsdMap.put("mobile", sysUser.getPhone());
		jsdMap.put("idCard", sysUser.getIdCardNumber());
		jsdMap.put("name", sysUser.getRealname());
		jsdList.add(jsdMap);

		LocalDateTime today = LocalDateTime.now();
		LocalDateTime startTime = today.minusDays(85);
		EmployPayResponse employPayResponse = dingService.payV3(config, subjectName,
				note.getSettleNo(), wmScorePackage.getScorePackageName(),
				note.getInvoiceType(), jsdList, startTime, today);

		LocalDateTime now = LocalDateTime.now();
		note.setSubToGigTime(now);
		note.setUpdateTime(now);

		// 进入结算环节  noticeInfo 失败原因
		StringBuilder noticeInfo = new StringBuilder();
		//提交结算接口失败
		if (!employPayResponse.isSuccess()) {
			log.info("人力家接口调用失败，结算解锁settleNoteId：{}", note.getId());
			redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());
			noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("：提交结算失败").append(employPayResponse.getCode()).append(employPayResponse.getMsg());
			note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());

		} else {
			//结算提交成功数据
			List<EmployPayResultSuccessItem> successResult = employPayResponse.getData().getSuccessResult();
			if (CollectionUtils.isNotEmpty(successResult)) {

				log.info("结算对象-" + sysUser.getRealname() + "：提交结算成功；");
				noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("：提交结算成功；");
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_SUBMIT.getType());
			}

			//结算提交失败的数据
			List<EmployPayResultfailItem> failResult = employPayResponse.getData().getFailResult();
			if (CollectionUtils.isNotEmpty(failResult)) {

				log.info("认证状态异常，结算解锁settleNoteId：{}", note.getId());
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + note.getId());

				log.info("结算对象-" + sysUser.getRealname() + "，结算失败：" + failResult.get(0).getCheckRemark() + "；");
				noticeInfo.append("结算对象-").append(sysUser.getRealname()).append("，结算失败：").append(failResult.get(0).getCheckRemark()).append("；");
				note.setSettleNoteStatus(DingEnum.NOTE_STATUS_FAIL.getType());
			}
		}

		this.baseMapper.updateById(note);

		return R.ok(Boolean.TRUE, noticeInfo.toString());
	}

	@Override
	public R settleNotify(String streamId, Integer settleStatus, Boolean izSingle) {


		log.info("=== 税邦云 回调处理开始 ===");

		WmScorePackageSettleNote query = new WmScorePackageSettleNote();
		query.setStreamId(streamId);
		List<WmScorePackageSettleNote> noteList = this.baseMapper.selectList(Wrappers.query(query));
		WmScorePackageSettleNote tmp = noteList.get(0);
		if (CollectionUtils.isNotEmpty(noteList)) {
			log.info("回调成功，流水号{}", streamId);

			List<WmScorePackageSettleNote> updateList = noteList.stream().map(note -> {
				WmScorePackageSettleNote updateEntity = new WmScorePackageSettleNote();
				updateEntity.setId(note.getId());
				updateEntity.setSettleNoteStatus(settleStatus);
				// 只有结算成功才记录回调时间
				if (settleStatus == 1){
					updateEntity.setNotifyTime(LocalDateTime.now());
				}
				// 失败的用更新时间暂存
				updateEntity.setUpdateTime(LocalDateTime.now());
				return updateEntity;
			}).collect(Collectors.toList());

			this.updateBatchById(updateList);

			WmScorePackage scorePackage = wmScorePackageMapper.selectById(tmp.getPackageId());

			// 遥领不修改
			if (!"1611890566".equals(scorePackage.getSendPackageDeptId())) {
				// 更新积分包状态
				log.info("更新积分包状态:{}", tmp);
				WmScorePackage wmScorePackage = new WmScorePackage();
				wmScorePackage.setId(tmp.getPackageId());
				wmScorePackage.setSettleStatus(settleStatus);
				wmScorePackage.setUpdateTime(LocalDateTime.now());
				wmScorePackageMapper.updateById(wmScorePackage);
			}

			// 如果结算失败删除缓存
			if (settleStatus == 2) {
				redisTemplate.delete(CacheConstants.SETTLE_PACKAGE_KEY + tmp.getId());
			}

			// 结算成功发送到账信息
			if (settleStatus == 1) {

				Set<Integer> ids = noteList.stream().mapToInt(WmScorePackageSettleNote::getId).boxed().collect(Collectors.toSet());

				if (CollUtil.isNotEmpty(ids)) {

					try {
						List<SettleNoteWarrantParams> settleNoteWarrantParams = baseMapper.listSettleNoteWarrantParams(ids);

						settleNoteWarrantParams.stream()
								.filter(params -> StrUtil.isNotBlank(params.getBankCard()))
								.filter(params -> StrUtil.contains(params.getDeptPermissions(), "1"))
								.collect(Collectors.toList())
								.forEach(item -> {
									String bankcard = StrUtil.subSuf(item.getBankCard(), -4);
									try {
										SendSms.sendWarrantSMS(bankcard, item.getSettleAmount(), LocalDateTime.now(), item.getScorePackageName(), item.getUsername());
									} catch (ClientException e) {
										log.error("发送短信失败:{}", item, e);
									}
								});
					} catch (Exception e) {
						log.error("获取短信发送信息失败", e);
					}

				}

			} else {
				log.info("回调成功，流水号异常： {}", streamId);
			}
		}
		log.info("=== 税邦云 回调处理结束 ===");
		return R.ok(noteList);

	}
}