package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.constant.UpmsType;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSubmissionPercentRule;
import com.qunzhixinxi.hnqz.admin.api.entity.WmTaskSubmissionRule;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.*;
import com.qunzhixinxi.hnqz.admin.enums.EnableEnum;
import com.qunzhixinxi.hnqz.admin.enums.LockEnum;
import com.qunzhixinxi.hnqz.admin.mapper.WmUserSignMapper;
import com.qunzhixinxi.hnqz.admin.service.*;
import com.qunzhixinxi.hnqz.common.core.constant.enums.CommonFlag;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.common.sequence.sequence.Sequence;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 用户签到表
 *
 * @author pigx code generator
 * @date 2020-07-14 20:10:02
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmUserSignServiceImpl extends ServiceImpl<WmUserSignMapper, WmUserSign> implements WmUserSignService {

	private final WmTaskContentService wmTaskContentService;
	private final SysUserService sysUserService;
	private final WmTaskTypeService wmTaskTypeService;
	private final WmScorePackageService wmScorePackageService;
	private final WmScorePackageStatusService wmScorePackageStatusService;
	private final WmTaskService wmTaskService;
	private final SysPublicParamService sysPublicParamService;
	private final WmDeptTaskTypeTemplateService wmDeptTaskTypeTemplateService;
	private final WmUserSignDetailService wmUserSignDetailService;
	private final WmTaskSubmissionPercentRuleService wmTaskSubmissionPercentRuleService;
	private final WmTaskSubmissionRuleService wmTaskSubmissionRuleService;
	private final UpmsConfig upmsConfig;
	private final Sequence taskSequence;

	@Override
	public R saveWmUserSign(WmUserSign wmUserSign) {
		if (StringUtils.isEmpty(wmUserSign.getPackageId())) {
			return R.failed("关联积分包异常");
		}
		if (StringUtils.isEmpty(wmUserSign.getSignEntId())) {
			return R.failed("打卡地址异常，请退出后重试");
		}

		if (StringUtils.isEmpty(wmUserSign.getSignEntName())) {
			return R.failed("打卡地址异常，请退出后重试");
		}
		if (StringUtils.isEmpty(wmUserSign.getSignEntType())) {
			wmUserSign.setSignEntType("1");
		}
		WmScorePackage tWmScorePackage = wmScorePackageService.getById(wmUserSign.getPackageId());
		if (StringUtils.isEmpty(tWmScorePackage.getId())) {
			log.warn("获取不到积分包：{}", wmUserSign.getPackageId());
			return R.failed("获取不到积分包");
		}
		if (StringUtils.isEmpty(tWmScorePackage.getRuleId())) {
			log.warn("积分包未配置积分规则：{}", wmUserSign.getPackageId());
			return R.failed("积分包未配置积分规则");
		}
		//TODO 测试通过后删除
		UserVO sysU = sysUserService.selectUserVoById(SecurityUtils.getUser().getId());
		if (!sysU.getUserId().toString().equals(wmUserSign.getSignUserid())) {
			log.warn("客户拜访任务，登录人不符：{}", wmUserSign.getSignUserid());
			return R.failed("客户拜访任务，登录人不符");
		}

		String taskTypeId;
		if ("1".equals(wmUserSign.getSignEntType())) {
			taskTypeId = "5";
		} else if ("2".equals(wmUserSign.getSignEntType())) {
			taskTypeId = "6";
		} else if ("3".equals(wmUserSign.getSignEntType())) {
			taskTypeId = "33";
		} else {
			return R.failed("拜访类型不存在");
		}
		wmUserSign.setTaskTypeId(taskTypeId);

		// 校验超过60周岁,不能做任务
		Map<String, String> checkResult = sysUserService.checkSixtyYearsOld(sysU);
		if (CollUtil.isNotEmpty(checkResult) && checkResult.containsKey("OVER")) {
			throw new RuntimeException(checkResult.get("OVER"));
		}

		// 查询任务类型启用状态
		List<WmTaskSubmissionPercentRule> taskTypeRules = wmTaskSubmissionPercentRuleService.list(Wrappers.<WmTaskSubmissionPercentRule>lambdaQuery()
				.eq(WmTaskSubmissionPercentRule::getDeptId, sysU.getDeptId())
				.eq(WmTaskSubmissionPercentRule::getSubCategory, UpmsType.TaskSubCategory2.TASK_TYPE_CONFIG));
		if (CollUtil.isEmpty(taskTypeRules)) {
			throw new RuntimeException("当前企业未开启该任务！");
		}
		String[] taskTypeIds = taskTypeRules.get(0).getRule().getTaskTypeIds();
		if (!ArrayUtil.contains(taskTypeIds, taskTypeId)) {
			throw new RuntimeException("当前企业未开启该任务！");
		}

		// 校验任务类型是否启用
		WmTaskType _baseTaskType = wmTaskTypeService.getById(taskTypeId);
		WmTaskType queryTaskType = new WmTaskType();
		queryTaskType.setRuleId(tWmScorePackage.getRuleId());
		queryTaskType.setTaskTypeName(_baseTaskType.getTaskTypeName());
		queryTaskType.setDelFlag("0");
		List<WmTaskType> deptTaskTypes = wmTaskTypeService.list(Wrappers.query(queryTaskType));
		if (CollUtil.isEmpty(deptTaskTypes)) {
			throw new RuntimeException(_baseTaskType.getTaskTypeName() + "任务类型不存在");
		}
		if (EnableEnum.DISABLE.val().equals(deptTaskTypes.get(0).getEnableFlag())) {
			throw new RuntimeException(_baseTaskType.getTaskTypeName() + "任务类型已停用");
		}

		if ("1".equals(tWmScorePackage.getPackageType1())) {
			String limitValue = "30000";
			// 查询当前企业配置的积分限制
			List<WmTaskSubmissionRule> scoreLimitRuleList = wmTaskSubmissionRuleService.list(Wrappers.<WmTaskSubmissionRule>lambdaQuery()
					.in(WmTaskSubmissionRule::getDeptId, CollUtil.newHashSet(sysU.getDeptId(), 1))
					.eq(WmTaskSubmissionRule::getOptFlag, CommonFlag.OptFlag.OK)
					.eq(WmTaskSubmissionRule::getSubCategory, UpmsType.TaskSubCategory.SCORE_LIMIT));

			// 公共参数配置的
			String userTaskSubScore = sysPublicParamService.getSysPublicParamKeyToValue("USER_TASK_SUB_SCORE");

			if (CollUtil.isNotEmpty(scoreLimitRuleList)) {
				Map<Integer, WmTaskSubmissionRule.SubmissionRule> submissionRuleMap = scoreLimitRuleList.stream()
						.collect(Collectors.toMap(WmTaskSubmissionRule::getDeptId, rules ->
								BeanUtil.toBean(rules.getRule().get(0), WmTaskSubmissionRule.SubmissionRule.class)));
				WmTaskSubmissionRule.SubmissionRule submissionRule = null;
				if (submissionRuleMap.containsKey(sysU.getDeptId())) {
					submissionRule = submissionRuleMap.get(sysU.getDeptId());
				} else {
					submissionRule = submissionRuleMap.get(1);
				}
				limitValue = String.valueOf(submissionRule.getFrequency().get(0).get("quantity"));
			} else if (StrUtil.isNotBlank(userTaskSubScore)) {
				limitValue = userTaskSubScore;
			}

			Boolean fullFlag = wmTaskService.checkUserTaskSubScore(sysU.getUserId() + "", limitValue);
			if (fullFlag) {
				String info = String.format("每日最多提交%s积分", limitValue);
				log.warn("系统当前限制积分上限：【{}】，提示：【{}】", limitValue, info);
				return R.failed(1, info);
			}
		}

		if ("33".equals(taskTypeId)) {
			// 一个用户，在同一打卡地点，4小时内只能打卡一次
			int total = this.count(Wrappers.<WmUserSign>lambdaQuery()
					.eq(WmUserSign::getSignUserid, wmUserSign.getSignUserid())
					.eq(WmUserSign::getSignEntId, wmUserSign.getSignEntId())
					.gt(WmUserSign::getSignDate, LocalDateTime.now().minus(4, ChronoUnit.HOURS)));
			// 一个用户，半小时内只能打卡一次
			int halfHourTotal = this.count(Wrappers.<WmUserSign>lambdaQuery()
					.eq(WmUserSign::getSignUserid, wmUserSign.getSignUserid())
					.gt(WmUserSign::getSignDate, LocalDateTime.now().minus(30, ChronoUnit.MINUTES)));
			if (total != 0 || halfHourTotal != 0) {
				log.warn("打卡间隔过短：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
				return R.failed("打卡间隔过短");
			}
		} else {

			List<WmTaskSubmissionRule.SubmissionRule> rules = wmTaskContentService.getTaskSubmissionRule(taskTypeId, tWmScorePackage.getRelatedService());

			int dailyLimit = 0;
			Map<String, Integer> eachLimitMap = new HashMap<>(2);

			if (CollUtil.isNotEmpty(rules)) {
				// 获取限制
				for (int i = 0, size = rules.size(); i < size; i++) {
					Map<String, Object> map = (Map<String, Object>) rules.get(i);
					WmTaskSubmissionRule.SubmissionRule r = BeanUtil.mapToBean(map, WmTaskSubmissionRule.SubmissionRule.class, true, new CopyOptions());

					if ("拜访次数上限".equals(r.getManual())) {
						Object o = r.getFrequency().get(0).get("quantity");
						dailyLimit = o instanceof String ? Integer.parseInt((String) o) : (int) o;

					}
					if (r.getManual().startsWith("同一个人")) {

						r.getFrequency().forEach(f -> {

							// 获取数量
							Object o = f.get("quantity");
							int limit = o instanceof String ? Integer.parseInt((String) o) : (int) o;

							// 获取单位
							String timeUnit = (String) f.get("timeUnit");

							// 同一个单位只记录最小的
							if (eachLimitMap.containsKey(timeUnit)) {
								Integer l1 = eachLimitMap.get(timeUnit);
								if (l1 != null && l1 > limit) {
									eachLimitMap.put(timeUnit, limit);
								}
							} else {
								eachLimitMap.put(timeUnit, limit);
							}

						});

					}
				}
			}

			// 按照最大周期进行查询
			LocalDateTime now = LocalDateTime.now();
			LocalDateTime start;
			boolean isMonth = false;
			if (eachLimitMap.containsKey("MONTH")) {
				start = now.minusDays(30L);
				isMonth = true;
			} else {
				start = LocalDateTime.of(now.toLocalDate(), LocalTime.MIN);
			}

			List<WmUserSign> signs = this.list(Wrappers.<WmUserSign>lambdaQuery()
					.eq(WmUserSign::getSignUserid, wmUserSign.getSignUserid())
					.eq(WmUserSign::getTaskTypeId, taskTypeId)
					.between(WmUserSign::getSignDate, start, now));


			// 月度限制
			if (isMonth) {

				Map<String, List<WmUserSign>> collect = signs.stream().collect(Collectors.groupingBy(WmUserSign::getSignEntId));
				List<WmUserSign> wmUserSigns = collect.get(wmUserSign.getSignEntId());
				if (CollUtil.isNotEmpty(wmUserSigns)) {
					int monthLimit = eachLimitMap.get("MONTH");
					if (monthLimit <= wmUserSigns.size()) {
						log.warn("超过打卡每月限制：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
						return R.failed("超过打卡每月限制");
					}
				}
			}

			// 获取当日的打卡记录
			LocalDateTime begin = LocalDateTime.of(now.toLocalDate(), LocalTime.MIN);
			List<WmUserSign> todaySign = signs.stream().filter(s -> s.getSignDate().isAfter(begin)).collect(Collectors.toList());

			// 没有超过日打卡限制
			int todaySignQty = CollUtil.isNotEmpty(todaySign) ? todaySign.size() : 0;
			if (dailyLimit > todaySignQty) {

				Map<String, List<WmUserSign>> collect = todaySign.stream().collect(Collectors.groupingBy(WmUserSign::getSignEntId));
				List<WmUserSign> wmUserSigns = collect.get(wmUserSign.getSignEntId());
				if (CollUtil.isNotEmpty(wmUserSigns)) {
					int eachLimit = eachLimitMap.get("DAY");
					if (wmUserSigns.size() < eachLimit) {

						// 一个用户，在同一打卡地点，4小时内只能打卡一次
						Optional<LocalDateTime> first = wmUserSigns.stream().map(WmUserSign::getSignDate).filter(sd -> Duration.between(sd, now).toMinutes() <= upmsConfig.getDurationLimit()).findFirst();

						if (first.isPresent()) {
							log.warn("打卡间隔过短：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
							return R.failed("打卡间隔过短");
						}

						// 一个用户，半小时内只能打卡一次
						Optional<LocalDateTime> first1 = signs.stream().map(WmUserSign::getSignDate).filter(sd -> Duration.between(sd, now).toMinutes() <= upmsConfig.getEachDurationLimit()).findFirst();

						if (first1.isPresent()) {
							log.warn("打卡间隔过短：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
							return R.failed("打卡间隔过短");
						}


					} else {
						log.warn("超过每个地点打卡每日限制：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
						return R.failed("超过每个地点打卡每日限制");
					}
				}

			} else {
				log.warn("超过打卡每日总限制：{} {}", wmUserSign.getSignUserid(), wmUserSign.getSignEntName());
				return R.failed("超过打卡每日总限制");
			}

		}

		// 校验任务百分比限制规则（如果积分包值大于【任务配置】-【百分比限制】配置的起始值，
		// 且【任务配置】-【百分比限制】所选的任务类型对应的任务总完成积分值>积分包值*【任务配置】-【百分比限制】配置的百分比，则校验不通过）
		R<?> checkPercentRuleResult =
				wmTaskSubmissionPercentRuleService.checkPercentRule(tWmScorePackage, SecurityUtils.getUser().getDeptId(), null, Arrays.asList("5", "6", "33"));
		if (checkPercentRuleResult.getCode() != 0) {
			throw new RuntimeException(checkPercentRuleResult.getMsg());
		}

		/**
		 * 积分包领取明细
		 */
		WmScorePackageStatus query = new WmScorePackageStatus();
		query.setUserId(sysU.getUserId() + "");
		query.setStatus("2");
		query.setTaskAddFlag("1");
		query.setPackageId(tWmScorePackage.getId());
		List<WmScorePackageStatus> statusList = wmScorePackageStatusService.list(Wrappers.lambdaQuery(query));
		if (CollectionUtils.isEmpty(statusList)) {
			log.warn("获取积分包异常，任务保存失败9001：{}", tWmScorePackage.getId());
			return R.failed("获取积分包异常，任务保存失败");
		}
		if (statusList.size() != 1) {
			log.warn("获取积分包异常，任务保存失败9002：{}", tWmScorePackage.getId());
			return R.failed("获取积分包异常，任务保存失败");
		}
		WmScorePackageStatus scorePackageStatus = statusList.get(0);

		boolean saveSign = this.save(wmUserSign);

		if (saveSign) {

			WmTask wmTask = new WmTask();
			wmTask.setTaskFrom("1");
			wmTask.setTaskUserId(sysU.getUserId() + "");    //任务提交人
			if ("1".equals(wmUserSign.getSignEntType())) {
				wmTask.setTaskTypeId("5");                           //任务类型
			} else if ("2".equals(wmUserSign.getSignEntType())) {
				wmTask.setTaskTypeId("6");                           //任务类型
			} else if ("3".equals(wmUserSign.getSignEntType())) {
				wmTask.setTaskTypeId("33");
			}
			wmTask.setSubmitStatus("0");
			LocalDateTime localDateTime = LocalDateTime.now();
			wmTask.setUpdateTime(localDateTime);
			wmTask.setLookintoDate(localDateTime);       //调研时间
			wmTask.setCreateTime(localDateTime);
			wmTask.setTaskStatus("2"); //直接提交
			wmTask.setDrugEntId(sysU.getDrugEntId());

			wmTask.setDeptId(sysU.getDeptId() + "");

			WmTaskType wmTaskType = new WmTaskType();

			if ("1".equals(wmUserSign.getSignEntType())) {
				WmTaskType wmTaskType1 = wmTaskTypeService.getById(5);
				wmTaskType.setTaskTypeName(wmTaskType1.getTaskTypeName());
			} else if ("2".equals(wmUserSign.getSignEntType())) {
				WmTaskType wmTaskType1 = wmTaskTypeService.getById(6);
				wmTaskType.setTaskTypeName(wmTaskType1.getTaskTypeName());
			} else if ("3".equals(wmUserSign.getSignEntType())) {
				WmTaskType wmTaskType1 = wmTaskTypeService.getById(33);
				wmTaskType.setTaskTypeName(wmTaskType1.getTaskTypeName());
			}

			wmTaskType.setRuleId(tWmScorePackage.getRuleId());
			int score = wmTaskTypeService.getWmTaskTypeByEnt(wmTaskType);
			if (score < 0) {
				return R.failed("获取任务积分失败");
			}
			wmTask.setScore(score);

			wmTask.setScorePackageId(tWmScorePackage.getId());
			wmTask.setTaskRuleId(tWmScorePackage.getTaskRuleId());

			WmTaskContent wmTaskContent = new WmTaskContent();
			wmTaskContent.setTemp1(wmUserSign.getLatitude());
			wmTaskContent.setTemp2(wmUserSign.getLongitude());
			wmTaskContent.setTemp3(wmUserSign.getId() + "");
			wmTaskContentService.save(wmTaskContent);

			wmTask.setTaskContentId(wmTaskContent.getId() + "");
			wmTask.setTaskNumber(taskSequence.nextNo());
			//保存任务
			wmTaskService.save(wmTask);

			/**
			 * 判断任务完成度
			 */
			WmScorePackage updatePke = wmScorePackageService.getById(tWmScorePackage.getId());
			if (null != updatePke) {
				/**
				 * 查询出 审核通过和审核中的任务
				 */
				WmTask taskQuery = new WmTask();
				taskQuery.setScorePackageId(tWmScorePackage.getId());
				QueryWrapper<WmTask> taskQueryWrapper = Wrappers.query(taskQuery);
				taskQueryWrapper.lambda()
						.ne(WmTask::getTaskStatus, "4")
						.eq(WmTask::getRealFlag, "0");
//						.ne(WmTask::getRealFlag, "1");
				List<WmTask> wmTaskList = wmTaskService.list(taskQueryWrapper);
				/**
				 * 判断定量还是定额 1- 定额   2-定量
				 * 20210120 目前 1-定额  1包1人  2-定量   1包多人
				 */

				if ("1".equals(updatePke.getPackageType1())) {

					int totalCount = 0;
					for (WmTask wmtask : wmTaskList) {
						totalCount += wmtask.getScore();
					}

					if (totalCount >= updatePke.getScore()) {
						updatePke.setTaskAddFlag("0");
						scorePackageStatus.setTaskAddFlag("0");
						wmScorePackageStatusService.updateById(scorePackageStatus);
					}
					wmTask.setType("1");

				} else if ("2".equals(updatePke.getPackageType1())) {

					List<WmTask> userTaskList = wmTaskList.stream().filter(item ->
							item.getTaskUserId().equals(sysU.getUserId() + "")).collect(Collectors.toList());

					if (userTaskList.size() >= scorePackageStatus.getTaskNum()) {
						scorePackageStatus.setTaskAddFlag("0");
						wmScorePackageStatusService.updateById(scorePackageStatus);

						/**
						 * 改变大包状态
						 */
						WmScorePackageStatus queryAll = new WmScorePackageStatus();
						queryAll.setPackageId(tWmScorePackage.getId());
						queryAll.setStatus("2");
						queryAll.setTaskAddFlag("0");
						List<WmScorePackageStatus> statusAll = wmScorePackageStatusService.
								list(Wrappers.query(queryAll));
						if (new BigDecimal(updatePke.getUserNum())
								.compareTo(new BigDecimal(statusAll.size())) <= 0) {
							updatePke.setTaskAddFlag("0");
						}
					}

//					if (wmTaskList.size() >= updatePke.getTaskNum()) {
//						updatePke.setTaskAddFlag("0");
//					}
					wmTask.setType("2");
				}
				updatePke.setIsConduct("1");
				wmScorePackageService.updateById(updatePke);

				// 如果配置了医院拜访的任务类型模板，则关联保存签到详情
				if (wmDeptTaskTypeTemplateService.checkDeptIdHospitalTemplate(sysU.getDeptId(), wmTask.getTaskTypeId())) {
					WmUserSignDetail userSignDetail = wmUserSign.getUserSignDetail();
					if (userSignDetail != null) {
						userSignDetail.setSignId(wmUserSign.getId());
						wmUserSignDetailService.save(userSignDetail);
					}
				}
				return R.ok(wmTask);
			}
		}

		return R.failed("签到保存失败");
	}


}
