/*
 *    Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the pig4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: hnqz
 */

package com.qunzhixinxi.hnqz.daemon.quartz.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.daemon.quartz.entity.WmTaskType;
import com.qunzhixinxi.hnqz.daemon.quartz.service.WmTaskTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;


/**
 * 任务类型打分表
 *
 * @author gaoyang
 * @date 2020-07-01 20:20:05
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wmtasktype" )
@Api(value = "wmtasktype", tags = "任务类型打分表管理")
public class WmTaskTypeController {

    private final WmTaskTypeService wmTaskTypeService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param wmTaskType 任务类型打分表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    public R getWmTaskTypePage(Page page, WmTaskType wmTaskType) {
        return R.ok(wmTaskTypeService.page(page, Wrappers.query(wmTaskType)));
    }


    /**
     * 通过id查询任务类型打分表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}" )
    public R getById(@PathVariable("id" ) Integer id) {
        return R.ok(wmTaskTypeService.getById(id));
    }

    /**
     * 新增任务类型打分表
     * @param wmTaskType 任务类型打分表
     * @return R
     */
    @ApiOperation(value = "新增任务类型打分表", notes = "新增任务类型打分表")
    @SysLog("新增任务类型打分表" )
    @PostMapping
    public R save(@RequestBody WmTaskType wmTaskType) {
		wmTaskType.setCreateTime(LocalDateTime.now());
		if(SecurityUtils.getRoles().contains(2))
		{
			wmTaskType.setStatus("0");
		}else{
			wmTaskType.setStatus("1");
		}
    	return R.ok(wmTaskTypeService.save(wmTaskType));
    }

    /**
     * 修改任务类型打分表
     * @param wmTaskType 任务类型打分表
     * @return R
     */
    @ApiOperation(value = "修改任务类型打分表", notes = "修改任务类型打分表")
    @SysLog("修改任务类型打分表" )
    @PutMapping
    public R updateById(@RequestBody WmTaskType wmTaskType) {
		if(SecurityUtils.getRoles().contains(2))
		{
			WmTaskType wmTaskRule1=wmTaskType;
			wmTaskRule1.setId(null);
			wmTaskRule1.setStatus("1");
			wmTaskTypeService.save(wmTaskRule1);
		}else{
			wmTaskTypeService.updateById(wmTaskType);
		}
    	return R.ok();
    }

    /**
     * 通过id删除任务类型打分表
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除任务类型打分表", notes = "通过id删除任务类型打分表")
    @SysLog("通过id删除任务类型打分表" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('admin_wmtasktype_del')" )
    public R removeById(@PathVariable Integer id) {
        return R.ok(wmTaskTypeService.removeById(id));
    }

	/**
	 * 获取药企列表
	 * @return 角色列表
	 */
	@GetMapping("/list")
	public R listWmScorePackage() {
		WmTaskType wmDaDrugEnt=new WmTaskType();
		wmDaDrugEnt.setDelFlag("0");
		wmDaDrugEnt.setEnableFlag("0");
		wmDaDrugEnt.setTaskTypeLevel("1");
		return R.ok(wmTaskTypeService.list(Wrappers.query(wmDaDrugEnt)));
	}
}
